# ------------------------------------------------------------------------------------------------------
#  Script to compile a CorSight2 OpenCamera FPGA design
#  Copyright (C) 2017 NET GmbH. All Rights Reserved
# ------------------------------------------------------------------------------------------------------

# ---------------------------------------------------------------------------------
# Procedure to display message on console and in log file
# ---------------------------------------------------------------------------------
proc LogMsg {LogFname MsgStr} {
    puts "$MsgStr"

    # Add message to log file
    set LogFileId [open $LogFname "a+"]
    puts -nonewline $LogFileId $MsgStr
    close $LogFileId
}

set LogTxt   ""
set ErrorTxt ""
set ErrorCnt 0

# ------------------------------------------------------------------------------------------------------
#  Set the OpenCamera FPGA compile parameters
# ------------------------------------------------------------------------------------------------------
# Define Cs2OpenCamCfg default values
set Cs2OpenCamCfg(OpenCamDir)  "Undefined"
set Cs2OpenCamCfg(CompileDir)  "Undefined"
set Cs2OpenCamCfg(ReleaseDir)  "Undefined"

# Read customer defined environment variables
if {[file exists Set_Customer_EnvVar.tcl] == 1} {
    source Set_Customer_EnvVar.tcl
} else {
    append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - OpenCamera environment file \"Set_Customer_EnvVar.tcl\" not found - Compile aborted\n\n"
    incr ErrorCnt
}

# Attempt to set local compile variables from the predefined computer environment script.
if {[file exists Set_PreDef_EnvVar.tcl] == 1 && [file isdirectory $Cs2OpenCamCfg(OpenCamDir)] == 0} {
    source Set_PreDef_EnvVar.tcl
}

# If the OpenCamera source directory still can not be found print an error message and abort the compilation.
if {[file isdirectory $Cs2OpenCamCfg(OpenCamDir)] == 0} {
    append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - OpenCamera source file directory \"$Cs2OpenCamCfg(OpenCamDir)\" not found - Compile aborted\n\n"
    incr ErrorCnt
}

if {$SCRIPT_DEBUG == "ON"} {
    puts "\nCorSight2 OpenCamera local compile environment variables:"
    puts "OpenCamDir  = $Cs2OpenCamCfg(OpenCamDir)"
    puts "CompileDir  = $Cs2OpenCamCfg(CompileDir)"
    puts "ReleaseDir  = $Cs2OpenCamCfg(ReleaseDir)"
}

# ------------------------------------------------------------------------------------------------------
# Set the OpenCamera project configuration variables
# ------------------------------------------------------------------------------------------------------
if {[file exists $Cs2OpenCamCfg(OpenCamDir)/CompileTools/Set_OpenCam_Config.tcl] == 1} {
    source $Cs2OpenCamCfg(OpenCamDir)/CompileTools/Set_OpenCam_Config.tcl
} else {
    append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - OpenCamera source file directory \" $Cs2OpenCamCfg(OpenCamDir)/CompileTools/Set_OpenCam_Config.tcl\" not found - Compile aborted\n\n"
    incr ErrorCnt
}

if {$BOARD_REV == "REV01" && $CS2_REV1_OPENCAM_ENABLE == "ENABLE"} {
    set PROJ_BASE_NAME "CS2_Rev1"
    set BOARD_REV_ID "R1"
} elseif {$BOARD_REV == "REV02" && $CS2_REV2_OPENCAM_ENABLE == "ENABLE"} {
    set PROJ_BASE_NAME "CS2_Rev2"
    set BOARD_REV_ID "R2"
} elseif {$BOARD_REV == "REV03" && $CS2_REV3_OPENCAM_ENABLE == "ENABLE"} {
    set PROJ_BASE_NAME "CS2_Rev3"
    set BOARD_REV_ID "R3"
} else {
    append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - CorSight2 Board Revision not supported in OpenCamera compile environment: $BOARD_REV - Compile aborted\n\n"
    incr ErrorCnt
}

# ------------------------------------------------------------------------------------------------------
# Set the OpenCamera project initialization variables
# ------------------------------------------------------------------------------------------------------
source $Cs2OpenCamCfg(OpenCamDir)/syn/InitFiles/CS2\_$FPGA_CUST_NAME\_Init.tcl

# Set undefined variables in selected example designs
if {$CUSTOM_COMPILE_MODE == "RELEASE" || $CUSTOM_DESIGN == "DEFAULT" || $CUSTOM_DESIGN == "DIFFPIC" || $CUSTOM_DESIGN == "CANNY" || $CUSTOM_DESIGN == "BLOCKDEMO" || $CUSTOM_DESIGN == "SCALED"} {
    set CUSTOM_DEBUG_MODULE  "NONE"
    set CUSTOM_DEBUG_DEVICE  "NONE"
}

# ------------------------------------------------------------------------------------------------------
# Setup the top-level compile directory
# ------------------------------------------------------------------------------------------------------
# Check if the top-level compile directory exists. If not, create it now.
if {[file isdirectory $Cs2OpenCamCfg(CompileDir)] == 0} {
    file mkdir $Cs2OpenCamCfg(CompileDir)

    # If the top-level compile directory still does not exist abort compilation.
    if {[file isdirectory $Cs2OpenCamCfg(CompileDir)] == 0} {
        append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - Can not create top-level compile directory \"$Cs2OpenCamCfg(CompileDir)\" - Compile aborted\n\n"
        incr ErrorCnt
    }
}

# ------------------------------------------------------------------------------------------------------
# Set the LogFile path and name and start logging
# ------------------------------------------------------------------------------------------------------
set LogFname "$Cs2OpenCamCfg(CompileDir)/$PROJ_BASE_NAME\_CompileLog.txt"

# Write the Header to the LogFile if the file doesn't already exists
if {![file exists $LogFname]} {
    set    LogHeader "-------------------------------------------------------------------------------\n"
    append LogHeader "-- Project    : CorSight2 - OpenCamera Compile Log\n"
    append LogHeader "-- Copyright (c) 2017 NET New Electronic Technology GmbH\n"
    append LogHeader "-------------------------------------------------------------------------------\n\n"

    set LogFileId [open $LogFname "w"]
    puts -nonewline $LogFileId $LogHeader
    close $LogFileId
}

# Display and log the compile options at start of compilation
set SysTime [clock seconds]
append LogTxt "Compiling $BOARD_NAME OpenCamera FPGA on [clock format $SysTime -format %d/%m/%Y] at [clock format $SysTime -format %H:%M:%S]:\n"
append LogTxt "    BOARD_REVISION      = $BOARD_REV\n"
append LogTxt "    BOARD_DESCRIPTOR    = $OpenCamConfig(BOARD_DESCRIPTOR)\n"
append LogTxt "    DDR3_DEVICE_ID      = $OpenCamConfig(DDR3_DEVICE_ID)\n\n"

append LogTxt "    FPGA_DEVICE_ID      = $OpenCamConfig(FPGA_DEVICE_ID)\n"
append LogTxt "    FPGA_PACKAGE_ID     = $OpenCamConfig(FPGA_PACKAGE_ID)\n"
append LogTxt "    FPGA_SPEED_GRADE    = $OpenCamConfig(FPGA_SPEED_GRADE)\n"
append LogTxt "    FPGA_MAJOR_REV      = $OpenCamConfig(FPGA_MAJOR_REV)\n"
append LogTxt "    FPGA_MINOR_REV      = $OpenCamConfig(FPGA_MINOR_REV)\n"
append LogTxt "    FPGA_DESIGN_ID      = $FPGA_DESIGN_ID\n\n"

append LogTxt "    CUSTOM_REV_NUM      = $CUSTOM_REV_NUM\n"
append LogTxt "    CUSTOM_DESIGN       = $CUSTOM_DESIGN\n"
append LogTxt "    CUSTOM_COMPILE_MODE = $CUSTOM_COMPILE_MODE\n"
append LogTxt "    CUSTOM_DEBUG_MODULE = $CUSTOM_DEBUG_MODULE\n"
append LogTxt "    CUSTOM_DEBUG_DEVICE = $CUSTOM_DEBUG_DEVICE\n"
append LogTxt "    VIVADO_MODE         = $VIVADO_MODE\n\n"

# Check for a valid CUSTOM_DEBUG_MODULE parameter
if {$CUSTOM_DEBUG_MODULE != "NONE" && $CUSTOM_DEBUG_MODULE != "CHIPSCOPE" && $CUSTOM_DEBUG_MODULE != "ANALYZER"} {
    append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - Invalid Debug Module parameter detected: $CUSTOM_DEBUG_MODULE - Compile aborted\n"
    append ErrorTxt "                             Supported Debug Module options are: \[NONE, CHIPSCOPE, ANALYZER\]\n\n"
    incr ErrorCnt
}

# Check for a valid CUSTOM_DEBUG_DEVICE parameter
if {$CUSTOM_DEBUG_MODULE != "NONE" && $CUSTOM_DEBUG_DEVICE != "ILA1x128x1024" && $CUSTOM_DEBUG_DEVICE != "ILA1x256x1024"} {
    append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - Invalid Debug Device parameter detected: $CUSTOM_DEBUG_DEVICE - Compile aborted\n"
    append ErrorTxt "                             Supported Debug Device options are: \[ILA1x128x1024, ILA1x256x1024\]\n\n"
    incr ErrorCnt
}

# Check for a valid VIVADO_MODE parameter
if {$VIVADO_MODE != "OPEN" && $VIVADO_MODE != "CLOSE"} {
    append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - Invalid Vivado Exit parameter detected: $VIVADO_MODE - Compile aborted\n"
    append ErrorTxt "                             Supported Vivado Exit options are: \[HOLD, CLOSE\]\n\n"
    incr ErrorCnt
}

# Append error messages (if any) to the log file message text and display the message.
# If errors have been detected stop compilation execution.
append LogTxt $ErrorTxt
LogMsg $LogFname $LogTxt
set LogTxt ""

if {$ErrorCnt > 0} {
    return error
}

# ------------------------------------------------------------------------------------------------------
#  Set the OpenCamera FPGA compile and project names
# ------------------------------------------------------------------------------------------------------
# Set the FPGA part name
if {$OpenCamConfig(FPGA_DEVICE_ID) == "XC7A75T"} {
    set FPGA_PART_NAME "A7-75"
} elseif {$OpenCamConfig(FPGA_DEVICE_ID) == "XC7A100T"} {
    set FPGA_PART_NAME "A7-100"
} elseif {$OpenCamConfig(FPGA_DEVICE_ID) == "XC7A200T"} {
    set FPGA_PART_NAME "A7-200"
} else {
    append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - Invalid OpenCamera compile parameter FPGA_DEVICE_ID: $OpenCamConfig(FPGA_DEVICE_ID) - Compile aborted\n\n"
    incr ErrorCnt
}

# ------------------------------------------------------------------------------------------------------
#  Set the FPGA compile, firmware and release names
# ------------------------------------------------------------------------------------------------------
# OpenCamera firmware name example: F114R1_A7-75-2_CFG00_DSG00_DEFAULT_OPC_REL_VER01-05-02
set OpenCompName "$OpenCamConfig(BOARD_DESCRIPTOR)$BOARD_REV_ID\_$FPGA_PART_NAME$OpenCamConfig(FPGA_SPEED_GRADE)\_$OpenCamConfig(BOARD_CONFIGURATION)\_$FPGA_DESIGN_ID\_$CUSTOM_DESIGN\_OPC\_$CUSTOM_COMPILE_NAME"
set OpenFwName   "$OpenCompName\_VER$OpenCamConfig(FPGA_MAJOR_REV)-$OpenCamConfig(FPGA_MINOR_REV)-$CUSTOM_REV_NUM"

# Example for OpenFwParent: CS2_REV01_OPC_VER01-03
set OpenFwParent "CS2\_$BOARD_REV\_OPC\_VER$OpenCamConfig(FPGA_MAJOR_REV)-$OpenCamConfig(FPGA_MINOR_REV)"

# Example for OpenFwDir: LUTDESIGN_DSG02_DBG
set OpenFwDir "$CUSTOM_DESIGN\_$FPGA_DESIGN_ID\_$CUSTOM_COMPILE_NAME"

if {$SCRIPT_DEBUG == "ON"} {
    puts "\nCorSight2 OpenCamera firmware release name    = $OpenFwName"
    puts "CorSight2 OpenCamera release parent directory   = $OpenFwParent"
    puts "CorSight2 OpenCamera release firmware directory = $OpenFwDir"
}

# ------------------------------------------------------------------------------------------------------
#  Set the FPGA compile environment variables
# ------------------------------------------------------------------------------------------------------
set TCL_SOURCE_DIR    $Cs2OpenCamCfg(OpenCamDir)
set CS2_COMPILE_DIR   $Cs2OpenCamCfg(CompileDir)/$OpenFwParent/$OpenFwDir
set CS2_RELEASE_DIR   $Cs2OpenCamCfg(ReleaseDir)/$OpenFwParent/$OpenFwDir
set CS2_RELEASE_FNAME $Cs2OpenCamCfg(ReleaseDir)/$OpenFwParent/$OpenFwDir/$OpenFwName.bin
set CS2_BIN_FNAME     "$PROJ_BASE_NAME.runs/impl_1/Cs2TopModule.bin"
set FPGA_PART_NO      $OpenCamConfig(FPGA_DEVICE_ID)$OpenCamConfig(FPGA_PACKAGE_ID)$OpenCamConfig(FPGA_SPEED_GRADE)

if {$SCRIPT_DEBUG == "ON"} {
    puts "\nCorSight2 OpenCamera environment variables:"
    puts "TCL_SOURCE_DIR  = $TCL_SOURCE_DIR"
    puts "CS2_COMPILE_DIR = $CS2_COMPILE_DIR"
    puts "CS2_BIN_FNAME   = $CS2_BIN_FNAME"
    puts "FPGA_PART_NO    = $FPGA_PART_NO"
}

# ------------------------------------------------------------------------------------------------------
#  Generate the Custom Module Version Number Package file
# ------------------------------------------------------------------------------------------------------
if {$CUSTOM_DESIGN == "USERDESIGN" || $CUSTOM_DESIGN == "LUTDESIGN" || $CUSTOM_DESIGN == "MEMTEST"} {
    if {$CUSTOM_DESIGN == "USERDESIGN"} {
        set VerPkgName "UserVerPkg"
        set VerDirName "UserDesign"
    } elseif {$CUSTOM_DESIGN == "LUTDESIGN"} {
        set VerPkgName "LutVerPkg"
        set VerDirName "LutDesign"
    } elseif {$CUSTOM_DESIGN == "MEMTEST"} {
        set VerPkgName "MemTestVerPkg"
        set VerDirName "MemTest"
    }

    set    OpenVerTxt "-------------------------------------------------------------------------------\n"
    append OpenVerTxt "-- Project    : CorSight2 - Custom Module\n"
    append OpenVerTxt "-- Title      : Version Package for OpenCamera $VerDirName\n"
    append OpenVerTxt "-------------------------------------------------------------------------------\n"
    append OpenVerTxt "-- Copyright (c) 2017 NET New Electronic Technology GmbH\n"
    append OpenVerTxt "-------------------------------------------------------------------------------\n"
    append OpenVerTxt "-- NOTE: This File is generated automatically - DO NOT MODIFY\n"
    append OpenVerTxt "-------------------------------------------------------------------------------\n"
    append OpenVerTxt "\n"
    append OpenVerTxt "library ieee;\n"
    append OpenVerTxt "use ieee.std_logic_1164.all;\n"
    append OpenVerTxt "\n"
    append OpenVerTxt "package $VerPkgName is\n"
    append OpenVerTxt "    constant cCUSTOM_VERSION_NUM      : integer := $CUSTOM_REV_NUM;          -- Set Custom Module version number\n"

    if {$CUSTOM_COMPILE_MODE == "DEBUG"} {
        append OpenVerTxt "    constant cCUSTOM_DEBUG_ENABLE     : boolean := True;        -- Enable/disable Custom Module debug mode\n"

        if {$CUSTOM_DEBUG_MODULE == "ANALYZER"} {
            append OpenVerTxt "    constant cCUSTOM_ANALYZER_ENABLE  : boolean := True;        -- Enable/disable Vivado Logic Analyzer\n"
        } else {
            append OpenVerTxt "    constant cCUSTOM_ANALYZER_ENABLE  : boolean := False;       -- Enable/disable Vivado Logic Analyzer\n"
        }

        if {$CUSTOM_DEBUG_MODULE == "CHIPSCOPE"} {
            append OpenVerTxt "    constant cCUSTOM_CHIPSCOPE_ENABLE : boolean := True;        -- Enable/disable ChipScopePro Analyzer\n"
            append OpenVerTxt "    constant cCUSTOM_CSP_ICON_ENABLE  : boolean := True;        -- Enable/disable ICON module (JTAG mode only)\n"
        } else {
            append OpenVerTxt "    constant cCUSTOM_CHIPSCOPE_ENABLE : boolean := False;       -- Enable/disable ChipScopePro Analyzer\n"
            append OpenVerTxt "    constant cCUSTOM_CSP_ICON_ENABLE  : boolean := False;       -- Enable/disable ICON module (JTAG mode only)\n"
        }
    } else {
        append OpenVerTxt "    constant cCUSTOM_DEBUG_ENABLE     : boolean := False;       -- Enable/disable Custom Module debug mode\n"
        append OpenVerTxt "    constant cCUSTOM_ANALYZER_ENABLE  : boolean := False;       -- Enable/disable Vivado Logic Analyzer\n"
        append OpenVerTxt "    constant cCUSTOM_CHIPSCOPE_ENABLE : boolean := False;       -- Enable/disable ChipScopePro Analyzer\n"
        append OpenVerTxt "    constant cCUSTOM_CSP_ICON_ENABLE  : boolean := False;       -- Enable/disable ICON module (JTAG mode only)\n"
    }

    if {$CUSTOM_DEBUG_DEVICE == "ILA1x128x1024"} {
        append OpenVerTxt "    constant cCUSTOM_DEBUG_PORT_WIDTH : integer := 128;         -- Select the Custom Module debug port width\n"
    } else {
        append OpenVerTxt "    constant cCUSTOM_DEBUG_PORT_WIDTH : integer := 256;         -- Select the Custom Module debug port width\n"
    }

    append OpenVerTxt "end package $VerPkgName;\n"

    set OpenVerFname "$Cs2OpenCamCfg(OpenCamDir)/src/$VerDirName/$VerPkgName.vhd"
    set OpenVerFileId [open $OpenVerFname "w"]
    puts -nonewline $OpenVerFileId $OpenVerTxt
    close $OpenVerFileId
}

# ------------------------------------------------------------------------------------------------------
# Bootstrap the Vivado compile environment
# ------------------------------------------------------------------------------------------------------
# Check if the FPGA compile directory exists. If it does, delete it now.
if {[file isdirectory $CS2_COMPILE_DIR] == 1} {
    file delete -force $CS2_COMPILE_DIR

    # If the FPGA compile directory still exist abort compilation.
    if {[file isdirectory $CS2_COMPILE_DIR] == 1} {
        append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - Can not delete FPGA compile directory \"$CS2_COMPILE_DIR\" - Compile aborted\n\n"
        incr ErrorCnt
    }
}

# Create FPGA compile directory and check if it exists.
file mkdir $CS2_COMPILE_DIR
if {[file isdirectory $CS2_COMPILE_DIR] == 0} {
    append ErrorTxt "\[Compile_OpenCam_FPGA.tcl\] ERROR - Can not create FPGA compile directory \"$CS2_COMPILE_DIR\" - Compile aborted\n\n"
    incr ErrorCnt
}

# ------------------------------------------------------------------------------------------------------
# Copy FPGA compile script and project file to FPGA compile directory.
# ------------------------------------------------------------------------------------------------------
file copy -force "$Cs2OpenCamCfg(OpenCamDir)/syn/ScriptFiles/$PROJ_BASE_NAME\_FpgaScript.tcl"                                     "$CS2_COMPILE_DIR/FpgaScript.tcl"
file copy -force "$Cs2OpenCamCfg(OpenCamDir)/syn/ProjectFiles/$PROJ_BASE_NAME/$PROJ_BASE_NAME\_$FPGA_DSG_NAME\_FpgaProject.tcl"   "$CS2_COMPILE_DIR/FpgaProject.tcl"
file copy -force "$Cs2OpenCamCfg(OpenCamDir)/syn/ProjectFiles/$PROJ_BASE_NAME/$PROJ_BASE_NAME\_$FPGA_CUST_NAME\_FpgaProject.tcl"  "$CS2_COMPILE_DIR/CustomProject.tcl"
file copy -force "$Cs2OpenCamCfg(OpenCamDir)/syn/ProjectFiles/CS2\_Debug\_FpgaProject.tcl"                                        "$CS2_COMPILE_DIR/DebugProject.tcl"
cd $CS2_COMPILE_DIR

# If errors have been detected display the error message and abort compilation.
if {$ErrorCnt > 0} {
    LogMsg $LogFname $ErrorTxt
    return error
}

# ------------------------------------------------------------------------------------------------------
# Run Vivado to synthesize and P&R the OpenCamera design
# ------------------------------------------------------------------------------------------------------
start_gui
create_project -force $PROJ_BASE_NAME $CS2_COMPILE_DIR -part $FPGA_PART_NO
source $CS2_COMPILE_DIR/FpgaScript.tcl

# Display log message from FpgaScript.tcl on console.
LogMsg $LogFname $LogTxt

cd $Cs2OpenCamCfg(OpenCamDir)/CompileTools
return ok
