# ---------------------------------------------------------------------------------
# Start a CorSight2 OpenCamera FPGA synthesis/implementation run
# Copyright (C) 2017 NET GmbH. All Rights Reserved
# ---------------------------------------------------------------------------------

# Set default FPGA compilation variables
set BOARD_NAME     CorSight2
set SCRIPT_DEBUG   OFF

# Define the supported Custom-Module-ID options
set cMODULE_OPTIONS  {DEFAULT USERDESIGN LUTDESIGN MEMTEST DIFFPIC CANNY BLOCKDEMO SCALED}
set cMODULE_NAME     {Default UserDesign LutDesign MemTest DiffPic Canny BlockDemo ScaleD}
set cMODULE_MAX_IDX  [expr {[llength $cMODULE_OPTIONS] - 1}]

# Define the supported FPGA Design ID options
set cDESIGN_OPTIONS  {DSG00 DSG01 DSG02}
set cDESIGN_NAME     {Dsg00 Dsg01 Dsg02}
set cDESIGN_MAX_IDX  [expr {[llength $cDESIGN_OPTIONS] - 1}]

# Define the supported Board Revision options
set cREVISION_OPTIONS  {REV01 REV02 REV03}
set cREVISION_NAME     {Rev01 Rev02 Rev03}
set cREVISION_MAX_IDX  [expr {[llength $cREVISION_OPTIONS] - 1}]

# Define the supported Compile Mode options
set cMODE_OPTIONS  {RELEASE DEBUG}
set cMODE_NAME     {REL     DBG}
set cMODE_MAX_IDX  [expr {[llength $cMODE_OPTIONS] - 1}]

# ---------------------------------------------------------------------------------
# Display Help message on console and in log file
# ---------------------------------------------------------------------------------
proc HelpMsg {MsgStr} {
    puts "$MsgStr"

    puts "USAGE: Start_OpenCam_FPGA.tcl <Custom_ID> <Design_ID> <Revision_ID> <Compile_Mode>\n"
    puts "Custom_ID    (required) = \[DEFAULT, USERDESIGN, LUTDESIGN, MEMTEST, DIFFPIC, CANNY, BLOCKDEMO, SCALED\]"
    puts "Design_ID    (required) = \[DSG00, DSG01, DSG02\]"
    puts "Revision_ID  (required) = \[REV01, REV02, REV03\]"
    puts "Compile_Mode (optional) = \[RELEASE, DEBUG\] - Default Value: RELEASE\n"
}

# ---------------------------------------------------------------------------------
# Read the available command line arguments
# ---------------------------------------------------------------------------------
if {$argc == 3} {
    set CUSTOM_DESIGN       [string toupper [lindex $argv 0]]
    set FPGA_DESIGN_ID      [string toupper [lindex $argv 1]]
    set BOARD_REV           [string toupper [lindex $argv 2]]
    set CUSTOM_COMPILE_MODE RELEASE
} elseif {$argc == 4} {
    set CUSTOM_DESIGN       [string toupper [lindex $argv 0]]
    set FPGA_DESIGN_ID      [string toupper [lindex $argv 1]]
    set BOARD_REV           [string toupper [lindex $argv 2]]
    set CUSTOM_COMPILE_MODE [string toupper [lindex $argv 3]]
} else {
    HelpMsg "\[Start_OpenCam_FPGA.tcl\] ERROR - Invalid number of command line arguments\n"
    return error
}

# ---------------------------------------------------------------------------------
# Check if invalid commmand line arguments has been provided
# ---------------------------------------------------------------------------------
# Check the CUSTOM_DESIGN parameter
if {$CUSTOM_DESIGN != "DEFAULT" && $CUSTOM_DESIGN != "USERDESIGN" && $CUSTOM_DESIGN != "LUTDESIGN" && $CUSTOM_DESIGN != "MEMTEST" && $CUSTOM_DESIGN != "DIFFPIC" && $CUSTOM_DESIGN != "CANNY" && $CUSTOM_DESIGN != "BLOCKDEMO" && $CUSTOM_DESIGN != "SCALED"} {
    HelpMsg "\[Start_OpenCam_FPGA.tcl\] ERROR - Invalid CUSTOM-MODULE-ID command line argument: $CUSTOM_DESIGN - Compile aborted\n"
    return error
}

# Check the FPGA_DESIGN_ID parameter
if {$FPGA_DESIGN_ID != "DSG00" && $FPGA_DESIGN_ID != "DSG01" && $FPGA_DESIGN_ID != "DSG02"} {
    HelpMsg "\[Start_OpenCam_FPGA.tcl\] ERROR - Invalid DESIGN-ID command line argument: $FPGA_DESIGN_ID - Compile aborted\n"
    return error
}

# Check the CorSight2 board revision parameter
if {$BOARD_REV != "REV01" && $BOARD_REV != "REV02" && $BOARD_REV != "REV03"} {
    HelpMsg "\[Start_OpenCam_FPGA.tcl\] ERROR - Invalid Board Revision parameter: $BOARD_REV - Compile aborted\n"
    return error
}

# Check the CUSTOM_COMPILE_MODE parameter
if {$CUSTOM_COMPILE_MODE != "RELEASE" && $CUSTOM_COMPILE_MODE != "DEBUG"} {
    HelpMsg "\[Start_OpenCam_FPGA.tcl\] ERROR - Invalid Compile Mode parameter: $CUSTOM_COMPILE_MODE - Compile aborted\n"
    return error
}

# Check valid CUSTOM_DESIGN / CUSTOM_COMPILE_MODE combinations
if {$CUSTOM_DESIGN == "DEFAULT" || $CUSTOM_DESIGN == "DIFFPIC" || $CUSTOM_DESIGN == "CANNY" || $CUSTOM_DESIGN == "BLOCKDEMO" || $CUSTOM_DESIGN == "SCALED"} {
    if {$CUSTOM_COMPILE_MODE != "RELEASE"} {
        puts "\[Start_OpenCam_FPGA.tcl\] ERROR - Compile Mode \"$CUSTOM_COMPILE_MODE\" not supported for Custom Module \"$CUSTOM_DESIGN\" - Compile aborted\n\n"
        return error
    }
}

# ---------------------------------------------------------------------------------
# Set the defined compilation name strings
# ---------------------------------------------------------------------------------
# Set the Custom Module name string
for {set Idx 0} {$Idx <= $cMODULE_MAX_IDX} {incr Idx} {
    if {$CUSTOM_DESIGN == [lindex $cMODULE_OPTIONS $Idx]} {
        set FPGA_CUST_NAME [lindex $cMODULE_NAME $Idx]
    }
}

# Set the FPGA Design-ID name string
for {set Idx 0} {$Idx <= $cDESIGN_MAX_IDX} {incr Idx} {
    if {$FPGA_DESIGN_ID == [lindex $cDESIGN_OPTIONS $Idx]} {
        set FPGA_DSG_NAME [lindex $cDESIGN_NAME $Idx]
    }
}

# Set the Compile Mode name string
for {set Idx 0} {$Idx <= $cMODE_MAX_IDX} {incr Idx} {
    if {$CUSTOM_COMPILE_MODE == [lindex $cMODE_OPTIONS $Idx]} {
        set CUSTOM_COMPILE_NAME [lindex $cMODE_NAME $Idx]
    }
}

# ---------------------------------------------------------------------------------
# Compile all selected CorSight2 OpenCamera FPGAs
# ---------------------------------------------------------------------------------
source Compile_OpenCam_FPGA.tcl -notrace
return ok
