-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Testbench Simulation Configuration Library
-------------------------------------------------------------------------------
-- Copyright (c) 2013 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2013-05-20  1.0      Manfred  Created
--              2015-04-16  2.0      a.kramer Changed for GigEpro
--              2017-03-02  3.0      Manfred  Changed for CorSight2
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_unsigned.all;  
use ieee.numeric_std.all;         
use ieee.math_real.all;

library std;
use std.textio.all;
use work.CfgModule.all;

package CustomCfgLib is

    ---------------------------------------------------------------------------
    -- Simulation  File Names
    ---------------------------------------------------------------------------
    constant cTESTBENCH_INI_FILE    : string  := cTB_INI_FILE;      -- Testbench initialization file name
    constant cLOG_FILE_NAME         : string  := "TestBench.log";   -- Name of simulation log file.

    ---------------------------------------------------------------------------
    -- Default Simulation  Parameters
    ---------------------------------------------------------------------------
    constant cSYS_RESET_PERIOD      : time    := 25 us;             -- System reset pulse duration after simulation start
    constant cMCB_RESET_PERIOD      : time    := 4 us;              -- MCB reset pulse duration after simulation start
    constant cMAX_SIM_TIME          : time    := 0 ms;              -- Set maximum simulation time. When '0' simulation time is set in simulator.

    constant cPIX_IN_FREQ_DEFAULT   : natural := 100;               -- Default value for the ImageBus Stimulus pixel rate 0..150 [MHz]
    constant cPIX_OUT_FREQ_DEFAULT  : natural := 150;               -- Default value for the ImageBus Analyzer pixel rate 0..150 [MHz]

    ---------------------------------------------------------------------------
    -- Simulation Log File Definitions
    ---------------------------------------------------------------------------
    type tLOG_ITEM_LIST is (LogError, LogExec, LogReturn, LogImgHeader, LogSimInit, LogRegInit, 
                            LogMemInit, LogWait, LogFlowCtl, LogDbgWait, LogAnalyzer);
    type tLOG_ITEM_ARRAY is array (natural range <>) of tLOG_ITEM_LIST;

    -- List of available log message types:
    -- LogError     = Messages are generated when simulation errors are detected.
    -- LogExec      = Messages are generated when simulation modules start execution.
    -- LogReturn    = Messages are generated when simulation modules stop execution.
    -- LogImgHeader = Messages are generated when to display the header of image source files.
    -- LogSimInit   = Messages are generated when reading the simulation parameter file.
    -- LogRegInit   = Messages are generated when performing write operations to the SystemBus register space.
    -- LogMemInit   = Messages are generated when performing write operations to the SystemBus memory space.
    -- LogWait      = Messages are generated when simulation wait states are executed (Wait command)
    -- LogFlowCtl   = Messages are generated at various events during simulation time, i.e. SOF, EOF, reset etc. 
    -- LogDbgWait   = Messages are generated when waiting for Debug Port signals.
    -- LogAnalyzer  = Display ImageBus comparison progress messages.

    -- Report log messages on stdout and in log file. To include a particular log
    -- message type (as listed above), add the relevant keyword to cLOG_ITEM_ARRAY 
    -- and adjust the upper range limit accordingly.
    constant cLOG_ITEM_ARRAY        : tLOG_ITEM_ARRAY(1 to 4) := (LogError, LogFlowCtl, LogDbgWait, LogAnalyzer);    
    constant cLOG_FILE_ENABLE       : boolean := True;          -- Enable simulation logging to file.

    ---------------------------------------------------------------------------
    -- SystemBus and ImageBus Definitions
    ---------------------------------------------------------------------------
    constant cCUSTOM_START_ADDR     : natural := 16#300000#;    -- Custom Module Register Space start address 0x300000
    constant cCUSTOM_END_ADDR       : natural := 16#31FFFF#;    -- Custom Module Memory Space start address 0x31FFFF
    constant cDDR3_MEMORY_ADDR      : natural := 16#60000000#;  -- DDR3-RAM start address 0x60000000

    constant cMAX_PIXEL_WIDTH       : natural := 24;            -- Maximum pixel data width
    constant cMAX_NUM_PIXEL         : natural := 4096;          -- Maximum number of pixels per image line
    constant cMAX_NUM_LINES         : natural := 1024;          -- Maximum number of lines per image

    constant cCMPIMG_PIXMSGCNT      : natural := 100000;        -- Generate pixel comparison messages every cCMPIMG_PIXMSGCNT pixel transfers
    constant cCMPIMG_FRMTMSGCNT     : natural := 1000;          -- Generate format comparison messages every cCMPIMG_FRMTMSGCNT frame status transfers

    constant cDBG_PORT_WIDTH        : integer := 32;            -- Width of Custom Module Debug port 

end package CustomCfgLib;
