-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Testbench Simulation Library
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2013-05-20  1.0      Manfred  Created
--              2015-04-16  2.0      a.kramer Changed for GigEpro
--              2017-09-11  3.0      Manfred  Changed for CorSight2
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library std;
use std.textio.all;
use work.CustomCfgLib.all;
use work.CustomSimLib.all;
use work.CustomPkg.all;

package CustomFctLib is
    type tINT_FILE       is file of integer;            -- Integer file type used to read/write binary data files.
    type tCOMPARE_ARG    is (Equal, NotEqual);          -- Argument in functions FindFirst and FindLast
    type tINTEGER_ENDIAN is (LittleEndian, BigEndian);  -- Integer endian decoding

    constant cINTEGER_BYTE_ORDER : tINTEGER_ENDIAN := BigEndian;

    ---------------------------------------------------------------------------
    -- Function and Procedure Declarations
    ---------------------------------------------------------------------------
    impure function StrgLen (StrgIn : string) return natural;
    impure function FindFirst (StrgIn : string; CmpMode : tCOMPARE_ARG; CharIn : character) return natural;
    impure function FindLast (StrgIn : string; CmpMode : tCOMPARE_ARG; CharIn : character) return natural;
    impure function RemoveBlanks (StrgIn : string) return string;
    impure function ConcatStrg (StrgIn0 : string; StrgIn1 : string) return string;
    impure function ConcatStrg (StrgIn0 : string; StrgIn1 : string; Padded : boolean; PadChar : character) return string;
    impure function Vec2Strg (InSlv : std_logic_vector) return string;
    impure function Vec2Strg (InBv : bit_vector) return string;
    impure function Strg2Int (InStrg : string) return integer;
    impure function Strg2Vec (InStrg : string; BitCnt : natural) return std_logic_vector;
    impure function Strg2Vec (InStrg : string; BitCnt : natural) return bit_vector;
    impure function IsNumber (InStrg : string) return boolean;
    impure function LowerCase (InChar : character) return character;

    procedure PrintNow (StrgIn : string);
    procedure PrintLog (StrgIn : string; LogItem : tLOG_ITEM_LIST);
    procedure AbortSim;
    procedure PrintFile (file fname : text; text_string: in string);
    procedure WriteImgFile (FileData : tIMG_FILE_RECORD; FileName : string);
    procedure ReadImgFile (FileName : in  string; ImgFileDatOut : out tIMG_FILE_RECORD);

    procedure Proc_SysBus_Write (
        CmdIniData          : in  tCMD_DATA_RECORD;
        signal SysClkxC     : in  std_logic;
        signal SysAckxM     : in  std_logic;
        signal SysErrxM     : in  std_logic;
        signal SysModReqxM  : out std_logic;
        signal SysDdrReqxM  : out std_logic;
        signal SysWrEnxM    : out std_logic;
        signal SysAddrxA    : out std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);
        signal SysWrDataxD  : out std_logic_vector(cCMD_DATA_WIDTH-1 downto 0));

    procedure Proc_SysBus_Read (
        CmdIniData          : in  tCMD_DATA_RECORD;
        signal SysClkxC     : in  std_logic;
        signal SysAckxM     : in  std_logic;
        signal SysErrxM     : in  std_logic;
        signal SysModReqxM  : out std_logic;
        signal SysDdrReqxM  : out std_logic;
        signal SysWrEnxM    : out std_logic;
        signal SysAddrxA    : out std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);
        signal SysRdDataxD  : in  std_logic_vector(cCMD_DATA_WIDTH-1 downto 0));

    procedure Proc_ImgBus_Write (
        ImgSimPara            : in  tIMG_SIM_PARAMETERS;
        ImgBusData            : in  tIMG_BUS_RECORD;
        signal ImgClkxC       : in  std_logic;
        signal ImgAbortxM     : in  boolean;
        signal ImgPixRdyxM    : in  std_logic;
        signal ImgPixValxM    : inout std_logic;
        signal ImgPixStatxS   : out std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);
        signal ImgPixDataxD   : out std_logic_vector(cIMG_DATA_WIDTH-1 downto 0));

end package CustomFctLib;

-------------------------------------------------------------------------------
-- Package Body
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_textio.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library std;
use std.textio.all;
use work.CustomCfgLib.all;
use work.CustomSimLib.all;
use work.CustomPkg.all;

package body CustomFctLib is

    ---------------------------------------------------------------------------
    -- Print a string to stdout and to file
    ---------------------------------------------------------------------------
    procedure PrintNow (StrgIn : string) is
        file     LogFile : text;
        variable LogLine : line;
    begin
        write (LogLine, String'("["));
        write (LogLine, now);
        write (LogLine, String'("] : "));
        write (LogLine, StrgIn);
        writeline (output, LogLine);

        -- Print the same message to a file. LogLine must be
        -- re-created since it was de-referenced in writeline.
        if (cLOG_FILE_ENABLE = True) then
            file_open(LogFile, cLOG_FILE_NAME, append_mode);
            write (LogLine, String'("["));
            write (LogLine, now);
            write (LogLine, String'("] : "));
            write (LogLine, StrgIn);
            writeline (LogFile, LogLine);
        end if;
    end PrintNow;

    ---------------------------------------------------------------------------
    -- Print a string to stdout and to file
    ---------------------------------------------------------------------------
    procedure PrintLog (StrgIn : string; LogItem : tLOG_ITEM_LIST) is
        file     LogFile : text;
        variable LogLine : line;
    begin
        LogPrintLoop: for i in cLOG_ITEM_ARRAY'range loop
            -- Print message if LogItem is defined in cLOG_ITEM_ARRAY
            if (LogItem = cLOG_ITEM_ARRAY(i)) then
                write (LogLine, String'("["));
                write (LogLine, now);
                write (LogLine, String'("] : "));
                write (LogLine, StrgIn);
                writeline (output, LogLine);

                -- Print the same message to a file. LogLine must be
                -- re-created since it was de-referenced in writeline.
                if (cLOG_FILE_ENABLE = True) then
                    file_open(LogFile, cLOG_FILE_NAME, append_mode);
                    write (LogLine, String'("["));
                    write (LogLine, now);
                    write (LogLine, String'("] : "));
                    write (LogLine, StrgIn);
                    writeline (LogFile, LogLine);
                end if;
            end if;
        end loop;
    end PrintLog;

    ---------------------------------------------------------------------------
    -- Print a string to file
    ---------------------------------------------------------------------------
    procedure PrintFile (file fname : text; text_string: in string) is
        variable L: line;
    begin
        write (L, String'("["));
        write (L, now);
        write (L, String'("] : "));
        write (L, text_string);
        writeline (fname, L);
    end PrintFile;

    ---------------------------------------------------------------------------
    -- Abort simulation run
    ---------------------------------------------------------------------------
    procedure AbortSim is
    begin
        report "[AbortSim] Simulation Aborted"
        severity failure;
    end AbortSim;

    ---------------------------------------------------------------------------
    -- Calculate the true (i.e. non-blank) length of a string
    ---------------------------------------------------------------------------
    impure function StrgLen (StrgIn : string) return natural is
        variable index : natural := StrgIn'length;
    begin
        loop
            exit when (index <= 1);
            exit when ((StrgIn(index) /= ' ') and (StrgIn(index) /= HT) and (StrgIn(index) /= NUL));
            index := index - 1;
        end loop;
        return index;
    end StrgLen;

    ---------------------------------------------------------------------------
    -- Find the position of the first occurrence of a char in a
    -- string or the position of the first non-occurrence of a char.
    ---------------------------------------------------------------------------
    impure function FindFirst (StrgIn : string; CmpMode : tCOMPARE_ARG; CharIn : character) return natural is
        variable index : natural := 1;
    begin
        loop
            exit when (index > StrgIn'length);
            exit when ((StrgIn(index) = CharIn) and (CmpMode = Equal));
            exit when ((StrgIn(index) /= CharIn) and (CmpMode = NotEqual));
            index := index + 1;
        end loop;
        return index;
    end FindFirst;

    ---------------------------------------------------------------------------
    -- Find the position of the last occurrence of a char in a
    -- string or the position of the last non-occurrence of a char.
    ---------------------------------------------------------------------------
    impure function FindLast (StrgIn : string; CmpMode : tCOMPARE_ARG; CharIn : character) return natural is
        variable index : natural := StrgIn'length;
    begin
        loop
            exit when (index <= 1);
            exit when ((StrgIn(index) = CharIn) and (CmpMode = Equal));
            exit when ((StrgIn(index) /= CharIn) and (CmpMode = NotEqual));
            index := index - 1;
        end loop;
        return index;
    end FindLast;

    ---------------------------------------------------------------------------
    -- Remove trailing blanks from string
    ---------------------------------------------------------------------------
    impure function RemoveBlanks (StrgIn : string) return string is
        variable index : natural := StrgIn'length;
    begin
        loop
            exit when (index <= 1);
            exit when ((StrgIn(index) /= ' ') and (StrgIn(index) /= HT));
            index := index - 1;
        end loop;
        return StrgIn(StrgIn'left to index);
    end RemoveBlanks;

    ---------------------------------------------------------------------------
    -- Concatenate the significant parts of two strings (no padding).
    ---------------------------------------------------------------------------
    impure function ConcatStrg (StrgIn0 : string; StrgIn1 : string) return string is
        variable Len0    : natural := StrgIn0'length;
        variable Len1    : natural := StrgIn1'length;
        variable StrgOut : string(1 to Len0+Len1) := (others => ' ');
    begin
        -- Determine the true lengths of StrgIn0 and StrgIn1 by finding the last non-blank character.
        loop
            exit when ((StrgIn0(Len0) /= ' ') and (StrgIn0(Len0) /= HT) and (StrgIn0(Len0) /= NUL));
            Len0 := Len0 - 1;
            exit when (Len0 = 0);
        end loop;

        loop
            exit when ((StrgIn1(Len1) /= ' ') and (StrgIn1(Len1) /= HT) and (StrgIn1(Len1) /= NUL));
            Len1 := Len1 - 1;
            exit when (Len1 = 0);
        end loop;

        -- Copy StrgIn0 to the lower part of StrgOut
        Strg0Loop: for i in 1 to Len0 loop
            StrgOut(i) := StrgIn0(i);
        end loop;

        -- Copy StrgIn1 to the upper part of StrgOut
        Strg1Loop: for i in 1 to Len1 loop
            StrgOut(Len0+i) := StrgIn1(i);
        end loop;

        return StrgOut(1 to Len0+Len1);
    end ConcatStrg;

    ---------------------------------------------------------------------------
    -- Concatenate the significant parts of two strings.
    -- Optionally pad the remainder of the output string.
    ---------------------------------------------------------------------------
    impure function ConcatStrg (StrgIn0 : string; StrgIn1 : string; Padded : boolean; PadChar : character) return string is
        variable Len0    : natural := StrgIn0'length;
        variable Len1    : natural := StrgIn1'length;
        variable StrgOut : string(1 to Len0+Len1) := (others => PadChar);
    begin
        -- Determine the true lengths of StrgIn0 and StrgIn1 by finding the last non-blank character.
        loop
            exit when ((StrgIn0(Len0) /= ' ') and (StrgIn0(Len0) /= HT) and (StrgIn0(Len0) /= NUL));
            Len0 := Len0 - 1;
            exit when (Len0 = 0);
        end loop;

        loop
            exit when ((StrgIn1(Len1) /= ' ') and (StrgIn1(Len1) /= HT) and (StrgIn1(Len1) /= NUL));
            Len1 := Len1 - 1;
            exit when (Len1 = 0);
        end loop;

        -- Copy StrgIn0 to the lower part of StrgOut
        Strg0Loop: for i in 1 to Len0 loop
            StrgOut(i) := StrgIn0(i);
        end loop;

        -- Copy StrgIn1 to the upper part of StrgOut
        Strg1Loop: for i in 1 to Len1 loop
            StrgOut(Len0+i) := StrgIn1(i);
        end loop;

        if (Padded = True) then
            return StrgOut;
        else
            return StrgOut(1 to Len0+Len1);
        end if;
    end ConcatStrg;

    ---------------------------------------------------------------------------
    -- Convert a std_logic_vector or bit_vector to a hex string (overloaded)
    ---------------------------------------------------------------------------
    impure function Vec2Strg (InSlv : std_logic_vector) return string is
        variable InLine    : line;
        variable SlvLen    : integer;
        variable HexVector : std_logic_vector(InSlv'length+3 downto 0) := (others => '0');
    begin
        if ((InSlv'length mod 4) = 0) then
            SlvLen := InSlv'length;
        else
            SlvLen := InSlv'length - (InSlv'length mod 4) + 4;
        end if;

        HexVector(InSlv'length-1 downto 0) := InSlv;
        hwrite (InLine, HexVector(SlvLen-1 downto 0));
        return InLine.all;
    end function;

    impure function Vec2Strg (InBv : bit_vector) return string is
        variable InLine    : line;
        variable SlvLen    : integer;
        variable HexVector : std_logic_vector(InBv'length+3 downto 0) := (others => '0');
    begin
        if ((InBv'length mod 4) = 0) then
            SlvLen := InBv'length;
        else
            SlvLen := InBv'length - (InBv'length mod 4) + 4;
        end if;

        HexVector(InBv'length-1 downto 0) :=  to_stdlogicvector(InBv);
        hwrite (InLine, HexVector(SlvLen-1 downto 0));
        return InLine.all;
    end function;

    ---------------------------------------------------------------------------
    -- Check if a string is a decimal or hexadecimal number
    ---------------------------------------------------------------------------
    impure function IsNumber (InStrg : string) return boolean is
        variable i, StartChar : integer := 0;
        variable CharFound    : boolean := False;
        variable NumFound     : boolean := False;
    begin
        if (InStrg(1 to 2) = cHEX_PREFIX) then
            StartChar := 3;
        else
            StartChar := 1;
        end if;

        ParseNumStrg: for i in StartChar to InStrg'high loop
            if ((InStrg(i) = ' ') or (InStrg(i) = HT) or (InStrg(i) = CR) or (InStrg(i) = LF)) then
                return CharFound;
            else
                CharFound := True;

                -- Check if the character is a decimal digit.
                if ((InStrg(i) >= '0') and (InStrg(i) <= '9')) then
                    NumFound := True;
                else
                    NumFound := False;
                end if;

                -- Check if the character is a hexadecimal digit.
                if ((StartChar = 3) and (((InStrg(i) >= 'A') and (InStrg(i) <= 'F')) or ((InStrg(i) >= 'a') and (InStrg(i) <= 'f')))) then
                    NumFound := True;
                end if;

                -- An invalid hex/dec character has been found, return false.
                if (NumFound = False) then
                    return False;
                end if;
            end if;
        end loop;

        return CharFound;
    end function;

    ---------------------------------------------------------------------------
    -- Convert characters to lower case
    ---------------------------------------------------------------------------
    impure function LowerCase (InChar : character) return character is
        variable Result : character := ' ';
    begin
        case InChar is
            when 'A' => Result := 'a';
            when 'B' => Result := 'b';
            when 'C' => Result := 'c';
            when 'D' => Result := 'd';
            when 'E' => Result := 'e';
            when 'F' => Result := 'f';
            when 'G' => Result := 'g';
            when 'H' => Result := 'h';
            when 'I' => Result := 'i';
            when 'J' => Result := 'j';
            when 'K' => Result := 'k';
            when 'L' => Result := 'l';
            when 'M' => Result := 'm';
            when 'N' => Result := 'n';
            when 'O' => Result := 'o';
            when 'P' => Result := 'p';
            when 'Q' => Result := 'q';
            when 'R' => Result := 'r';
            when 'S' => Result := 's';
            when 'T' => Result := 't';
            when 'U' => Result := 'u';
            when 'V' => Result := 'v';
            when 'W' => Result := 'w';
            when 'X' => Result := 'x';
            when 'Y' => Result := 'y';
            when 'Z' => Result := 'z';
            when others => Result := InChar;
        end case;

        return Result;
    end function;

    ---------------------------------------------------------------------------
    -- Convert a hex or decimal string to std_logic_vector
    ---------------------------------------------------------------------------
    impure function Strg2Vec (InStrg : string; BitCnt : natural) return std_logic_vector is
        variable IsHex      : boolean := False;
        variable StartChar  : natural := 1;
        variable CharVal    : natural := 0;
        variable IntVal     : natural := 0;
        variable SlvVal     : std_logic_vector(BitCnt+3 downto 0) := (others => '0');
    begin
        -- Determine if the input string represents a hexadecimal or a decimal number.
        if (InStrg(1 to 2) = cHEX_PREFIX) then
            IsHex     := True;
            StartChar := 3;
        end if;

        -- Convert each hex character from InStrg to an integer,
        -- which is then converted to std_logic_vector.
        ParseStrg: for i in StartChar to InStrg'high loop
            if ((InStrg(i) /= ' ') and (InStrg(i) /= HT) and (InStrg(i) /= CR) and (InStrg(i) /= LF)) then
                if ((InStrg(i) >= '0') and (InStrg(i) <= '9')) then
                    CharVal := character'pos(InStrg(i)) - character'pos('0');
                elsif ((IsHex = True) and (InStrg(i) >= 'A') and (InStrg(i) <= 'F')) then
                    CharVal := character'pos(InStrg(i)) - character'pos('A') + 10;
                elsif ((IsHex = True) and (InStrg(i) >= 'a') and (InStrg(i) <= 'f')) then
                    CharVal := character'pos(InStrg(i)) - character'pos('a') + 10;
                else
                    report " " & time'image(now) & "  -  (Strg2Vec) Invalid hexadecimal character in String: " & String'(InStrg)
                    severity failure;
                end if;

                if (IsHex = True) then
                    SlvVal := SlvVal(BitCnt-1 downto 0) & std_logic_vector(to_unsigned(CharVal, 4));
                else
                    IntVal := IntVal*10 + CharVal;
                end if;
            end if;
        end loop ParseStrg;

        if (IsHex = True) then
            return SlvVal(BitCnt-1 downto 0);
        else
            return std_logic_vector(to_unsigned(IntVal, BitCnt));
        end if;
    end function;

    impure function Strg2Vec (InStrg : string; BitCnt : natural) return bit_vector is
        variable IsHex      : boolean := False;
        variable StartChar  : natural := 1;
        variable CharVal    : natural := 0;
        variable IntVal     : natural := 0;
        variable BvVal      : bit_vector(BitCnt+3 downto 0) := (others => '0');
    begin
        -- Determine if the input string represents a hexadecimal or a decimal number.
        if (InStrg(1 to 2) = cHEX_PREFIX) then
            IsHex     := True;
            StartChar := 3;
        end if;

        -- Convert each hex character from InStrg to an integer,
        -- which is then converted to std_logic_vector.
        ParseStrg: for i in StartChar to InStrg'high loop
            if ((InStrg(i) /= ' ') and (InStrg(i) /= HT) and (InStrg(i) /= CR) and (InStrg(i) /= LF)) then
                if ((InStrg(i) >= '0') and (InStrg(i) <= '9')) then
                    CharVal := character'pos(InStrg(i)) - character'pos('0');
                elsif ((IsHex = True) and (InStrg(i) >= 'A') and (InStrg(i) <= 'F')) then
                    CharVal := character'pos(InStrg(i)) - character'pos('A') + 10;
                elsif ((IsHex = True) and (InStrg(i) >= 'a') and (InStrg(i) <= 'f')) then
                    CharVal := character'pos(InStrg(i)) - character'pos('a') + 10;
                else
                    report " " & time'image(now) & "  -  (Strg2Vec) Invalid hexadecimal character in String: " & String'(InStrg)
                    severity failure;
                end if;

                if (IsHex = True) then
                    BvVal := BvVal(BitCnt-1 downto 0) & to_bitvector(std_logic_vector(to_unsigned(CharVal, 4)));
                else
                    IntVal := IntVal*10 + CharVal;
                end if;
            end if;
        end loop ParseStrg;

        if (IsHex = True) then
            return BvVal(BitCnt-1 downto 0);
        else
            return to_bitvector(std_logic_vector(to_unsigned(IntVal, BitCnt)));
        end if;
    end function;

    ---------------------------------------------------------------------------
    -- Convert a decimal string to integer
    ---------------------------------------------------------------------------
    impure function Strg2Int (InStrg : string) return integer is
        variable IntVal  : integer := 0;
    begin
        -- Determine if the input string represents a hexadecimal or a decimal number.
        -- Convert each character from InStrg to an integer,
        -- which is then converted to std_logic_vector.
        if (InStrg(1 to 2) = cHEX_PREFIX) then
            HexParseStrg: for i in InStrg'low+2 to InStrg'high loop
                if ((InStrg(i) >= '0') and (InStrg(i) <= '9')) then
                    IntVal := (IntVal*16) + character'pos(InStrg(i)) - character'pos('0');
                elsif ((InStrg(i) >= 'A') and (InStrg(i) <= 'F')) then
                    IntVal := (IntVal*16) + character'pos(InStrg(i)) - character'pos('A') + 10;
                elsif ((InStrg(i) >= 'a') and (InStrg(i) <= 'f')) then
                    IntVal := (IntVal*16) + character'pos(InStrg(i)) - character'pos('a') + 10;
                else
                    exit HexParseStrg;
                end if;
            end loop HexParseStrg;
        else
            IntParseStrg: for i in InStrg'low to InStrg'high loop
                if ((InStrg(i) >= '0') and (InStrg(i) <= '9')) then
                    IntVal := (IntVal*10) + character'pos(InStrg(i)) - character'pos('0');
                else
                    exit IntParseStrg;
                end if;
            end loop IntParseStrg;
        end if;

        return IntVal;
    end function;

    ---------------------------------------------------------------------------
    -- Perform SystemBus write cycles as per the Register Data Record
    ---------------------------------------------------------------------------
    procedure Proc_SysBus_Write (
        CmdIniData          : in  tCMD_DATA_RECORD;
        signal SysClkxC     : in  std_logic;
        signal SysAckxM     : in  std_logic;
        signal SysErrxM     : in  std_logic;
        signal SysModReqxM  : out std_logic;
        signal SysDdrReqxM  : out std_logic;
        signal SysWrEnxM    : out std_logic;
        signal SysAddrxA    : out std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);
        signal SysWrDataxD  : out std_logic_vector(cCMD_DATA_WIDTH-1 downto 0))
    is
        variable SysAddrCnt : std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0) := (others => '0');
    begin
        -- Check if a valid Custom Module or DDR3-RAM write cycle is requested.
        if ((CmdIniData.DdrCycle = True) or (CmdIniData.ModCycle = True)) then
            PrintLog ("Executing Proc_SysBus_Write", LogExec);
            SysAddrCnt := to_stdlogicvector(CmdIniData.CmdAddr(cCMD_ADDR_WIDTH-1 downto 0));
        
            for i in 0 to CmdIniData.DataCnt-1 loop
                if (CmdIniData.DdrCycle = True) then
                    PrintLog ("[Proc_SysBus_Write] SystemBus DDR3-RAM Write - MemAddr = " & cHEX_PREFIX & Vec2Strg(SysAddrCnt) & " - MemData = " & cHEX_PREFIX & Vec2Strg(CmdIniData.CmdData(i)), LogMemInit);
                else
                    PrintLog ("[Proc_SysBus_Write] SystemBus Custom Module Write - RegAddr = " & cHEX_PREFIX & Vec2Strg(SysAddrCnt) & " - RegData = " & cHEX_PREFIX & Vec2Strg(CmdIniData.CmdData(i)), LogRegInit);
                end if;
    
                -- Wait for the SystemBus slave to deassert SysAckxM and SysErrxM.
                -- Execute the "wait until" statement only if either SysAckxM or
                -- SysErrxM are actually high (otherwise the simulation hangs).
                if ((SysAckxM = '1') or (SysErrxM = '1')) then
                    wait until ((SysAckxM = '0') and (SysErrxM = '0'));
                end if;
                wait until rising_edge(SysClkxC);
    
                -- Assert SystemBus Module/Memory Requests
                if (CmdIniData.DdrCycle = True) then
                    SysModReqxM <= '0';
                    SysDdrReqxM <= '1';
                else
                    SysModReqxM <= '1';
                    SysDdrReqxM <= '0';
                end if;
    
                SysWrEnxM   <= '1';
                SysAddrxA   <= SysAddrCnt;
                SysWrDataxD <= to_stdlogicvector(CmdIniData.CmdData(i)(cCMD_DATA_WIDTH-1 downto 0));
    
                -- Similar as the above wait statement: Execute "wait until"
                -- only if the desired signal state is not already active.
                if ((SysAckxM = '0') and (SysErrxM = '0')) then
                    wait until ((SysAckxM = '1') or (SysErrxM = '1'));
                end if;
                wait until rising_edge(SysClkxC);
    
                SysModReqxM <= '0';
                SysDdrReqxM <= '0';
                SysAddrCnt := SysAddrCnt + x"4";
            end loop;

            PrintLog ("Exiting Proc_SysBus_Write", LogReturn);
        else
            PrintLog ("[Proc_SysBus_Write] ERROR - Invalid SystemBus write operation detected - Address = " & cHEX_PREFIX & Vec2Strg(CmdIniData.CmdAddr), LogRegInit);
        end if;

    end Proc_SysBus_Write;

    ---------------------------------------------------------------------------
    -- Perform a single SystemBus read cycle
    ---------------------------------------------------------------------------
    procedure Proc_SysBus_Read (
        CmdIniData          : in  tCMD_DATA_RECORD;
        signal SysClkxC     : in  std_logic;
        signal SysAckxM     : in  std_logic;
        signal SysErrxM     : in  std_logic;
        signal SysModReqxM  : out std_logic;
        signal SysDdrReqxM  : out std_logic;
        signal SysWrEnxM    : out std_logic;
        signal SysAddrxA    : out std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);
        signal SysRdDataxD  : in  std_logic_vector(cCMD_DATA_WIDTH-1 downto 0))
    is
        variable VerifyError : boolean := False;
        variable ExpData     : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0) := (others => '0');
    begin
        -- Check if a valid Custom Module or DDR3-RAM read cycle is requested.
        if ((CmdIniData.DdrCycle = True) or (CmdIniData.ModCycle = True)) then
            PrintLog ("Executing Proc_SysBus_Read", LogExec);
    
            if ((SysAckxM = '1') or (SysErrxM = '1')) then
                wait until ((SysAckxM = '0') and (SysErrxM = '0'));
            end if;
            wait until rising_edge(SysClkxC);
    
            -- Assert SystemBus Module/Memory Requests
            if (CmdIniData.DdrCycle = True) then
                SysModReqxM <= '0';
                SysDdrReqxM <= '1';
            else
                SysModReqxM <= '1';
                SysDdrReqxM <= '0';
            end if;
    
            SysWrEnxM <= '0';
            SysAddrxA <=  to_stdlogicvector(CmdIniData.CmdAddr(cCMD_ADDR_WIDTH-1 downto 0));
    
            if ((SysAckxM = '0') and (SysErrxM = '0')) then
                wait until ((SysAckxM = '1') or (SysErrxM = '1'));
            end if;
            wait until rising_edge(SysClkxC);
    
            if (CmdIniData.Verify = False) then
                PrintNow ("[Proc_SysBus_Read] SystemBus Read: Addr = " & cHEX_PREFIX & Vec2Strg(CmdIniData.CmdAddr) & " - Data = " & cHEX_PREFIX & Vec2Strg(SysRdDataxD));
            else
                -- Verify read data with expected data
                for i in 0 to cCMD_DATA_WIDTH-1 loop
                    if (CmdIniData.CmdMask(i) = '1') then
                        ExpData(i) := SysRdDataxD(i);
                        if (CmdIniData.CmdData(0)(i) /= to_bit(SysRdDataxD(i))) then
                            VerifyError := True;
                        end if;
                    end if;
                end loop;
    
                if (VerifyError = False) then
                    PrintNow ("[Proc_SysBus_Read] SystemBus Verify OK: Addr = " & cHEX_PREFIX & Vec2Strg(CmdIniData.CmdAddr) & " - ReadData = " & cHEX_PREFIX & Vec2Strg(ExpData));
                else
                    PrintNow ("[Proc_SysBus_Read] SystemBus Verify Error: Addr = " & cHEX_PREFIX & Vec2Strg(CmdIniData.CmdAddr) & " - ExpData = " & cHEX_PREFIX & Vec2Strg(CmdIniData.CmdData(0)) & " - ActData = " & cHEX_PREFIX & Vec2Strg(ExpData));
                end if;
            end if;
    
            SysModReqxM <= '0';
            SysDdrReqxM <= '0';
            PrintLog ("Exiting Proc_SysBus_Read", LogReturn);
        else
            PrintLog ("[Proc_SysBus_Read] ERROR - Invalid SystemBus read operation detected - Address = " & cHEX_PREFIX & Vec2Strg(CmdIniData.CmdAddr), LogRegInit);
        end if;
    end Proc_SysBus_Read;

    ---------------------------------------------------------------------------
    -- Perform an entire ImageBus line transfer incl. hor/ver sync.
    ---------------------------------------------------------------------------
    procedure Proc_ImgBus_Write (
        ImgSimPara            : in  tIMG_SIM_PARAMETERS;
        ImgBusData            : in  tIMG_BUS_RECORD;
        signal ImgClkxC       : in  std_logic;
        signal ImgAbortxM     : in  boolean;
        signal ImgPixRdyxM    : in  std_logic;
        signal ImgPixValxM    : inout std_logic;
        signal ImgPixStatxS   : out std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);
        signal ImgPixDataxD   : out std_logic_vector(cIMG_DATA_WIDTH-1 downto 0))
    is
        variable PixelCnt     : natural := 0;
        variable TimeCnt      : integer := 0;
        variable ClkCycTime   : integer := 0;
        variable ValCycTime   : integer := 0;
        variable NumCycles    : natural := 0;
        variable TimeWrap     : boolean := False;
        variable SetPixel     : boolean := False;
        variable ResetPixel   : boolean := False;
    begin
        PrintLog ("Executing Proc_ImgBus_Write", LogExec);

        ClkCycTime := 1000/cIMGBUS_PIXEL_FREQ;              -- ImageBus pixel clock cycle time, in ns
        ValCycTime := 1000/ImgSimPara.ImgMode.PixInFreq;    -- ImageBus Stimulus Valid signal cycle time, in ns

        -- Calculate the number of pixel cycles to be performed
        -- for the current line. If the last line is active add
        -- the vertical blanking time.
        if (ImgBusData.LastLin = True) then
            NumCycles := (ImgSimPara.ImgIn.NumPix + ImgSimPara.ImgMode.HorSync) * (ImgSimPara.ImgMode.VerSync + 1);
        else
            NumCycles := (ImgSimPara.ImgIn.NumPix + ImgSimPara.ImgMode.HorSync);
        end if;

        PixTxLoop: loop
            -- Control ImageBus pixel transfers and handshake.
            wait until rising_edge(ImgClkxC);

            -- An external image transfer abort is pending.
            if (ImgAbortxM = True) then
                ImgPixValxM  <= '0';
                ImgPixDataxD <= (others => '0');
                ImgPixStatxS <= cIMG_STAT_NONE;
                exit PixTxLoop;
            end if;

            -- Increment the Time Counter by pixel clock cycle time every clock cycle.
            TimeCnt := TimeCnt + ClkCycTime;

            -- If the Time Counter reaches the requested pixel input time period
            -- reset the Counter and enable pixel transfer.
            if (TimeCnt >= ValCycTime) then
                TimeCnt  := TimeCnt - ValCycTime;
                TimeWrap := True;
            else
                TimeWrap := False;
            end if;


            -- Implement a pixel clock divider during active line time.
            if (PixelCnt <= ImgSimPara.ImgIn.NumPix-1) then
                if ((ImgPixValxM = '1') and (ImgPixRdyxM = '1')) then
                    if (TimeWrap = True) then
                        SetPixel := True;
                    else
                        ImgPixValxM <= '0';
                    end if;
                elsif (ImgPixValxM = '0') then
                    if (TimeWrap = True) then
                        SetPixel := True;
                    end if;
                end if;
            elsif (PixelCnt = ImgSimPara.ImgIn.NumPix) then
                -- Wait for the last pixel transfer of a line to complete before resetting ImgPixValxM.
                if ((ImgPixValxM = '1') and (ImgPixRdyxM = '1')) then
                    ResetPixel := True;
                end if;
            else
                ResetPixel := True;
            end if;

            -- Start a new pixel transfer
            if (SetPixel = True) then
                ImgPixValxM <= '1';

                -- Set the ImageBus status
                if (PixelCnt = 0) then
                    if (ImgSimPara.ImgMode.ScanLineMode = True) then
                        ImgPixStatxS <= cIMG_STAT_SOSF;
                    elsif (ImgBusData.FirstLin = True) then
                        ImgPixStatxS <= cIMG_STAT_SOF;
                    elsif (ImgBusData.LastLin = True) then
                        ImgPixStatxS <= cIMG_STAT_SOLL;
                    else
                        ImgPixStatxS <= cIMG_STAT_SOL;
                    end if;
                elsif (PixelCnt = ImgSimPara.ImgIn.NumPix-1) then
                    ImgPixStatxS <= cIMG_STAT_EOL;
                else
                    ImgPixStatxS <= cIMG_STAT_NONE;
                end if;

                -- Default pixel value
                ImgPixDataxD <= (others => '0');

                -- Assign 8- or 12-bit pixel values determined by the
                -- pixel format specified in the image file.
                if (ImgBusData.NumBits = 1) then        -- 1-bit bit map data
                    ImgPixDataxD(24) <= to_stdlogicvector(ImgBusData.LineBuf(PixelCnt)(0 downto 0))(0);
                elsif (ImgBusData.NumBits = 8) then     -- 8-bit monochrome pixel data
                    ImgPixDataxD(35 downto 28) <= to_stdlogicvector(ImgBusData.LineBuf(PixelCnt)(7 downto 0));
                elsif (ImgBusData.NumBits = 12) then    -- 12-bit monochrome pixel data
                    ImgPixDataxD(35 downto 24) <= to_stdlogicvector(ImgBusData.LineBuf(PixelCnt)(11 downto 0));
                else                                    -- 24-bit RGB data
                    ImgPixDataxD(35 downto 28) <= to_stdlogicvector(ImgBusData.LineBuf(PixelCnt)(23 downto 16));
                    ImgPixDataxD(23 downto 16) <= to_stdlogicvector(ImgBusData.LineBuf(PixelCnt)(15 downto 8));
                    ImgPixDataxD(11 downto 4)  <= to_stdlogicvector(ImgBusData.LineBuf(PixelCnt)(7 downto 0));
                end if;

                PixelCnt := PixelCnt + 1;
                SetPixel := False;
            end if;

            -- Reset ImageBus (blanking period)
            if (ResetPixel = True) then
                ImgPixValxM  <= '0';
                ImgPixDataxD <= (others => '0');
                ImgPixStatxS <= cIMG_STAT_NONE;
                PixelCnt   := PixelCnt + 1;
                ResetPixel := False;
            end if;

            -- Line transfer complete - exit Proc_ImgBus_Write.
            if (PixelCnt >= NumCycles) then
                exit PixTxLoop;
            end if;
        end loop PixTxLoop;

        PrintLog ("Exiting Proc_ImgBus_Write", LogReturn);
    end Proc_ImgBus_Write;

    ---------------------------------------------------------------------------
    -- Read the contents of an image file into memory
    ---------------------------------------------------------------------------
    procedure ReadImgFile (FileName : in  string; ImgFileDatOut : out tIMG_FILE_RECORD)
    is
        file     CharFile    : text;
        file     IntFile     : tINT_FILE;
        variable ImgBusLine  : line;
        variable InChar      : character := ' ';
        variable EndOfWord   : boolean := False;
        variable EndOfLine   : boolean := False;
        variable IsBinary    : boolean := False;
        variable Comment     : boolean := False;
        variable HeaderDone  : boolean := False;
        variable CharCnt     : natural range 0 to cMAX_WORD_LENGTH := 0;
        variable LinCnt      : natural := 0;
        variable WordCnt     : natural := 0;
        variable ByteCnt     : natural := 0;
        variable CurPixCnt   : natural := 0;
        variable CurLinCnt   : natural := 0;
        variable LineNumber  : natural := 0;
        variable PixIntVal   : natural := 0;
        variable IntBuffer   : integer := 0;
        variable StdVecVal   : unsigned(31 downto 0) := (others => '0');
        variable PixBitVal   : bit_vector(7 downto 0) := (others => '0');
        variable DataMode    : tIMG_FILE_MODE := MagicNo;
        variable WordBuffer  : string(1 to cMAX_WORD_LENGTH) := (others => ' ');
        variable ImgFileData : tIMG_FILE_RECORD   := cIMG_FILE_DEFAULT;
        variable FileErrCode : tSIM_ERROR_MESSAGE := cSIM_ERROR_DEFAULT;
    begin

        PrintLog ("Executing ReadImgFile", LogExec);
        PrintLog ("[ReadImgFile] Opening image file " & FileName & " in read mode", LogFlowCtl);
        file_open(CharFile, FileName, read_mode);
        ImgFileLoop: loop
            if endfile(CharFile) then
                exit ImgFileLoop;
            else
                -- Read a line from the Register setup file into a line buffer.
                readline(CharFile, ImgBusLine);

                -- Check basic read line error conditions.
                if ((ImgBusLine /= NULL) and (ImgBusLine'length > 0)) then
                    -- Read a line from the Register setup file into a line buffer.
                    CharCnt    := 0;
                    WordCnt    := 0;
                    ByteCnt    := 0;
                    EndOfLine  := False;
                    EndOfWord  := False;
                    WordBuffer := (others => ' ');
                    LineNumber := LineNumber + 1;

                    ImgLineLoop: for CharNum in ImgBusLine'range loop
                        if (CharNum = ImgBusLine'high) then
                            EndOfLine := True;
                        end if;

                        case ImgBusLine(CharNum) is
                            when ' ' | HT =>
                                if (CharCnt > 0) then       -- Skip leading white spaces
                                    EndOfWord := True;
                                end if;

                            when cIMG_COMMENT | CR | LF =>
                                EndOfLine := True;

                            when others =>
                                -- Check if the WordBuffer is overflowing.
                                if (CharCnt < cMAX_WORD_LENGTH) then
                                    CharCnt := CharCnt + 1;

                                    if (DataMode = ImgBinData) then
                                        -- Read binary image data
                                        WordBuffer(CharCnt) := ImgBusLine(CharNum);

                                        if (CharCnt = cMAX_WORD_LENGTH) then
                                            EndOfWord := True;
                                        end if;
                                    else
                                        -- Read ASCII image data or header data
                                        WordBuffer(CharCnt) := LowerCase(ImgBusLine(CharNum));
                                    end if;
                                else
                                    FileErrCode := cRDIMGFILE_BUFOVRFLOW;
                                end if;
                        end case;

                        -- Analyze WordBuffer if valid characters have been detected.
                        if (((EndOfLine = True) or (EndOfWord = True)) and (CharCnt > 0)) then
                            case DataMode is
                                when MagicNo =>
                                    PrintLog ("[ReadImgFile] Image MagicNum = " & WordBuffer, LogImgHeader);
                                    DataMode := ImgSize;
                                    ImgFileData.MagicNum := WordBuffer(1 to 2);

                                    -- Check if Magic Number is correct.
                                    case WordBuffer(1 to 2) is
                                        when cPBM_ASCII_MAGIC_NUM | cPBM_BIN_MAGIC_NUM =>
                                            ImgFileData.NumBits := 1;   -- pbm file use 1 bit pixel depth
                                            FileErrCode := cRDIMGFILE_ERRNUM_2;
                                        when cPGM_ASCII_MAGIC_NUM | cPGM_BIN_MAGIC_NUM =>
                                            ImgFileData.NumBits := 8;   -- Default pgm bit depth, may change in MaxPixVal
                                        when cPPM_ASCII_MAGIC_NUM | cPPM_BIN_MAGIC_NUM =>
                                            ImgFileData.NumBits := 24;  -- ppm files use 24 bit pixel depth (RGB)
                                        when  others => FileErrCode := cRDIMGFILE_ERRNUM_3;
                                    end case;

                                    if ((WordBuffer(1 to 2) = cPBM_BIN_MAGIC_NUM) or (WordBuffer(1 to 2) = cPGM_BIN_MAGIC_NUM) or (WordBuffer(1 to 2) = cPPM_BIN_MAGIC_NUM)) then
                                        IsBinary := True;
                                    else
                                        IsBinary := False;
                                    end if;

                                when ImgSize =>
                                    if (WordCnt = 0) then
                                        PrintLog ("[ReadImgFile] Image MaxPix = " & WordBuffer, LogImgHeader);
                                        if (IsNumber(WordBuffer) = True) then
                                            ImgFileData.MaxPix := Strg2Int(WordBuffer);
                                        else
                                            FileErrCode := cRDIMGFILE_ERRNUM_5;
                                        end if;
                                    elsif (WordCnt = 1) then
                                        PrintLog ("[ReadImgFile] Image MaxLin = " & WordBuffer, LogImgHeader);
                                        if (IsNumber(WordBuffer) = True) then
                                            ImgFileData.MaxLin := Strg2Int(WordBuffer);

                                            -- pbm (bit map) files do not contain a max. pixel value, skip PixValue.
                                            if ((ImgFileData.MagicNum = cPBM_ASCII_MAGIC_NUM) or (ImgFileData.MagicNum = cPBM_BIN_MAGIC_NUM)) then
                                                if (IsBinary = False) then
                                                    DataMode := ImgAsciiData;
                                                else
                                                    exit ImgFileLoop;
                                                    -- DataMode := ImgBinData;
                                                end if;
                                            else
                                                DataMode := PixValue;
                                            end if;
                                        else
                                            FileErrCode := cRDIMGFILE_ERRNUM_7;
                                        end if;
                                    end if;

                                when PixValue =>
                                    PrintLog ("[ReadImgFile] Image MaxPixVal = " & WordBuffer, LogImgHeader);
                                    if (IsNumber(WordBuffer) = True) then
                                        -- Check in pgm files if the maximum pixel value is greater than 255. Assign 12 bit pixel depth.
                                        if (((ImgFileData.MagicNum = cPGM_ASCII_MAGIC_NUM) or (ImgFileData.MagicNum = cPGM_BIN_MAGIC_NUM)) and (Strg2Int(WordBuffer) > 255)) then
                                            ImgFileData.NumBits := 12;
                                        end if;

                                        if (IsBinary = False) then
                                            DataMode := ImgAsciiData;
                                        else
                                            exit ImgFileLoop;
                                            -- DataMode := ImgBinData;
                                        end if;
                                    else
                                        FileErrCode := cRDIMGFILE_ERRNUM_8;
                                    end if;

                                when ImgAsciiData =>
                                    if (ImgFileData.MagicNum = cPPM_ASCII_MAGIC_NUM) then
                                        -- Read ASCII ppm image file, sequence of 3x 8-bit values for RGB. First Red, second Green, third Blue.
                                        if (IsNumber(WordBuffer) = True) then
                                            case ByteCnt is
                                                when 0 => ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(23 downto 16) := Strg2Vec(WordBuffer, 8);   -- Red
                                                when 1 => ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(15 downto 8)  := Strg2Vec(WordBuffer, 8);   -- Green
                                                when 2 => ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(7 downto 0)   := Strg2Vec(WordBuffer, 8);   -- Blue
                                                when others => NULL;
                                            end case;
                                        else
                                            FileErrCode := cRDIMGFILE_ERRNUM_12;
                                        end if;

                                        if (ByteCnt = 2) then
                                            ByteCnt := 0;
                                            if (CurPixCnt < ImgFileData.MaxPix-1) then
                                                CurPixCnt := CurPixCnt + 1;
                                            else
                                                CurPixCnt := 0;
                                                CurLinCnt := CurLinCnt + 1;
                                            end if;
                                        else
                                            ByteCnt := ByteCnt + 1;
                                        end if;
                                    elsif ((ImgFileData.MagicNum = cPGM_ASCII_MAGIC_NUM) or (ImgFileData.MagicNum = cPBM_ASCII_MAGIC_NUM)) then
                                        -- Read ASCII pbm or pgm image file
                                        if (IsNumber(WordBuffer) = True) then
                                            ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(ImgFileData.NumBits-1 downto 0) := Strg2Vec(WordBuffer, ImgFileData.NumBits);
                                        else
                                            FileErrCode := cRDIMGFILE_ERRNUM_13;
                                        end if;

                                        if (CurPixCnt < ImgFileData.MaxPix-1) then
                                            CurPixCnt := CurPixCnt + 1;
                                        else
                                            CurPixCnt := 0;
                                            CurLinCnt := CurLinCnt + 1;
                                        end if;
                                    else
                                        FileErrCode := cRDIMGFILE_ERRNUM_14;
                                    end if;

                                when others => NULL;
                            end case;

                            CharCnt    := 0;
                            EndOfWord  := False;
                            WordBuffer := (others => ' ');
                            WordCnt    := WordCnt + 1;
                        end if;

                        if (EndOfLine = True) then
                            exit ImgLineLoop;
                        end if;

                        if (FileErrCode /= cSIM_ERROR_DEFAULT) then
                            exit ImgFileLoop;
                        end if;
                    end loop ImgLineLoop;
                end if;
            end if;
        end loop ImgFileLoop;

        file_close(CharFile);

        if (IsBinary = True) then
            -- Check that an 8-bit character format is used by the simulator.
            -- Binary files can only be read if characters are 8-bit wide.
            if (character'pos(character'right) /= 255) then
                PrintLog ("[ReadImgFile] Simulator does not support 8-bit characters - Reading of binary image files is not supported", LogError);
                AbortSim;
            end if;

            if (integer'high /= 16#7FFF_FFFF#) then
                PrintLog ("[ReadImgFile] Simulator does not support 32-bit integers - Reading of binary image files is not supported", LogError);
                AbortSim;
            end if;

            -- If a binary image file is read reopen the image file in integer mode.
            file_open(IntFile, FileName, read_mode);

            CharCnt    := 0;
            LinCnt     := 0;
            ByteCnt    := 0;
            Comment    := False;
            HeaderDone := False;

            IntFileLoop: loop
                -- Read an integer from file and analyze the header content.
                read(IntFile, IntBuffer);

                -- Circumvent the sign of the integer conversion by always converting a positive
                -- integer value and explicitly setting the MSB of the unsigned StdVecVal.
                if (IntBuffer < 0) then
                    IntBuffer := IntBuffer-16#4000_0000#;   -- Subtract 16#8000_0000# from IntBuffer in 2 separate
                    IntBuffer := IntBuffer-16#4000_0000#;   -- operations to avoid simulator warning message
                    StdVecVal := to_unsigned(IntBuffer, StdVecVal'length);
                    StdVecVal(31) := '1';
                else
                    StdVecVal := to_unsigned(IntBuffer, StdVecVal'length);
                end if;

                for i in 0 to 3 loop
                    PixIntVal := to_integer(StdVecVal((i*8)+7 downto (i*8)));

                    -- Find the beginning of image data, i.e. skip
                    -- the file header (already read previously).
                    if (HeaderDone = False) then
                        InChar := character'val(PixIntVal);

                        if (InChar = LF) then               -- End of a line
                            if (CharCnt > 0) then           -- Check if valid header characters have been found.
                                LinCnt := LinCnt + 1;

                                if (LinCnt = 3) then        -- After 3 valid lines the header is complete.
                                    HeaderDone := True;
                                end if;
                            end if;

                            CharCnt := 0;
                            Comment := False;
                        elsif (InChar = cIMG_COMMENT) then  -- Flag the rest of the line as comment
                            Comment := True;
                        elsif ((Comment = False) and (InChar /= ' ') and (InChar /= HT) and (InChar /= CR)) then -- Skip leading white spaces
                            CharCnt := CharCnt + 1;
                        end if;
                    else
                        if (ImgFileData.MagicNum = cPBM_BIN_MAGIC_NUM) then
                            -- Read binary pbm image file - to be completed
                            FileErrCode := cRDIMGFILE_ERRNUM_10;

                        elsif (ImgFileData.MagicNum = cPGM_BIN_MAGIC_NUM) then
                            if (ImgFileData.NumBits = 8) then

                                -- Read 8-bit binary pgm image file
                                ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(7 downto 0) := to_bitvector(std_logic_vector(to_unsigned(PixIntVal, 8)));
                                if (CurPixCnt < ImgFileData.MaxPix-1) then
                                    CurPixCnt := CurPixCnt + 1;
                                else
                                    CurPixCnt := 0;
                                    CurLinCnt := CurLinCnt + 1;
                                end if;
                            elsif (ImgFileData.NumBits = 12) then
                                -- Read 12-bit binary pgm image file
                                PixBitVal := to_bitvector(std_logic_vector(to_unsigned(PixIntVal, 8)));

                                -- Assign bit_vector value to frame buffer array.
                                case ByteCnt is
                                    when 0 =>
                                        ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(7 downto 0)  := PixBitVal;
                                        ByteCnt := 1;
                                    when 1 =>
                                        -- Assign the lower bit_vector nibble to the
                                        -- current pixel and increment the pixel counter.
                                        ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(11 downto 8) := PixBitVal(3 downto 0);
                                        if (CurPixCnt < ImgFileData.MaxPix-1) then
                                            CurPixCnt := CurPixCnt + 1;
                                        else
                                            CurPixCnt := 0;
                                            CurLinCnt := CurLinCnt + 1;
                                        end if;

                                        -- Assign the upper bit_vector nibble to the (next) pixel.
                                        ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(3 downto 0) := PixBitVal(7 downto 4);
                                        ByteCnt := 2;
                                    when 2 =>
                                        ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(11 downto 4) := PixBitVal;
                                        if (CurPixCnt < ImgFileData.MaxPix-1) then
                                            CurPixCnt := CurPixCnt + 1;
                                        else
                                            CurPixCnt := 0;
                                            CurLinCnt := CurLinCnt + 1;
                                        end if;
                                        ByteCnt := 0;
                                    when others => NULL;
                                end case;
                            else
                                FileErrCode := cRDIMGFILE_ERRNUM_9;
                            end if;

                        elsif (ImgFileData.MagicNum = cPPM_BIN_MAGIC_NUM) then
                            -- Read binary ppm image file
                            case ByteCnt is
                                when 0 =>
                                    ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(23 downto 16) := to_bitvector(std_logic_vector(to_unsigned(PixIntVal, 8)));   -- Red
                                    ByteCnt := 1;
                                when 1 =>
                                    ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(15 downto 8)  := to_bitvector(std_logic_vector(to_unsigned(PixIntVal, 8)));   -- Green
                                    ByteCnt := 2;
                                when 2 =>
                                    ImgFileData.FrmBuffer(CurLinCnt)(CurPixCnt)(7 downto 0)   := to_bitvector(std_logic_vector(to_unsigned(PixIntVal, 8)));   -- Blue
                                    if (CurPixCnt < ImgFileData.MaxPix-1) then
                                        CurPixCnt := CurPixCnt + 1;
                                    else
                                        CurPixCnt := 0;
                                        CurLinCnt := CurLinCnt + 1;
                                    end if;
                                    ByteCnt := 0;
                                when others => NULL;
                            end case;

                        else
                            FileErrCode := cRDIMGFILE_ERRNUM_11;
                        end if;
                    end if;
                end loop;
                if (endfile(IntFile) or (FileErrCode /= cSIM_ERROR_DEFAULT)) then
                    exit IntFileLoop;
                end if;
            end loop IntFileLoop;
        end if;
        -- Print error message and abort simulation
        if (FileErrCode /= cSIM_ERROR_DEFAULT) then
            PrintLog("[ReadImgFile] File read error in file " & FileName & ", line " & integer'image(LineNumber) & ": " & RemoveBlanks(FileErrCode), LogError);
            AbortSim;
        end if;

        ImgFileData.Valid := True;
        file_close(CharFile);
        file_close(IntFile);

        PrintLog ("Exiting ReadImgFile", LogReturn);
        ImgFileDatOut := ImgFileData;
    end procedure ReadImgFile;

    ---------------------------------------------------------------------------
    -- Write an image to file. PPM or PGM ASCII file formats only.
    ---------------------------------------------------------------------------
    procedure WriteImgFile (FileData : tIMG_FILE_RECORD; FileName : string) is
        file     CharFile   : text;
        file     IntFile    : tINT_FILE;
        variable ImgBusLine : line;
        variable WrChar     : character;
        variable IntPix, j  : integer;
        variable PixCnt     : natural := 0;
    begin
        PrintLog ("Executing WriteImgFile", LogExec);

        -- Check frame size requirements for binary image data.
        if (FileData.MagicNum = cPGM_BIN_MAGIC_NUM) then
            if (FileData.NumBits = 8) then
                if (((FileData.MaxPix * FileData.MaxLin) mod 4) /= 0) then
                    PrintLog ("[WriteImgFile] Error writing image file '" & FileName & "' - Frame must contain an even multiple of 4 pixels" , LogError);
                    AbortSim;
                end if;
            else
                PrintLog ("[WriteImgFile] Error writing image file '" & FileName & "' - Saving " & integer'image(FileData.NumBits) & "-bit binary pgm image data not supported. Please select an 8-bit format" , LogError);
                AbortSim;

                -- if ((FileData.NumBits = 12) and ((FileData.MaxPix * FileData.MaxLin) mod 8) /= 0)) then
                --     PrintLog ("[WriteImgFile] Error writing image file '" & FileName & "' - Frame must contain an even multiple of 8 pixels" , LogError);
                --     AbortSim;
                -- end if;
            end if;
        elsif (FileData.MagicNum = cPPM_BIN_MAGIC_NUM) then
            if (((FileData.MaxPix * FileData.MaxLin) mod 4) /= 0) then
                PrintLog ("[WriteImgFile] Error writing image file '" & FileName & "' - Frame must contain an even multiple of 4 pixels" , LogError);
                AbortSim;
            end if;
        end if;

        PrintLog ("[WriteImgFile] Opening image file " & FileName & " in write mode", LogFlowCtl);
        file_open(CharFile, FileName, write_mode);

        -- Assign Magic Number to write buffer
        case FileData.MagicNum is
            when cPGM_ASCII_MAGIC_NUM => write (ImgBusLine, string'("P2"));
            when cPPM_ASCII_MAGIC_NUM => write (ImgBusLine, string'("P3"));
            when cPGM_BIN_MAGIC_NUM   => write (ImgBusLine, string'("P5"));
            when cPPM_BIN_MAGIC_NUM   => write (ImgBusLine, string'("P6"));
            when others => PrintLog ("[WriteImgFile] Invalid Magic Number detected", LogError);
                           AbortSim;
        end case;

        writeline (CharFile, ImgBusLine);         -- Write Magic Number to file
        write (ImgBusLine, integer'image(FileData.MaxPix) & " " & integer'image(FileData.MaxLin));
        writeline (CharFile, ImgBusLine);         -- Write frame size parameters

        ------------------------------------------------------------------------------------------

        -- Write image to ASCII pgm fille
        if (FileData.MagicNum = cPGM_ASCII_MAGIC_NUM) then
            if (FileData.NumBits = 8) then
                -- Write 8-bit pgm format
                write (ImgBusLine, string'("255"));
                writeline (CharFile, ImgBusLine);

                for i in 0 to FileData.MaxLin-1 loop
                    for j in 0 to FileData.MaxPix-1 loop
                        IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0))));
                        write (ImgBusLine, string'(integer'image(IntPix) & ' '));
                    end loop;
                    writeline (CharFile, ImgBusLine);
                end loop;
            else
                -- Write 12-bit pgm format
                write (ImgBusLine, string'("4095"));
                writeline (CharFile, ImgBusLine);

                for i in 0 to FileData.MaxLin-1 loop
                    for j in 0 to FileData.MaxPix-1 loop
                        IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0))));
                        write (ImgBusLine, string'(integer'image(IntPix) & ' '));
                    end loop;
                    writeline (CharFile, ImgBusLine);
                end loop;
            end if;
        end if;

        ------------------------------------------------------------------------------------------

        -- Write image to ASCII ppm fille
        if (FileData.MagicNum = cPPM_ASCII_MAGIC_NUM) then
            write (ImgBusLine, string'("255"));
            writeline (CharFile, ImgBusLine);

            -- Pixel data is written as integers.
            for i in 0 to FileData.MaxLin-1 loop
                for j in 0 to FileData.MaxPix-1 loop
                    IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(23 downto 16))));   -- Red
                    write (ImgBusLine, string'(integer'image(IntPix) & ' '));
                    IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(15 downto 8))));    -- Green
                    write (ImgBusLine, string'(integer'image(IntPix) & ' '));
                    IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0))));     -- Blue
                    write (ImgBusLine, string'(integer'image(IntPix) & ' '));
                end loop;
                writeline (CharFile, ImgBusLine);
            end loop;
        end if;

        ------------------------------------------------------------------------------------------

        -- Write image to binary pgm file
        if (FileData.MagicNum = cPGM_BIN_MAGIC_NUM) then
            if (FileData.NumBits = 8) then
                write (ImgBusLine, string'("255"));
                writeline (CharFile, ImgBusLine);

                -- After writing the image header as text, close the file and
                -- reopen it as integer file to write the binary image data.
                file_close(CharFile);
                file_open(IntFile, FileName, append_mode);

                PixCnt := 0;
                for i in 0 to FileData.MaxLin-1 loop
                    for j in 0 to FileData.MaxPix-1 loop
                        case PixCnt is
                            when 0 =>
                                IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0))));
                                PixCnt := 1;
                            when 1 =>
                                IntPix := IntPix + (256 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0)))));
                                PixCnt := 2;
                            when 2 =>
                                IntPix := IntPix + (65536 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0)))));
                                PixCnt := 3;
                            when 3 =>
                                IntPix := IntPix + (16777216 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0)))));
                                write(IntFile, IntPix);
                                PixCnt := 0;
                            when others => NULL;
                        end case;
                    end loop;
                end loop;
            else
                write (ImgBusLine, string'("4095"));
                writeline (CharFile, ImgBusLine);

                -- After writing the image header as text, close the file and
                -- reopen it as integer file to write the binary image data.
                file_close(CharFile);
                file_open(IntFile, FileName, append_mode);

                PixCnt := 0;
                if (cINTEGER_BYTE_ORDER = BigEndian) then
                    -- Integer Byte Order (big endianness): 0xPix0Pix1Pix2Pix3 ...
                    for i in 0 to FileData.MaxLin-1 loop
                        for j in 0 to FileData.MaxPix-1 loop
                            case PixCnt is
                                when 0 =>
                                    IntPix := 16777216 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0))));
                                    PixCnt := 1;
                                when 1 =>
                                    IntPix := IntPix + (256 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0)))));
                                    PixCnt := 2;
                                when 2 =>
                                    IntPix := IntPix + to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 4))));
                                    write(IntFile, IntPix);
                                    IntPix := 268435456 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(3 downto 0))));
                                    PixCnt := 3;
                                when 3 =>
                                    IntPix := IntPix + (65536 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0)))));
                                    PixCnt := 4;
                                when 4 =>
                                    IntPix := IntPix + (16 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0)))));
                                    PixCnt := 5;
                                when 5 =>
                                    IntPix := IntPix + to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 8))));
                                    write(IntFile, IntPix);
                                    IntPix := 16777216 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0))));
                                    PixCnt := 6;
                                when 6 =>
                                    IntPix := IntPix + (4096 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0)))));
                                    PixCnt := 7;
                                when 7 =>
                                    IntPix := IntPix + to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(3 downto 0))));
                                    write(IntFile, IntPix);
                                    PixCnt := 0;
                                when others => NULL;
                            end case;
                        end loop;
                    end loop;
                else
                    -- Integer Byte Order (little endianness): 0xPix3Pix2Pix1Pix0 ...
                    for i in 0 to FileData.MaxLin-1 loop
                        for j in 0 to FileData.MaxPix-1 loop
                            case PixCnt is
                                when 0 =>
                                    IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0))));
                                    PixCnt := 1;
                                when 1 =>
                                    IntPix := IntPix + (4096 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0)))));
                                    PixCnt := 2;
                                when 2 =>
                                    IntPix := IntPix + (16777216 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0)))));
                                    write(IntFile, IntPix);
                                    IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 4))));
                                    PixCnt := 3;
                                when 3 =>
                                    IntPix := IntPix + (16 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0)))));
                                    PixCnt := 4;
                                when 4 =>
                                    IntPix := IntPix + (65536 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0)))));
                                    PixCnt := 5;
                                when 5 =>
                                    IntPix := IntPix + (268435456 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(3 downto 0)))));
                                    write(IntFile, IntPix);
                                    IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 4))));
                                    PixCnt := 6;
                                when 6 =>
                                    IntPix := IntPix + (256 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(11 downto 0)))));
                                    PixCnt := 7;
                                when 7 =>
                                    IntPix := IntPix + (1048576 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(3 downto 0)))));
                                    write(IntFile, IntPix);
                                    PixCnt := 0;
                                when others => NULL;
                            end case;
                        end loop;
                    end loop;
                end if;
            end if;
        end if;

        ------------------------------------------------------------------------------------------

        -- Write image to binary ppm fille
        if (FileData.MagicNum = cPPM_BIN_MAGIC_NUM) then
            write (ImgBusLine, string'("255"));
            writeline (CharFile, ImgBusLine);

            -- After writing the image header as text, close the file and
            -- reopen it as integer file to write the binary image data.
            file_close(CharFile);
            file_open(IntFile, FileName, append_mode);

            PixCnt := 0;
            for i in 0 to FileData.MaxLin-1 loop
                for j in 0 to FileData.MaxPix-1 loop
                    case PixCnt is
                        when 0 =>
                            IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(23 downto 16))));
                            IntPix := IntPix + (256 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(15 downto 8)))));
                            IntPix := IntPix + (65536 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0)))));
                            PixCnt := 1;
                        when 1 =>
                            IntPix := IntPix + (16777216 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(23 downto 16)))));
                            write(IntFile, IntPix);
                            IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(15 downto 8))));
                            IntPix := IntPix + (256 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0)))));
                            PixCnt := 2;
                        when 2 =>
                            IntPix := IntPix + (65536 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(23 downto 16)))));
                            IntPix := IntPix + (16777216 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(15 downto 8)))));
                            write(IntFile, IntPix);
                            IntPix := to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0))));
                            PixCnt := 3;
                        when 3 =>
                            IntPix := IntPix + (256 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(23 downto 16)))));
                            IntPix := IntPix + (65536 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(15 downto 8)))));
                            IntPix := IntPix + (16777216 * to_integer(unsigned(to_stdlogicvector(FileData.FrmBuffer(i)(j)(7 downto 0)))));
                            write(IntFile, IntPix);
                            PixCnt := 0;
                        when others => NULL;
                    end case;
                end loop;
            end loop;
        end if;

        file_close(CharFile);
        file_close(IntFile);
        PrintLog ("Exiting WriteImgFile", LogReturn);
    end procedure WriteImgFile;

end package body CustomFctLib;
