-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Memory Controller Block Interface
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-09-05  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--use ieee.math_real.all;

library UNISIM;
use UNISIM.vcomponents.all;
library unimacro;
use unimacro.vcomponents.all;

library work;
use work.CustomCfgLib.all;
use work.CustomSimLib.all;

entity CustomMcbInterface is
    generic (
        gCUST_INSTR_BUS_WIDTH : integer := 3;                                           -- Width of Custom Module Instruction bus
        gCUST_BLEN_BUS_WIDTH  : integer := 6;                                           -- Width of Custom Module Burst-Length bus
        gCUST_ADDR_BUS_WIDTH  : integer := 30;                                          -- Width of Custom Module Address bus
        gCUST_DATA_BUS_WIDTH  : integer := 32;                                          -- Width of Custom Module Data bus

        gMCB_CMD_BUS_WIDTH    : integer := 3;                                           -- Width of MCB Command bus
        gMCB_ADDR_BUS_WIDTH   : integer := 27;                                          -- Width of MCB Address bus
        gMCB_MASK_BUS_WIDTH   : integer := 16;                                          -- Width of MCB Mask bus
        gMCB_DATA_BUS_WIDTH   : integer := 128;                                         -- Width of MCB Data bus

        gMCB_DEBUG_ENABLE     : boolean := True;                                        -- Enable Debug Port logic
        gMCB_DEBUG_WIDTH      : integer := 256);                                        -- Width of Debug Port
    port (
        -- Infrastructure Signals
        SysClkxCI           : in  std_logic;                                            -- SystemBus Clock
        SysRstxRI           : in  std_logic;                                            -- SystemBus Reset

        -- SystemBus Control Interface
        SysMemReqxMI        : in  std_logic;                                            -- SystemBus Memory Request
        SysMemAckxMO        : out std_logic;                                            -- SystemBus Memory Acknowledge
        SysMemErrxMO        : out std_logic;                                            -- SystemBus Memory Error
        SysMemWrEnxMI       : in  std_logic;                                            -- SystemBus Memory Write Enable
        SysMemAddrxAI       : in  std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);         -- SystemBus Memory Address
        SysMemWrDataxDI     : in  std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);         -- SystemBus Memory Write Data
        SysMemRdDataxDO     : out std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);         -- SystemBus Memory Read Data

        -------------------------------------------------------------------------------

        -- Custom Module Command Port
        CustMcbCmdValxMI    : in  std_logic;                                            -- Custom Module Command Valid
        CustMcbCmdRdyxMO    : out std_logic;                                            -- Custom Module Command Ready
        CustMcbCmdInstxMI   : in  std_logic_vector(gCUST_INSTR_BUS_WIDTH-1 downto 0);   -- Custom Module Command Instruction
        CustMcbCmdLenxMI    : in  std_logic_vector(gCUST_BLEN_BUS_WIDTH-1 downto 0);    -- Custom Module Command Burst Length
        CustMcbCmdAddrxAI   : in  std_logic_vector(gCUST_ADDR_BUS_WIDTH-1 downto 0);    -- Custom Module Command Address

        -- Custom Module Write Port
        CustMcbWrValxMI     : in  std_logic;                                            -- Custom Module Write Data Valid
        CustMcbWrRdyxMO     : out std_logic;                                            -- Custom Module Write Data Ready
        CustMcbWrDataxDI    : in  std_logic_vector(gCUST_DATA_BUS_WIDTH-1 downto 0);    -- Custom Module Write Data Bus

        -- Custom Module Read Port
        CustMcbRdValxMI     : in  std_logic;                                            -- Custom Module Read Data Valid
        CustMcbRdRdyxMO     : out std_logic;                                            -- Custom Module Read Data Ready
        CustMcbRdDataxDO    : out std_logic_vector(gCUST_DATA_BUS_WIDTH-1 downto 0);    -- Custom Module Read Data Bus

        -------------------------------------------------------------------------------

        -- MCB User-Application Infrastructure
        McbAppClkxCI        : in  std_logic;                                            -- MCB User-Application Interface Clock
        McbAppRstxRI        : in  std_logic;                                            -- MCB User-Application Interface Reset
        McbCalibDonexSI     : in  std_logic;                                            -- MCB Calibration complete
        McbRdFifoOvrFlxMO   : out std_logic;                                            -- MCB Read FIFO Overflow

        -- MCB User-Application Command-Interface
        McbAppCmdEnxMO      : out std_logic;                                            -- MCB User-Application Command Enable
        McbAppCmdRdyxMI     : in  std_logic;                                            -- MCB User-Application Command Ready
        McbAppCmdCmdxMO     : out std_logic_vector(gMCB_CMD_BUS_WIDTH-1 downto 0);      -- MCB User-Application Command Instruction
        McbAppCmdAddrxAO    : out std_logic_vector(gMCB_ADDR_BUS_WIDTH-1 downto 0);     -- MCB User-Application Command Address

        -- MCB User-Application Write-Data-Interface
        McbAppWrEnxMO       : out std_logic;                                            -- MCB User-Application Write Enable
        McbAppWrRdyxMI      : in  std_logic;                                            -- MCB User-Application Write Ready
        McbAppWrEndxMO      : out std_logic;                                            -- MCB User-Application Write End
        McbAppWrMaskxMO     : out std_logic_vector(gMCB_MASK_BUS_WIDTH-1 downto 0);     -- MCB User-Application Write Maks
        McbAppWrDataxDO     : out std_logic_vector(gMCB_DATA_BUS_WIDTH-1 downto 0);     -- MCB User-Application Write Data

        -- MCB User-Application Read-Data-Interface
        McbAppRdValxMI      : in  std_logic;                                            -- MCB User-Application Read Valid
        McbAppRdEndxMI      : in  std_logic;                                            -- MCB User-Application Read End
        McbAppRdDataxDI     : in  std_logic_vector(gMCB_DATA_BUS_WIDTH-1 downto 0);     -- MCB User-Application Read Data

        -- Debug Port
        McbDebugPortxMO     : out std_logic_vector(gMCB_DEBUG_WIDTH-1 downto 0));      -- Debug Port
end CustomMcbInterface;

architecture Behavioral of CustomMcbInterface is

    -------------------------------------------------------------------------------
    -- Constant and Type Definitions
    -------------------------------------------------------------------------------
    constant cMCB_NUM_READ_BUFFER : integer   := 4;                     -- Number of MCB Read Input Buffers
    constant cSYNC_NUM_STAGES     : integer   := 3;                     -- Number of SystemBus synchronization stages

    constant cMCB_DATA_INVALID    : std_logic := '1';                   -- MCB Data Mask Invalid
    constant cMCB_DATA_VALID      : std_logic := '0';                   -- MCB Data Mask Valid
    constant cMCB_WRITE_COMMAND   : std_logic := '0';                   -- MCB Write Command
    constant cMCB_READ_COMMAND    : std_logic := '1';                   -- MCB Read Command

    constant cMCB_BURST_LEN_ONE   : std_logic_vector(gCUST_BLEN_BUS_WIDTH-1 downto 0) :=  std_logic_vector(to_unsigned(1, gCUST_BLEN_BUS_WIDTH));     -- MCB Burst Length one
    constant cMCB_NUM_CYC_ZERO    : std_logic_vector(gCUST_BLEN_BUS_WIDTH-2 downto 0) :=  std_logic_vector(to_unsigned(0, gCUST_BLEN_BUS_WIDTH-1));   -- MCB Number of cycles zero

    type tSYS_MEM_FSM_STATE   is (SysIdle, SysStart, SysEnd);
    type tMCB_SYS_FSM_STATE   is (McbSysIdle, McbSysStart, McbSysEnd);

    type tMCB_CMD_FSM_STATE   is (CmdIdle, CmdWrite, CmdRead, CmdTransfer, CmdCycEnd, CmdRdWait);
    type tMCB_WRITE_FSM_STATE is (WrIdle, WrSysBus, WrFifoRead, WrMcbTransfer);
    type tMCB_READ_FSM_STATE  is (RdIdle, RdWait, RdActive);

    type tMCB_READ_COMMAND is record
        CmdValid   : std_logic;                                         -- MCB Read Command Buffer Valid
        DataValid  : std_logic;                                         -- MCB Read Data Buffer Valid
        SysBusReq  : std_logic;                                         -- MCB Read SystemBus Request
        DwordPtr   : integer range 0 to 3;                              -- MCB Read DWORD Pointer
        BstLenCnt  : integer range 0 to (2**gCUST_BLEN_BUS_WIDTH)-1;    -- MCB Read Burst Length Counter
        DataBuffer : std_logic_vector(127 downto 0);                    -- MCB Read Data Buffer
    end record;

    constant cMCB_READ_CMD_DEFAULT : tMCB_READ_COMMAND := (
        CmdValid   => '0',
        DataValid  => '0',
        SysBusReq  => '0',
        DwordPtr   => 0,
        BstLenCnt  => 0,
        DataBuffer => (others => '0'));

    type tMCB_COMMAND_ARRAY is array (0 to cMCB_NUM_READ_BUFFER-1) of tMCB_READ_COMMAND;

    -------------------------------------------------------------------------------
    -- Component Declarations
    -------------------------------------------------------------------------------
    -- FIFO18E1: In FIFO18 mode, supports port widths of x4, x9, x18
    --           In FIFO18_36 mode, port width is x36
    --           Supported FIFO sizes: 4Kx4, 2Kx9, 1Kx18, 512x36
    component FIFO18E1
    generic (
        ALMOST_EMPTY_OFFSET     : bit_vector := X"0080";
        ALMOST_FULL_OFFSET      : bit_vector := X"0080";
        DATA_WIDTH              : integer    := 4;
        FIFO_MODE               : string     := "FIFO18";
        DO_REG                  : integer    := 1;
        EN_SYN                  : boolean    := FALSE;
        INIT                    : bit_vector := x"000000000";
        SRVAL                   : bit_vector := x"000000000";
        FIRST_WORD_FALL_THROUGH : boolean    := FALSE;
        SIM_DEVICE              : string     := "7SERIES";
        IS_RDCLK_INVERTED       : bit        := '0';
        IS_RDEN_INVERTED        : bit        := '0';
        IS_RSTREG_INVERTED      : bit        := '0';
        IS_RST_INVERTED         : bit        := '0';
        IS_WRCLK_INVERTED       : bit        := '0';
        IS_WREN_INVERTED        : bit        := '0');
    port (
        -- FIFO Infrastructure
        RST         : in std_ulogic;
        REGCE       : in std_ulogic;
        RSTREG      : in std_ulogic;

        -- Write Port
        WRCLK       : in std_ulogic;
        WREN        : in std_ulogic;
        DI          : in std_logic_vector(31 downto 0);
        DIP         : in std_logic_vector(3 downto 0);

        -- Read Port
        RDCLK       : in std_ulogic;
        RDEN        : in std_ulogic;
        DO          : out std_logic_vector(31 downto 0);
        DOP         : out std_logic_vector(3 downto 0);

        -- FIFO status signals
        EMPTY       : out std_ulogic;
        ALMOSTEMPTY : out std_ulogic;
        ALMOSTFULL  : out std_ulogic;
        FULL        : out std_ulogic;

        WRCOUNT     : out std_logic_vector(11 downto 0);
        WRERR       : out std_ulogic;
        RDCOUNT     : out std_logic_vector(11 downto 0);
        RDERR       : out std_ulogic);
    end component;

    attribute BOX_TYPE of FIFO18E1 : component is "PRIMITIVE";

    -------------------------------------------------------------------------------
    -- Signal Definitions
    -------------------------------------------------------------------------------
    signal SysMemStatexM      : tSYS_MEM_FSM_STATE;                             -- SystemBus State Machine (SysBus Clock Domain)
    signal SysMemReqxM        : std_logic;                                      -- SystemBus Memory Request
    signal SysMemAckxM        : std_logic;                                      -- SystemBus Memory Acknowledge
    signal SysMemErrxM        : std_logic;                                      -- SystemBus Memory Error
    signal SysMemWrEnxM       : std_logic;                                      -- SystemBus Memory Write Enable
    signal SysMemAddrxA       : std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);   -- SystemBus Memory Address
    signal SysMemWrDataxD     : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);   -- SystemBus Memory Write Data
    signal SysMemRdDataxD     : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);   -- SystemBus Memory Read Data

    signal SyncMcbAckxM       : std_logic_vector(cSYNC_NUM_STAGES-1 downto 0);  -- Synchronized SystemBus Memory Acknowledge
    signal SyncMcbErrxM       : std_logic_vector(cSYNC_NUM_STAGES-1 downto 0);  -- Synchronized SystemBus Memory Error
    signal SyncMcbRdDataxD    : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);   -- Synchronized SystemBus Memory Read Data

    signal McbSysStatexM      : tMCB_SYS_FSM_STATE;                             -- MCB SystemBus State Machine (MCB User Clock Domain)
    signal McbSysWrReqxM      : std_logic;                                      -- MCB SystemBus Write Cycle Request
    signal McbSysRdReqxM      : std_logic;                                      -- MCB SystemBus Read Cycle Request
    signal McbSysAckxM        : std_logic;                                      -- MCB SystemBus Acknowledge
    signal McbSysErrxM        : std_logic;                                      -- MCB SystemBus Error
    signal McbSysAddrxA       : std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);   -- MCB SystemBus Address
    signal McbSysWrDataxD     : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);   -- MCB SystemBus Write Data
    signal McbSysRdDataxD     : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);   -- MCB SystemBus Read Data

    signal SyncMemReqxM       : std_logic_vector(cSYNC_NUM_STAGES-1 downto 0);  -- Synchronized MCB SystemBus Request
    signal SyncMemWrEnxM      : std_logic;                                      -- Synchronized MCB SystemBus Write Enable
    signal SyncMemAddrxA      : std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);   -- Synchronized MCB SystemBus Address
    signal SyncMemWrDataxD    : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);   -- Synchronized MCB SystemBus Write Data

    signal McbCmdAckxM        : std_logic;                                      -- MCB Command Interface Acknowledge
    signal McbCmdErrxM        : std_logic;                                      -- MCB Command Interface Error
    signal McbCmdRdDataxD     : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);   -- MCB Command Interface Read Data

    signal McbMemRdValidxM    : std_logic;                                      -- MCB Memory Read Data Valid
    signal McbMemRdDataxD     : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);   -- MCB Memory Read Data

    signal CustMcbCmdRdyxM    : std_logic;                          -- Command Ready
    signal CustMcbRdRdyxM     : std_logic;                          -- Read Data Ready
    signal CustMcbRdDataxD    : std_logic_vector(31 downto 0);      -- Read Data Bus
    signal CustMcbWrRdyxM     : std_logic;                          -- Write Data Ready

    signal McbCmdStatexM      : tMCB_CMD_FSM_STATE;                 -- MCB Command State Machine
    signal McbCmdSysReqxM     : std_logic;                          -- MCB Command SystemBus Request
    signal McbCmdWrReqxM      : std_logic;                          -- MCB Command Write Request
    signal McbCmdWrAckxM      : std_logic;                          -- MCB Command Write Acknowledge
    signal McbCmdRdReqxM      : std_logic;                          -- MCB Command Read Request
    signal McbCmdRdAckxM      : std_logic;                          -- MCB Command Read Acknowledge
    signal McbCmdFifoWrEnxM   : std_logic;                          -- MCB Command FIFO Write Enable
    signal McbCmdFifoRdEnxM   : std_logic;                          -- MCB Command FIFO Read Enable
    signal McbCmdFifoEmptyxS  : std_logic;                          -- MCB Command FIFO Empty
    signal McbCmdFifoFullxS   : std_logic;                          -- MCB Command FIFO Full
    signal McbCmdFifoWrDataxD : std_logic_vector(35 downto 0);      -- MCB Command FIFO Read Data
    signal McbCmdFifoRdDataxD : std_logic_vector(35 downto 0);      -- MCB Command FIFO Read Data

    signal McbAppCmdEnxM      : std_logic;                          -- MCB User-Application Command Enable
    signal McbAppCmdCmdxM     : std_logic_vector(2 downto 0);       -- MCB User-Application Command Instruction
    signal McbAppCmdAddrxA    : std_logic_vector(25 downto 0);      -- MCB User-Application Command Address (address of 128-bit data word)
    signal McbAppCmdWdSelxM   : std_logic_vector(1 downto 0);       -- MCB User-Application DWord Select (LSB of Command Address)
    signal McbAppCmdWdNumxM   : std_logic_vector(2 downto 0);       -- MCB User-Application Number of DWORDS per MCB data transfer cycle
    signal McbAppCmdBLenxM    : std_logic_vector(gCUST_BLEN_BUS_WIDTH-1 downto 0);  -- MCB Burst Length, in Number-of-DWORDS - 1
    signal McbAppCmdNumxM     : std_logic_vector(gCUST_BLEN_BUS_WIDTH-2 downto 0);  -- Number of required MCB commands/data cycles
    signal McbCmdEndAddrxA    : std_logic_vector(gCUST_BLEN_BUS_WIDTH downto 0);    -- Partial MCB burst end address, used to calculate McbAppCmdNumxM.
    signal McbCmdFifoBLenxM   : std_logic_vector(gCUST_BLEN_BUS_WIDTH downto 0);
    signal McbCmdFifoStartxM  : std_logic_vector(gCUST_BLEN_BUS_WIDTH downto 0);

    signal McbWrStatexM       : tMCB_WRITE_FSM_STATE;               -- MCB Write Interface State Machine
    signal McbWrFifoWrEnxM    : std_logic;                          -- MCB Write FIFO Write Enable
    signal McbWrFifoRdEnxM    : std_logic;                          -- MCB Write FIFO Read Enable
    signal McbWrFifoEmptyxS   : std_logic;                          -- MCB Write FIFO Empty
    signal McbWrFifoFullxS    : std_logic;                          -- MCB Write FIFO Full
    signal McbWrFifoWrDataxD  : std_logic_vector(31 downto 0);      -- MCB Write FIFO Write Data
    signal McbWrFifoRdDataxD  : std_logic_vector(31 downto 0);      -- MCB Write FIFO Read Data
    signal McbWrDWordPtrxM    : integer range 0 to 3;               -- MCB Write DWORD Pointer
    signal McbWrDWordCntxM    : integer range 0 to (2**gCUST_BLEN_BUS_WIDTH)-1;  -- MCB Write DWORD Counter

    signal McbAppWrEnxM       : std_logic;                          -- MCB User-Application Write Enable
    signal McbAppWrEndxM      : std_logic;                          -- MCB User-Application Write End
    signal McbAppWrMaskxM     : std_logic_vector(3 downto 0);       -- MCB User-Application Write Mask
    signal McbAppWrDataxD     : std_logic_vector(127 downto 0);     -- MCB User-Application Write Data

    signal McbRdStatexM       : tMCB_READ_FSM_STATE;                -- MCB Read Interface State Machine
    signal McbRdFifoWrValxM   : std_logic;                          -- MCB Read FIFO Write Valid
    signal McbRdFifoWrReqxM   : std_logic;                          -- MCB Read FIFO Write Request
    signal McbRdFifoWrEndxM   : std_logic;                          -- MCB Read FIFO Write End
    signal McbRdFifoWrEnxM    : std_logic;                          -- MCB Read FIFO Write Enable
    signal McbRdFifoRdEnxM    : std_logic;                          -- MCB Read FIFO Read Enable
    signal McbRdFifoEmptyxS   : std_logic;                          -- MCB Read FIFO Empty
    signal McbRdFifoFullxS    : std_logic;                          -- MCB Read FIFO Full
    signal McbRdFifoWrDataxD  : std_logic_vector(31 downto 0);      -- MCB Read FIFO Write Data
    signal McbRdFifoRdDataxD  : std_logic_vector(31 downto 0);      -- MCB Read FIFO Read Data

    signal McbRdCmdDatBufxM   : tMCB_COMMAND_ARRAY;                 -- MCB Read Command/Data Buffer
    signal McbRdCmdInPtrxM    : integer range 0 to cMCB_NUM_READ_BUFFER-1;  -- MCB Read Command Input Buffer Pointer
    signal McbRdDatInPtrxM    : integer range 0 to cMCB_NUM_READ_BUFFER-1;  -- MCB Read Data Input Buffer Pointer
    signal McbRdCmdDatPtrxM   : integer range 0 to cMCB_NUM_READ_BUFFER-1;  -- MCB Read Command/Data Buffer Pointer
    signal McbRdFifoOvrFlxM   : std_logic;                                  -- MCB Read FIFO Overflow

begin

    -------------------------------------------------------------------------
    -- SystemBus Interface (System Clock Domain)
    -------------------------------------------------------------------------
    SysMcbProc: process (SysClkxCI)
    begin
        if rising_edge(SysClkxCI) then
            if (SysRstxRI = '1') then
                SysMemReqxM     <= '0';
                SysMemAckxM     <= '0';
                SysMemErrxM     <= '0';
                SysMemWrEnxM    <= '0';
                SysMemAddrxA    <= (others => '0');
                SysMemWrDataxD  <= (others => '0');
                SysMemRdDataxD  <= (others => '0');
                SysMemStatexM   <= SysIdle;

                SyncMcbAckxM    <= (others => '0');
                SyncMcbErrxM    <= (others => '0');
                SyncMcbRdDataxD <= (others => '0');
            else
                -- SystemBus Request/Acknowledge handshake
                case SysMemStatexM is
                    when SysIdle =>
                        SysMemReqxM <= '0';
                        SysMemAckxM <= '0';
                        SysMemErrxM <= '0';

                        -- Detect a new SystemBus request from SysBusStimulus
                        if (SysMemReqxMI = '1') then
                            SysMemReqxM    <= '1';
                            SysMemWrEnxM   <= SysMemWrEnxMI;
                            SysMemAddrxA   <= SysMemAddrxAI;
                            SysMemWrDataxD <= SysMemWrDataxDI;
                            SysMemStatexM  <= SysStart;
                        end if;

                    when SysStart =>
                        -- The SystemBus access is in progress - wait for the
                        -- Acknowledge or Error Response from the MCB.
                        if (SyncMcbAckxM(cSYNC_NUM_STAGES-1) = '1') then
                            SysMemReqxM    <= '0';
                            SysMemAckxM    <= '1';
                            SysMemRdDataxD <= SyncMcbRdDataxD;
                            SysMemStatexM  <= SysEnd;
                        end if;

                        if (SyncMcbErrxM(cSYNC_NUM_STAGES-1) = '1') then
                            SysMemReqxM   <= '0';
                            SysMemErrxM   <= '1';
                            SysMemStatexM <= SysEnd;
                        end if;

                    when SysEnd =>
                        SysMemAckxM <= '0';
                        SysMemErrxM <= '0';

                        -- The Acknowledge or Error Response has been sent to the SysBusStimulus module.
                        -- Now wait for the synchronization pipeline to clear before returning to Idle.
                        if ((SyncMcbAckxM(cSYNC_NUM_STAGES-1) = '0') and (SyncMcbErrxM(cSYNC_NUM_STAGES-1) = '0')) then
                            SysMemStatexM <= SysIdle;
                        end if;

                    when others => NULL;
                end case;

                -- Synchronize outgoing SystemBus signals to SysClkxCI User clock
                SyncMcbAckxM    <= SyncMcbAckxM(cSYNC_NUM_STAGES-2 downto 0) & McbSysAckxM;
                SyncMcbErrxM    <= SyncMcbErrxM(cSYNC_NUM_STAGES-2 downto 0) & McbSysErrxM;
                SyncMcbRdDataxD <= McbSysRdDataxD;
            end if;
        end if;
    end process;

    -- Assign SystemBus handshake signals
    SysMemAckxMO    <= SysMemAckxM;
    SysMemErrxMO    <= SysMemErrxM;
    SysMemRdDataxDO <= SysMemRdDataxD;

    -------------------------------------------------------------------------
    -- SystemBus Interface (MCB User Clock Domain)
    -------------------------------------------------------------------------
    McbSysProc: process (McbAppClkxCI)
    begin
        if rising_edge(McbAppClkxCI) then
            if (McbAppRstxRI = '1') then
                McbSysWrReqxM   <= '0';
                McbSysRdReqxM   <= '0';
                McbSysAckxM     <= '0';
                McbSysErrxM     <= '0';
                McbSysAddrxA    <= (others => '0');
                McbSysWrDataxD  <= (others => '0');
                McbSysRdDataxD  <= (others => '0');
                McbSysStatexM   <= McbSysIdle;

                SyncMemReqxM    <= (others => '0');
                SyncMemWrEnxM   <= '0';
                SyncMemAddrxA   <= (others => '0');
                SyncMemWrDataxD <= (others => '0');
            else
                -- MCB SystemBus Request/Acknowledge handshake
                case McbSysStatexM is
                    when McbSysIdle =>
                        McbSysAckxM   <= '0';
                        McbSysErrxM   <= '0';
                        McbSysWrReqxM <= '0';
                        McbSysRdReqxM <= '0';

                        -- Detect a new SystemBus request from SystemBus Interface
                        if (SyncMemReqxM(cSYNC_NUM_STAGES-1) = '1') then
                            if (SyncMemWrEnxM = '1') then
                                McbSysWrReqxM <= '1';
                            else
                                McbSysRdReqxM <= '1';
                            end if;

                            -- Capture synchronized SystemBus signals
                            McbSysAddrxA   <= SyncMemAddrxA;
                            McbSysWrDataxD <= SyncMemWrDataxD;
                            McbSysStatexM  <= McbSysStart;
                        end if;

                    when McbSysStart =>
                        -- An MCB read or write cycle is in progress. Wait for
                        -- the MCB Command Interface to acknowledge the request.
                        if (McbCmdAckxM = '1') then
                            McbSysWrReqxM  <= '0';
                            McbSysRdReqxM  <= '0';
                            McbSysAckxM    <= '1';
                            McbSysRdDataxD <= McbCmdRdDataxD;
                            McbSysStatexM  <= McbSysEnd;
                        end if;

                        -- If an Error Response is received from the MCB Command
                        -- Interface pass it on to the MCB SystemBus Interface. 
                        if (McbCmdErrxM = '1') then
                            McbSysWrReqxM <= '0';
                            McbSysRdReqxM <= '0';
                            McbSysErrxM   <= '1';
                            McbSysStatexM <= McbSysEnd;
                        end if;

                    when McbSysEnd =>
                        -- Wait for the MCB SystemBus Interface to de-assert
                        -- SysMemReqxM before returning to Idle.
                        if (SyncMemReqxM(cSYNC_NUM_STAGES-1) = '0') then
                            McbSysAckxM   <= '0';
                            McbSysErrxM   <= '0';
                            McbSysStatexM <= McbSysIdle;
                        end if;

                    when others => NULL;
                end case;

                -- Synchronize incoming SystemBus signals to MCB User clock
                SyncMemReqxM    <= SyncMemReqxM(cSYNC_NUM_STAGES-2 downto 0) & SysMemReqxM;
                SyncMemWrEnxM   <= SysMemWrEnxM;
                SyncMemAddrxA   <= SysMemAddrxA;
                SyncMemWrDataxD <= SysMemWrDataxD;
            end if;
        end if;
    end process;

    -------------------------------------------------------------------------
    -- MCB Command FIFO
    -------------------------------------------------------------------------
    McbCmdFifoInst: FIFO18E1
    generic map (
        ALMOST_EMPTY_OFFSET     => X"0020",                 -- 13-bit almost empty threshold
        ALMOST_FULL_OFFSET      => X"0030",                 -- 13-bit almost full threshold
        DATA_WIDTH              => 36,                      -- Sets data width to [4, 9, 18, 36]
        FIFO_MODE               => "FIFO18_36",             -- Sets mode to "FIFO18" or "FIFO18_36" (FIFO18_36 requires DATA_WIDTH = 36)
        DO_REG                  => 1,                       -- Enable output register (1-0) Must be 1 if EN_SYN = FALSE
        EN_SYN                  => False,                   -- Specifies FIFO as dual-clock (FALSE) or Synchronous (TRUE)
        INIT                    => x"000000000",            -- Initial values on output port
        SRVAL                   => x"000000000",            -- Set/Reset value for output port
        FIRST_WORD_FALL_THROUGH => True,                    -- Sets the FIFO FWFT to FALSE, TRUE
        SIM_DEVICE              => "7SERIES")               -- Must be set to "7SERIES" for simulation behavior
    port map (
        -- FIFO Infrastructure
        RST         => SysRstxRI,                           -- 1-bit input: Asynchronous Reset
        REGCE       => '1',                                 -- 1-bit input: Clock enable
        RSTREG      => SysRstxRI,                           -- 1-bit input: Output register set/reset

        -- Write Port
        WRCLK       => SysClkxCI,                           -- 1-bit input: Write clock
        WREN        => McbCmdFifoWrEnxM,                    -- 1-bit input: Write enable
        DI          => McbCmdFifoWrDataxD(31 downto 0),     -- 32-bit input: Data input
        DIP         => McbCmdFifoWrDataxD(35 downto 32),    -- 4-bit input: Parity input

        -- Read Port
        RDCLK       => McbAppClkxCI,                        -- 1-bit input: Read clock
        RDEN        => McbCmdFifoRdEnxM,                    -- 1-bit input: Read enable
        DO          => McbCmdFifoRdDataxD(31 downto 0),     -- 32-bit output: Data output
        DOP         => McbCmdFifoRdDataxD(35 downto 32),    -- 4-bit output: Parity data output

        -- FIFO status signals
        EMPTY       => McbCmdFifoEmptyxS,                   -- 1-bit output: Empty flag
        ALMOSTEMPTY => open,                                -- 1-bit output: Almost empty flag
        ALMOSTFULL  => open,                                -- 1-bit output: Almost full flag
        FULL        => McbCmdFifoFullxS,                    -- 1-bit output: Full flag

        WRCOUNT     => open,                                -- 12-bit output: Write count
        WRERR       => open,                                --  1-bit output: Write error
        RDCOUNT     => open,                                -- 12-bit output: Read count
        RDERR       => open);                               --  1-bit output: Read error

    -- Indicate a valid write operation to the MCB Command FIFO.
    McbCmdFifoWrEnxM <= '1' when ((CustMcbCmdValxMI = '1') and (CustMcbCmdRdyxM = '1')) else '0';
    CustMcbCmdRdyxM  <= '1' when (McbCmdFifoFullxS = '0') else '0';

    -- Assign MCB Command FIFO data input bus. Ignore the 2 LSB address lines for a
    -- 32-bit data port width, CustMcbCmdAddrxAI(1..0) must always be always "00".
    McbCmdFifoWrDataxD(0)           <= CustMcbCmdInstxMI(0);
    McbCmdFifoWrDataxD(6 downto 1)  <= CustMcbCmdLenxMI;
    McbCmdFifoWrDataxD(34 downto 7) <= CustMcbCmdAddrxAI(gCUST_ADDR_BUS_WIDTH-1 downto 2);
    McbCmdFifoWrDataxD(35)          <= '0';

    -------------------------------------------------------------------------
    -- MCB Write FIFO
    -------------------------------------------------------------------------
    McbWrFifoInst: FIFO18E1
    generic map (
        ALMOST_EMPTY_OFFSET     => X"0020",             -- 13-bit almost empty threshold
        ALMOST_FULL_OFFSET      => X"0030",             -- 13-bit almost full threshold
        DATA_WIDTH              => 36,                  -- Sets data width to [4, 9, 18, 36]
        FIFO_MODE               => "FIFO18_36",         -- Sets mode to "FIFO18" or "FIFO18_36" (FIFO18_36 requires DATA_WIDTH = 36)
        DO_REG                  => 1,                   -- Enable output register (1-0) Must be 1 if EN_SYN = FALSE
        EN_SYN                  => False,               -- Specifies FIFO as dual-clock (FALSE) or Synchronous (TRUE)
        INIT                    => x"000000000",        -- Initial values on output port
        SRVAL                   => x"000000000",        -- Set/Reset value for output port
        FIRST_WORD_FALL_THROUGH => True,                -- Sets the FIFO FWFT to FALSE, TRUE
        SIM_DEVICE              => "7SERIES")           -- Must be set to "7SERIES" for simulation behavior
    port map (
        -- FIFO Infrastructure
        RST         => SysRstxRI,                       -- 1-bit input: Asynchronous Reset
        REGCE       => '1',                             -- 1-bit input: Clock enable
        RSTREG      => SysRstxRI,                       -- 1-bit input: Output register set/reset

        -- Write Port
        WRCLK       => SysClkxCI,                       -- 1-bit input: Write clock
        WREN        => McbWrFifoWrEnxM,                 -- 1-bit input: Write enable
        DI          => McbWrFifoWrDataxD,               -- 32-bit input: Data input
        DIP         => (others => '0'),                 -- 4-bit input: Parity input

        -- Read Port
        RDCLK       => McbAppClkxCI,                    -- 1-bit input: Read clock
        RDEN        => McbWrFifoRdEnxM,                 -- 1-bit input: Read enable
        DO          => McbWrFifoRdDataxD,               -- 32-bit output: Data output
        DOP         => open,                            -- 4-bit output: Parity data output

        -- FIFO status signals
        EMPTY       => McbWrFifoEmptyxS,                -- 1-bit output: Empty flag
        ALMOSTEMPTY => open,                            -- 1-bit output: Almost empty flag
        ALMOSTFULL  => open,                            -- 1-bit output: Almost full flag
        FULL        => McbWrFifoFullxS,                 -- 1-bit output: Full flag

        WRCOUNT     => open,                            -- 12-bit output: Write count
        WRERR       => open,                            --  1-bit output: Write error
        RDCOUNT     => open,                            -- 12-bit output: Read count
        RDERR       => open);                           --  1-bit output: Read error

    -- Indicate a valid write operation to the MCB Write FIFO.
    McbWrFifoWrEnxM <= '1' when ((CustMcbWrValxMI = '1') and (CustMcbWrRdyxM = '1')) else '0';
    CustMcbWrRdyxM  <= '1' when (McbWrFifoFullxS = '0') else '0';

    -- Write data multiplexer to the MCB Write FIFO
    McbWrFifoWrDataxD <= CustMcbWrDataxDI;

    -------------------------------------------------------------------------
    -- MCB Read FIFO
    -------------------------------------------------------------------------
    McbRdFifoInst: FIFO18E1
    generic map (
        ALMOST_EMPTY_OFFSET     => X"0020",             -- 13-bit almost empty threshold
        ALMOST_FULL_OFFSET      => X"0030",             -- 13-bit almost full threshold
        DATA_WIDTH              => 36,                  -- Sets data width to [4, 9, 18, 36]
        FIFO_MODE               => "FIFO18_36",         -- Sets mode to "FIFO18" or "FIFO18_36" (FIFO18_36 requires DATA_WIDTH = 36)
        DO_REG                  => 1,                   -- Enable output register (1-0) Must be 1 if EN_SYN = FALSE
        EN_SYN                  => False,               -- Specifies FIFO as dual-clock (FALSE) or Synchronous (TRUE)
        INIT                    => x"000000000",        -- Initial values on output port
        SRVAL                   => x"000000000",        -- Set/Reset value for output port
        FIRST_WORD_FALL_THROUGH => True,                -- Sets the FIFO FWFT to FALSE, TRUE
        SIM_DEVICE              => "7SERIES")           -- Must be set to "7SERIES" for simulation behavior
    port map (
        -- FIFO Infrastructure
        RST         => SysRstxRI,                       -- 1-bit input: Asynchronous Reset
        REGCE       => '1',                             -- 1-bit input: Clock enable
        RSTREG      => SysRstxRI,                       -- 1-bit input: Output register set/reset

        -- Write Port
        WRCLK       => McbAppClkxCI,                    -- 1-bit input: Write clock
        WREN        => McbRdFifoWrEnxM,                 -- 1-bit input: Write enable
        DI          => McbRdFifoWrDataxD,               -- 32-bit input: Data input
        DIP         => (others => '0'),                 -- 4-bit input: Parity input

        -- Read Port
        RDCLK       => SysClkxCI,                       -- 1-bit input: Read clock
        RDEN        => McbRdFifoRdEnxM,                 -- 1-bit input: Read enable
        DO          => McbRdFifoRdDataxD,               -- 32-bit output: Data output
        DOP         => open,                            -- 4-bit output: Parity data output

        -- FIFO status signals
        EMPTY       => McbRdFifoEmptyxS,                -- 1-bit output: Empty flag
        ALMOSTEMPTY => open,                            -- 1-bit output: Almost empty flag
        ALMOSTFULL  => open,                            -- 1-bit output: Almost full flag
        FULL        => McbRdFifoFullxS,                 -- 1-bit output: Full flag

        WRCOUNT     => open,                            -- 12-bit output: Write count
        WRERR       => open,                            --  1-bit output: Write error
        RDCOUNT     => open,                            -- 12-bit output: Read count
        RDERR       => open);                           --  1-bit output: Read error

    -- Indicate a valid write operation to the MCB Read FIFO.
    McbRdFifoWrEnxM <= '1' when ((McbRdFifoWrReqxM = '1') and (McbRdFifoFullxS = '0')) else '0';

    -- Indicate a valid read operation from the MCB Read FIFO.
    McbRdFifoRdEnxM <= '1' when ((CustMcbRdValxMI = '1') and (CustMcbRdRdyxM = '1')) else '0';
    CustMcbRdRdyxM  <= '1' when (McbRdFifoEmptyxS = '0') else '0';
    CustMcbRdDataxD <= McbRdFifoRdDataxD;

    -------------------------------------------------------------------------
    -- MCB Command Interface
    -------------------------------------------------------------------------
    McbCmdProc: process (McbAppClkxCI)
    begin
        if rising_edge(McbAppClkxCI) then
            if (McbAppRstxRI = '1') then
                McbCmdAckxM      <= '0';
                McbCmdErrxM      <= '0';
                McbCmdRdDataxD   <= (others => '0');

                McbCmdFifoRdEnxM <= '0';
                McbCmdSysReqxM   <= '0';
                McbCmdWrReqxM    <= '0';
                McbCmdRdReqxM    <= '0';
                McbAppCmdEnxM    <= '0';
                McbAppCmdNumxM   <= (others => '0');
                McbAppCmdBLenxM  <= (others => '0');
                McbAppCmdCmdxM   <= (others => '0');
                McbAppCmdWdSelxM <= (others => '0');
                McbAppCmdAddrxA  <= (others => '0');
                McbCmdStatexM    <= CmdIdle;
            else
                -- MCB Command Interface Controller
                case McbCmdStatexM is
                    when CmdIdle =>
                        McbCmdAckxM      <= '0';
                        McbCmdErrxM      <= '0';
                        McbAppCmdEnxM    <= '0';
                        McbCmdSysReqxM   <= '0';
                        McbCmdWrReqxM    <= '0';
                        McbCmdRdReqxM    <= '0';
                        McbCmdFifoRdEnxM <= '0';

                        -- A SystemBus access to the MCB is pending and enabled.
                        if ((McbSysWrReqxM = '1') or (McbSysRdReqxM = '1')) then
                            -- Indicate that an MCB SystemBus access is requested.
                            McbCmdSysReqxM <= '1';

                            -- The number of required MCB command/data cycles and the burst length is 
                            -- always 0 (i.e. 1 MCB cycle) for SystemBus cycles.
                            McbAppCmdCmdxM  <= "00" & McbSysRdReqxM;
                            McbAppCmdNumxM  <= (others => '0');
                            McbAppCmdBLenxM <= (others => '0');

                            -- Set the 2-bit start address of the first DWORD within the first 128-bit MCB data word.
                            McbAppCmdWdSelxM <= McbSysAddrxA(3 downto 2);
                            McbAppCmdAddrxA  <= McbSysAddrxA(29 downto 4);

                            -- Request one or more MCB data transfers from the selected data interface.
                            if (McbSysWrReqxM = '1') then
                                McbCmdWrReqxM <= '1';
                                McbCmdStatexM <= CmdWrite;
                            else
                                McbCmdRdReqxM <= '1';
                                McbCmdStatexM <= CmdRead;
                            end if;

                        elsif (McbCmdFifoEmptyxS = '0') then
                            -- If a valid command is pending at the MCB Command FIFO Calculate the number of
                            -- required MCB command/data cycles to transfer the requested burst by adding the
                            -- 2-bit DWORD start address to the Burst-Length register value and then disregard
                            -- the lower 2 LSBs from the result. McbAppCmdNumxM is zero-based,
                            -- i.e. McbAppCmdNumxM = Actual-Number-of-Cycles - 1.
                            McbAppCmdNumxM <= McbCmdEndAddrxA(gCUST_BLEN_BUS_WIDTH downto 2);

                            -- Set the command type and the required burst length.
                            -- McbAppCmdBLenxM contains the number of required DWORD transfers - 1.
                            McbAppCmdCmdxM  <= "00" & McbCmdFifoRdDataxD(0);
                            McbAppCmdBLenxM <= McbCmdFifoRdDataxD(6 downto 1);

                            -- Set the 2-bit start address of the first DWORD within the first 128-bit MCB data word.
                            McbAppCmdWdSelxM <= McbCmdFifoRdDataxD(8 downto 7);

                            -- Set the address of the first 128-bit MCB data word.
                            McbAppCmdAddrxA <= McbCmdFifoRdDataxD(34 downto 9);

                            -- All data has been captured from the MCB Command
                            -- FIFO. Remove the command from the FIFO.
                            McbCmdFifoRdEnxM <= '1';

                            -- Request one or more MCB data transfers from the selected data interface.
                            if (McbCmdFifoRdDataxD(0) = cMCB_WRITE_COMMAND) then
                                McbCmdWrReqxM <= '1';
                                McbCmdStatexM <= CmdWrite;
                            else
                                McbCmdRdReqxM <= '1';
                                McbCmdStatexM <= CmdRead;
                            end if;
                        end if;

                    when CmdWrite =>
                        McbCmdFifoRdEnxM <= '0';

                        -- When the MCB Write Interface acknowledges that the write data has
                        -- been transferred to the MCB, issue the corresponding MCB Write Command.
                        if (McbCmdWrAckxM = '1') then
                            McbAppCmdEnxM <= '1';
                            McbCmdWrReqxM <= '0';
                            McbCmdStatexM <= CmdTransfer;
                        end if;

                    when CmdRead =>
                        McbCmdFifoRdEnxM <= '0';

                        -- The MCB Read Interface acknowledges that it is ready to accept
                        -- new read data from the MCB. Issue the MCB Read Command.
                        if (McbCmdRdAckxM = '1') then
                            McbAppCmdEnxM  <= '1';
                            McbCmdRdReqxM  <= '0';
                            McbCmdStatexM  <= CmdTransfer;
                        end if;

                    when CmdTransfer =>
                        -- When a read or write command has been accepted by the MCB prepare
                        -- for the next MCB data transfer, unless the last data transfer has
                        -- just been processed, i.e. when McbAppCmdNumxM = 0.
                        if ((McbAppCmdEnxM = '1') and (McbAppCmdRdyxMI = '1')) then
                            McbAppCmdEnxM <= '0';

                            -- Decrease the value of the MCB burst length register by the
                            -- number of DWORDs transmitted in the current MCB data transfer.
                            McbAppCmdBLenxM <= std_logic_vector(unsigned(McbAppCmdBLenxM) - unsigned(McbAppCmdWdNumxM));

                            -- Count the number of required MCB command/data cycles to transfer the entire Custom Module burst.
                            McbAppCmdNumxM <= std_logic_vector(unsigned(McbAppCmdNumxM) - 1);

                            -- Calculate the address of the next 128-bit data word.
                            McbAppCmdWdSelxM <= (others => '0');
                            McbAppCmdAddrxA  <= std_logic_vector(unsigned(McbAppCmdAddrxA) + 1);

                            -- Determine the next MCB Write state
                            if (McbCmdSysReqxM = '1') then
                                -- Acknowledge a SystemBus write cycle when the 
                                -- Write Command has been transferred to the MCB.
                                if (McbSysWrReqxM = '1') then
                                    McbCmdAckxM   <= '1';
                                    McbCmdErrxM   <= '0';
                                    McbCmdStatexM <= CmdCycEnd;
                                else
                                    -- For a SystemBus read cycle wait until the 
                                    -- requested read data is returned from the MCB.
                                    McbCmdStatexM <= CmdRdWait;
                                end if;
                            elsif (McbAppCmdNumxM = cMCB_NUM_CYC_ZERO) then
                                McbCmdStatexM <= CmdIdle;
                            elsif (McbAppCmdCmdxM(0) = cMCB_WRITE_COMMAND) then
                                McbCmdWrReqxM <= '1';
                                McbCmdStatexM <= CmdWrite;
                            elsif (McbAppCmdCmdxM(0) = cMCB_READ_COMMAND) then
                                McbCmdRdReqxM <= '1';
                                McbCmdStatexM <= CmdRead;
                            end if;
                        end if;

                    when CmdRdWait =>
                        -- For a SystemBus read cycle wait for the MCB read data to arrive.
                        if (McbMemRdValidxM = '1') then
                            McbCmdAckxM    <= '1';
                            McbCmdErrxM    <= '0';
                            McbCmdRdDataxD <= McbMemRdDataxD;
                            McbCmdStatexM  <= CmdCycEnd;
                        end if;

                    when CmdCycEnd =>
                        -- After a SystemBus write cycle wait for the MCB Write Request signal to de-assert. Then return to Idle.
                        if ((McbSysWrReqxM = '0') and (McbSysRdReqxM = '0')) then
                            McbCmdAckxM   <= '0';
                            McbCmdErrxM   <= '0';
                            McbCmdStatexM <= CmdIdle;
                        end if;

                    when others => NULL;
                end case;
            end if;
        end if;
    end process;

    -- Partial MCB burst end address, used to calculate the number of required MCB command/data cycles.
    McbCmdFifoBLenxM  <= '0' & McbCmdFifoRdDataxD(gCUST_BLEN_BUS_WIDTH downto 1);
    McbCmdFifoStartxM <= "00000" & McbCmdFifoRdDataxD(gCUST_BLEN_BUS_WIDTH+2 downto gCUST_BLEN_BUS_WIDTH+1);
    McbCmdEndAddrxA <= std_logic_vector(unsigned(McbCmdFifoBLenxM) + unsigned(McbCmdFifoStartxM));

    -- Calculate the number of DWORDs transmitted in the current MCB read/write data transfer.
    -- 4 DWORDs are transferred if the 2-bit DWORD start address (McbAppCmdWdSelxM) is "00".
    McbAppCmdWdNumxM <= std_logic_vector(to_unsigned(4, 3) - unsigned(McbAppCmdWdSelxM));

    -------------------------------------------------------------------------
    -- MCB Write Interface
    -------------------------------------------------------------------------
    McbWrProc: process (McbAppClkxCI)
    begin
        if rising_edge(McbAppClkxCI) then
            if (McbAppRstxRI = '1') then
                McbCmdWrAckxM   <= '0';
                McbWrFifoRdEnxM <= '0';
                McbWrDWordPtrxM <= 0;
                McbWrDWordCntxM <= 0;

                McbAppWrEnxM    <= '0';
                McbAppWrEndxM   <= '1';
                McbAppWrMaskxM  <= (others => cMCB_DATA_INVALID);
                McbAppWrDataxD  <= (others => '0');
                McbWrStatexM    <= WrIdle;
            else
                -- MCB Write Interface Controller
                case McbWrStatexM is
                    when WrIdle =>
                        -- NOTE: McbAppWrEndxM is permanently set to '1', because the MCB
                        -- Interface runs in 4:1 mode and the Memory Burst Type is BL8.
                        -- This results in a single MCB data transfer per MCB command.
                        McbCmdWrAckxM   <= '0';
                        McbWrFifoRdEnxM <= '0';
                        McbAppWrEnxM    <= '0';
                        McbAppWrEndxM   <= '1';
                        McbAppWrMaskxM  <= (others => cMCB_DATA_INVALID);

                        -- A new MCB Write Request is currently pending. Read data
                        -- from the MCB Write FIFO and transfer it to the MCB.
                        if (McbCmdWrReqxM = '1') then
                            if (McbCmdSysReqxM = '1') then
                                McbWrStatexM <= WrSysBus;
                            else
                                -- Read data from the MCB Write FIFO and store it in the 128-bit McbAppWrDataxD register.
                                McbWrFifoRdEnxM <= '1';
                                McbWrStatexM    <= WrFifoRead;
                            end if;

                            -- The 2 least significant MCB address bits determine the location
                            -- of the first valid DWORD within the 128-bit McbAppWrDataxD bus.
                            McbWrDWordPtrxM <= to_integer(unsigned(McbAppCmdWdSelxM));

                            -- Load the DWORD counter with the burst length value of the MCB Command.
                            McbWrDWordCntxM <= to_integer(unsigned(McbAppCmdBLenxM));
                        end if;

                    when WrSysBus =>
                        -- Perform an MCB write transfer from the SystemBus Interface 
                        -- and acknowledge the request to the MCB Command Interface.
                        McbAppWrEnxM  <= '1';
                        McbCmdWrAckxM <= '1';
                        McbWrStatexM  <= WrMcbTransfer;
        
                        -- Set the MCB write mask and data registers
                        McbAppWrMaskxM(McbWrDWordPtrxM) <= cMCB_DATA_VALID;
                        McbAppWrDataxD((McbWrDWordPtrxM*32)+31 downto (McbWrDWordPtrxM*32)) <= McbSysWrDataxD;

                    when WrFifoRead =>
                        -- Read the MCB Write FIFO when it contains valid output data.
                        if ((McbWrFifoRdEnxM = '1') and (McbWrFifoEmptyxS = '0')) then
                            -- Load the MCB data output register with the read data from the MCB Write
                            -- FIFO and set the Write Mask bits of the currently selected DWORD.
                            McbAppWrMaskxM(McbWrDWordPtrxM) <= cMCB_DATA_VALID;
                            McbAppWrDataxD((McbWrDWordPtrxM*32)+31 downto (McbWrDWordPtrxM*32)) <= McbWrFifoRdDataxD;

                            -- The FIFO read cycle is complete when either the most significant DWORD is
                            -- loaded into McbAppWrDataxD or when the burst length counter reaches zero.
                            -- Perform the actual data transfer to the MCB next.
                            if ((McbWrDWordPtrxM = 3) or (McbWrDWordCntxM = 0)) then
                                McbWrFifoRdEnxM <= '0';
                                McbAppWrEnxM    <= '1';
                                McbCmdWrAckxM   <= '1';
                                McbWrStatexM    <= WrMcbTransfer;
                            else
                                -- Decrement the DWORD counter and increment the DWORD pointer
                                -- with every read operation from the MCB Write FIFO.
                                McbWrDWordPtrxM <= McbWrDWordPtrxM + 1;
                                McbWrDWordCntxM <= McbWrDWordCntxM - 1;
                            end if;
                        end if;

                    when WrMcbTransfer =>
                        McbCmdWrAckxM <= '0';

                        -- Transfer the 128-bit data word to the MCB. When done, acknowledge
                        -- the transfer to the MCB Command Interface and return to idle.
                        if ((McbAppWrEnxM = '1') and (McbAppWrRdyxMI = '1')) then
                            McbAppWrEnxM <= '0';
                            McbWrStatexM <= WrIdle;
                        end if;

                    when others => NULL;
                end case;
            end if;
        end if;
    end process;

    -------------------------------------------------------------------------
    -- MCB Read Interface
    -------------------------------------------------------------------------
    McbRdProc: process (McbAppClkxCI)
        variable LoBit : natural;
    begin
        if rising_edge(McbAppClkxCI) then
            if (McbAppRstxRI = '1') then
                McbCmdRdAckxM     <= '0';
                McbRdFifoWrValxM  <= '0';
                McbRdFifoOvrFlxM  <= '0';
                McbRdCmdInPtrxM   <= 0;
                McbRdDatInPtrxM   <= 0;
                McbRdCmdDatPtrxM  <= 0;

                McbMemRdValidxM   <= '0';
                McbMemRdDataxD    <= (others => '0');

                McbRdFifoWrReqxM  <= '0';
                McbRdFifoWrEndxM  <= '0';
                McbRdFifoWrDataxD <= (others => '0');
                McbRdCmdDatBufxM  <= (others => cMCB_READ_CMD_DEFAULT);
                McbRdStatexM      <= RdIdle;
            else
                -- An MCB Read Request is pending from the MCB Command Interface. Store the Read
                -- Request data in a 4-stage rotating input buffer. Wait until an input buffer
                -- becomes available before acknowledging the request.
                if ((McbCmdRdReqxM = '1') and (McbCmdRdAckxM = '0') and (McbRdCmdDatBufxM(McbRdCmdInPtrxM).CmdValid = '0')) then
                    McbRdCmdDatBufxM(McbRdCmdInPtrxM).CmdValid <= '1';

                    -- The 2 least significant MCB address bits determine the location
                    -- of the first valid DWORD within the 128-bit McbAppRdDataxD bus.
                    McbRdCmdDatBufxM(McbRdCmdInPtrxM).DwordPtr <= to_integer(unsigned(McbAppCmdWdSelxM));

                    -- Load the DWORD counter with the burst length value of the MCB Command.
                    McbRdCmdDatBufxM(McbRdCmdInPtrxM).BstLenCnt <= to_integer(unsigned(McbAppCmdBLenxM));

                    -- Indicate that the pending read request originated from the SystemBus Interface
                    McbRdCmdDatBufxM(McbRdCmdInPtrxM).SysBusReq <= McbCmdSysReqxM;

                    -- Increment the MCB Read Command Pointer when the command has been stored.
                    if (McbRdCmdInPtrxM = cMCB_NUM_READ_BUFFER-1) then
                        McbRdCmdInPtrxM <= 0;
                    else
                        McbRdCmdInPtrxM <= McbRdCmdInPtrxM + 1;
                    end if;

                    -- Acknowledge the Read Command to the MCB Command Interface.
                    McbCmdRdAckxM <= '1';
                else
                    McbCmdRdAckxM <= '0';
                end if;

                -- An MCB Read Data transfer is pending from the MCB.
                -- Store the data in an 4-stage rotating data input buffer.
                if (McbAppRdValxMI = '1') then
                    if (McbRdCmdDatBufxM(McbRdDatInPtrxM).DataValid = '0') then
                        McbRdCmdDatBufxM(McbRdDatInPtrxM).DataValid  <= '1';
                        McbRdCmdDatBufxM(McbRdDatInPtrxM).DataBuffer <= McbAppRdDataxDI;

                        -- Increment the MCB Read Data Pointer when the data has been stored.
                        if (McbRdDatInPtrxM = cMCB_NUM_READ_BUFFER-1) then
                            McbRdDatInPtrxM <= 0;
                        else
                            McbRdDatInPtrxM <= McbRdDatInPtrxM + 1;
                        end if;
                    else
                        -- MCB Read Buffer Overflow. If no free data
                        -- buffer is available set the overflow bit.
                        McbRdFifoOvrFlxM <= '1';
                    end if;
                end if;

                -- MCB Read Interface Controller
                case McbRdStatexM is
                    when RdIdle =>
                        McbRdFifoWrValxM <= '0';
                        McbRdFifoWrReqxM <= '0';
                        McbRdFifoWrEndxM <= '0';
                        McbMemRdValidxM  <= '0';

                        -- Start the transfer of data from the MCB Read Input Buffer to the MCB Read FIFO
                        -- when the Read Command and Read Data are both available in an MCB Read Input Buffer.
                        if ((McbRdCmdDatBufxM(McbRdCmdDatPtrxM).CmdValid = '1') and (McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DataValid = '1')) then
                            if (McbRdCmdDatBufxM(McbRdCmdDatPtrxM).SysBusReq = '1') then
                                McbRdCmdDatBufxM(McbRdCmdDatPtrxM).CmdValid  <= '0';
                                McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DataValid <= '0';
    
                                -- Increment the MCB Read Command/Data Pointer
                                if (McbRdCmdDatPtrxM = cMCB_NUM_READ_BUFFER-1) then
                                    McbRdCmdDatPtrxM <= 0;
                                else
                                    McbRdCmdDatPtrxM <= McbRdCmdDatPtrxM + 1;
                                end if;

                                LoBit := McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DwordPtr*32;
                                McbMemRdDataxD  <= McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DataBuffer(LoBit+31 downto LoBit);
                                McbMemRdValidxM <= '1';
                            else
                                McbRdStatexM <= RdActive;
                            end if;
                        end if;

                    when RdActive =>
                        -- McbRdFifoWrValxM is used to pre-load data into the MCB Read FIFO for the first transfer.
                        McbRdFifoWrValxM <= '1';

                        -- Transfer one DWORD at a time from the 128-bit input buffer to the MCB Read FIFO.
                        if ((McbRdFifoWrEnxM = '1') or (McbRdFifoWrValxM = '0')) then
                            LoBit := McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DwordPtr*32;
                            McbRdFifoWrDataxD <= McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DataBuffer(LoBit+31 downto LoBit);
    
                            -- Decrement the Burst Length counter and increment the DWORD pointer
                            -- with every write operation to the MCB Read FIFO.
                            if (McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DwordPtr < 3) then
                                McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DwordPtr <= McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DwordPtr + 1;
                            end if;
    
                            if (McbRdCmdDatBufxM(McbRdCmdDatPtrxM).BstLenCnt > 0) then
                                McbRdCmdDatBufxM(McbRdCmdDatPtrxM).BstLenCnt <= McbRdCmdDatBufxM(McbRdCmdDatPtrxM).BstLenCnt - 1;
                            end if;
    
                            -- Indicate the last data transfer into the MCB Read FIFO.
                            if ((McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DwordPtr = 3) or (McbRdCmdDatBufxM(McbRdCmdDatPtrxM).BstLenCnt = 0)) then
                                McbRdFifoWrEndxM <= '1';
                            end if;
                        end if;
    
                        if ((McbRdFifoWrEnxM = '1') and (McbRdFifoWrEndxM = '1')) then
                            -- When the MCB read data has been transferred from the input buffer to the
                            -- MCB Read FIFO flag the buffer as invalid and increment the buffer pointer.
                            McbRdCmdDatBufxM(McbRdCmdDatPtrxM).CmdValid  <= '0';
                            McbRdCmdDatBufxM(McbRdCmdDatPtrxM).DataValid <= '0';
    
                            -- Increment the MCB Read Command/Data Pointer
                            if (McbRdCmdDatPtrxM = cMCB_NUM_READ_BUFFER-1) then
                                McbRdCmdDatPtrxM <= 0;
                            else
                                McbRdCmdDatPtrxM <= McbRdCmdDatPtrxM + 1;
                            end if;
    
                            -- Reset the MCB Read FIFO write request and return to idle.
                            McbRdFifoWrReqxM <= '0';
                            McbRdStatexM     <= RdIdle;
                        else
                            -- Request an MCB Read FIFO write cycle.
                            McbRdFifoWrReqxM <= '1';
                        end if;

                    when others => NULL;
                end case;
            end if;
        end if;
    end process;

    -------------------------------------------------------------------------
    -- Assign internally used output ports
    -------------------------------------------------------------------------
    CustMcbCmdRdyxMO  <= CustMcbCmdRdyxM;
    CustMcbRdRdyxMO   <= CustMcbRdRdyxM;
    CustMcbRdDataxDO  <= CustMcbRdDataxD;
    CustMcbWrRdyxMO   <= CustMcbWrRdyxM;
    McbRdFifoOvrFlxMO <= McbRdFifoOvrFlxM;

    McbAppCmdEnxMO   <= McbAppCmdEnxM;
    McbAppCmdCmdxMO  <= McbAppCmdCmdxM;
    McbAppCmdAddrxAO <= McbAppCmdAddrxA(gMCB_ADDR_BUS_WIDTH-4 downto 0) & "000";

    McbAppWrEnxMO    <= McbAppWrEnxM;
    McbAppWrEndxMO   <= McbAppWrEndxM;
    McbAppWrMaskxMO  <= McbAppWrMaskxM(3) & McbAppWrMaskxM(3) & McbAppWrMaskxM(3) & McbAppWrMaskxM(3) &
                        McbAppWrMaskxM(2) & McbAppWrMaskxM(2) & McbAppWrMaskxM(2) & McbAppWrMaskxM(2) &
                        McbAppWrMaskxM(1) & McbAppWrMaskxM(1) & McbAppWrMaskxM(1) & McbAppWrMaskxM(1) &
                        McbAppWrMaskxM(0) & McbAppWrMaskxM(0) & McbAppWrMaskxM(0) & McbAppWrMaskxM(0);
    McbAppWrDataxDO  <= McbAppWrDataxD;

    -------------------------------------------------------------------------
    -- Debug Logic
    -------------------------------------------------------------------------
    DebugGen: if (gMCB_DEBUG_ENABLE = True) generate
        signal McbCmdStateDecodexM : std_logic_vector(1 downto 0);
        signal McbWrStateDecodexM  : std_logic_vector(1 downto 0);
        signal McbRdStateDecodexM  : std_logic_vector(1 downto 0);
        signal McbDebugPortxM      : std_logic_vector(gMCB_DEBUG_WIDTH-1 downto 0); -- Debug Port
    begin
        McbCmdStateDecodexM <= std_logic_vector(to_unsigned(tMCB_CMD_FSM_STATE'pos(McbCmdStatexM), McbCmdStateDecodexM'length));
        McbWrStateDecodexM  <= std_logic_vector(to_unsigned(tMCB_WRITE_FSM_STATE'pos(McbWrStatexM), McbWrStateDecodexM'length));
        McbRdStateDecodexM  <= std_logic_vector(to_unsigned(tMCB_READ_FSM_STATE'pos(McbRdStatexM), McbRdStateDecodexM'length));

        DbgProc: process (McbAppClkxCI)
            variable i : natural;
        begin
            if rising_edge(McbAppClkxCI) then
                -- MCB Command-Interface
                i := 0;
                McbDebugPortxM(i+1 downto i)     <= McbCmdStateDecodexM;
                McbDebugPortxM(i+2)              <= McbCmdWrReqxM;
                McbDebugPortxM(i+3)              <= McbCmdWrAckxM;
                McbDebugPortxM(i+4)              <= McbCmdRdReqxM;
                McbDebugPortxM(i+5)              <= McbCmdRdAckxM;
                McbDebugPortxM(i+6)              <= McbCmdFifoWrEnxM;
                McbDebugPortxM(i+7)              <= McbCmdFifoRdEnxM;
                McbDebugPortxM(i+8)              <= McbCmdFifoEmptyxS;
                McbDebugPortxM(i+9)              <= McbCmdFifoFullxS;
                McbDebugPortxM(i+45 downto i+10) <= McbCmdFifoWrDataxD;
                McbDebugPortxM(i+81 downto i+46) <= McbCmdFifoRdDataxD;

                i := 82;
                McbDebugPortxM(i)                <= McbAppCmdEnxM;
                McbDebugPortxM(i+1)              <= McbAppCmdRdyxMI;
                McbDebugPortxM(i+4 downto i+2)   <= McbAppCmdCmdxM;
                McbDebugPortxM(i+30 downto i+5)  <= McbAppCmdAddrxA;

                i := 113;
                McbDebugPortxM(i+1 downto i)     <= McbAppCmdWdSelxM;
                McbDebugPortxM(i+4 downto i+2)   <= McbAppCmdWdNumxM;
                McbDebugPortxM(i+11 downto i+6)  <= McbAppCmdBLenxM;
                McbDebugPortxM(i+16 downto i+12) <= McbAppCmdNumxM;
                McbDebugPortxM(i+23 downto i+17) <= McbCmdEndAddrxA;

                -- MCB Write-Interface
                i := 137;
                McbDebugPortxM(i+1 downto i)     <= McbWrStateDecodexM;
                McbDebugPortxM(i+2)              <= McbWrFifoWrEnxM;
                McbDebugPortxM(i+3)              <= McbWrFifoRdEnxM;
                McbDebugPortxM(i+4)              <= McbWrFifoEmptyxS;
                McbDebugPortxM(i+5)              <= McbWrFifoFullxS;
                McbDebugPortxM(i+37 downto i+6)  <= McbWrFifoRdDataxD;
                McbDebugPortxM(i+39 downto i+38) <= std_logic_vector(to_unsigned(McbWrDWordPtrxM, 2));
                McbDebugPortxM(i+45 downto i+40) <= std_logic_vector(to_unsigned(McbWrDWordCntxM, 6));

                i := 183;
                McbDebugPortxM(i)                <= McbAppWrEnxM;
                McbDebugPortxM(i+1)              <= McbAppWrRdyxMI;
                McbDebugPortxM(i+2)              <= McbAppWrEndxM;
                McbDebugPortxM(i+6 downto i+3)   <= McbAppWrMaskxM;
                -- McbDebugPortxM(i+134 downto i+7) <= McbAppWrDataxD;

                -- MCB Read Interface
                i := 190;
                McbDebugPortxM(i)                <= McbAppRdValxMI;
                McbDebugPortxM(i+1)              <= McbAppRdEndxMI;
                -- McbDebugPortxM(i+129 downto i+2) <= McbAppRdDataxDI;

                i := 192;
                McbDebugPortxM(i+1 downto i)     <= McbRdStateDecodexM;
                McbDebugPortxM(i+2)              <= McbRdFifoWrValxM;
                McbDebugPortxM(i+3)              <= McbRdFifoWrReqxM;
                McbDebugPortxM(i+4)              <= McbRdFifoWrEndxM;
                McbDebugPortxM(i+5)              <= McbRdFifoWrEnxM;
                McbDebugPortxM(i+6)              <= McbRdFifoRdEnxM;
                McbDebugPortxM(i+7)              <= McbRdFifoEmptyxS;
                McbDebugPortxM(i+8)              <= McbRdFifoFullxS;

                i := 201;
                McbDebugPortxM(gMCB_DEBUG_WIDTH-1 downto i) <= (others => '0');
            end if;
        end process;

        -- Assign debug output port
        McbDebugPortxMO <= McbDebugPortxM;
    end generate;

    -- Define Debug Port, if not used
    NoDebugGen: if (gMCB_DEBUG_ENABLE = False) generate
        McbDebugPortxMO <= (others => '0');
    end generate;

end Behavioral;
