-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Simulation Testbench
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-09-06  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library std;
use std.textio.all;

library UNISIM;
use UNISIM.vcomponents.all;
library unimacro;
use unimacro.vcomponents.all;

library work;
use work.CfgModule.all;
use work.CustomCfgLib.all;
use work.CustomSimLib.all;
use work.CustomFctLib.all;
use work.CustomPkg.all;

entity CustomTB is
    -- Empty entity declaration for top level simulation test bench
end CustomTB;

architecture Behavioral of CustomTB is

    -----------------------------------------------------------------------------
    -- Component Definitions
    -----------------------------------------------------------------------------
    component CustomModule is
    port (
        -- Module Infrastructure Signals
        SysClkxCI           : in  std_logic;                        -- Custom Module Clock 150 MHz
        SysRstxRI           : in  std_logic;                        -- Custom Module Reset
        ImgRstxRI           : in  std_logic;                        -- ImageBus Reset

        -- SystemBus Control Interface
        SysReqxMI           : in  std_logic;                        -- SystemBus Cycle Request
        SysAckxMO           : out std_logic;                        -- SystemBus Cycle Acknowledge
        SysErrxMO           : out std_logic;                        -- SystemBus Cycle Error
        SysWrEnxMI          : in  std_logic;                        -- SystemBus Write Enable
        SysAddrxAI          : in  std_logic_vector(16 downto 0);    -- SystemBus Address
        SysWrDataxDI        : in  std_logic_vector(31 downto 0);    -- SystemBus Write Data
        SysRdDataxDO        : out std_logic_vector(31 downto 0);    -- SystemBus Read Data

        -- Custom Module GPIO
        CustIoCtrlxMO       : out std_logic_vector(5 downto 0);     -- GPIO Output Control
        CustIoDataOutxDO    : out std_logic_vector(5 downto 0);     -- GPIO Output Data
        CustIoDataInxDI     : in  std_logic_vector(5 downto 0);     -- GPIO Input Data

        -- ImageBus Receive Port
        ImgRxPixValxMI      : in  std_logic;                        -- ImageBus Receive Pixel Valid
        ImgRxPixRdyxMO      : out std_logic;                        -- ImageBus Receive Pixel Ready
        ImgRxPixStatxSI     : in  std_logic_vector(2 downto 0);     -- ImageBus Receive Pixel Status
        ImgRxPixDataxDI     : in  std_logic_vector(35 downto 0);    -- ImageBus Receive Pixel Data

        -- ImageBus Transmit Port
        ImgTxPixValxMO      : out std_logic;                        -- ImageBus Transmit Pixel Valid
        ImgTxPixRdyxMI      : in  std_logic;                        -- ImageBus Transmit Pixel Ready
        ImgTxPixStatxSO     : out std_logic_vector(2 downto 0);     -- ImageBus Transmit Pixel Status
        ImgTxPixDataxDO     : out std_logic_vector(35 downto 0);    -- ImageBus Transmit Pixel Data

        -- Custom Module MCB Command Port
        px_cmd_en_o         : out std_logic;                        -- MCB Command Valid
        px_cmd_full_i       : in  std_logic;                        -- MCB Command Buffer Full
        px_cmd_instr_o      : out std_logic_vector(2 downto 0);     -- MCB Command Instruction
        px_cmd_bl_o         : out std_logic_vector(5 downto 0);     -- MCB Command Burst Length
        px_cmd_addr_o       : out std_logic_vector(29 downto 0);    -- MCB Command Address

        -- Custom Module MCB Read Port
        px_rd_en_o          : out std_logic;                        -- MCB Read Data Valid
        px_rd_empty_i       : in  std_logic;                        -- MCB Read Data Buffer Empty
        px_rd_data_i        : in  std_logic_vector(31 downto 0);    -- MCB Read Data Bus

        -- Custom Module MCB Write Port
        px_wr_en_o          : out std_logic;                        -- MCB Write Data Valid
        px_wr_full_i        : in  std_logic;                        -- MCB Write Data Buffer Full
        px_wr_data_o        : out std_logic_vector(31 downto 0));   -- MCB Write Data Bus
    end component;

    component CustomMcbInterface is
    generic (
        gCUST_INSTR_BUS_WIDTH : integer := 3;                                           -- Width of Custom Module Instruction bus
        gCUST_BLEN_BUS_WIDTH  : integer := 6;                                           -- Width of Custom Module Burst-Length bus
        gCUST_ADDR_BUS_WIDTH  : integer := 30;                                          -- Width of Custom Module Address bus
        gCUST_DATA_BUS_WIDTH  : integer := 32;                                          -- Width of Custom Module Data bus

        gMCB_CMD_BUS_WIDTH    : integer := 3;                                           -- Width of MCB Command bus
        gMCB_ADDR_BUS_WIDTH   : integer := 27;                                          -- Width of MCB Address bus
        gMCB_MASK_BUS_WIDTH   : integer := 16;                                          -- Width of MCB Mask bus
        gMCB_DATA_BUS_WIDTH   : integer := 128;                                         -- Width of MCB Data bus

        gMCB_DEBUG_ENABLE     : boolean := True;                                        -- Enable Debug Port logic
        gMCB_DEBUG_WIDTH      : integer := 256);                                        -- Width of Debug Port
    port (
        -- Infrastructure Signals
        SysClkxCI           : in  std_logic;                                            -- SystemBus Clock
        SysRstxRI           : in  std_logic;                                            -- SystemBus Reset

        -- SystemBus Control Interface
        SysMemReqxMI        : in  std_logic;                                            -- SystemBus Memory Request
        SysMemAckxMO        : out std_logic;                                            -- SystemBus Memory Acknowledge
        SysMemErrxMO        : out std_logic;                                            -- SystemBus Memory Error
        SysMemWrEnxMI       : in  std_logic;                                            -- SystemBus Memory Write Enable
        SysMemAddrxAI       : in  std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);         -- SystemBus Memory Address
        SysMemWrDataxDI     : in  std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);         -- SystemBus Memory Write Data
        SysMemRdDataxDO     : out std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);         -- SystemBus Memory Read Data

        -------------------------------------------------------------------------------

        -- Custom Module Command Port
        CustMcbCmdValxMI    : in  std_logic;                                            -- Custom Module Command Valid
        CustMcbCmdRdyxMO    : out std_logic;                                            -- Custom Module Command Ready
        CustMcbCmdInstxMI   : in  std_logic_vector(gCUST_INSTR_BUS_WIDTH-1 downto 0);   -- Custom Module Command Instruction
        CustMcbCmdLenxMI    : in  std_logic_vector(gCUST_BLEN_BUS_WIDTH-1 downto 0);    -- Custom Module Command Burst Length
        CustMcbCmdAddrxAI   : in  std_logic_vector(gCUST_ADDR_BUS_WIDTH-1 downto 0);    -- Custom Module Command Address

        -- Custom Module Write Port
        CustMcbWrValxMI     : in  std_logic;                                            -- Custom Module Write Data Valid
        CustMcbWrRdyxMO     : out std_logic;                                            -- Custom Module Write Data Ready
        CustMcbWrDataxDI    : in  std_logic_vector(gCUST_DATA_BUS_WIDTH-1 downto 0);    -- Custom Module Write Data Bus

        -- Custom Module Read Port
        CustMcbRdValxMI     : in  std_logic;                                            -- Custom Module Read Data Valid
        CustMcbRdRdyxMO     : out std_logic;                                            -- Custom Module Read Data Ready
        CustMcbRdDataxDO    : out std_logic_vector(gCUST_DATA_BUS_WIDTH-1 downto 0);    -- Custom Module Read Data Bus

        -------------------------------------------------------------------------------

        -- MCB User-Application Infrastructure
        McbAppClkxCI        : in  std_logic;                                            -- MCB User-Application Interface Clock
        McbAppRstxRI        : in  std_logic;                                            -- MCB User-Application Interface Reset
        McbCalibDonexSI     : in  std_logic;                                            -- MCB Calibration complete
        McbRdFifoOvrFlxMO   : out std_logic;                                            -- MCB Read FIFO Overflow

        -- MCB User-Application Command-Interface
        McbAppCmdEnxMO      : out std_logic;                                            -- MCB User-Application Command Enable
        McbAppCmdRdyxMI     : in  std_logic;                                            -- MCB User-Application Command Ready
        McbAppCmdCmdxMO     : out std_logic_vector(gMCB_CMD_BUS_WIDTH-1 downto 0);      -- MCB User-Application Command Instruction
        McbAppCmdAddrxAO    : out std_logic_vector(gMCB_ADDR_BUS_WIDTH-1 downto 0);     -- MCB User-Application Command Address

        -- MCB User-Application Write-Data-Interface
        McbAppWrEnxMO       : out std_logic;                                            -- MCB User-Application Write Enable
        McbAppWrRdyxMI      : in  std_logic;                                            -- MCB User-Application Write Ready
        McbAppWrEndxMO      : out std_logic;                                            -- MCB User-Application Write End
        McbAppWrMaskxMO     : out std_logic_vector(gMCB_MASK_BUS_WIDTH-1 downto 0);     -- MCB User-Application Write Maks
        McbAppWrDataxDO     : out std_logic_vector(gMCB_DATA_BUS_WIDTH-1 downto 0);     -- MCB User-Application Write Data

        -- MCB User-Application Read-Data-Interface
        McbAppRdValxMI      : in  std_logic;                                            -- MCB User-Application Read Valid
        McbAppRdEndxMI      : in  std_logic;                                            -- MCB User-Application Read End
        McbAppRdDataxDI     : in  std_logic_vector(gMCB_DATA_BUS_WIDTH-1 downto 0);     -- MCB User-Application Read Data

        -- Debug Port
        McbDebugPortxMO     : out std_logic_vector(gMCB_DEBUG_WIDTH-1 downto 0));       -- Debug Port
    end component;

    component CustomRAM is
    generic (
        gCUSTOM_RAM_SIZE    : integer := 32768;                     -- Size of CustomRAM memory [gMCB_DATA_BUS_WIDTH]
        gMCB_CMD_BUS_WIDTH  : integer := 3;                         -- Width of MCB Command bus
        gMCB_ADDR_BUS_WIDTH : integer := 27;                        -- Width of MCB Address bus
        gMCB_MASK_BUS_WIDTH : integer := 16;                        -- Width of MCB Mask bus
        gMCB_DATA_BUS_WIDTH : integer := 128);                      -- Width of MCB Data bus
    port (
        -- MCB User-Application Infrastructure
        McbAppClkxCO        : out std_logic;                                            -- MCB User-Application Interface Clock
        McbAppRstxRO        : out std_logic;                                            -- MCB User-Application Interface Reset
        McbCalibDonexSO     : out std_logic;                                            -- MCB Calibration complete

        -- MCB User-Application Command-Interface
        McbAppCmdEnxMI      : in  std_logic;                                            -- MCB User-Application Command Enable
        McbAppCmdRdyxMO     : out std_logic;                                            -- MCB User-Application Command Ready
        McbAppCmdCmdxMI     : in  std_logic_vector(gMCB_CMD_BUS_WIDTH-1 downto 0);      -- MCB User-Application Command Instruction
        McbAppCmdAddrxAI    : in  std_logic_vector(gMCB_ADDR_BUS_WIDTH-1 downto 0);     -- MCB User-Application Command Address

        -- MCB User-Application Write-Data-Interface
        McbAppWrEnxMI       : in  std_logic;                                            -- MCB User-Application Write Enable
        McbAppWrRdyxMO      : out std_logic;                                            -- MCB User-Application Write Ready
        McbAppWrEndxMI      : in  std_logic;                                            -- MCB User-Application Write End
        McbAppWrMaskxMI     : in  std_logic_vector(gMCB_MASK_BUS_WIDTH-1 downto 0);     -- MCB User-Application Write Maks
        McbAppWrDataxDI     : in  std_logic_vector(gMCB_DATA_BUS_WIDTH-1 downto 0);     -- MCB User-Application Write Data

        -- MCB User-Application Read-Data-Interface
        McbAppRdValxMO      : out std_logic;                                            -- MCB User-Application Read Valid
        McbAppRdEndxMO      : out std_logic;                                            -- MCB User-Application Read End
        McbAppRdDataxDO     : out std_logic_vector(gMCB_DATA_BUS_WIDTH-1 downto 0));    -- MCB User-Application Read Data
    end component;

    component MCB_SIM_MT41K64M16XX107 is
    port (
        -- Global Infrastructure Signals
        clk_ref_i           : in    std_logic;
        sys_rst             : in    std_logic;

        -- User Infrastructure Signals
        sys_clk_i           : in    std_logic;
        ui_clk              : out   std_logic;
        ui_clk_sync_rst     : out   std_logic;
        init_calib_complete : out   std_logic;
        device_temp_o       : out   std_logic_vector(11 downto 0);

        -- User-Command-Interface
        app_en              : in    std_logic;
        app_rdy             : out   std_logic;
        app_cmd             : in    std_logic_vector(2 downto 0);
        app_addr            : in    std_logic_vector(26 downto 0);

        -- User-Write-Data-Interface
        app_wdf_wren        : in    std_logic;
        app_wdf_rdy         : out   std_logic;
        app_wdf_end         : in    std_logic;
        app_wdf_mask        : in    std_logic_vector(15 downto 0);
        app_wdf_data        : in    std_logic_vector(127 downto 0);

        -- User-Read-Data-Interface
        app_rd_data_valid   : out   std_logic;
        app_rd_data_end     : out   std_logic;
        app_rd_data         : out   std_logic_vector(127 downto 0);

        -- Application Interface
        app_sr_req          : in    std_logic;
        app_sr_active       : out   std_logic;
        app_ref_req         : in    std_logic;
        app_ref_ack         : out   std_logic;
        app_zq_req          : in    std_logic;
        app_zq_ack          : out   std_logic;

        -- DDR3-RAM Interface
        ddr3_reset_n        : out   std_logic;
        ddr3_ck_p           : out   std_logic_vector(0 downto 0);
        ddr3_ck_n           : out   std_logic_vector(0 downto 0);
        ddr3_cke            : out   std_logic_vector(0 downto 0);

        ddr3_ras_n          : out   std_logic;
        ddr3_cas_n          : out   std_logic;
        ddr3_we_n           : out   std_logic;
        ddr3_odt            : out   std_logic_vector(0 downto 0);
        ddr3_ba             : out   std_logic_vector(2 downto 0);
        ddr3_addr           : out   std_logic_vector(12 downto 0);
        ddr3_dm             : out   std_logic_vector(1 downto 0);

        ddr3_dqs_p          : inout std_logic_vector(1 downto 0);
        ddr3_dqs_n          : inout std_logic_vector(1 downto 0);
        ddr3_dq             : inout std_logic_vector(15 downto 0));
    end component;

    component ddr3_1Gbit_model
    port (
        cs_n    : in    std_logic;
        rst_n   : in    std_logic;
        ck      : in    std_logic;
        ck_n    : in    std_logic;
        cke     : in    std_logic;
        ras_n   : in    std_logic;
        cas_n   : in    std_logic;
        we_n    : in    std_logic;
        odt     : in    std_logic;
        ba      : in    std_logic_vector(2 downto 0);
        addr    : in    std_logic_vector(12 downto 0);
        dm_tdqs : inout std_logic_vector(1 downto 0);
        dqs     : inout std_logic_vector(1 downto 0);
        dqs_n   : inout std_logic_vector(1 downto 0);
        dq      : inout std_logic_vector(15 downto 0);
        tdqs_n  : out   std_logic_vector(1 downto 0));
    end component;

    component SimParaInit is
    port (
        -- ImageBus Simulation Parameters
        ImgSimParaxTO     : out tIMG_SIM_PARAMETERS);
    end component;

    component SysBusStimulus is
    port (
        -- Infrastructure Signals
        SysClkxCI         : in  std_logic;              -- SystemBus Clock
        SysRstxRI         : in  std_logic;              -- SystemBus Reset
        SimParaxTI        : in  tSIMULATION_PARAMETERS; -- System Simulation Parameters
        DebugPortxTI      : in  std_logic_vector(cDBG_PORT_WIDTH-1 downto 0);   -- Debug Port
    
        -- SystemBus Control Interface                      
        SysModReqxMO      : out std_logic;              -- SystemBus Custom Module Request
        SysDdrReqxMO      : out std_logic;              -- SystemBus DDR3-RAM Memory Request
        SysAckxMI         : in  std_logic;              -- SystemBus Cycle Acknowledge
        SysErrxMI         : in  std_logic;              -- SystemBus Cycle Error
        SysWrEnxMO        : out std_logic;              -- SystemBus Write Enable
        SysAddrxAO        : out std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);  -- SystemBus Address
        SysWrDataxDO      : out std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);  -- SystemBus Write Data
        SysRdDataxDI      : in  std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);  -- SystemBus Read Data

        -- Simulation Flow Control
        ImgRstxRO         : out std_logic;              -- ImageBus Reset
        StartImgBusxMO    : out boolean;                -- Start ImageBus frame transfers
        StopImgBusxMO     : out boolean;                -- Stop ImageBus frame transfers
        AbortImgBusxMO    : out boolean);               -- Abort ImageBus frame transfers
    end component;

    component ImgBusStimulus is
    port (
        -- Infrastructure Signals
        ImgClkxCI         : in  std_logic;              -- ImageBus Clock
        ImgRstxRI         : in  std_logic;              -- ImageBus Reset
        ImgSimParaxTI     : in  tIMG_SIM_PARAMETERS;    -- ImageBus Simulation Parameters

        -- Simulation Flow Control
        StartImgBusxMI    : in  boolean;                -- Start ImageBus frame transfers
        StopImgBusxMI     : in  boolean;                -- Stop ImageBus frame transfers
        AbortImgBusxMI    : in  boolean;                -- Abort ImageBus frame transfers

        -- ImageBus Receive Port
        ImgRxPixValxMO    : out std_logic;              -- ImageBus Receive Pixel Valid
        ImgRxPixRdyxMI    : in  std_logic;              -- ImageBus Receive Pixel Ready - ignore if cIMG_HANDSHAKE_EN = False
        ImgRxPixStatxSO   : out std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);  -- ImageBus Receive Pixel Status
        ImgRxPixDataxDO   : out std_logic_vector(cIMG_DATA_WIDTH-1 downto 0)); -- ImageBus Receive Pixel Data
    end component;

    component ImgBusAnalyzer is
    port (
        -- Infrastructure Signals
        ImgClkxCI         : in  std_logic;              -- ImageBus Clock
        ImgRstxRI         : in  std_logic;              -- ImageBus Reset
        ImgSimParaxTI     : in  tIMG_SIM_PARAMETERS;    -- ImageBus Simulation Parameters

        -- ImageBus Transmit Port
        ImgTxPixValxMI    : in  std_logic;              -- ImageBus Transmit Pixel Valid
        ImgTxPixRdyxMO    : out std_logic;              -- ImageBus Transmit Pixel Ready - ignore if cIMG_HANDSHAKE_EN = False
        ImgTxPixStatxSI   : in  std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);  -- ImageBus Transmit Pixel Status
        ImgTxPixDataxDI   : in  std_logic_vector(cIMG_DATA_WIDTH-1 downto 0)); -- ImageBus Transmit Pixel Data
    end component;

    -----------------------------------------------------------------------------
    -- Signal Declarations
    -----------------------------------------------------------------------------
    signal ImgSimParaxT     : tIMG_SIM_PARAMETERS;                                  -- ImageBus Simulation Parameters
    signal StartImgBusxM    : boolean := False;                                     -- Start ImageBus frame transfers
    signal StopImgBusxM     : boolean := False;                                     -- Stop ImageBus frame transfers
    signal AbortImgBusxM    : boolean := False;                                     -- Abort ImageBus frame transfers
    signal DebugPortxT      : std_logic_vector(cDBG_PORT_WIDTH-1 downto 0);         -- Event Logger Port

    -- Infrastructure Signals
    signal SysClkxC         : std_logic := '0';                                     -- SystemBus Clock
    signal SysRstxR         : std_logic := '1';                                     -- SystemBus Reset (powerup)
    signal SysRstnxR        : std_logic := '0';                                     -- SystemBus NotReset (powerup)
    signal ImgRstxR         : std_logic := '1';                                     -- ImageBus Reset

    -- SystemBus Control Interface
    signal SysModReqxM      : std_logic;                                            -- SystemBus Custom Module Request
    signal SysModAckxM      : std_logic;                                            -- SystemBus Custom Module Acknowledge
    signal SysModErrxM      : std_logic;                                            -- SystemBus Custom Module Error
    signal SysModRdDataxD   : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);         -- SystemBus Custom Module Read Data

    signal SysDdrReqxM      : std_logic;                                            -- SystemBus DDR3-RAM Memory Request
    signal SysDdrAckxM      : std_logic;                                            -- SystemBus DDR3-RAM Memory Acknowledge
    signal SysDdrErrxM      : std_logic;                                            -- SystemBus DDR3-RAM Memory Error
    signal SysDdrRdDataxD   : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);         -- SystemBus DDR3-RAM Memory Read Data

    signal SysStimAckxM     : std_logic;                                            -- SystemBus Stimulus Acknowledge
    signal SysStimErrxM     : std_logic;                                            -- SystemBus Stimulus Error
    signal SysStimWrEnxM    : std_logic;                                            -- SystemBus Stimulus Write Enable
    signal SysStimAddrxA    : std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);         -- SystemBus Stimulus Address
    signal SysStimWrDataxD  : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);         -- SystemBus Stimulus Write Data
    signal SysStimRdDataxD  : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);         -- SystemBus Stimulus Read Data

    -- Custom Module GPIO
    signal CustIoCtrlxM     : std_logic_vector(cCUST_GPIO_BUS_WIDTH-1 downto 0);    -- GPIO Output Control
    signal CustIoDataOutxD  : std_logic_vector(cCUST_GPIO_BUS_WIDTH-1 downto 0);    -- GPIO Output Data
    signal CustIoDataInxD   : std_logic_vector(cCUST_GPIO_BUS_WIDTH-1 downto 0);    -- GPIO Input Data

    -- ImageBus Receive Port
    signal ImgRxPixValxM    : std_logic;                                            -- ImageBus Receive Pixel Valid
    signal ImgRxPixRdyxM    : std_logic;                                            -- ImageBus Receive Pixel Ready - ignore if cIMG_HANDSHAKE_EN = False
    signal ImgRxPixStatxS   : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);         -- ImageBus Receive Pixel Status
    signal ImgRxPixDataxD   : std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);         -- ImageBus Receive Pixel Data

    -- ImageBus Transmit Port
    signal ImgTxPixValxM    : std_logic;                                            -- ImageBus Transmit Pixel Valid
    signal ImgTxPixRdyxM    : std_logic;                                            -- ImageBus Transmit Pixel Ready - ignore if cIMG_HANDSHAKE_EN = False
    signal ImgTxPixStatxS   : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);         -- ImageBus Transmit Pixel Status
    signal ImgTxPixDataxD   : std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);         -- ImageBus Transmit Pixel Data
    signal ImgTxLastLinexM  : std_logic;                                            -- ImageBus Last Line Indicator, for simulation only

    -- Custom Module Command Port
    signal CustMcbCmdValxM  : std_logic;                                            -- Command Valid
    signal CustMcbCmdRdyxM  : std_logic;                                            -- Command Ready
    signal CustMcbCmdInstxM : std_logic_vector(cCUST_INSTR_BUS_WIDTH-1 downto 0);   -- Command Instruction
    signal CustMcbCmdLenxM  : std_logic_vector(cCUST_BLEN_BUS_WIDTH-1 downto 0);    -- Command Burst Length
    signal CustMcbCmdAddrxA : std_logic_vector(cCUST_ADDR_BUS_WIDTH-1 downto 0);    -- Command Address

    -- Custom Module Read Port
    signal CustMcbRdValxM   : std_logic;                                            -- Read Data Valid
    signal CustMcbRdRdyxM   : std_logic;                                            -- Read Data Ready
    signal CustMcbRdDataxD  : std_logic_vector(cCUST_DATA_BUS_WIDTH-1 downto 0);    -- Read Data Bus

    -- Custom Module Write Port
    signal CustMcbWrValxM   : std_logic;                                            -- Write Data Valid
    signal CustMcbWrRdyxM   : std_logic;                                            -- Write Data Ready
    signal CustMcbWrDataxD  : std_logic_vector(cCUST_DATA_BUS_WIDTH-1 downto 0);    -- Write Data Bus

    signal px_cmd_full_i    : std_logic;                                            -- Custom Module MCB Command Buffer Full
    signal px_rd_empty_i    : std_logic;                                            -- Custom Module MCB Read Data Buffer Empty
    signal px_wr_full_i     : std_logic;                                            -- Custom Module MCB Write Data Buffer Full

    -- MCB User-Command-Interface
    signal McbAppEnxM       : std_logic;                                            -- MCB User Command Enable
    signal McbAppRdyxM      : std_logic;                                            -- MCB User Command Ready
    signal McbAppCmdxM      : std_logic_vector(cMCB_CMD_BUS_WIDTH-1 downto 0);      -- MCB User Command Instruction
    signal McbAppAddrxA     : std_logic_vector(cMCB_ADDR_BUS_WIDTH-1 downto 0);     -- MCB User Command Address

    -- MCB User-Read-Data-Interface
    signal McbAppRdValxM    : std_logic;                                            -- MCB User Read Valid
    signal McbAppRdEndxM    : std_logic;                                            -- MCB User Read End
    signal McbAppRdDataxD   : std_logic_vector(cMCB_DATA_BUS_WIDTH-1 downto 0);     -- MCB User Read Data

    -- MCB User-Write-Data-Interface
    signal McbAppWrEnxM     : std_logic;                                            -- MCB User Write Enable
    signal McbAppWrRdyxM    : std_logic;                                            -- MCB User Write Ready
    signal McbAppWrEndxM    : std_logic;                                            -- MCB User Write End
    signal McbAppWrMaskxM   : std_logic_vector(cMCB_MASK_BUS_WIDTH-1 downto 0);     -- MCB User Write Maks
    signal McbAppWrDataxD   : std_logic_vector(cMCB_DATA_BUS_WIDTH-1 downto 0);     -- MCB User Write Data

    -- MCB Control Signals
    signal McbSysClkxC      : std_logic := '0';                                     -- MCB System Clock (400 MHz, 2.5ns)
    signal McbRefClkxC      : std_logic := '0';                                     -- MCB Reference Clock (200 MHz, 5.0ns)
    signal McbUserClkxC     : std_logic := '0';                                     -- MCB User Interface Clock (100 MHz, 10.0ns)
    signal McbUserRstxR     : std_logic := '1';                                     -- MCB User Interface Reset
    signal McbSysRstxR      : std_logic := '1';                                     -- MCB System Interface Reset
    signal McbCalibDonexS   : std_logic := '0';                                     -- MCB Calibration complete

    -- DDR3-RAM Interface Signals
    signal ddr3_reset_n     : std_logic;
    signal ddr3_ck_p        : std_logic_vector(0 downto 0);
    signal ddr3_ck_n        : std_logic_vector(0 downto 0);
    signal ddr3_cke         : std_logic_vector(0 downto 0);

    signal ddr3_ras_n       : std_logic;
    signal ddr3_cas_n       : std_logic;
    signal ddr3_we_n        : std_logic;
    signal ddr3_odt         : std_logic_vector(0 downto 0);
    signal ddr3_ba          : std_logic_vector(2 downto 0);
    signal ddr3_addr        : std_logic_vector(12 downto 0);
    signal ddr3_dm          : std_logic_vector(1 downto 0);

    signal ddr3_dqs_p       : std_logic_vector(1 downto 0);
    signal ddr3_dqs_n       : std_logic_vector(1 downto 0);
    signal ddr3_dq          : std_logic_vector(15 downto 0);

    signal mcb3_enable1     : std_logic;
    signal mcb3_enable2     : std_logic;
    signal mcb3_command     : std_logic_vector(2 downto 0);
    signal ddr3_dqs_p_vec   : std_logic_vector(1 downto 0);
    signal ddr3_dqs_n_vec   : std_logic_vector(1 downto 0);
    signal McbDebugPortxM   : std_logic_vector(cMCB_DEBUG_WIDTH-1 downto 0);

begin

    -----------------------------------------------------------------------------
    -- Custom Module (UUT)
    -----------------------------------------------------------------------------
    CustModInst: CustomModule
    port map (
        -- Module Infrastructure Signals
        SysClkxCI         => SysClkxC,
        SysRstxRI         => SysRstxR,
        ImgRstxRI         => ImgRstxR,

        -- SystemBus Control Interface
        SysReqxMI         => SysModReqxM,
        SysAckxMO         => SysModAckxM,
        SysErrxMO         => SysModErrxM,
        SysWrEnxMI        => SysStimWrEnxM,
        SysAddrxAI        => SysStimAddrxA(cSYS_ADDR_WIDTH-1 downto 0),
        SysWrDataxDI      => SysStimWrDataxD(cSYS_DATA_WIDTH-1 downto 0),
        SysRdDataxDO      => SysModRdDataxD(cSYS_DATA_WIDTH-1 downto 0),

        -- Custom Module GPIO
        CustIoCtrlxMO     => CustIoCtrlxM,
        CustIoDataOutxDO  => CustIoDataOutxD,
        CustIoDataInxDI   => CustIoDataInxD,

        -- ImageBus Receive Port
        ImgRxPixValxMI    => ImgRxPixValxM,
        ImgRxPixRdyxMO    => ImgRxPixRdyxM,
        ImgRxPixStatxSI   => ImgRxPixStatxS,
        ImgRxPixDataxDI   => ImgRxPixDataxD,

        -- ImageBus Transmit Port
        ImgTxPixValxMO    => ImgTxPixValxM,
        ImgTxPixRdyxMI    => ImgTxPixRdyxM,
        ImgTxPixStatxSO   => ImgTxPixStatxS,
        ImgTxPixDataxDO   => ImgTxPixDataxD,

        -- MCB command
        px_cmd_en_o       => CustMcbCmdValxM,
        px_cmd_full_i     => px_cmd_full_i,
        px_cmd_instr_o    => CustMcbCmdInstxM,
        px_cmd_bl_o       => CustMcbCmdLenxM,
        px_cmd_addr_o     => CustMcbCmdAddrxA,

        -- MCB read
        px_rd_en_o        => CustMcbRdValxM,
        px_rd_empty_i     => px_rd_empty_i,
        px_rd_data_i      => CustMcbRdDataxD,

        -- MCB write
        px_wr_en_o        => CustMcbWrValxM,
        px_wr_full_i      => px_wr_full_i,
        px_wr_data_o      => CustMcbWrDataxD);

    -- Convert MCB Ready signals to Custom Module Empty/Full polarity
    px_cmd_full_i <= not(CustMcbCmdRdyxM);
    px_rd_empty_i <= not(CustMcbRdRdyxM);
    px_wr_full_i  <= not(CustMcbWrRdyxM);

    -----------------------------------------------------------------------------
    -- DDR3-RAM interface
    -----------------------------------------------------------------------------
    NoDDR3_RAM_Gen: if (cDDR3_SIM_ID = NONE) generate
        SysDdrAckxM     <= '0';
        SysDdrErrxM     <= '0';
        SysDdrRdDataxD  <= (others => '0');

        CustMcbCmdRdyxM <= '0';
        CustMcbRdRdyxM  <= '0';
        CustMcbWrRdyxM  <= '0';
        CustMcbRdDataxD <= (others => '0');

        McbAppEnxM     <= '0';
        McbAppRdyxM    <= '0';
        McbAppCmdxM    <= (others => '0');
        McbAppAddrxA   <= (others => '0');

        McbAppRdValxM  <= '0';
        McbAppRdEndxM  <= '0';
        McbAppRdDataxD <= (others => '0');

        McbAppWrEnxM   <= '0';
        McbAppWrRdyxM  <= '0';
        McbAppWrEndxM  <= '0';
        McbAppWrMaskxM <= (others => '0');
        McbAppWrDataxD <= (others => '0');

        McbSysClkxC    <= '0';
        McbRefClkxC    <= '0';
        McbSysRstxR    <= '0';
        McbUserClkxC   <= '0';
        McbUserRstxR   <= '0';
        McbCalibDonexS <= '0';

        mcb3_enable1 <= '0';
        mcb3_enable2 <= '0';
        mcb3_command <= (others => '1');

        ddr3_reset_n <= '1';
        ddr3_ck_p    <= (others => '0');
        ddr3_ck_n    <= (others => '0');
        ddr3_cke     <= (others => '0');

        ddr3_ras_n   <= '1';
        ddr3_cas_n   <= '1';
        ddr3_we_n    <= '1';
        ddr3_odt     <= (others => '0');
        ddr3_ba      <= (others => '0');
        ddr3_addr    <= (others => '0');
        ddr3_dm      <= (others => '0');

        ddr3_dqs_p   <= (others => 'Z');
        ddr3_dqs_n   <= (others => 'Z');
        ddr3_dq      <= (others => 'Z');

        ddr3_dqs_p_vec <= (others => 'Z');
        ddr3_dqs_n_vec <= (others => 'Z');

        McbDebugPortxM <= (others => '0');
    end generate;

    DDR3_RAM_Gen: if (cDDR3_SIM_ID /= NONE) generate
        -- Instantiate Custom-to-MCB Interface
        CustMcbInst: CustomMcbInterface
        generic map (
            gCUST_INSTR_BUS_WIDTH => cCUST_INSTR_BUS_WIDTH,
            gCUST_BLEN_BUS_WIDTH  => cCUST_BLEN_BUS_WIDTH,
            gCUST_ADDR_BUS_WIDTH  => cCUST_ADDR_BUS_WIDTH,
            gCUST_DATA_BUS_WIDTH  => cCUST_DATA_BUS_WIDTH,

            gMCB_CMD_BUS_WIDTH    => cMCB_CMD_BUS_WIDTH,
            gMCB_ADDR_BUS_WIDTH   => cMCB_ADDR_BUS_WIDTH,
            gMCB_MASK_BUS_WIDTH   => cMCB_MASK_BUS_WIDTH,
            gMCB_DATA_BUS_WIDTH   => cMCB_DATA_BUS_WIDTH,

            gMCB_DEBUG_ENABLE     => cMCB_DEBUG_ENABLE,
            gMCB_DEBUG_WIDTH      => cMCB_DEBUG_WIDTH)
        port map (
            -- Infrastructure Signals
            SysClkxCI           => SysClkxC,
            SysRstxRI           => SysRstxR,

            -- SystemBus Control Interface
            SysMemReqxMI        => SysDdrReqxM,
            SysMemAckxMO        => SysDdrAckxM,
            SysMemErrxMO        => SysDdrErrxM,
            SysMemWrEnxMI       => SysStimWrEnxM,
            SysMemAddrxAI       => SysStimAddrxA,
            SysMemWrDataxDI     => SysStimWrDataxD,
            SysMemRdDataxDO     => SysDdrRdDataxD,

            ------------------------------------------

            -- Custom Module Command Port
            CustMcbCmdValxMI    => CustMcbCmdValxM,
            CustMcbCmdRdyxMO    => CustMcbCmdRdyxM,
            CustMcbCmdInstxMI   => CustMcbCmdInstxM,
            CustMcbCmdLenxMI    => CustMcbCmdLenxM,
            CustMcbCmdAddrxAI   => CustMcbCmdAddrxA,

            -- Custom Module Read Port
            CustMcbRdValxMI     => CustMcbRdValxM,
            CustMcbRdRdyxMO     => CustMcbRdRdyxM,
            CustMcbRdDataxDO    => CustMcbRdDataxD,

            -- Custom Module Write Port
            CustMcbWrValxMI     => CustMcbWrValxM,
            CustMcbWrRdyxMO     => CustMcbWrRdyxM,
            CustMcbWrDataxDI    => CustMcbWrDataxD,

            ------------------------------------------

            -- MCB Memory User Infrastructure
            McbAppClkxCI        => McbUserClkxC,
            McbAppRstxRI        => McbUserRstxR,
            McbCalibDonexSI     => McbCalibDonexS,
            McbRdFifoOvrFlxMO   => open,

            -- MCB User-Command-Interface
            McbAppCmdEnxMO      => McbAppEnxM,
            McbAppCmdRdyxMI     => McbAppRdyxM,
            McbAppCmdCmdxMO     => McbAppCmdxM,
            McbAppCmdAddrxAO    => McbAppAddrxA,

            -- MCB User-Read-Data-Interface
            McbAppRdValxMI      => McbAppRdValxM,
            McbAppRdEndxMI      => McbAppRdEndxM,
            McbAppRdDataxDI     => McbAppRdDataxD,

            -- MCB User-Write-Data-Interface
            McbAppWrEnxMO       => McbAppWrEnxM,
            McbAppWrRdyxMI      => McbAppWrRdyxM,
            McbAppWrEndxMO      => McbAppWrEndxM,
            McbAppWrMaskxMO     => McbAppWrMaskxM,
            McbAppWrDataxDO     => McbAppWrDataxD,

            -- Debug Port
            McbDebugPortxMO     => McbDebugPortxM);

        -- Instantiate Custom RAM
        CustomRAM_Gen: if (cDDR3_SIM_ID = CUSTOM_RAM) generate
            CustomRAM_Inst: CustomRAM
            generic map (
                gCUSTOM_RAM_SIZE    => cCUSTOM_RAM_SIZE,
                gMCB_CMD_BUS_WIDTH  => cMCB_CMD_BUS_WIDTH,
                gMCB_ADDR_BUS_WIDTH => cMCB_ADDR_BUS_WIDTH,
                gMCB_MASK_BUS_WIDTH => cMCB_MASK_BUS_WIDTH,
                gMCB_DATA_BUS_WIDTH => cMCB_DATA_BUS_WIDTH)
            port map (
                -- MCB User-Application Infrastructure
                McbAppClkxCO        => McbUserClkxC,
                McbAppRstxRO        => McbUserRstxR,
                McbCalibDonexSO     => McbCalibDonexS,

                -- MCB User-Application Command-Interface
                McbAppCmdEnxMI      => McbAppEnxM,
                McbAppCmdRdyxMO     => McbAppRdyxM,
                McbAppCmdCmdxMI     => McbAppCmdxM,
                McbAppCmdAddrxAI    => McbAppAddrxA,

                -- MCB User-Application Write-Data-Interface
                McbAppWrEnxMI       => McbAppWrEnxM,
                McbAppWrRdyxMO      => McbAppWrRdyxM,
                McbAppWrEndxMI      => McbAppWrEndxM,
                McbAppWrMaskxMI     => McbAppWrMaskxM,
                McbAppWrDataxDI     => McbAppWrDataxD,

                -- MCB User-Application Read-Data-Interface
                McbAppRdValxMO      => McbAppRdValxM,
                McbAppRdEndxMO      => McbAppRdEndxM,
                McbAppRdDataxDO     => McbAppRdDataxD);

            -- Assign unused DDR3-RAM signals
            mcb3_enable1 <= '0';
            mcb3_enable2 <= '0';
            mcb3_command <= (others => '1');

            ddr3_reset_n <= '1';
            ddr3_ck_p    <= (others => '0');
            ddr3_ck_n    <= (others => '0');
            ddr3_cke     <= (others => '0');

            ddr3_ras_n   <= '1';
            ddr3_cas_n   <= '1';
            ddr3_we_n    <= '1';
            ddr3_odt     <= (others => '0');
            ddr3_ba      <= (others => '0');
            ddr3_addr    <= (others => '0');
            ddr3_dm      <= (others => '0');

            ddr3_dqs_p   <= (others => 'Z');
            ddr3_dqs_n   <= (others => 'Z');
            ddr3_dq      <= (others => 'Z');

            ddr3_dqs_p_vec <= (others => 'Z');
            ddr3_dqs_n_vec <= (others => 'Z');
        end generate;

        -- Instantiate DDR3 RAM
        DDR3_1GB_Gen: if (cDDR3_SIM_ID = MT41K64M16XX107) generate
            -- Instantiate the Memory Controller Block for 1-Gbit DDR3-RAM
            Mcb1GbInst: MCB_SIM_MT41K64M16XX107
            port map (
                -- Global Infrastructure Signals
                clk_ref_i           => McbRefClkxC,                   -- in    std_logic;
                sys_rst             => McbSysRstxR,                   -- in    std_logic;

                -- User Infrastructure Signals
                sys_clk_i           => McbSysClkxC,                   -- in    std_logic;
                ui_clk              => McbUserClkxC,                  -- out   std_logic;
                ui_clk_sync_rst     => McbUserRstxR,                  -- out   std_logic;
                init_calib_complete => McbCalibDonexS,                -- out   std_logic;
                device_temp_o       => open,                          -- out   std_logic_vector(11 downto 0);

                -- User-Command-Interface
                app_en              => McbAppEnxM,                    -- in    std_logic;
                app_rdy             => McbAppRdyxM,                   -- out   std_logic;
                app_cmd             => McbAppCmdxM,                   -- in    std_logic_vector(2 downto 0);
                app_addr            => McbAppAddrxA,                  -- in    std_logic_vector(26 downto 0);

                -- User-Write-Data-Interface
                app_wdf_wren        => McbAppWrEnxM,                  -- in    std_logic;
                app_wdf_rdy         => McbAppWrRdyxM,                 -- out   std_logic;
                app_wdf_end         => McbAppWrEndxM,                 -- in    std_logic;
                app_wdf_mask        => McbAppWrMaskxM,                -- in    std_logic_vector(15 downto 0);
                app_wdf_data        => McbAppWrDataxD,                -- in    std_logic_vector(127 downto 0);

                -- User-Read-Data-Interface
                app_rd_data_valid   => McbAppRdValxM,                 -- out   std_logic;
                app_rd_data_end     => McbAppRdEndxM,                 -- out   std_logic;
                app_rd_data         => McbAppRdDataxD,                -- out   std_logic_vector(127 downto 0);

                -- Application Interface
                app_sr_req          => '0',                           -- in    std_logic;
                app_sr_active       => open,                          -- out   std_logic;
                app_ref_req         => '0',                           -- in    std_logic;
                app_ref_ack         => open,                          -- out   std_logic;
                app_zq_req          => '0',                           -- in    std_logic;
                app_zq_ack          => open,                          -- out   std_logic;

                -- DDR3-RAM Interface
                ddr3_reset_n        => ddr3_reset_n,                  -- out   std_logic;
                ddr3_ck_p           => ddr3_ck_p,                     -- out   std_logic_vector(0 downto 0);
                ddr3_ck_n           => ddr3_ck_n,                     -- out   std_logic_vector(0 downto 0);
                ddr3_cke            => ddr3_cke,                      -- out   std_logic_vector(0 downto 0);

                ddr3_ras_n          => ddr3_ras_n,                    -- out   std_logic;
                ddr3_cas_n          => ddr3_cas_n,                    -- out   std_logic;
                ddr3_we_n           => ddr3_we_n,                     -- out   std_logic;
                ddr3_odt            => ddr3_odt,                      -- out   std_logic_vector(0 downto 0);
                ddr3_ba             => ddr3_ba,                       -- out   std_logic_vector(2 downto 0);
                ddr3_addr           => ddr3_addr,                     -- out   std_logic_vector(12 downto 0);
                ddr3_dm             => ddr3_dm,                       -- out   std_logic_vector(1 downto 0);

                ddr3_dqs_p          => ddr3_dqs_p,                    -- inout std_logic_vector(1 downto 0);
                ddr3_dqs_n          => ddr3_dqs_n,                    -- inout std_logic_vector(1 downto 0);
                ddr3_dq             => ddr3_dq);                      -- inout std_logic_vector(15 downto 0));

            -- DDR3-RAM control logic
            process (ddr3_ck_p, SysRstxR)
            begin
                if (SysRstxR = '1') then
                    mcb3_enable1 <= '0';
                    mcb3_enable2 <= '0';
                elsif (rising_edge(ddr3_ck_p(0))) then
                    if (mcb3_command = "100") then
                        mcb3_enable2 <= '0';
                    elsif (mcb3_command = "101") then
                        mcb3_enable2 <= '1';
                    else
                        mcb3_enable2 <= mcb3_enable2;
                    end if;

                    mcb3_enable1 <= mcb3_enable2;
                end if;
            end process;

            -- Generate MCB system and reference clocks
            McbSysClkxC <= not(McbSysClkxC) after (1 us/(2*cMCB_SYSTEM_FREQ));
            McbRefClkxC <= not(McbRefClkxC) after (1 us/(2*cMCB_REFERENCE_FREQ));
            McbSysRstxR <= '0' after cMCB_RESET_PERIOD;

            -- DDR3-RAM Pullup/Pulldown resistors
            reset_pd  : PULLDOWN port map (O => ddr3_reset_n);
            dqs_p0_pd : PULLDOWN port map (O => ddr3_dqs_p(0));
            dqs_p1_pd : PULLDOWN port map (O => ddr3_dqs_p(1));
            dqs_n0_pu : PULLUP   port map (O => ddr3_dqs_n(0));
            dqs_n1_pu : PULLUP   port map (O => ddr3_dqs_n(1));

            mcb3_command <= (ddr3_ras_n & ddr3_cas_n & ddr3_we_n);

            ddr3_dqs_p_vec(1 downto 0) <= (ddr3_dqs_p(1) & ddr3_dqs_p(0)) when ((mcb3_enable2 = '0') and (mcb3_enable1 = '0')) else "ZZ";
            ddr3_dqs_n_vec(1 downto 0) <= (ddr3_dqs_n(1) & ddr3_dqs_n(0)) when ((mcb3_enable2 = '0') and (mcb3_enable1 = '0')) else "ZZ";

            ddr3_dqs_p(0) <= ddr3_dqs_p_vec(0) when (mcb3_enable1 = '1') else 'Z';
            ddr3_dqs_p(1) <= ddr3_dqs_p_vec(1) when (mcb3_enable1 = '1') else 'Z';
            ddr3_dqs_n(0) <= ddr3_dqs_n_vec(0) when (mcb3_enable1 = '1') else 'Z';
            ddr3_dqs_n(1) <= ddr3_dqs_n_vec(1) when (mcb3_enable1 = '1') else 'Z';

            -- Instantiate 1Gbit DDR3-RAM
            DDR3_1GB_Inst: ddr3_1Gbit_model
            port map (
                cs_n    => '0',
                rst_n   => ddr3_reset_n,
                ck      => ddr3_ck_p(0),
                ck_n    => ddr3_ck_n(0),
                cke     => ddr3_cke(0),
                ras_n   => ddr3_ras_n,
                cas_n   => ddr3_cas_n,
                we_n    => ddr3_we_n,
                odt     => ddr3_odt(0),
                ba      => ddr3_ba,
                addr    => ddr3_addr,
                dm_tdqs => ddr3_dm,
                dqs     => ddr3_dqs_p_vec,
                dqs_n   => ddr3_dqs_n_vec,
                dq      => ddr3_dq,
                tdqs_n  => open);
        end generate;
    end generate;

    -----------------------------------------------------------------------------
    -- Simulation Stimulus and Analysis
    -----------------------------------------------------------------------------
    -- Simulation Parameter Setup
    SimParaInst: SimParaInit
    port map (
        -- ImageBus Simulation Parameters
        ImgSimParaxTO     => ImgSimParaxT);

    -- Setup SystemBus Registers for UUT
    SysStimulus: SysBusStimulus
    port map (
        -- Infrastructure Signals
        SysClkxCI         => SysClkxC,
        SysRstxRI         => SysRstxR,
        SimParaxTI        => ImgSimParaxT.SimPara,
        DebugPortxTI      => DebugPortxT,

        -- SystemBus Control Interface
        SysModReqxMO      => SysModReqxM,
        SysDdrReqxMO      => SysDdrReqxM,
        SysAckxMI         => SysStimAckxM,
        SysErrxMI         => SysStimErrxM,
        SysWrEnxMO        => SysStimWrEnxM,
        SysAddrxAO        => SysStimAddrxA,
        SysWrDataxDO      => SysStimWrDataxD,
        SysRdDataxDI      => SysStimRdDataxD,

        -- Simulation Flow Control
        ImgRstxRO         => ImgRstxR,
        StartImgBusxMO    => StartImgBusxM,
        StopImgBusxMO     => StopImgBusxM,
        AbortImgBusxMO    => AbortImgBusxM);

    SysStimAckxM    <= SysDdrAckxM    when (SysDdrReqxM = '1') else SysModAckxM;
    SysStimErrxM    <= SysDdrErrxM    when (SysDdrReqxM = '1') else SysModErrxM;
    SysStimRdDataxD <= SysDdrRdDataxD when (SysDdrReqxM = '1') else SysModRdDataxD;

    -- Generate LVPP input data for UUT
    ImgStimulus: ImgBusStimulus
    port map (
        -- Infrastructure Signals
        ImgClkxCI         => SysClkxC,
        ImgRstxRI         => ImgRstxR,
        ImgSimParaxTI     => ImgSimParaxT,

        -- Simulation Flow Control
        StartImgBusxMI    => StartImgBusxM,
        StopImgBusxMI     => StopImgBusxM,
        AbortImgBusxMI    => AbortImgBusxM,

        -- ImageBus Receive Port
        ImgRxPixValxMO    => ImgRxPixValxM,
        ImgRxPixRdyxMI    => ImgRxPixRdyxM,
        ImgRxPixStatxSO   => ImgRxPixStatxS,
        ImgRxPixDataxDO   => ImgRxPixDataxD);

    -- Analyze LVPP output data from UUT
    ImgAnalyzer: ImgBusAnalyzer
    port map (
        -- Infrastructure Signals
        ImgClkxCI         => SysClkxC,
        ImgRstxRI         => ImgRstxR,
        ImgSimParaxTI     => ImgSimParaxT,

        -- ImageBus Transmit Port
        ImgTxPixValxMI    => ImgTxPixValxM,
        ImgTxPixRdyxMO    => ImgTxPixRdyxM,
        ImgTxPixStatxSI   => ImgTxPixStatxS,
        ImgTxPixDataxDI   => ImgTxPixDataxD);

    -----------------------------------------------------------------------------
    -- System Clock and Reset
    -----------------------------------------------------------------------------
    PcieRstProc : process
        file LogFile : text;
    begin
        -- Open and close the log file to delete it's contents (if any).
        if (cLOG_FILE_ENABLE = True) then
            file_open(LogFile, cLOG_FILE_NAME, write_mode);
            file_close(LogFile);
        end if;

        PrintNow ("         CorSight2 Custom Module Testbench");
        PrintNow ("Copyright (c) 2017 NET New Electronic Technology GmbH");
        PrintNow (" ");

        PrintLog ("Executing CustomTB", LogExec);
        SysRstxR  <= '1';
        SysRstnxR <= '0';
        wait for 1 ns;

        -- Deassert Reset asynchronously to clock.
        wait for ImgSimParaxT.SimPara.RstPeriod;
        SysRstxR  <= '0';
        SysRstnxR <= '1';
        PrintLog ("[PcieRstProc] Reset deasserted", LogFlowCtl);
        wait;
    end process;

    -- Generate SystemBus clock.
    SysClkxC <= not(SysClkxC) after (1 us/(2*cIMGBUS_PIXEL_FREQ));

    -- Abort simulation run when the maximum simulation time has been exceeded.
    SimTimeProc : process
    begin
        wait for 1 us;
        if ((ImgSimParaxT.SimPara.MaxSimTime > 0 ns) and (now >= ImgSimParaxT.SimPara.MaxSimTime)) then
            PrintNow ("Maximum Simulation Time Exceeded (MaxSimTime) - Simulation Terminated");
            report " "
            severity failure;
        end if;
    end process;

    ----------------------------------------------------------------------------
    -- GPIO Ports (not used)
    ----------------------------------------------------------------------------
    CustIoDataInxD <= "000000";

    ----------------------------------------------------------------------------
    -- Debug port for simulation flow control
    ----------------------------------------------------------------------------
    -- Detect SOF on the ImageBus input port and EOF on the output port.
    DbgProc: process (SysClkxC, ImgRstxR)
    begin
        if (ImgRstxR = '1') then
            ImgTxLastLinexM <= '0';
        elsif rising_edge(SysClkxC) then
            -- Detect Start-of-Last-Line on the ImageBus Transmit port.
            -- Set LastLine flag and hold until the end of the line.
            if ((ImgTxPixValxM = '1') and (ImgTxPixRdyxM = '1') and (ImgTxPixStatxS = cIMG_STAT_SOLL)) then
                ImgTxLastLinexM <= '1';
            elsif ((ImgTxPixValxM = '1') and (ImgTxPixRdyxM = '1') and (ImgTxPixStatxS /= cIMG_STAT_NONE)) then
                ImgTxLastLinexM <= '0';
            end if;
        end if;
    end process;

    -- Debug(0): Start-of-Frame in ImageBus Receive port
    DebugPortxT(0) <= '1' when ((ImgRxPixValxM = '1') and (ImgRxPixRdyxM = '1') and ((ImgRxPixStatxS = cIMG_STAT_SOF) or (ImgRxPixStatxS = cIMG_STAT_SOSF))) else '0';

    -- Debug(1): End-of-Frame in ImageBus Transmit port
    DebugPortxT(1) <= '1' when ((ImgTxPixValxM = '1') and (ImgTxPixRdyxM = '1') and (ImgTxPixStatxS = cIMG_STAT_EOL) and (ImgTxLastLinexM = '1')) else '0';

    -- Debug(11..2): ImageBus Receive data
    DebugPortxT(11 downto 2) <= ImgRxPixDataxD(11 downto 2) when ((ImgRxPixValxM = '1') and (ImgRxPixRdyxM = '1')) else "0000000000";
    DebugPortxT(cDBG_PORT_WIDTH-1 downto 12) <= (others => '0');

end;
