-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : DDR3-RAM Replacement Module
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-09-05  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--use ieee.math_real.all;

library UNISIM;
use UNISIM.vcomponents.all;
library unimacro;
use unimacro.vcomponents.all;

library work;
use work.CustomCfgLib.all;
use work.CustomSimLib.all;
use work.CustomFctLib.all;

entity CustomRAM is
    generic (
        gCUSTOM_RAM_SIZE    : integer := 32768;                     -- Size of CustomRAM memory [gMCB_DATA_BUS_WIDTH]
        gMCB_CMD_BUS_WIDTH  : integer := 3;                         -- Width of MCB Command bus
        gMCB_ADDR_BUS_WIDTH : integer := 27;                        -- Width of MCB Address bus
        gMCB_MASK_BUS_WIDTH : integer := 16;                        -- Width of MCB Mask bus
        gMCB_DATA_BUS_WIDTH : integer := 128);                      -- Width of MCB Data bus
    port (
        -- MCB User-Application Infrastructure
        McbAppClkxCO        : out std_logic;                                            -- MCB User-Application Interface Clock
        McbAppRstxRO        : out std_logic;                                            -- MCB User-Application Interface Reset
        McbCalibDonexSO     : out std_logic;                                            -- MCB Calibration complete

        -- MCB User-Application Command-Interface
        McbAppCmdEnxMI      : in  std_logic;                                            -- MCB User-Application Command Enable
        McbAppCmdRdyxMO     : out std_logic;                                            -- MCB User-Application Command Ready
        McbAppCmdCmdxMI     : in  std_logic_vector(gMCB_CMD_BUS_WIDTH-1 downto 0);      -- MCB User-Application Command Instruction
        McbAppCmdAddrxAI    : in  std_logic_vector(gMCB_ADDR_BUS_WIDTH-1 downto 0);     -- MCB User-Application Command Address

        -- MCB User-Application Write-Data-Interface
        McbAppWrEnxMI       : in  std_logic;                                            -- MCB User-Application Write Enable
        McbAppWrRdyxMO      : out std_logic;                                            -- MCB User-Application Write Ready
        McbAppWrEndxMI      : in  std_logic;                                            -- MCB User-Application Write End
        McbAppWrMaskxMI     : in  std_logic_vector(gMCB_MASK_BUS_WIDTH-1 downto 0);     -- MCB User-Application Write Maks
        McbAppWrDataxDI     : in  std_logic_vector(gMCB_DATA_BUS_WIDTH-1 downto 0);     -- MCB User-Application Write Data

        -- MCB User-Application Read-Data-Interface
        McbAppRdValxMO      : out std_logic;                                            -- MCB User-Application Read Valid
        McbAppRdEndxMO      : out std_logic;                                            -- MCB User-Application Read End
        McbAppRdDataxDO     : out std_logic_vector(gMCB_DATA_BUS_WIDTH-1 downto 0));    -- MCB User-Application Read Data
end CustomRAM;

architecture Behavioral of CustomRAM is

    ---------------------------------------------------------------------------
    -- Constant Definitions
    ---------------------------------------------------------------------------
    constant cMCB_USER_RST_PERIOD   : time    := 5 us;              -- MCB User-Application Reset duration after simulation start
    constant cMCB_CALIB_DONE_PERIOD : time    := 10 us;             -- MCB Calibration duration after simulation start
    constant cMCB_DATA_INVALID      : bit     := '1';               -- MCB Data Mask Invalid
    constant cMCB_DATA_VALID        : bit     := '0';               -- MCB Data Mask Valid
    constant cMCB_READ_DELAY        : integer := 4;                 -- MCB Read Buffer Delay, in MCB User clock cycles
    constant cMCB_WRITE_DELAY       : integer := 4;                 -- MCB Write Buffer Delay, in MCB User clock cycles

    ---------------------------------------------------------------------------
    -- MCB Command Buffer Definitions
    ---------------------------------------------------------------------------
    constant cMCB_COMMAND_BUFFER_SIZE : integer := 16;              -- MCB CustomRAM command buffer size
    constant cMCB_COMMAND_WRITE       : std_logic_vector(gMCB_CMD_BUS_WIDTH-1 downto 0) := std_logic_vector(to_unsigned(0, gMCB_CMD_BUS_WIDTH));
    constant cMCB_COMMAND_READ        : std_logic_vector(gMCB_CMD_BUS_WIDTH-1 downto 0) := std_logic_vector(to_unsigned(1, gMCB_CMD_BUS_WIDTH));

    type tMCB_COMMAND_BUFFER is record
        CmdVal  : std_logic;
        CmdType : std_logic_vector(gMCB_CMD_BUS_WIDTH-1 downto 0);
        CmdAddr : integer range 0 to gCUSTOM_RAM_SIZE-1;
    end record;

    constant cMCB_COMMAND_DEFAULT : tMCB_COMMAND_BUFFER := (
        CmdVal  => '0',
        CmdType => (others => '0'),
        CmdAddr => 0);

    type tMCB_COMMAND_ARRAY is array (0 to cMCB_COMMAND_BUFFER_SIZE-1) of tMCB_COMMAND_BUFFER;

    ---------------------------------------------------------------------------
    -- MCB Write Buffer Definitions
    ---------------------------------------------------------------------------
    constant cMCB_WRITE_BUFFER_SIZE : integer := 64;                -- MCB CustomRAM write buffer size

    type tMCB_WRITE_BUFFER is record
        WrVal   : std_logic;
        WrEnd   : std_logic;
        WrMask  : bit_vector(gMCB_MASK_BUS_WIDTH-1 downto 0);
        WrData  : bit_vector(gMCB_DATA_BUS_WIDTH-1 downto 0);
    end record;

    constant cMCB_WRITE_DEFAULT : tMCB_WRITE_BUFFER := (
        WrVal   => '0',
        WrEnd   => '0',
        WrMask  => (others => '0'),
        WrData  => (others => '0'));

    type tMCB_WR_DATA_ARRAY is array (0 to cMCB_WRITE_BUFFER_SIZE-1) of tMCB_WRITE_BUFFER;

    ---------------------------------------------------------------------------
    -- MCB Memory Definitions
    ---------------------------------------------------------------------------
    type tMCB_MEMORY_ARRAY is array (0 to gCUSTOM_RAM_SIZE-1) of bit_vector(gMCB_DATA_BUS_WIDTH-1 downto 0);
    constant cMCB_MEMORY_DEFAULT : bit_vector(gMCB_DATA_BUS_WIDTH-1 downto 0) := (others => '0');

    ---------------------------------------------------------------------------
    -- Signal Declarations
    ---------------------------------------------------------------------------
    -- MCB Command Buffer signals
    signal McbCmdCntxM      : integer range 0 to cMCB_COMMAND_BUFFER_SIZE   := 0;
    signal McbCmdWrPtrxM    : integer range 0 to cMCB_COMMAND_BUFFER_SIZE-1 := 0;
    signal McbCmdRdPtrxM    : integer range 0 to cMCB_COMMAND_BUFFER_SIZE-1 := 0;
    signal McbCmdRdNxtxM    : integer range 0 to cMCB_COMMAND_BUFFER_SIZE-1 := 0;
    signal McbMemRdPtrxM    : integer range 0 to gCUSTOM_RAM_SIZE-1 := 0;
    signal McbMemWrPtrxM    : integer range 0 to gCUSTOM_RAM_SIZE-1 := 0;
    signal McbCmdArrayxM    : tMCB_COMMAND_ARRAY := (others => cMCB_COMMAND_DEFAULT);

    signal McbAppCmdValxM   : std_logic := '0';                                     -- MCB Command Buffer Valid
    signal McbRdBufReqxM    : std_logic := '0';                                     -- MCB Read Buffer Data Transfer Request
    signal McbWrBufReqxM    : std_logic := '0';                                     -- MCB Write Buffer Data Transfer Request

    -- MCB Write Buffer signals
    signal McbWrBufCntxM    : integer range 0 to cMCB_WRITE_BUFFER_SIZE   := 0;
    signal McbWrBufWrPtrxM  : integer range 0 to cMCB_WRITE_BUFFER_SIZE-1 := 0;
    signal McbWrBufRdPtrxM  : integer range 0 to cMCB_WRITE_BUFFER_SIZE-1 := 0;
    signal McbWrBufArrayxM  : tMCB_WR_DATA_ARRAY := (others => cMCB_WRITE_DEFAULT);
    signal McbMemoryArrayxM : tMCB_MEMORY_ARRAY  := (others => cMCB_MEMORY_DEFAULT);

    signal McbAppWrValxM    : std_logic := '0';                                     -- MCB Write-Data-Interface Transfer Valid
    signal McbWrBufValxM    : std_logic := '0';                                     -- MCB Write Buffer Data Valid
    signal McbWrBufRdyxM    : std_logic := '0';                                     -- MCB Write Buffer Data Valid
    signal McbWrBufDelayxM  : integer   := 0;                                       -- MCB Write Buffer Delay Counter

    -- MCB Read Buffer signals
    signal McbRdBufValxM    : std_logic := '0';                                     -- MCB Read Buffer Data Valid
    signal McbRdBufRdyxM    : std_logic := '0';                                     -- MCB Read Buffer Data Ready
    signal McbRdBufDelayxM  : integer   := 0;                                       -- MCB Read Buffer Delay Counter

    signal McbAppClkxC      : std_logic := '0';                                     -- MCB User-Application Interface Clock
    signal McbAppRstxR      : std_logic := '1';                                     -- MCB User-Application Interface Reset
    signal McbCalibDonexS   : std_logic := '0';                                     -- MCB Calibration complete
    signal McbAppCmdRdyxM   : std_logic := '0';                                     -- MCB User-Application Command Ready
    signal McbAppWrRdyxM    : std_logic := '0';                                     -- MCB User-Application Write Ready
    signal McbAppRdValxM    : std_logic := '0';                                     -- MCB User-Application Read Valid
    signal McbAppRdEndxM    : std_logic := '0';                                     -- MCB User-Application Read End
    signal McbAppRdDataxD   : std_logic_vector(gMCB_DATA_BUS_WIDTH-1 downto 0) := (others => '0');    -- MCB User-Application Read Data

begin

    ---------------------------------------------------------------------------
    -- MCB User-Application Infrastructure
    ---------------------------------------------------------------------------
    McbAppClkxC    <= not(McbAppClkxC) after (1 us/(2*cMCB_USER_FREQ));
    McbAppRstxR    <= '0' after cMCB_USER_RST_PERIOD;
    McbCalibDonexS <= '1' after cMCB_CALIB_DONE_PERIOD;

    ---------------------------------------------------------------------------
    -- MCB User-Application Command-Interface
    ---------------------------------------------------------------------------
    process (McbAppClkxC, McbAppRstxR)
    begin
        if (McbAppRstxR = '1') then
            McbCmdCntxM     <= 0;
            McbCmdWrPtrxM   <= 0;
            McbCmdRdPtrxM   <= 0;
            McbMemRdPtrxM   <= 0;
            McbMemWrPtrxM   <= 0;
            McbRdBufReqxM   <= '0';
            McbWrBufReqxM   <= '0';
            McbCmdArrayxM   <= (others => cMCB_COMMAND_DEFAULT);
        elsif (rising_edge(McbAppClkxC)) then
            -- Receive a MCB command and write it into the command buffer
            if (McbAppCmdValxM = '1') then
                McbCmdArrayxM(McbCmdWrPtrxM).CmdVal  <= '1';
                McbCmdArrayxM(McbCmdWrPtrxM).CmdType <= McbAppCmdCmdxMI;
                McbCmdArrayxM(McbCmdWrPtrxM).CmdAddr <= to_integer(unsigned(McbAppCmdAddrxAI(gMCB_ADDR_BUS_WIDTH-1 downto 3)));

                -- Increment the MCB command buffer write pointer
                if (McbCmdWrPtrxM = cMCB_COMMAND_BUFFER_SIZE-1) then
                    McbCmdWrPtrxM <= 0;
                else
                    McbCmdWrPtrxM <= McbCmdWrPtrxM + 1;
                end if;
            end if;

            -- Count the number of valid commands present in the command buffer.
            if ((McbAppCmdValxM = '1') and (McbRdBufValxM = '0') and (McbWrBufValxM = '0')) then
                McbCmdCntxM <= McbCmdCntxM + 1;
            elsif ((McbAppCmdValxM = '0') and ((McbRdBufValxM = '1') or (McbWrBufValxM = '1'))) then
                McbCmdCntxM <= McbCmdCntxM - 1;
            end if;

            -- Execute an MCB read or write command
            if (McbCmdArrayxM(McbCmdRdPtrxM).CmdVal = '1') then
                if ((McbCmdArrayxM(McbCmdRdPtrxM).CmdType = cMCB_COMMAND_READ) and (McbRdBufReqxM = '0') and (McbRdBufRdyxM = '1')) then
                    McbCmdArrayxM(McbCmdRdPtrxM).CmdVal <= '0';
                    McbCmdRdPtrxM <= McbCmdRdNxtxM;
                    McbRdBufReqxM <= '1';
                    McbWrBufReqxM <= '0';
                    McbMemRdPtrxM <= McbCmdArrayxM(McbCmdRdPtrxM).CmdAddr;
                elsif ((McbCmdArrayxM(McbCmdRdPtrxM).CmdType = cMCB_COMMAND_WRITE) and (McbWrBufReqxM = '0') and (McbWrBufRdyxM = '1')) then
                    McbCmdArrayxM(McbCmdRdPtrxM).CmdVal <= '0';
                    McbCmdRdPtrxM <= McbCmdRdNxtxM;
                    McbRdBufReqxM <= '0';
                    McbWrBufReqxM <= '1';
                    McbMemWrPtrxM <= McbCmdArrayxM(McbCmdRdPtrxM).CmdAddr;
                else
                    McbRdBufReqxM <= '0';
                    McbWrBufReqxM <= '0';
                end if;
            else
                McbRdBufReqxM <= '0';
                McbWrBufReqxM <= '0';
            end if;
        end if;
    end process;

    -- Indicate a valid MCB command transfer
    McbAppCmdValxM <= '1' when ((McbAppCmdEnxMI = '1') and (McbAppCmdRdyxM = '1')) else '0';

    -- Indicate that the MCB command buffer can accept new commands
    McbAppCmdRdyxM <= '1' when (McbCmdCntxM < cMCB_COMMAND_BUFFER_SIZE) else '0';

    -- Calculate the next MCB command read buffer pointer value
    McbCmdRdNxtxM <= 0 when (McbCmdRdPtrxM = cMCB_COMMAND_BUFFER_SIZE-1) else McbCmdRdPtrxM + 1;

    ---------------------------------------------------------------------------
    -- MCB User-Application Write-Data-Interface
    ---------------------------------------------------------------------------
    process (McbAppClkxC, McbAppRstxR)
    begin
        if (McbAppRstxR = '1') then
            McbWrBufWrPtrxM  <= 0;
            McbWrBufRdPtrxM  <= 0;
            McbWrBufCntxM    <= 0;
            McbWrBufDelayxM  <= 0;
            McbWrBufArrayxM  <= (others => cMCB_WRITE_DEFAULT);
            McbMemoryArrayxM <= (others => cMCB_MEMORY_DEFAULT);
        elsif (rising_edge(McbAppClkxC)) then
            -- Receive MCB write data and write it into the write data buffer
            if (McbAppWrValxM = '1') then
                -- McbAppWrEndxMI must be set to '1' for all write data transfers
                McbWrBufArrayxM(McbWrBufWrPtrxM).WrVal  <= '1';
                McbWrBufArrayxM(McbWrBufWrPtrxM).WrEnd  <= McbAppWrEndxMI;
                McbWrBufArrayxM(McbWrBufWrPtrxM).WrMask <= to_bitvector(McbAppWrMaskxMI);
                McbWrBufArrayxM(McbWrBufWrPtrxM).WrData <= to_bitvector(McbAppWrDataxDI);

                -- Increment the MCB write buffer write pointer
                if (McbWrBufWrPtrxM = cMCB_WRITE_BUFFER_SIZE-1) then
                    McbWrBufWrPtrxM <= 0;
                else
                    McbWrBufWrPtrxM <= McbWrBufWrPtrxM + 1;
                end if;
            end if;

            -- Count the number of valid data words present in the write data buffer.
            if ((McbAppWrValxM = '1') and (McbWrBufValxM = '0')) then
                McbWrBufCntxM <= McbWrBufCntxM + 1;
            elsif ((McbAppWrValxM = '0') and (McbWrBufValxM = '1')) then
                McbWrBufCntxM <= McbWrBufCntxM - 1;
            end if;

            -- Transfer write data from the MCB write data buffer to the memory array.
            if (McbWrBufValxM = '1') then
                McbWrBufArrayxM(McbWrBufRdPtrxM).WrVal <= '0';

                for i in 0 to gMCB_MASK_BUS_WIDTH-1 loop
                    if (McbWrBufArrayxM(McbWrBufRdPtrxM).WrMask(i) = cMCB_DATA_VALID) then
                        McbMemoryArrayxM(McbMemWrPtrxM)((i*8)+7 downto (i*8)) <= McbWrBufArrayxM(McbWrBufRdPtrxM).WrData((i*8)+7 downto (i*8));
                    end if;
                end loop;

                -- Increment the MCB write buffer write pointer
                if (McbWrBufRdPtrxM = cMCB_WRITE_BUFFER_SIZE-1) then
                    McbWrBufRdPtrxM <= 0;
                else
                    McbWrBufRdPtrxM <= McbWrBufRdPtrxM + 1;
                end if;
            end if;

            -- Delay execution of MCB write operations to mimic real DDR3 behaviour.
            if (McbWrBufValxM = '1') then
                McbWrBufDelayxM <= cMCB_WRITE_DELAY;
            elsif (McbWrBufDelayxM > 0) then
                McbWrBufDelayxM <= McbWrBufDelayxM - 1;
            end if;
        end if;
    end process;

    -- Indicate a valid MCB write data transfer
    McbAppWrValxM <= '1' when ((McbAppWrEnxMI = '1') and (McbAppWrRdyxM = '1')) else '0';

    -- Indicate that the MCB write data buffer can accept new commands
    McbAppWrRdyxM <= '1' when (McbWrBufCntxM < cMCB_WRITE_BUFFER_SIZE) else '0';

    -- Indicate that the MCB write data buffer contains one or more valid data words
    -- and that the MCB delay period has passed since the last write operation.
    McbWrBufRdyxM <= '1' when ((McbWrBufArrayxM(McbWrBufRdPtrxM).WrVal = '1') and (McbWrBufDelayxM = 0)) else '0';

    -- Indicate a valid MCB write data transfer into the memory array
    McbWrBufValxM <= '1' when ((McbWrBufReqxM = '1') and (McbWrBufRdyxM = '1')) else '0';

    ---------------------------------------------------------------------------
    -- MCB User-Application Read-Data-Interface
    ---------------------------------------------------------------------------
    process (McbAppClkxC, McbAppRstxR)
    begin
        if (McbAppRstxR = '1') then
            McbAppRdValxM   <= '0';
            McbAppRdEndxM   <= '0';
            McbAppRdDataxD  <= (others => '0');
            McbRdBufDelayxM <= 0;
         elsif (rising_edge(McbAppClkxC)) then
            -- Retrieve data from the memory array and transfer it over the MCB Read-Data-Interface.
            if (McbRdBufValxM = '1') then
                McbAppRdValxM  <= '1';
                McbAppRdEndxM  <= '1';
                McbAppRdDataxD <= to_stdlogicvector(McbMemoryArrayxM(McbMemRdPtrxM));
            else
                McbAppRdValxM <= '0';
            end if;

            -- Delay execution of MCB read operations to mimic real DDR3 behaviour.
            if (McbRdBufValxM = '1') then
                McbRdBufDelayxM <= cMCB_READ_DELAY;
            elsif (McbRdBufDelayxM > 0) then
                McbRdBufDelayxM <= McbRdBufDelayxM - 1;
            end if;
        end if;
    end process;

    -- Control the read process by delaying the read ready signal.
    McbRdBufRdyxM <= '1' when (McbRdBufDelayxM = 0) else '0';

    -- Indicate a valid MCB read data transfer from the memory array
    McbRdBufValxM <= '1' when ((McbRdBufReqxM = '1') and (McbRdBufRdyxM = '1')) else '0';

    ---------------------------------------------------------------------------
    --Assign internally used output ports
    ---------------------------------------------------------------------------
    McbAppClkxCO    <= McbAppClkxC;
    McbAppRstxRO    <= McbAppRstxR;
    McbCalibDonexSO <= McbCalibDonexS;

    McbAppCmdRdyxMO <= McbAppCmdRdyxM;
    McbAppWrRdyxMO  <= McbAppWrRdyxM;
    McbAppRdValxMO  <= McbAppRdValxM;
    McbAppRdEndxMO  <= McbAppRdEndxM;
    McbAppRdDataxDO <= McbAppRdDataxD;

end Behavioral;
