-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Testbench Simulation Library
-------------------------------------------------------------------------------
-- Copyright (c) 2013 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2013-05-20  1.0      Manfred  Created
--              2015-04-16  2.0      a.kramer Changed for GigEpro
--              2017-03-02  3.0      Manfred  Changed for CorSight2
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_unsigned.all;  
use ieee.numeric_std.all;         
use ieee.math_real.all;

library std;
use std.textio.all;
use work.CustomCfgLib.all;

package CustomSimLib is 

    ---------------------------------------------------------------------------
    -- INI File Key Words and Characters
    ---------------------------------------------------------------------------
    constant cEQUAL_CHAR            : character := '=';         -- Assignment char in INI file
    constant cINVERT_CHAR           : character := '!';         -- Logic inversion char in INI file
    constant cINI_COMMENT           : character := ';';         -- Comment in INI file 
    constant cSECTION_START         : character := '[';         -- Section start char in INI file
    constant cSECTION_END           : character := ']';         -- Section end char in INI file
    constant cIMG_COMMENT           : character := '#';         -- Comment in image file
    constant cTRAP_CHAR             : character := '\';         -- Trap control characters in string messages
    constant cHEX_PREFIX            : string    := "0x";        -- Define prefix for hexadecimal numbers

    constant cPBM_ASCII_MAGIC_NUM   : string    := "p1";        -- Magic number for ASCII pbm (bitmap) files
    constant cPGM_ASCII_MAGIC_NUM   : string    := "p2";        -- Magic number for ASCII pgm (monochrome) files
    constant cPPM_ASCII_MAGIC_NUM   : string    := "p3";        -- Magic number for ASCII ppm (colour) files
    constant cPBM_BIN_MAGIC_NUM     : string    := "p4";        -- Magic number for binary pbm (bitmap) files
    constant cPGM_BIN_MAGIC_NUM     : string    := "p5";        -- Magic number for binary pgm (monochrome) files
    constant cPPM_BIN_MAGIC_NUM     : string    := "p6";        -- Magic number for binary ppm (colour) files
    
    constant cMAX_WORD_LENGTH       : natural   := 16;          -- Number of characters per key word in simulatin INI file
    constant cMAX_STRING_LENGTH     : natural   := 256;         -- Number of characters in a message string

    -- Section names in simulatin INI file
    constant cSECTION_SIMULATION    : string    := "simulation      ";
    constant cSECTION_IMAGEMODE     : string    := "imagemode       ";
    constant cSECTION_IMAGEINPUT    : string    := "imageinput      ";
    constant cSECTION_IMAGEOUTPUT   : string    := "imageoutput     ";
    constant cSECTION_IMAGECOMPARE  : string    := "imagecompare    ";

    -- Simulation section key names
    constant cSIMKEY_RSTPERIOD      : string    := "resetperiod     ";
    constant cSIMKEY_MAXSIMTIME     : string    := "maxsimtime      ";
    constant cSIMKEY_COMMANDFILE    : string    := "commandfile     ";

    -- ImageMode section key names
    constant cSIMKEY_SCANLINEMODE   : string    := "scanlinemode    ";
    constant cSIMKEY_SINGLEFRM      : string    := "singlefrm       ";
    constant cSIMKEY_PIXINFREQ      : string    := "pixinfreq       ";
    constant cSIMKEY_PIXOUTFREQ     : string    := "pixoutfreq      ";
    constant cSIMKEY_PIXWAITSEQ     : string    := "pixwaitseq      ";
    constant cSIMKEY_PPMBGRMODE     : string    := "ppmbgrmode      ";
    constant cSIMKEY_REPEATIMGSEQ   : string    := "repeatimgseq    ";
    constant cSIMKEY_HORSYNC        : string    := "horsync         ";
    constant cSIMKEY_VERSYNC        : string    := "versync         ";

    -- ImageInput section key names
    constant cSIMKEY_STARTPIX       : string    := "startpixel      ";
    constant cSIMKEY_STARTLIN       : string    := "startline       ";
    constant cSIMKEY_NUMPIX         : string    := "numpixel        ";
    constant cSIMKEY_NUMLIN         : string    := "numline         ";
    constant cSIMKEY_SRCIMGFILE     : string    := "srcimgfile      ";

    -- ImageOutput section key names
    constant cSIMKEY_OUTIMGENABLE   : string    := "outimgenable    ";
    constant cSIMKEY_OUTIMGTYPE     : string    := "outimgtype      ";
    constant cSIMKEY_OUTIMGBITS     : string    := "outimgbits      ";
    constant cSIMKEY_OUTIMGFRMT     : string    := "outimgfrmt      ";
    constant cSIMKEY_OUTIMGFILE     : string    := "outimgfile      ";

    -- ImageAnalyzer section key names
    constant cSIMKEY_CMPIMGPIXEL    : string    := "cmpimgpixel     ";
    constant cSIMKEY_CMPIMGFORMAT   : string    := "cmpimgformat    ";
    constant cSIMKEY_CMPIMGABORT    : string    := "cmpimgabort     ";
    constant cSIMKEY_CMPIMGERRCNT   : string    := "cmpimgpixerr    ";
    constant cSIMKEY_CMPIMGFILE     : string    := "cmpimgfile      ";
                                               
    -- Command file key names
    constant cCMDKEY_SYSWRITE       : string    := "syswrite        ";
    constant cCMDKEY_SYSREAD        : string    := "sysread         ";
    constant cCMDKEY_SYSVERIFY      : string    := "sysverify       ";
    constant cCMDKEY_SIMWAIT        : string    := "wait            ";
    constant cCMDKEY_EVENTSIG       : string    := "eventsignal     ";
    constant cCMDKEY_EVENTBUS       : string    := "eventbus        ";

    constant cCMDKEY_RISEEDGE       : string    := "rising          ";
    constant cCMDKEY_FALLEDGE       : string    := "falling         ";
    constant cCMDKEY_HIGHLEVEL      : string    := "high            ";
    constant cCMDKEY_LOWLEVEL       : string    := "low             ";
    constant cCMDKEY_EQUALOP        : string    := "eq              ";
    constant cCMDKEY_NOTEQOP        : string    := "ne              ";

    constant cCMDKEY_IMGBUSRESET    : string    := "imgbusreset     ";
    constant cCMDKEY_STARTIMGBUS    : string    := "startimgbus     ";
    constant cCMDKEY_STOPIMGBUS     : string    := "stopimgbus      ";
    constant cCMDKEY_ABORTIMGBUS    : string    := "abortimgbus     ";
    constant cCMDKEY_PRINTMSG       : string    := "printmsg        ";

    -- List of available Key Words in cTESTBENCH_INI_FILE
    type tSIM_KEY_NAME_LIST is (none, RstPeriod, MaxSimTime, CommandFile, ScanLineMode, SingleFrm, PixInFreq, 
                                PixOutFreq, PixWaitSeq, PpmBGRMode, StartPix, StartLin, NumPix, NumLin, HorSync, VerSync, 
                                SrcImgFile, CmpImgFile, CmpImgPixErr, CmpImgFrmt, CmpImgAbort, CmpImgPix, RepeatImgSeq, 
                                OutImgEnable, OutImgType, OutImgBits, OutImgFile);

    ---------------------------------------------------------------------------
    -- Simulation Definitions
    ---------------------------------------------------------------------------
    constant cMAX_FILE_NAMES        : natural := 128;           -- Maximum number of ImageBus source files
    constant cMAX_FILENAME_LENGTH   : natural := 256;           -- Maximum number of characters for file name strings
    constant cBLANK_FILENAME        : string(1 to cMAX_FILENAME_LENGTH) := (others => ' '); -- Default (empty) file name

    type tSIM_MODULE_NAME is (default, Stimulus, Analyzer);     -- Define the names of simulation modules
    type tIMG_FILE_ARRAY is array (0 to cMAX_FILE_NAMES-1) of string(1 to cMAX_FILENAME_LENGTH);

    ---------------------------------------------------------------------------

    -- Simulation section parameters
    type tSIMULATION_PARAMETERS is record
        RstPeriod   : time;
        MaxSimTime  : time;
        CommandFile : string(1 to cMAX_FILENAME_LENGTH);
    end record;

    constant cSIMULATION_PARA_DEFAULT : tSIMULATION_PARAMETERS := (
        RstPeriod   => cSYS_RESET_PERIOD,
        MaxSimTime  => cMAX_SIM_TIME,
        CommandFile => cBLANK_FILENAME);

    ---------------------------------------------------------------------------

    -- ImageMode section parameters
    type tIMAGE_MODE_PARAMETERS is record
        ScanLineMode : boolean;
        SingleFrm    : boolean;
        PixInFreq    : natural; 
        PixOutFreq   : natural; 
        WaitPeriod   : natural; 
        WaitStates   : natural; 
        WaitModule   : tSIM_MODULE_NAME;
        PpmBgrMode   : boolean;
        RepeatImgSeq : boolean;
        HorSync      : natural;
        VerSync      : natural;
    end record;

    constant cIMAGE_MODE_PARA_DEFAULT : tIMAGE_MODE_PARAMETERS := (
        ScanLineMode => False,
        SingleFrm    => False,
        PixInFreq    => cPIX_IN_FREQ_DEFAULT,
        PixOutFreq   => cPIX_OUT_FREQ_DEFAULT,
        WaitPeriod   => 0,
        WaitStates   => 0,
        WaitModule   => Stimulus,
        PpmBgrMode   => False,
        RepeatImgSeq => False,
        HorSync      => 0,
        VerSync      => 0);

    ---------------------------------------------------------------------------

    -- ImageInput section parameters
    type tIMAGE_INPUT_PARAMETERS is record
        StartPix     : natural;   
        StartLin     : natural;
        NumPix       : natural;
        NumLin       : natural;

        SrcNumFile   : natural;
        SrcImgFile   : tIMG_FILE_ARRAY;
    end record;

    constant cIMAGE_INPUT_PARA_DEFAULT : tIMAGE_INPUT_PARAMETERS := (
        StartPix     => 0,
        StartLin     => 0,
        NumPix       => 0,
        NumLin       => 0,

        SrcNumFile   => 0,
        SrcImgFile   => (others => (others => ' ')));

    ---------------------------------------------------------------------------

    -- ImageCompare section parameters
    type tIMAGE_COMPARE_PARAMETERS is record
        CmpImgFrmt   : boolean;
        CmpImgPix    : boolean;
        CmpImgAbort  : boolean;
        CmpImgPixErr : natural;
        CmpNumFile   : natural;
        CmpFileName  : tIMG_FILE_ARRAY;
    end record;

    constant cIMAGE_COMPARE_PARA_DEFAULT : tIMAGE_COMPARE_PARAMETERS := (
        CmpImgFrmt   => False,
        CmpImgPix    => False,
        CmpImgAbort  => False,
        CmpImgPixErr => 0,
        CmpNumFile   => 0,
        CmpFileName  => (others => (others => ' ')));

    ---------------------------------------------------------------------------

    -- ImageOutput section parameters
    type tIMAGE_OUTPUT_PARAMETERS is record
        OutImgEnable : boolean;
        OutImgBits   : natural;
        OutImgType   : string(1 to 3);
        OutImgFrmt   : string(1 to 5);
        OutImgFile   : string(1 to cMAX_FILENAME_LENGTH);
    end record;

    constant cIMAGE_OUTPUT_PARA_DEFAULT : tIMAGE_OUTPUT_PARAMETERS := (
        OutImgEnable => False,
        OutImgBits   => 8,
        OutImgType   => (others => ' '),
        OutImgFrmt   => (others => ' '),
        OutImgFile   => (others => ' '));

    ---------------------------------------------------------------------------

    -- Simulation parameter record, read from simulation ini file.
    type tIMG_SIM_PARAMETERS is record
        SimPara  : tSIMULATION_PARAMETERS;
        ImgMode  : tIMAGE_MODE_PARAMETERS;
        ImgIn    : tIMAGE_INPUT_PARAMETERS;
        ImgOut   : tIMAGE_OUTPUT_PARAMETERS;
        ImgCmp   : tIMAGE_COMPARE_PARAMETERS;
    end record;

    constant cIMG_SIM_PARA_DEFAULT : tIMG_SIM_PARAMETERS := (
        SimPara  => cSIMULATION_PARA_DEFAULT,
        ImgMode  => cIMAGE_MODE_PARA_DEFAULT,
        ImgIn    => cIMAGE_INPUT_PARA_DEFAULT,
        ImgOut   => cIMAGE_OUTPUT_PARA_DEFAULT,
        ImgCmp   => cIMAGE_COMPARE_PARA_DEFAULT);

    ---------------------------------------------------------------------------
    -- Command File Definitions
    ---------------------------------------------------------------------------
    constant cCMD_ADDR_WIDTH : natural := 32;           -- Stimulus command address bus width [bits]
    constant cCMD_DATA_WIDTH : natural := 32;           -- Stimulus command data bus width [bits]
    constant cMAX_CMD_ARGS   : natural := 128;          -- Maximum arguments per command line

    -- List of available command key words in simulation command file
    type tCMD_KEY_NAME_LIST is (none, SysWrite, SysRead, SysVerify, SimWait, DebugSig, DebugBus, PrintMsg);

    type tCMD_DATA_ARRAY is array (0 to cMAX_CMD_ARGS-1) of bit_vector(cCMD_DATA_WIDTH-1 downto 0);

    -----------------------------------------------------------

    type tCMD_DATA_RECORD is record
        Valid    : boolean;
        WrCycle  : boolean;
        Verify   : boolean;
        ModCycle : boolean;
        DdrCycle : boolean;
        DataCnt  : natural range 0 to cMAX_CMD_ARGS;
        CmdAddr  : bit_vector(cCMD_ADDR_WIDTH-1 downto 0);
        CmdMask  : bit_vector(cCMD_DATA_WIDTH-1 downto 0);
        CmdData  : tCMD_DATA_ARRAY;
    end record;

    constant cCMD_DATA_DEFAULT : tCMD_DATA_RECORD := (
        Valid    => False,
        WrCycle  => False,
        Verify   => False,
        ModCycle => False,
        DdrCycle => False,
        DataCnt  => 0,
        CmdAddr  => (others => '0'), 
        CmdMask  => (others => '0'), 
        CmdData  => (others => (others => '0'))); 

    ---------------------------------------------------------------------------
    -- ImageBus  Definitions
    ---------------------------------------------------------------------------
    subtype tPIXEL_BUFFER    is bit_vector(cMAX_PIXEL_WIDTH-1 downto 0);
    type tLINE_BUFFER_ARRAY  is array (0 to cMAX_NUM_PIXEL-1) of tPIXEL_BUFFER;
    type tFRAME_BUFFER_ARRAY is array (0 to cMAX_NUM_LINES-1) of tLINE_BUFFER_ARRAY;
    type tFRAME_BUFFER_PTR   is access tFRAME_BUFFER_ARRAY;

    constant cIMGBUS_PIXEL_FREQ     : integer             := 150;      -- ImageBus clock frequency
    constant cPIXEL_BUFFER_DEFAULT  : tPIXEL_BUFFER       := (others => '0');
    constant cLINE_BUFFER_DEFAULT   : tLINE_BUFFER_ARRAY  := (others => cPIXEL_BUFFER_DEFAULT);
    constant cFRAME_BUFFER_DEFAULT  : tFRAME_BUFFER_ARRAY := (others => cLINE_BUFFER_DEFAULT);

    type tIMG_FILE_MODE is (MagicNo, ImgSize, PixValue, ImgAsciiData, ImgBinData);      -- Sequence of PGM image file parameters

    -----------------------------------------------------------

    type tIMG_FILE_RECORD is record
        Valid      : boolean;
        MagicNum   : string(1 to 2);
        MaxPix     : natural;
        MaxLin     : natural;
        NumBits    : natural;
        FrmBuffer  : tFRAME_BUFFER_ARRAY;
    end record;

    constant cIMG_FILE_DEFAULT : tIMG_FILE_RECORD := (
        Valid      => False,
        MagicNum   => "  ",
        MaxPix     => 0,
        MaxLin     => 0,
        NumBits    => 8,
        FrmBuffer  => cFRAME_BUFFER_DEFAULT); 

    -----------------------------------------------------------

    type tIMG_BUS_RECORD is record
        FirstLin   : boolean;
        LastLin    : boolean;
        NumBits    : natural;
        LineBuf    : tLINE_BUFFER_ARRAY;
    end record;

    constant cIMG_BUS_DEFAULT : tIMG_BUS_RECORD := (
        FirstLin   => False,
        LastLin    => False,
        NumBits    => 8,
        LineBuf    => cLINE_BUFFER_DEFAULT);

    -----------------------------------------------------------------------------
    -- Custom Module Definitions
    -----------------------------------------------------------------------------
    constant cCUST_INSTR_BUS_WIDTH : integer := 3;              -- Width of Custom Module Instruction bus
    constant cCUST_BLEN_BUS_WIDTH  : integer := 6;              -- Width of Custom Module Burst-Length bus
    constant cCUST_ADDR_BUS_WIDTH  : integer := 30;             -- Width of Custom Module Address bus
    constant cCUST_DATA_BUS_WIDTH  : integer := 32;             -- Width of Custom Module Data bus
    constant cCUST_GPIO_BUS_WIDTH  : integer := 6;              -- Width of Custom Module GPIO bus

    ---------------------------------------------------------------------------
    -- Memory Controller Bus  Definitions
    ---------------------------------------------------------------------------
    constant cMCB_SYSTEM_FREQ      : integer := 400;            -- MCB System clock frequency
    constant cMCB_REFERENCE_FREQ   : integer := 200;            -- MCB Reference clock frequency
    constant cMCB_USER_FREQ        : integer := 100;            -- MCB User Interface clock frequency

    constant cMCB_CMD_BUS_WIDTH    : integer := 3;              -- Width of MCB Command bus
    constant cMCB_ADDR_BUS_WIDTH   : integer := 27;             -- Width of MCB Address bus
    constant cMCB_MASK_BUS_WIDTH   : integer := 16;             -- Width of MCB Mask bus
    constant cMCB_DATA_BUS_WIDTH   : integer := 128;            -- Width of MCB Data bus

    constant cMCB_DEBUG_ENABLE     : boolean := True;           -- MCB Debug Port enable
    constant cMCB_DEBUG_WIDTH      : integer := 256;            -- Width of MCB Debug Port

    ---------------------------------------------------------------------------
    -- Error Message Definitions
    ---------------------------------------------------------------------------
    subtype tSIM_ERROR_MESSAGE is string(1 to 64);
    constant cSIM_ERROR_DEFAULT     : tSIM_ERROR_MESSAGE := (others => ' ');

    -- ReadImgFile Error Messages
    constant cRDIMGFILE_BUFOVRFLOW      : tSIM_ERROR_MESSAGE := "Input Buffer Overflow                                           ";
    constant cRDIMGFILE_ERRNUM_2        : tSIM_ERROR_MESSAGE := "Error 2                                                         ";
    constant cRDIMGFILE_ERRNUM_3        : tSIM_ERROR_MESSAGE := "Error 3                                                         ";
    constant cRDIMGFILE_ERRNUM_5        : tSIM_ERROR_MESSAGE := "Error 5                                                         ";
    constant cRDIMGFILE_ERRNUM_7        : tSIM_ERROR_MESSAGE := "Error 7                                                         ";
    constant cRDIMGFILE_ERRNUM_8        : tSIM_ERROR_MESSAGE := "Error 8                                                         ";
    constant cRDIMGFILE_ERRNUM_9        : tSIM_ERROR_MESSAGE := "Error 9                                                         ";
    constant cRDIMGFILE_ERRNUM_10       : tSIM_ERROR_MESSAGE := "Error 10                                                        ";
    constant cRDIMGFILE_ERRNUM_11       : tSIM_ERROR_MESSAGE := "Error 11                                                        ";
    constant cRDIMGFILE_ERRNUM_12       : tSIM_ERROR_MESSAGE := "Error 12                                                        ";
    constant cRDIMGFILE_ERRNUM_13       : tSIM_ERROR_MESSAGE := "Error 13                                                        ";
    constant cRDIMGFILE_ERRNUM_14       : tSIM_ERROR_MESSAGE := "Error 14                                                        ";

    ---------------------------------------------------------------------------

    -- SysBusStimulus Error Messages
    constant cSYSBUSSTIM_ERRNUM_1       : tSIM_ERROR_MESSAGE := "Error 1                                                         ";
    constant cSYSBUSSTIM_ERRNUM_2       : tSIM_ERROR_MESSAGE := "Error 2                                                         ";
    constant cSYSBUSSTIM_ERRNUM_3       : tSIM_ERROR_MESSAGE := "Error 3                                                         ";
    constant cSYSBUSSTIM_ERRNUM_4       : tSIM_ERROR_MESSAGE := "Error 4                                                         ";
    constant cSYSBUSSTIM_ERRNUM_5       : tSIM_ERROR_MESSAGE := "Error 5                                                         ";
    constant cSYSBUSSTIM_ERRNUM_6       : tSIM_ERROR_MESSAGE := "Error 6                                                         ";
    constant cSYSBUSSTIM_ERRNUM_7       : tSIM_ERROR_MESSAGE := "Error 7                                                         ";
    constant cSYSBUSSTIM_ERRNUM_8       : tSIM_ERROR_MESSAGE := "Error 8                                                         ";
    constant cSYSBUSSTIM_ERRNUM_9       : tSIM_ERROR_MESSAGE := "Error 9                                                         ";
    constant cSYSBUSSTIM_ERRNUM_10      : tSIM_ERROR_MESSAGE := "Error 10                                                        ";
    constant cSYSBUSSTIM_ERRNUM_11      : tSIM_ERROR_MESSAGE := "Error 11                                                        ";
    constant cSYSBUSSTIM_ERRNUM_12      : tSIM_ERROR_MESSAGE := "Error 12                                                        ";
    constant cSYSBUSSTIM_ERRNUM_13      : tSIM_ERROR_MESSAGE := "Error 13                                                        ";
    constant cSYSBUSSTIM_ERRNUM_14      : tSIM_ERROR_MESSAGE := "Error 14                                                        ";
    constant cSYSBUSSTIM_ERRNUM_15      : tSIM_ERROR_MESSAGE := "Error 15                                                        ";
    constant cSYSBUSSTIM_ERRNUM_16      : tSIM_ERROR_MESSAGE := "Error 16                                                        ";
    constant cSYSBUSSTIM_ERRNUM_17      : tSIM_ERROR_MESSAGE := "Error 17                                                        ";
    constant cSYSBUSSTIM_ERRNUM_18      : tSIM_ERROR_MESSAGE := "Error 18                                                        ";
    constant cSYSBUSSTIM_ERRNUM_19      : tSIM_ERROR_MESSAGE := "Error 19                                                        ";
    constant cSYSBUSSTIM_ERRNUM_20      : tSIM_ERROR_MESSAGE := "Error 20                                                        ";
    constant cSYSBUSSTIM_ERRNUM_21      : tSIM_ERROR_MESSAGE := "Error 21                                                        ";
    constant cSYSBUSSTIM_ERRNUM_22      : tSIM_ERROR_MESSAGE := "Error 22                                                        ";

    ---------------------------------------------------------------------------

    -- SysBusStimulus Error Messages
    constant cSIMPARAINIT_INVSECTNAME       : tSIM_ERROR_MESSAGE := "Invalid Section Name                                            ";
    constant cSIMPARAINIT_INVKEYNAME        : tSIM_ERROR_MESSAGE := "Invalid Key Name                                                ";
    constant cSIMPARAINIT_INVTIMEBASE       : tSIM_ERROR_MESSAGE := "Invalid Time Base Specifier                                     ";
    constant cSIMPARAINIT_INVSIMKEY         : tSIM_ERROR_MESSAGE := "Invalid Key Name in Simulation Section                          ";
    constant cSIMPARAINIT_INVSCANVAL        : tSIM_ERROR_MESSAGE := "Invalid ScanLineMode Value [0, 1]                               ";
    constant cSIMPARAINIT_INVSGLFRMVAL      : tSIM_ERROR_MESSAGE := "Invalid SingleFrm Value [0, 1]                                  ";
    constant cSIMPARAINIT_PIXINFREQRANGE    : tSIM_ERROR_MESSAGE := "PixInFreq Value out of range [Max = 150 MHz]                    ";
    constant cSIMPARAINIT_INVPIXINFREQVAL   : tSIM_ERROR_MESSAGE := "Invalid PixInFreq Value                                         ";
    constant cSIMPARAINIT_PIXOUTFREQRANGE   : tSIM_ERROR_MESSAGE := "PixOutFreq Value out of range [Max = 150 MHz]                   ";
    constant cSIMPARAINIT_INVPIXOUTFREQVAL  : tSIM_ERROR_MESSAGE := "Invalid PixOutFreq Value                                        ";
    constant cSIMPARAINIT_INVPIXWAITSEQVAL  : tSIM_ERROR_MESSAGE := "Invalid PixWaitSeq Value                                        ";
    constant cSIMPARAINIT_INVBGRMODEVAL     : tSIM_ERROR_MESSAGE := "Invalid PpmBGRMode Value                                        ";
    constant cSIMPARAINIT_ERRNUM_11         : tSIM_ERROR_MESSAGE := "Error 11                                                        ";
    constant cSIMPARAINIT_ERRNUM_12         : tSIM_ERROR_MESSAGE := "Error 12                                                        ";
    constant cSIMPARAINIT_ERRNUM_13         : tSIM_ERROR_MESSAGE := "Error 13                                                        ";
    constant cSIMPARAINIT_ERRNUM_14         : tSIM_ERROR_MESSAGE := "Error 14                                                        ";
    constant cSIMPARAINIT_ERRNUM_15         : tSIM_ERROR_MESSAGE := "Error 15                                                        ";
    constant cSIMPARAINIT_ERRNUM_16         : tSIM_ERROR_MESSAGE := "Error 16                                                        ";
    constant cSIMPARAINIT_ERRNUM_17         : tSIM_ERROR_MESSAGE := "Error 17                                                        ";
    constant cSIMPARAINIT_ERRNUM_18         : tSIM_ERROR_MESSAGE := "Error 18                                                        ";
    constant cSIMPARAINIT_ERRNUM_19         : tSIM_ERROR_MESSAGE := "Error 19                                                        ";
    constant cSIMPARAINIT_ERRNUM_20         : tSIM_ERROR_MESSAGE := "Error 20                                                        ";
    constant cSIMPARAINIT_ERRNUM_21         : tSIM_ERROR_MESSAGE := "Error 21                                                        ";
    constant cSIMPARAINIT_ERRNUM_22         : tSIM_ERROR_MESSAGE := "Error 22                                                        ";
    constant cSIMPARAINIT_ERRNUM_23         : tSIM_ERROR_MESSAGE := "Error 23                                                        ";
    constant cSIMPARAINIT_ERRNUM_24         : tSIM_ERROR_MESSAGE := "Error 24                                                        ";
    constant cSIMPARAINIT_ERRNUM_25         : tSIM_ERROR_MESSAGE := "Error 25                                                        ";
    constant cSIMPARAINIT_ERRNUM_26         : tSIM_ERROR_MESSAGE := "Error 26                                                        ";
    constant cSIMPARAINIT_ERRNUM_27         : tSIM_ERROR_MESSAGE := "Error 27                                                        ";
    constant cSIMPARAINIT_ERRNUM_28         : tSIM_ERROR_MESSAGE := "Error 28                                                        ";
    constant cSIMPARAINIT_ERRNUM_29         : tSIM_ERROR_MESSAGE := "Error 29                                                        ";

end package CustomSimLib;
