-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Testbench ImageBus Analyzer
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2013-05-20  1.0      Manfred  Created
--              2015-04-16  2.0      a.kramer Changed for GigEpro
--              2017-03-02  3.0      Manfred  Modified for CorSight2
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library std;
use std.textio.all;

library work;
use work.CustomCfgLib.all;
use work.CustomSimLib.all;
use work.CustomFctLib.all;
use work.CustomPkg.all;

entity ImgBusAnalyzer is
    port (  
        -- Infrastructure Signals
        ImgClkxCI         : in  std_logic;              -- ImageBus Clock
        ImgRstxRI         : in  std_logic;              -- ImageBus Reset
        ImgSimParaxTI     : in  tIMG_SIM_PARAMETERS;    -- ImageBus Simulation Parameters

        -- ImageBus Transmit Port              
        ImgTxPixValxMI    : in  std_logic;              -- ImageBus Transmit Pixel Valid
        ImgTxPixRdyxMO    : out std_logic;              -- ImageBus Transmit Pixel Ready - ignore if cIMG_HANDSHAKE_EN = False
        ImgTxPixStatxSI   : in  std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);  -- ImageBus Transmit Pixel Status
        ImgTxPixDataxDI   : in  std_logic_vector(cIMG_DATA_WIDTH-1 downto 0)); -- ImageBus Transmit Pixel Data
end ImgBusAnalyzer;

architecture Behavioral of ImgBusAnalyzer is

    signal ImgTxPixRdyxM    : std_logic := '0';              -- ImageBus Transmit Pixel Ready

begin

    ----------------------------------------------------------------------------
    -- ImageBus Analyzer Pixel Flow Control
    ----------------------------------------------------------------------------
    AnalyzeClkProc: process 
        variable TimeCnt    : integer := 0;
        variable ClkCycTime : integer := 0;
        variable RdyCycTime : integer := 0;
        variable WaitPeriod : integer := 1;
    begin
        ImgTxPixRdyxM <= '0'; 
        wait until (ImgRstxRI = '0');
        ImgTxPixRdyxM <= '1'; 

        ClkCycTime := 1000/cIMGBUS_PIXEL_FREQ;                  -- ImageBus pixel clock cycle time, in ns
        RdyCycTime := 1000/ImgSimParaxTI.ImgMode.PixOutFreq;    -- ImageBus Analyzer Ready signal cycle time, in ns

        -- Control ImageBus pixel transfers and handshake.
        AnalyzerClkLoop: loop
            wait until rising_edge(ImgClkxCI);
            -- Implement a pixel clock divider.
            if (ImgTxPixValxMI = '1') then
                -- Increment TimeCnt by the value of the ImageBus pixel clock period
                -- until the requested Ready cycle time has been reached. Assert
                -- ImgTxPixRdyxM at that point and 
                --                __    __    __    __    __    __    __    __   
                -- ImgBus Clk  __/  \__/  \__/  \__/  \__/  \__/  \__/  \__/  \__
                --                _____       _____       _____       _____      
                -- Ready Clk   __/     \_____/     \_____/     \_____/     \_____
                --                __          __          __          __         
                -- ImgTxPixRdy __/  \________/  \________/  \________/  \________

                TimeCnt := TimeCnt + ClkCycTime;

                if (TimeCnt >= RdyCycTime) then
                    TimeCnt := TimeCnt - RdyCycTime;
                    ImgTxPixRdyxM <= '1';
                else
                    ImgTxPixRdyxM <= '0';
                end if;

                --------------------------------------------------------------------------

                -- Implement a method to insert wait states into the pixel stream.
                -- Insert a pre-determined number of wait states during a defined period.
                if ((ImgSimParaxTI.ImgMode.WaitModule = Analyzer) and (ImgSimParaxTI.ImgMode.WaitStates > 0)) then
                    if (WaitPeriod >= (ImgSimParaxTI.ImgMode.WaitPeriod-ImgSimParaxTI.ImgMode.WaitStates)) then
                        ImgTxPixRdyxM <= '0';
                    else
                        ImgTxPixRdyxM <= '1';
                    end if;
        
                    WaitPeriod := WaitPeriod + 1;
                    if (WaitPeriod = ImgSimParaxTI.ImgMode.WaitPeriod) then
                        WaitPeriod := 0;
                    end if;
                end if;
            end if;
        end loop;

        wait;
    end process;

    ImgTxPixRdyxMO <= ImgTxPixRdyxM;

    ----------------------------------------------------------------------------
    -- ImageBus Analyzer 
    ----------------------------------------------------------------------------
    ImgAnalyzeProc: process 
        variable CurPixCnt   : natural := 0;
        variable CurLinCnt   : natural := 0;
        variable CurFrmCnt   : natural := 0;
        variable FrmNumPix   : natural := 0;
        variable FrmNumLin   : natural := 0;
        variable FileNumber  : natural := 0;
        variable FrmtErrCnt  : natural := 0;
        variable FrmtCycCnt  : natural := 0;
        variable FrmtTotCnt  : natural := 0;
        variable PixErrCnt   : natural := 0;
        variable PixCmpCnt   : natural := 0;
        variable PixCycCnt   : natural := 0;
        variable LastLine    : boolean := False;
        variable LineActive  : boolean := False;
        variable FrameActive : boolean := False;
        variable ImgCmpData  : tIMG_FILE_RECORD := cIMG_FILE_DEFAULT;
        variable ImgSavData  : tIMG_FILE_RECORD := cIMG_FILE_DEFAULT;
        variable ImgBusData  : tIMG_BUS_RECORD  := cIMG_BUS_DEFAULT;
        variable ExpPixVal   : bit_vector(cMAX_PIXEL_WIDTH-1 downto 0) := (others => '0');
        variable ActPixVal   : bit_vector(cMAX_PIXEL_WIDTH-1 downto 0) := (others => '0');
    begin
        PrintLog ("Executing ImgBusAnalyzer", LogExec);
        wait until (ImgRstxRI = '0');

        AnalyzeLoop: loop
            wait until rising_edge(ImgClkxCI);

            -- A valid pixel transfer is detected on the ImageBus Transmit Port.
            if ((ImgTxPixValxMI = '1') and (ImgTxPixRdyxM = '1')) then
                if ((ImgTxPixStatxSI = cIMG_STAT_SOF) or (ImgTxPixStatxSI = cIMG_STAT_SOSF)) then
                    -- Start of a new frame.
                    PrintLog ("", LogFlowCtl);
                    PrintLog ("[ImgBusAnalyzer] ImageBus Start-Of-Frame " & integer'image(CurFrmCnt) & " detected", LogFlowCtl);

                    FrmtCycCnt := FrmtCycCnt + 1;
                    if ((ImgSimParaxTI.ImgCmp.CmpImgFrmt = True) and ((LineActive = True) or (FrameActive = True))) then
                        if ((PixErrCnt+FrmtErrCnt) <= ImgSimParaxTI.ImgCmp.CmpImgPixErr) then
                            PrintNow ("[ImgBusAnalyzer] ImageBus Error: Invalid Start-Of-Frame detected");
                        end if;
                        FrmtErrCnt := FrmtErrCnt + 1;
                    end if;

                    -- Read comparison image file, if enabled
                    if (ImgSimParaxTI.ImgCmp.CmpImgPix = True) then
                        -- Open and read a new image compare file and return the image descriptor.
                        ReadImgFile(RemoveBlanks(ImgSimParaxTI.ImgCmp.CmpFileName(FileNumber)), ImgCmpData);

                        -- Check basic image comparison file properties.
                        if ((ImgCmpData.MagicNum = cPGM_ASCII_MAGIC_NUM) or (ImgCmpData.MagicNum = cPGM_BIN_MAGIC_NUM)) then
                            --pgm file format
                            if ((ImgCmpData.NumBits /= 8) and (ImgCmpData.NumBits /= 12)) then
                                PrintLog ("[ImgBusAnalyzer] Invalid pixel depth selected in file " & ImgSimParaxTI.ImgCmp.CmpFileName(FileNumber) & " - Simulation aborted", LogError);
                                AbortSim;
                            end if;
                        elsif ((ImgCmpData.MagicNum = cPPM_ASCII_MAGIC_NUM) or (ImgCmpData.MagicNum = cPPM_BIN_MAGIC_NUM)) then
                            --ppm file format
                            if (ImgCmpData.NumBits /= 24) then
                                PrintLog ("[ImgBusAnalyzer] Invalid pixel depth selected in file " & ImgSimParaxTI.ImgCmp.CmpFileName(FileNumber) & " - Simulation aborted", LogError);
                                AbortSim;
                            end if;
                        else
                            PrintLog ("[ImgBusAnalyzer] Invalid image format selected in file " & ImgSimParaxTI.ImgCmp.CmpFileName(FileNumber) & " - Simulation aborted", LogError);
                            AbortSim;
                        end if;
            
                        if (FileNumber < ImgSimParaxTI.ImgCmp.CmpNumFile-1) then
                            FileNumber := FileNumber + 1;
                        else 
                            FileNumber := 0;
                        end if;
                    end if;

                    CurPixCnt   := 0;
                    CurLinCnt   := 0;
                    LineActive  := True;
                    FrameActive := True;
                    ImgSavData  := cIMG_FILE_DEFAULT;

                    if (ImgTxPixStatxSI = cIMG_STAT_SOSF) then
                        LastLine := True;
                    else
                        LastLine := False;
                    end if;

                elsif (ImgTxPixStatxSI = cIMG_STAT_SOL) then
                    -- Start of a new Line.
                    FrmtCycCnt := FrmtCycCnt + 1;
                    if ((ImgSimParaxTI.ImgCmp.CmpImgFrmt = True) and ((LineActive = True) or (FrameActive = False))) then
                        if ((PixErrCnt+FrmtErrCnt) <= ImgSimParaxTI.ImgCmp.CmpImgPixErr) then
                            PrintNow ("[ImgBusAnalyzer] ImageBus Error: Invalid Start-Of-Line detected");
                        end if;
                        FrmtErrCnt := FrmtErrCnt + 1;
                    end if;

                    CurPixCnt  := 0;
                    CurLinCnt  := CurLinCnt + 1;
                    LineActive := True;
                    LastLine   := False;

                elsif (ImgTxPixStatxSI = cIMG_STAT_SOLL) then
                    -- Start of the last Line.
                    FrmtCycCnt := FrmtCycCnt + 1;
                    if ((ImgSimParaxTI.ImgCmp.CmpImgFrmt = True) and ((LineActive = True) or (FrameActive = False))) then
                        if ((PixErrCnt+FrmtErrCnt) <= ImgSimParaxTI.ImgCmp.CmpImgPixErr) then
                            PrintNow ("[ImgBusAnalyzer] ImageBus Error: Invalid Start-Of-Last-Line detected");
                        end if;
                        FrmtErrCnt := FrmtErrCnt + 1;
                    end if;

                    CurPixCnt  := 0;
                    CurLinCnt  := CurLinCnt + 1;
                    LineActive := True;
                    LastLine   := True;

                elsif (ImgTxPixStatxSI = cIMG_STAT_EOL) then
                    -- End of a Line.
                    FrmtCycCnt := FrmtCycCnt + 1;
                    if ((ImgSimParaxTI.ImgCmp.CmpImgFrmt = True) and ((LineActive = False) or (FrameActive = False))) then
                        if ((PixErrCnt+FrmtErrCnt) <= ImgSimParaxTI.ImgCmp.CmpImgPixErr) then
                            PrintNow ("[ImgBusAnalyzer] ImageBus Error: Invalid End-Of-Line detected");
                        end if;
                        FrmtErrCnt := FrmtErrCnt + 1;
                    end if;

                    -- End of frame 
                    if (LastLine = True) then
                        PrintLog ("[ImgBusAnalyzer] ImageBus End-Of-Frame " & integer'image(CurFrmCnt) & " detected", LogFlowCtl);

                        -- Detect frame size mismatch
                        if (CurFrmCnt > 0) then
                            if ((ImgSimParaxTI.ImgCmp.CmpImgFrmt = True) and (CurLinCnt /= FrmNumLin)) then
                                if ((PixErrCnt+FrmtErrCnt) <= ImgSimParaxTI.ImgCmp.CmpImgPixErr) then
                                    PrintNow ("[ImgBusAnalyzer] Frame size error detected in frame " & integer'image(CurFrmCnt) & ": ExpNumLin = " & integer'image(FrmNumLin) & " - ActNumLin = " & integer'image(CurLinCnt));
                                end if;
                                FrmtErrCnt := FrmtErrCnt + 1;
                            end if;
                        else
                            -- Save the line counter at the end of the first frame.
                            FrmNumLin := CurLinCnt;
                        end if;

                        FrameActive := False;
                        CurFrmCnt   := CurFrmCnt + 1;
                    end if;

                    if ((CurFrmCnt > 0) or (CurLinCnt > 0)) then
                        if ((ImgSimParaxTI.ImgCmp.CmpImgFrmt = True) and (CurPixCnt /= FrmNumPix)) then
                            if ((PixErrCnt+FrmtErrCnt) <= ImgSimParaxTI.ImgCmp.CmpImgPixErr) then
                                PrintNow ("[ImgBusAnalyzer] Line size error detected in frame " & integer'image(CurFrmCnt) & ", line " & integer'image(CurLinCnt) & ": ExpNumPix = " & integer'image(FrmNumPix) & " - ActNumPix = " & integer'image(CurPixCnt));
                            end if;
                            FrmtErrCnt := FrmtErrCnt + 1;
                        end if;
                    else
                        -- Save the pixel counter at the end of the first line of the first frame.
                        FrmNumPix := CurPixCnt;
                    end if;

                    LineActive := False;

                elsif (ImgTxPixStatxSI = cIMG_STAT_ERR) then
                    -- Pixel transfer error.
                    FrmtCycCnt := FrmtCycCnt + 1;
                    if (ImgSimParaxTI.ImgCmp.CmpImgFrmt = True) then
                        if ((PixErrCnt+FrmtErrCnt) <= ImgSimParaxTI.ImgCmp.CmpImgPixErr) then
                            PrintNow ("[ImgBusAnalyzer] ImageBus Error: Error Status detected");
                        end if;
                        FrmtErrCnt := FrmtErrCnt + 1;
                    end if;

                    FrameActive := False;
                    LineActive  := False;

                else
                    if ((ImgSimParaxTI.ImgCmp.CmpImgFrmt = True) and ((LineActive = False) or (FrameActive = False))) then
                        FrmtCycCnt := FrmtCycCnt + 1;
                        if ((PixErrCnt+FrmtErrCnt) <= ImgSimParaxTI.ImgCmp.CmpImgPixErr) then
                            PrintNow ("[ImgBusAnalyzer] ImageBus Error: Invalid pixel transfer detected");
                        end if;
                        FrmtErrCnt := FrmtErrCnt + 1;
                    end if;
                end if;

                ------------------------------------------------------------------------------------------

                -- Save ImageBus pixel stream to file
                if (ImgSimParaxTI.ImgOut.OutImgEnable = True) then
                 
                    -- Save input image pixel data
                    if (ImgSimParaxTI.ImgOut.OutImgType = "ppm") then
                        ImgSavData.FrmBuffer(CurLinCnt)(CurPixCnt)(23 downto 0) :=   to_bitvector(ImgTxPixDataxDI(35 downto 28))
                                                                                   & to_bitvector(ImgTxPixDataxDI(23 downto 16))
                                                                                   & to_bitvector(ImgTxPixDataxDI(11 downto 4));
--                      ImgSavData.FrmBuffer(CurLinCnt)(CurPixCnt)(cMAX_PIXEL_WIDTH-1 downto 24) := (others => '0');

                    elsif (ImgSimParaxTI.ImgOut.OutImgType = "pgm") then
                        if (ImgSimParaxTI.ImgOut.OutImgBits = 8) then
                            ImgSavData.FrmBuffer(CurLinCnt)(CurPixCnt)(7 downto 0) := to_bitvector(ImgTxPixDataxDI(35 downto 28));
                            ImgSavData.FrmBuffer(CurLinCnt)(CurPixCnt)(cMAX_PIXEL_WIDTH-1 downto 8) := (others => '0');
                        else
                            ImgSavData.FrmBuffer(CurLinCnt)(CurPixCnt)(11 downto 0) := to_bitvector(ImgTxPixDataxDI(35 downto 24));
                            ImgSavData.FrmBuffer(CurLinCnt)(CurPixCnt)(cMAX_PIXEL_WIDTH-1 downto 12) := (others => '0');
                        end if;
                    end if;
    
                    -- Save the entire input image to file on the last pixel transfer of the frame.
                    if ((ImgTxPixStatxSI = cIMG_STAT_EOL) and (LastLine = True)) then
                        -- Do a basic compatibility check between the input image format (being the format
                        -- of the image comparison file) and the format selected for the output image.
                        if (ImgSimParaxTI.ImgCmp.CmpImgPix = True) then
                            if ((ImgCmpData.MagicNum = cPGM_ASCII_MAGIC_NUM) or (ImgCmpData.MagicNum = cPGM_BIN_MAGIC_NUM)) then
                                --pgm comparison file format 
                                if (ImgSimParaxTI.ImgOut.OutImgType /= "pgm") then
                                    PrintLog ("[ImgBusAnalyzer] Warning: Selected image output file type (." & ImgSimParaxTI.ImgOut.OutImgType & ") is not compatible with the ImageBus data stream (.pgm)", LogError);
                                end if;
                            else
                                --ppm comparison file format 
                                if (ImgSimParaxTI.ImgOut.OutImgType /= "ppm") then
                                    PrintLog ("[ImgBusAnalyzer] Warning: Selected image output file type (." & ImgSimParaxTI.ImgOut.OutImgType & ") is not compatible with the ImageBus data stream (.ppm)", LogError);
                                end if;
                            end if;

                            if (ImgCmpData.NumBits /= ImgSimParaxTI.ImgOut.OutImgBits) then
                                PrintLog ("[ImgBusAnalyzer] Warning: Possible image pixel depth mismatch - Output Image = '" & integer'image(ImgSimParaxTI.ImgOut.OutImgBits) & "' - Input Image = " & integer'image(ImgCmpData.NumBits), LogError);
                            end if;
                        end if;

                        -- Save the image header information.
                        if (ImgSimParaxTI.ImgOut.OutImgType = "pgm") then
                            if (ImgSimParaxTI.ImgOut.OutImgFrmt(1 to 5) = "ascii") then
                                ImgSavData.MagicNum := cPGM_ASCII_MAGIC_NUM;
                            elsif (ImgSimParaxTI.ImgOut.OutImgFrmt(1 to 3) = "bin") then
                                ImgSavData.MagicNum := cPGM_BIN_MAGIC_NUM;
                            else
                                PrintLog ("[ImgBusAnalyzer] ImageBus Error: Invalid image output file format detected: " & ImgSimParaxTI.ImgOut.OutImgFrmt, LogError);
                                AbortSim;
                            end if;
                        elsif (ImgSimParaxTI.ImgOut.OutImgType = "ppm") then
                            if (ImgSimParaxTI.ImgOut.OutImgFrmt(1 to 5) = "ascii") then
                                ImgSavData.MagicNum := cPPM_ASCII_MAGIC_NUM;
                            elsif (ImgSimParaxTI.ImgOut.OutImgFrmt(1 to 3) = "bin") then
                                ImgSavData.MagicNum := cPPM_BIN_MAGIC_NUM;
                            else
                                PrintLog ("[ImgBusAnalyzer] ImageBus Error: Invalid image output file format detected: " & ImgSimParaxTI.ImgOut.OutImgFrmt, LogError);
                                AbortSim;
                            end if;
                        else
                            PrintLog ("[ImgBusAnalyzer] ImageBus Error: Invalid image output file extension detected", LogError);
                            AbortSim;
                        end if;
    
                        ImgSavData.Valid   := True;
                        ImgSavData.MaxPix  := CurPixCnt + 1;
                        ImgSavData.MaxLin  := CurLinCnt + 1;  
                        ImgSavData.NumBits := ImgSimParaxTI.ImgOut.OutImgBits;
                        WriteImgFile(ImgSavData, RemoveBlanks(ImgSimParaxTI.ImgOut.OutImgFile) & "_" & RemoveBlanks(ImgSimParaxTI.ImgOut.OutImgFrmt) & "_" & integer'image(CurFrmCnt) & "." & ImgSimParaxTI.ImgOut.OutImgType);
                    end if;
                end if;

                ------------------------------------------------------------------------------------------

                -- Compare ImageBus pixel values with reference image
                PixCycCnt := PixCycCnt + 1;
                PixCmpCnt := PixCmpCnt + 1;
                if (ImgSimParaxTI.ImgCmp.CmpImgPix = True) then
                    ExpPixVal := (others => '0');
                    ActPixVal := (others => '0');
                    if ((ImgCmpData.MagicNum = cPGM_ASCII_MAGIC_NUM) or (ImgCmpData.MagicNum = cPGM_BIN_MAGIC_NUM)) then
                        --pgm file format
                        if (ImgCmpData.NumBits = 8) then
                            ExpPixVal(7 downto 0) := ImgCmpData.FrmBuffer(CurLinCnt+ImgSimParaxTI.ImgIn.StartLin)(CurPixCnt+ImgSimParaxTI.ImgIn.StartPix)(7 downto 0);
                            ActPixVal(7 downto 0) := to_bitvector(ImgTxPixDataxDI(35 downto 28));
                        else
                            ExpPixVal(11 downto 0) := ImgCmpData.FrmBuffer(CurLinCnt+ImgSimParaxTI.ImgIn.StartLin)(CurPixCnt+ImgSimParaxTI.ImgIn.StartPix)(11 downto 0);
                            ActPixVal(11 downto 0) := to_bitvector(ImgTxPixDataxDI(35 downto 24));
                        end if;

                    elsif ((ImgCmpData.MagicNum = cPPM_ASCII_MAGIC_NUM) or (ImgCmpData.MagicNum = cPPM_BIN_MAGIC_NUM)) then
                        --ppm file format
                        ExpPixVal(cMAX_PIXEL_WIDTH-1 downto 0) := ImgCmpData.FrmBuffer(CurLinCnt+ImgSimParaxTI.ImgIn.StartLin)(CurPixCnt+ImgSimParaxTI.ImgIn.StartPix)(cMAX_PIXEL_WIDTH-1 downto 0);
                        ActPixVal(23 downto 0) :=   to_bitvector(ImgTxPixDataxDI(35 downto 28))
                                                  & to_bitvector(ImgTxPixDataxDI(23 downto 16))
                                                  & to_bitvector(ImgTxPixDataxDI(11 downto 4));  
                        
                    end if;

                    if ((ExpPixVal /= ActPixVal)) then
                        PixErrCnt := PixErrCnt + 1;
                        if ((PixErrCnt+FrmtErrCnt) <= ImgSimParaxTI.ImgCmp.CmpImgPixErr) then
                            PrintNow ("[ImgBusAnalyzer] ImageBus Error(" & integer'image(PixErrCnt) & "): Pixel mismatch detected in frame " & integer'image(CurFrmCnt) & 
                                    ", line " & integer'image(CurLinCnt) & ", pixel " & integer'image(CurPixCnt) & 
                                    ": ExpPix = 0x" & Vec2Strg(ExpPixVal) & 
                                    " - ActPix = 0x" & Vec2Strg(ActPixVal));
                        end if;

                        if ((ImgSimParaxTI.ImgCmp.CmpImgAbort = True) and (PixErrCnt >= ImgSimParaxTI.ImgCmp.CmpImgPixErr)) then
                            PrintNow ("[ImgBusAnalyzer] Maximum ImageBus pixel comparison errors detected - Simulation aborted");
                            AbortSim;
                        end if;
                    elsif (PixCycCnt >= cCMPIMG_PIXMSGCNT) then
                        PixCycCnt := 0;
                        PrintLog ("[ImgBusAnalyzer] " & integer'image(PixCmpCnt) & " ImageBus pixel transfers analyzed - " & integer'image(PixErrCnt) & " Errors detected", LogAnalyzer);
                    end if;
                elsif (PixCycCnt >= cCMPIMG_PIXMSGCNT) then
                    PixCycCnt := 0;
                    PrintLog ("[ImgBusAnalyzer] " & integer'image(PixCmpCnt) & " ImageBus pixel transfers detected - Pixel Comparison Disabled", LogAnalyzer);
                end if;

                if ((ImgSimParaxTI.ImgCmp.CmpImgFrmt = True) and (FrmtCycCnt >= cCMPIMG_FRMTMSGCNT)) then
                    FrmtTotCnt := FrmtTotCnt + FrmtCycCnt;
                    PrintLog ("[ImgBusAnalyzer] " & integer'image(FrmtTotCnt) & " ImageBus frame format events analyzed - " & integer'image(FrmtErrCnt) & " Errors detected", LogAnalyzer);
                    FrmtCycCnt := 0;
                end if;

                -- Increment pixel counter at the end of the loop.
                CurPixCnt := CurPixCnt + 1;
            end if;
        end loop AnalyzeLoop;

        PrintLog ("Exiting ImgBusAnalyzer", LogReturn);
        wait;
    end process;

end;
