-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Testbench ImageBus Stimulus
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2013-05-20  1.0      Manfred  Created
--              2015-04-16  2.0      a.kramer Changed for GigEpro
--              2017-03-02  2.0      Manfred  Modified for CorSight2
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library std;
use std.textio.all;

library work;
use work.CustomCfgLib.all;
use work.CustomSimLib.all;
use work.CustomFctLib.all;
use work.CustomPkg.all;

entity ImgBusStimulus is
    port (  
        -- Infrastructure Signals
        ImgClkxCI         : in  std_logic;              -- ImageBus Clock
        ImgRstxRI         : in  std_logic;              -- ImageBus Reset
        ImgSimParaxTI     : in  tIMG_SIM_PARAMETERS;    -- ImageBus Simulation Parameters

        -- Simulation Flow Control
        StartImgBusxMI    : in  boolean;                -- Start ImageBus frame transfers
        StopImgBusxMI     : in  boolean;                -- Stop ImageBus frame transfers
        AbortImgBusxMI    : in  boolean;                -- Abort ImageBus frame transfers
    
        -- ImageBus Receive Port                
        ImgRxPixValxMO    : out std_logic;              -- ImageBus Receive Pixel Valid
        ImgRxPixRdyxMI    : in  std_logic;              -- ImageBus Receive Pixel Ready - ignore if cIMG_HANDSHAKE_EN = False
        ImgRxPixStatxSO   : out std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);  -- ImageBus Receive Pixel Status
        ImgRxPixDataxDO   : out std_logic_vector(cIMG_DATA_WIDTH-1 downto 0)); -- ImageBus Receive Pixel Data
end ImgBusStimulus;

architecture Behavioral of ImgBusStimulus is

    signal ImgBusEnablexM   : boolean   := False;       -- ImageBus frame transfer enable
    signal ImgAbortReqxM    : boolean   := False;       -- ImageBus abort frame transfer request
    signal ImgAbortAckxM    : boolean   := False;       -- ImageBus abort frame transfer acknowledge
    signal ImgRxPixValxM    : std_logic := '0';         -- ImageBus Receive Pixel Valid
    signal ImgRxPixStatxS   : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0) := (others => '0');  -- ImageBus Receive Pixel Status
    signal ImgRxPixDataxD   : std_logic_vector(cIMG_DATA_WIDTH-1 downto 0) := (others => '0');  -- ImageBus Receive Pixel Data

begin
    ----------------------------------------------------------------------------
    -- ImageBus Flow Control
    ----------------------------------------------------------------------------
    ImgFlowProc: process 
    begin
        PrintLog ("Executing ImgFlowProc", LogExec);
        loop
            wait until rising_edge(ImgClkxCI);
    
            if (StartImgBusxMI = True) then
                ImgBusEnablexM <= True;
            elsif (StopImgBusxMI = True) then
                ImgBusEnablexM <= False;
            elsif ((ImgRxPixValxM = '1') and (ImgRxPixRdyxMI = '1') and ((ImgRxPixStatxS = cIMG_STAT_SOF) or (ImgRxPixStatxS = cIMG_STAT_SOSF)) and (ImgSimParaxTI.ImgMode.SingleFrm = True)) then
                ImgBusEnablexM <= False;
            end if;

            if (AbortImgBusxMI = True) then
                ImgAbortReqxM <= True;
            elsif (ImgAbortAckxM = True) then
                ImgAbortReqxM <= False;
            end if;
        end loop;
    end process;

    ----------------------------------------------------------------------------
    -- ImageBus Stimulus
    ----------------------------------------------------------------------------
    ImgBusProc: process 
        variable MaxPixel    : natural := 0;
        variable MaxLines    : natural := 0;
        variable ImgFileData : tIMG_FILE_RECORD := cIMG_FILE_DEFAULT;
        variable ImgBusData  : tIMG_BUS_RECORD  := cIMG_BUS_DEFAULT;
    begin
        PrintLog ("Executing ImgBusStimulus", LogExec);

        ImgAbortAckxM  <= False; 
        ImgRxPixValxM  <= '0'; 
        ImgRxPixStatxS <= (others => '0');
        ImgRxPixDataxD <= (others => '0');
        wait until (ImgRstxRI = '0');

        MaxPixel := ImgSimParaxTI.ImgIn.StartPix+ImgSimParaxTI.ImgIn.NumPix;
        MaxLines := ImgSimParaxTI.ImgIn.StartLin+ImgSimParaxTI.ImgIn.NumLin;

        SimSeqLoop: loop
            ImgSeqLoop: for FileNumber in 0 to ImgSimParaxTI.ImgIn.SrcNumFile-1 loop
                if (ImgAbortAckxM = True) then
                    if (ImgAbortReqxM = True) then
                        wait until (ImgAbortReqxM = False);
                    end if;

                    ImgAbortAckxM <= False; 
                end if;

                -- Wait until the next frame can be transferred.
                if (ImgBusEnablexM = False) then
                    wait until (ImgBusEnablexM = True);
                end if;

                -- Open and read a new image file and return the image descriptor.
                ReadImgFile(RemoveBlanks(ImgSimParaxTI.ImgIn.SrcImgFile(FileNumber)), ImgFileData);

                -- Transfer the frame line-by-line over the ImageBus
                LineLoop: for LineCnt in ImgSimParaxTI.ImgIn.StartLin to MaxLines-1 loop
                    if (LineCnt = ImgSimParaxTI.ImgIn.StartLin) then
                        ImgBusData.NumBits  := ImgFileData.NumBits;
                        ImgBusData.FirstLin := True;
                        ImgBusData.LastLin  := False;
                    elsif (LineCnt = MaxLines-1) then
                        ImgBusData.FirstLin := False;
                        ImgBusData.LastLin  := True;
                    else
                        ImgBusData.FirstLin := False;
                        ImgBusData.LastLin  := False;
                    end if;

                    -- Copy the next line of the image to ImgBusData.
                    for PixCnt in ImgSimParaxTI.ImgIn.StartPix to MaxPixel-1 loop
                        ImgBusData.LineBuf(PixCnt-ImgSimParaxTI.ImgIn.StartPix) := ImgFileData.FrmBuffer(LineCnt)(PixCnt);
                    end loop;

                    -- Perform ImageBus transfer cycles as described in ImgBusData.
                    Proc_ImgBus_Write(ImgSimParaxTI, ImgBusData, ImgClkxCI, ImgAbortReqxM, ImgRxPixRdyxMI, 
                                      ImgRxPixValxM, ImgRxPixStatxS, ImgRxPixDataxD);

                    -- Exit current frame transfer loop if an Frame Abort is requested.
                    if (ImgAbortReqxM = True) then
                        ImgAbortAckxM <= True; 
                        wait until rising_edge(ImgClkxCI);
                        exit LineLoop;
                    end if;
                end loop;
            end loop ImgSeqLoop;
    
            -- Exit the simulation loop at the end of an image 
            -- sequence if the RepeatImgSeq flag is deasserted.
            if (ImgSimParaxTI.ImgMode.RepeatImgSeq = False) then
                exit SimSeqLoop;
            end if;
        end loop SimSeqLoop;

        PrintLog ("Exiting ImgBusStimulus", LogReturn);
        wait;
    end process;

    -- Assign internally used output signals.
    ImgRxPixValxMO  <= ImgRxPixValxM; 
    ImgRxPixStatxSO <= ImgRxPixStatxS;
    ImgRxPixDataxDO <= ImgRxPixDataxD;

end;
