-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Testbench Simulation Parameter Initialization
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2013-05-20  1.0      Manfred  Created
--              2015-04-16  2.0      a.kramer Changed for GigEpro
--              2017-03-02  3.0      Manfred  Changed for CorSight2
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library std;
use std.textio.all;

library work;
use work.SimCfg.all;
use work.CustomPkg.all;
use work.CustomCfgLib.all;
use work.CustomSimLib.all;
use work.CustomFctLib.all;

entity SimParaInit is
    port (  
        -- ImageBus Simulation Parameters
        ImgSimParaxTO     : out tIMG_SIM_PARAMETERS);   
end SimParaInit;

architecture Behavioral of SimParaInit is

begin

    ----------------------------------------------------------------------------
    -- Simulation Parameter Setup 
    ----------------------------------------------------------------------------
    SimParaProc: process 
        file     SimIniFile   : text;
        variable SimIniLine   : line;
        variable LineNumber   : natural := 0;
        variable EndOfWord    : boolean := False;
        variable EndOfLine    : boolean := False;
        variable SimKeyName   : tSIM_KEY_NAME_LIST := none;
        variable SectionName  : boolean := False;
        variable WordCnt      : natural := 0;
        variable TempInt      : natural := 0;
        variable NumFound     : natural := 0;
        variable TimeVar      : time    := 0 ps;
        variable CharCnt      : natural range 1 to cMAX_FILENAME_LENGTH := 1;
        variable WordBuffer   : string(1 to cMAX_FILENAME_LENGTH) := (others => ' ');
        variable ValBuffer    : string(1 to cMAX_WORD_LENGTH) := (others => ' ');
        variable UnitBuffer   : string(1 to cMAX_WORD_LENGTH) := (others => ' ');
        variable ImgSimPara   : tIMG_SIM_PARAMETERS    := cIMG_SIM_PARA_DEFAULT;
        variable CurSectName  : string(1 to cMAX_WORD_LENGTH) := (others => ' ');
        variable FileErrCode  : tSIM_ERROR_MESSAGE := cSIM_ERROR_DEFAULT;
    begin
        PrintLog ("Executing SimParaInit", LogExec);
        PrintLog ("[SimParaInit] Opening TestBench initialization file " & RemoveBlanks(cTESTBENCH_INI_FILE) & " in read mode", LogFlowCtl);
        file_open(SimIniFile, RemoveBlanks(cTESTBENCH_INI_FILE), read_mode);

        SimIniFileLoop: loop
            if endfile(SimIniFile) then
                exit SimIniFileLoop;
            end if;

            -- Read a line from the Simulatin Parameter setup file into a line buffer.
            readline(SimIniFile, SimIniLine);
            LineNumber := LineNumber + 1;
        
            -- Check basic read line error conditions. Skip the rest
            -- of the loop iteration if no valid line has been read.
            next when ((SimIniLine = NULL) or (SimIniLine'length = 0));

            CharCnt     := 1;
            WordCnt     := 0;
            EndOfLine   := False;
            EndOfWord   := False;
            EndOfWord   := False;
            SectionName := False;
            SimKeyName  := none;
            WordBuffer  := (others => ' ');
            
            SimIniLineLoop: for CharNum in SimIniLine'range loop
                if (CharNum = SimIniLine'high) then 
                    EndOfLine := True;
                end if;

                case SimIniLine.all(CharNum) is     -- Skip leading white spaces
                    when ' ' | HT | cEQUAL_CHAR => 
                        if (CharCnt > 1) then       
                            if (SectionName = True) then
                                EndOfLine := True;
                            else
                                EndOfWord := True;      
                            end if;     
                        end if;     
            
                    when cSECTION_START =>          -- Start of Section Name
                        if ((WordCnt = 0) and (CharCnt = 1)) then       
                            SectionName := True;
                        else
                            WordBuffer(CharCnt) := SimIniLine.all(CharNum);
                            CharCnt := CharCnt + 1;
                        end if;
            
                    when cSECTION_END =>            -- End of Section Name
                        if (SectionName = True) then                    
                           EndOfLine := True;
                        else
                            WordBuffer(CharCnt) := SimIniLine.all(CharNum);
                            CharCnt := CharCnt + 1;
                        end if;
            
                    when cINI_COMMENT | CR | LF =>
                        EndOfLine := True;
            
                    when others => 
                        -- Convert the Section Name or Key Name to lower case
                        if (WordCnt = 0) then
                            WordBuffer(CharCnt) := LowerCase(SimIniLine.all(CharNum));
                        else
                            WordBuffer(CharCnt) := SimIniLine.all(CharNum);
                        end if;
                        CharCnt := CharCnt + 1;
                end case;
            
                -- Analyze WordBuffer if valid characters have been detected.
                if ((SectionName = True) and (EndOfLine = True) and (CharCnt > 1)) then
                    CurSectName := WordBuffer(1 to cMAX_WORD_LENGTH);

                    -- Check correct Section Name
                    case CurSectName is
                        when cSECTION_SIMULATION   => NULL;
                        when cSECTION_IMAGEMODE    => NULL;
                        when cSECTION_IMAGEINPUT   => NULL;
                        when cSECTION_IMAGEOUTPUT  => NULL;
                        when cSECTION_IMAGECOMPARE => NULL;
                        when others => FileErrCode := cSIMPARAINIT_INVSECTNAME;
                    end case;

                    CharCnt     := 1;
                    EndOfWord   := False;
                    WordBuffer  := (others => ' ');
                    WordCnt     := WordCnt + 1;

                elsif (((EndOfLine = True) or (EndOfWord = True)) and (CharCnt > 1)) then
                    -- First word of a line is a Key Name.
                    if (WordCnt = 0) then
                        case WordBuffer(1 to cMAX_WORD_LENGTH) is
                            -- Simulation section
                            when cSIMKEY_RSTPERIOD    => SimKeyName := RstPeriod;
                            when cSIMKEY_MAXSIMTIME   => SimKeyName := MaxSimTime;
                            when cSIMKEY_COMMANDFILE  => SimKeyName := CommandFile; 
   
                            -- ImageMode section
                            when cSIMKEY_SCANLINEMODE => SimKeyName := ScanLineMode;
                            when cSIMKEY_SINGLEFRM    => SimKeyName := SingleFrm;
                            when cSIMKEY_PIXINFREQ    => SimKeyName := PixInFreq;
                            when cSIMKEY_PIXOUTFREQ   => SimKeyName := PixOutFreq;
                            when cSIMKEY_PIXWAITSEQ   => SimKeyName := PixWaitSeq;
                            when cSIMKEY_PPMBGRMODE   => SimKeyName := PpmBGRMode;
                            when cSIMKEY_REPEATIMGSEQ => SimKeyName := RepeatImgSeq;
                            when cSIMKEY_HORSYNC      => SimKeyName := HorSync;
                            when cSIMKEY_VERSYNC      => SimKeyName := VerSync;

                            -- ImageInput section
                            when cSIMKEY_STARTPIX     => SimKeyName := StartPix;
                            when cSIMKEY_STARTLIN     => SimKeyName := StartLin;
                            when cSIMKEY_NUMPIX       => SimKeyName := NumPix;
                            when cSIMKEY_NUMLIN       => SimKeyName := NumLin;
                            when cSIMKEY_SRCIMGFILE   => SimKeyName := SrcImgFile;

                            -- ImageAnalyzer section
                            when cSIMKEY_CMPIMGPIXEL  => SimKeyName := CmpImgPix;
                            when cSIMKEY_CMPIMGFORMAT => SimKeyName := CmpImgFrmt;
                            when cSIMKEY_CMPIMGABORT  => SimKeyName := CmpImgAbort;
                            when cSIMKEY_CMPIMGERRCNT => SimKeyName := CmpImgPixErr;
                            when cSIMKEY_CMPIMGFILE   => SimKeyName := CmpImgFile;

                            -- ImageOutput section
                            when cSIMKEY_OUTIMGENABLE => SimKeyName := OutImgEnable;
                            when cSIMKEY_OUTIMGTYPE   => SimKeyName := OutImgType;
                            when cSIMKEY_OUTIMGBITS   => SimKeyName := OutImgBits;      -- allow only 8-bit image formats for file output
                            when cSIMKEY_OUTIMGFILE   => SimKeyName := OutImgFile;

                            when others => FileErrCode := cSIMPARAINIT_INVKEYNAME;
                        end case;

                    elsif (CurSectName = cSECTION_SIMULATION) then
                        -- Assign key values for the Simulation section
                        case SimKeyName is
                            when RstPeriod | MaxSimTime => 
                                if (WordCnt = 1) then
                                    case SimKeyName is
                                        when RstPeriod  => PrintLog ("[SimParaInit] Reading simulation parameter: RstPeriod = " & WordBuffer, LogSimInit);
                                        when MaxSimTime => PrintLog ("[SimParaInit] Reading simulation parameter: MaxSimTime = " & WordBuffer, LogSimInit);
                                        when others     => NULL;
                                    end case;
    
                                    -- Analyze the Reset Period parameter. The argument can be written with or
                                    -- without a white space between the value and unit eg. "20us" or "20 us".
                                    ValBuffer := (others => ' ');
                                    if (IsNumber(WordBuffer) = True) then
                                        -- WordBuffer contains a pure number value. Copy it to ValBuffer
                                        -- and wait for the next word which defines the Time Unit.
                                        ValBuffer := WordBuffer(1 to cMAX_WORD_LENGTH);
                                    else
                                        -- WordBuffer does not contain a pure number value. Assume that the 
                                        -- Time Value and Unit are both present with no spaces in between.

                                        -- Copy all decimal numbers into ValBuffer 
                                        ParseValue: for i in 1 to cMAX_WORD_LENGTH loop
                                            if ((WordBuffer(i) >= '0') and (WordBuffer(i) <= '9')) then
                                                ValBuffer(i) := WordBuffer(i);
                                            else    
                                                TempInt := i;
                                                exit ParseValue;
                                            end if;
                                        end loop;

                                        -- Copy all non-blank characters into UnitBuffer 
                                        ParseUnit: for i in 1 to cMAX_WORD_LENGTH loop
                                            if ((WordBuffer(TempInt) = ' ') or (WordBuffer(TempInt) = HT)) then
                                                exit ParseUnit;
                                            else
                                                UnitBuffer(i) := WordBuffer(TempInt);
                                            end if;

                                            TempInt := TempInt + 1;
                                        end loop;


                                        -- The first non-decimal character is assumed to be the Unit character.
                                        case UnitBuffer(1 to 2) is
                                            when "ns" => TimeVar := Strg2Int(ValBuffer) * 1 ns;
                                            when "us" => TimeVar := Strg2Int(ValBuffer) * 1 us;
                                            when "ms" => TimeVar := Strg2Int(ValBuffer) * 1 ms;
                                            when "se" => TimeVar := Strg2Int(ValBuffer) * 1 sec;
                                            when others => FileErrCode := cSIMPARAINIT_INVTIMEBASE;
                                        end case;
                                    end if;
                                elsif (WordCnt = 2) then
                                    -- The second word must contain the Time Unit.
                                    case WordBuffer(1 to 2) is
                                        when "ns" => TimeVar := Strg2Int(ValBuffer) * 1 ns;
                                        when "us" => TimeVar := Strg2Int(ValBuffer) * 1 us;
                                        when "ms" => TimeVar := Strg2Int(ValBuffer) * 1 ms;
                                        when "se" => TimeVar := Strg2Int(ValBuffer) * 1 sec;
                                        when others => FileErrCode := cSIMPARAINIT_INVTIMEBASE;
                                    end case;
                                end if;
    
                                case SimKeyName is
                                    when RstPeriod  => ImgSimPara.SimPara.RstPeriod := TimeVar;
                                    when MaxSimTime => ImgSimPara.SimPara.MaxSimTime := TimeVar;
                                    when others     => NULL;
                                end case;

                            when CommandFile => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: CommandFile = " & WordBuffer, LogSimInit);
                                ImgSimPara.SimPara.CommandFile := cSIM_ROOT_DIR & WordBuffer(1 to cMAX_FILENAME_LENGTH-cSIM_ROOT_DIR'length);

                            when others => 
                                FileErrCode := cSIMPARAINIT_INVSIMKEY;
                        end case;

                    elsif (CurSectName = cSECTION_IMAGEMODE) then
                        -- Assign key values for the ImageMode section
                        case SimKeyName is
                            when ScanLineMode => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: ScanLineMode = " & WordBuffer, LogSimInit);
                                if (WordBuffer(1) = '1') then
                                    ImgSimPara.ImgMode.ScanLineMode := True;
                                elsif (WordBuffer(1) = '0') then
                                    ImgSimPara.ImgMode.ScanLineMode := False;
                                else
                                    FileErrCode := cSIMPARAINIT_INVSCANVAL;
                                end if;

                            when SingleFrm => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: SingleFrm = " & WordBuffer, LogSimInit);
                                if (WordBuffer(1) = '1') then
                                    ImgSimPara.ImgMode.SingleFrm := True;
                                elsif (WordBuffer(1) = '0') then
                                    ImgSimPara.ImgMode.SingleFrm := False;
                                else
                                    FileErrCode := cSIMPARAINIT_INVSGLFRMVAL;
                                end if;

                            when PixInFreq => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: PixInFreq = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgMode.PixInFreq := Strg2Int(WordBuffer);
    
                                    if (ImgSimPara.ImgMode.PixInFreq > cIMGBUS_PIXEL_FREQ) then
                                        FileErrCode := cSIMPARAINIT_PIXINFREQRANGE;
                                    end if; 
                                else
                                    FileErrCode := cSIMPARAINIT_INVPIXINFREQVAL;
                                end if;

                            when PixOutFreq => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: PixOutFreq = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgMode.PixOutFreq := Strg2Int(WordBuffer);
    
                                    if (ImgSimPara.ImgMode.PixOutFreq > cIMGBUS_PIXEL_FREQ) then
                                        FileErrCode := cSIMPARAINIT_PIXOUTFREQRANGE;
                                    end if; 
                                else
                                    FileErrCode := cSIMPARAINIT_INVPIXOUTFREQVAL;
                                end if;
    
                            when PixWaitSeq => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: PixWaitSeq = " & WordBuffer, LogSimInit);

                                -- Parse the PixWaitSeq parameters. 
                                -- Expected format is 2 numbers separated by '/': <Num>/<Num>
                                -- Assume an invalid PixWaitSeq value is pending. Reset 
                                -- FileErrCode only if the correct format is found.
                                FileErrCode := cSIMPARAINIT_INVPIXWAITSEQVAL;
                                TempInt   := 1;
                                NumFound  := 0;
                                ValBuffer := (others => ' ');

                                ParseWaitSeq: for i in 1 to cMAX_WORD_LENGTH loop
                                    if ((WordBuffer(i) >= '0') and (WordBuffer(i) <= '9')) then
                                        ValBuffer(TempInt) := WordBuffer(i);
                                        TempInt := TempInt + 1;
                                    elsif (WordBuffer(i) = '/') then
                                        -- The divide character has been found. 
                                        -- Store the number of wait states per wait period.
                                        if (IsNumber(ValBuffer) = True) then
                                            ImgSimPara.ImgMode.WaitStates := Strg2Int(ValBuffer);
                                            NumFound := 1;
                                        else
                                            exit ParseWaitSeq;
                                        end if;

                                        TempInt   := 1;
                                        ValBuffer := (others => ' ');
                                    elsif (WordBuffer(i) = ' ') then
                                        -- The end of the second number has been reached. 
                                        -- Store the total number of clock cycles per wait period.
                                        if ((NumFound > 0) and (TempInt > 1)) then
                                            if (IsNumber(ValBuffer) = True) then
                                                ImgSimPara.ImgMode.WaitPeriod := Strg2Int(ValBuffer);
                                                FileErrCode := cSIM_ERROR_DEFAULT;
                                            end if;

                                            exit ParseWaitSeq;
                                        end if;
                                    end if;
                                end loop;

                            when PpmBGRMode => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: PpmBGRMode = " & WordBuffer, LogSimInit);
                                if (WordBuffer(1) = '1') then
                                    ImgSimPara.ImgMode.PpmBgrMode := True;
                                elsif (WordBuffer(1) = '0') then
                                    ImgSimPara.ImgMode.PpmBgrMode := False;
                                else
                                    FileErrCode := cSIMPARAINIT_INVBGRMODEVAL;
                                end if;

                            when RepeatImgSeq => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: RepeatImgSeq = " & WordBuffer, LogSimInit);
                                if (WordBuffer(1) = '1') then
                                    ImgSimPara.ImgMode.RepeatImgSeq := True;
                                elsif (WordBuffer(1) = '0') then
                                    ImgSimPara.ImgMode.RepeatImgSeq := False;
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_11;
                                end if;

                            when HorSync => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: HorSync = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgMode.HorSync := Strg2Int(WordBuffer);
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_12;
                                end if;
        
                            when VerSync => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: VerSync = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgMode.VerSync := Strg2Int(WordBuffer);
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_13;
                                end if;

                            when others => 
                                FileErrCode := cSIMPARAINIT_ERRNUM_14;
                        end case;

                    elsif (CurSectName = cSECTION_IMAGEINPUT) then
                        -- Assign key values for the ImageInput section
                        case SimKeyName is
                            when StartPix => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: StartPix = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgIn.StartPix := Strg2Int(WordBuffer);
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_15;
                                end if;
        
                            when StartLin => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: StartLin = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgIn.StartLin := Strg2Int(WordBuffer);
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_16;
                                end if;
        
                            when NumPix => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: NumPix = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgIn.NumPix := Strg2Int(WordBuffer);
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_17;
                                end if;
        
                            when NumLin => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: NumLin = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgIn.NumLin := Strg2Int(WordBuffer);
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_18;
                                end if;
        
                            when SrcImgFile => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: SrcImgFile(" & integer'image(ImgSimPara.ImgIn.SrcNumFile) & ") = " & WordBuffer, LogSimInit);
                                ImgSimPara.ImgIn.SrcImgFile(ImgSimPara.ImgIn.SrcNumFile) := cSIM_ROOT_DIR & WordBuffer(1 to cMAX_FILENAME_LENGTH-cSIM_ROOT_DIR'length);
                                ImgSimPara.ImgIn.SrcNumFile := ImgSimPara.ImgIn.SrcNumFile + 1;

                            when others => 
                                FileErrCode := cSIMPARAINIT_ERRNUM_19;
                        end case;

                    elsif (CurSectName = cSECTION_IMAGECOMPARE) then
                        -- Assign key values for the ImageAnalyzer section
                        case SimKeyName is
                            when CmpImgFrmt => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: CmpImgFrmt = " & WordBuffer, LogSimInit);
                                if (WordBuffer(1) = '1') then
                                    ImgSimPara.ImgCmp.CmpImgFrmt := True;
                                elsif (WordBuffer(1) = '0') then
                                    ImgSimPara.ImgCmp.CmpImgFrmt := False;
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_20;
                                end if;

                            when CmpImgPix => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: CmpImgPix = " & WordBuffer, LogSimInit);
                                if (WordBuffer(1) = '1') then
                                    ImgSimPara.ImgCmp.CmpImgPix := True;
                                elsif (WordBuffer(1) = '0') then
                                    ImgSimPara.ImgCmp.CmpImgPix := False;
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_21;
                                end if;
            
                            when CmpImgAbort => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: CmpImgAbort = " & WordBuffer, LogSimInit);
                                if (WordBuffer(1) = '1') then
                                    ImgSimPara.ImgCmp.CmpImgAbort := True;
                                elsif (WordBuffer(1) = '0') then
                                    ImgSimPara.ImgCmp.CmpImgAbort := False;
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_22;
                                end if;

                            when CmpImgPixErr => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: CmpImgPixErr = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgCmp.CmpImgPixErr := Strg2Int(WordBuffer);
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_23;
                                end if;

                            when CmpImgFile => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: CmpImgFile(" & integer'image(ImgSimPara.ImgCmp.CmpNumFile) & ") = " & WordBuffer, LogSimInit);
                                ImgSimPara.ImgCmp.CmpFileName(ImgSimPara.ImgCmp.CmpNumFile) := cSIM_ROOT_DIR & WordBuffer(1 to cMAX_FILENAME_LENGTH-cSIM_ROOT_DIR'length);
                                ImgSimPara.ImgCmp.CmpNumFile := ImgSimPara.ImgCmp.CmpNumFile + 1;
        
                            when others => 
                                FileErrCode := cSIMPARAINIT_ERRNUM_24;
                        end case;

                    elsif (CurSectName = cSECTION_IMAGEOUTPUT) then
                        -- Assign key values for the ImageOutput section
                        case SimKeyName is

                            when OutImgEnable => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: OutImgEnable = " & WordBuffer, LogSimInit);
                                if (WordBuffer(1) = '1') then
                                    ImgSimPara.ImgOut.OutImgEnable := True;
                                elsif (WordBuffer(1) = '0') then
                                    ImgSimPara.ImgOut.OutImgEnable := False;
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_25;
                                end if;

                            when OutImgType => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: OutImgType = " & WordBuffer, LogSimInit);

                                -- Set default pixel depth (may change in OutImgBits)
                                if (WordBuffer(1 to 7) = "bin_pgm") then
                                    --ImgSimPara.ImgOut.OutImgBits := 8;
                                    ImgSimPara.ImgOut.OutImgFrmt := "bin  ";
                                    ImgSimPara.ImgOut.OutImgType := "pgm";
                                elsif (WordBuffer(1 to 9) = "ascii_pgm") then
                                    --ImgSimPara.ImgOut.OutImgBits := 8;
                                    ImgSimPara.ImgOut.OutImgFrmt := "ascii";
                                    ImgSimPara.ImgOut.OutImgType := "pgm";
                                elsif (WordBuffer(1 to 7) = "bin_ppm") then
                                    ImgSimPara.ImgOut.OutImgBits := 24;
                                    ImgSimPara.ImgOut.OutImgFrmt := "bin  ";
                                    ImgSimPara.ImgOut.OutImgType := "ppm";
                                elsif (WordBuffer(1 to 9) = "ascii_ppm") then
                                    ImgSimPara.ImgOut.OutImgBits := 24;
                                    ImgSimPara.ImgOut.OutImgFrmt := "ascii";
                                    ImgSimPara.ImgOut.OutImgType := "ppm";
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_26;
                                end if;

                            when OutImgBits => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: OutImgBits = " & WordBuffer, LogSimInit);
                                if (IsNumber(WordBuffer) = True) then
                                    ImgSimPara.ImgOut.OutImgBits := Strg2Int(WordBuffer);
                                else
                                    FileErrCode := cSIMPARAINIT_ERRNUM_27;
                                end if;

                            when OutImgFile => 
                                PrintLog ("[SimParaInit] Reading simulation parameter: OutImgFile = " & WordBuffer, LogSimInit);
                                ImgSimPara.ImgOut.OutImgFile := WordBuffer(1 to cMAX_FILENAME_LENGTH);

                            when others => 
                                FileErrCode := cSIMPARAINIT_ERRNUM_28;
                        end case;
                    else
                        FileErrCode := cSIMPARAINIT_ERRNUM_29;      -- Unknow Section name
                    end if;
            
                    CharCnt    := 1;
                    EndOfWord  := False;
                    WordBuffer := (others => ' ');
                    WordCnt    := WordCnt + 1;
                end if;
            
                if ((EndOfLine = True) or (FileErrCode /= cSIM_ERROR_DEFAULT)) then
                    exit SimIniLineLoop;
                end if;
            end loop SimIniLineLoop;
            
            if (FileErrCode /= cSIM_ERROR_DEFAULT) then
                exit SimIniFileLoop;
            end if;
        end loop SimIniFileLoop;

        file_close(SimIniFile);

        -- Print error message and abort simulation
        if (FileErrCode /= cSIM_ERROR_DEFAULT) then
            PrintLog("[SimParaInit] Simulation parameter read error in file " & cTESTBENCH_INI_FILE & ", line " & integer'image(LineNumber) & ": " & RemoveBlanks(FileErrCode), LogError);
            AbortSim;
        end if;

        -- Assign internally used output signals.
        ImgSimParaxTO <= ImgSimPara;  

        PrintLog ("[SimParaInit] TestBench initialization parameters valid", LogFlowCtl);
        PrintLog ("Exiting SimParaInit", LogReturn);
        wait;
    end process;


end;

