-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Testbench SystemBus Stimulus
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2013-05-20  1.0      Manfred  Created
--              2015-04-16  2.0      a.kramer Changed for GigEpro
--              2017-03-02  3.0      Manfred  Changed for CorSight2
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library std;
use std.textio.all;

library work;
use work.CustomPkg.all;
use work.CustomCfgLib.all;
use work.CustomSimLib.all;
use work.CustomFctLib.all;

entity SysBusStimulus is
    port (  
        -- Infrastructure Signals
        SysClkxCI         : in  std_logic;              -- SystemBus Clock
        SysRstxRI         : in  std_logic;              -- SystemBus Reset
        SimParaxTI        : in  tSIMULATION_PARAMETERS; -- System Simulation Parameters
        DebugPortxTI      : in  std_logic_vector(cDBG_PORT_WIDTH-1 downto 0);   -- Debug Port
    
        -- SystemBus Control Interface                      
        SysModReqxMO      : out std_logic;              -- SystemBus Custom Module Request
        SysDdrReqxMO      : out std_logic;              -- SystemBus DDR3-RAM Memory Request
        SysAckxMI         : in  std_logic;              -- SystemBus Cycle Acknowledge
        SysErrxMI         : in  std_logic;              -- SystemBus Cycle Error
        SysWrEnxMO        : out std_logic;              -- SystemBus Write Enable
        SysAddrxAO        : out std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0);  -- SystemBus Address
        SysWrDataxDO      : out std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);  -- SystemBus Write Data
        SysRdDataxDI      : in  std_logic_vector(cCMD_DATA_WIDTH-1 downto 0);  -- SystemBus Read Data

        -- Simulation Flow Control
        ImgRstxRO         : out std_logic;              -- ImageBus Reset
        StartImgBusxMO    : out boolean;                -- Start ImageBus frame transfers
        StopImgBusxMO     : out boolean;                -- Stop ImageBus frame transfers
        AbortImgBusxMO    : out boolean);               -- Abort ImageBus frame transfers
end SysBusStimulus;

architecture Behavioral of SysBusStimulus is

    signal StartImgBusxM  : boolean   := False;         -- Start ImageBus frame transfers
    signal StopImgBusxM   : boolean   := False;         -- Stop ImageBus frame transfers
    signal AbortImgBusxM  : boolean   := False;         -- Abort ImageBus frame transfers
    signal ImgRstxR       : std_logic := '0';           -- ImageBus Reset
    signal SysModReqxM    : std_logic := '0';           -- SystemBus Custom Module Request
    signal SysDdrReqxM    : std_logic := '0';           -- SystemBus DDR3-RAM Memory Request
    signal SysWrEnxM      : std_logic := '0';           -- SystemBus Write Enable
    signal SysAddrxA      : std_logic_vector(cCMD_ADDR_WIDTH-1 downto 0) := (others => '0');  -- SystemBus Address
    signal SysWrDataxD    : std_logic_vector(cCMD_DATA_WIDTH-1 downto 0) := (others => '0');  -- SystemBus Write Data

begin

    ----------------------------------------------------------------------------
    -- SystemBus Host Interface
    ----------------------------------------------------------------------------
    SysBusProc: process 
        file     SimCmdFile   : text;
        variable SimCmdLine   : line;
        variable CurChar      : character := ' ';
        variable LastChar     : character := ' ';
        variable CommandOp    : string(1 to 2) := "  ";
        variable EventOp      : string(1 to 2) := "  ";
        variable CmdIniData   : tCMD_DATA_RECORD := cCMD_DATA_DEFAULT;
        variable LineNumber   : natural := 0;
        variable EndOfWord    : boolean := False;
        variable EndOfLine    : boolean := False;
        variable ImgBusReset  : boolean := False;
        variable StartImgTx   : boolean := False;
        variable StopImgTx    : boolean := False;
        variable AbortImgTx   : boolean := False;
        variable MessageAct   : boolean := False;
        variable CharTrap     : boolean := False;
        variable WordCnt      : natural := 0;
        variable WaitCyc      : natural := 0;
        variable DbgIdxLo     : natural := 0;
        variable DbgIdxHi     : natural := 0;
        variable DebugBusVal  : natural := 0;
        variable CharCnt      : natural range 1 to cMAX_WORD_LENGTH := 1;
        variable WordBuffer   : string(1 to cMAX_WORD_LENGTH)   := (others => ' ');
        variable StrCharCnt   : natural range 1 to cMAX_STRING_LENGTH := 1;
        variable StringBuffer : string(1 to cMAX_STRING_LENGTH) := (others => ' ');
        variable CmdKeyName   : tCMD_KEY_NAME_LIST := none;
        variable FileErrCode  : tSIM_ERROR_MESSAGE := cSIM_ERROR_DEFAULT;
    begin
        PrintLog ("Executing SysBusStimulus", LogExec);

        wait for 1 ns;

        -- Use the default command file name constant if SimParaxTI.CommandFile is not initialized.
        if (SimParaxTI.CommandFile = cBLANK_FILENAME) then
            PrintLog("[SysBusStimulus] Simulation Command file name missing", LogError);
            AbortSim;
        else
            PrintLog ("[SysBusStimulus] Opening Simulation Command file " & RemoveBlanks(SimParaxTI.CommandFile) & " in read mode", LogFlowCtl);
            file_open(SimCmdFile, RemoveBlanks(SimParaxTI.CommandFile), read_mode);
        end if;

        StartImgBusxM <= False;
        StopImgBusxM  <= False;
        SysModReqxM   <= '0';
        SysDdrReqxM   <= '0';
        SysWrEnxM     <= '0';
        SysAddrxA     <= (others => '0');
        SysWrDataxD   <= (others => '0');
        wait until (SysRstxRI = '0');

        ReadFileLoop: loop
            if endfile(SimCmdFile) then
                file_close(SimCmdFile);
                exit ReadFileLoop;
            else
                -- Read a line from the Register setup file into a line buffer.
                readline(SimCmdFile, SimCmdLine);
                LineNumber := LineNumber + 1;
        
                -- Check basic read line error conditions.
                if ((SimCmdLine /= NULL) and (SimCmdLine'length > 0)) then
                    -- Read a line from the Command file into a line buffer.
                    CharCnt     := 1;
                    WordCnt     := 0;
                    StrCharCnt  := 1;
                    CurChar     := ' ';
                    LastChar    := ' ';
                    CommandOp   := "  ";
                    EventOp     := "  ";
                    EndOfLine   := False;
                    EndOfWord   := False;
                    ImgBusReset := False;
                    StartImgTx  := False;
                    StopImgTx   := False;
                    AbortImgTx  := False;
                    CmdKeyName  := none;
                    WordBuffer  := (others => ' ');
                    CmdIniData  := cCMD_DATA_DEFAULT; 
            
                    SimCmdLineLoop: for CharNum in SimCmdLine'range loop
                        LastChar := CurChar;
                        CurChar  := SimCmdLine.all(CharNum);

                        if (CharNum = SimCmdLine'high) then 
                            EndOfLine := True;
                        end if;

                        if (CmdKeyName /= PrintMsg) then
                            case CurChar is
                                when ' ' | HT | '.' | '(' | ')' => 
                                    if (CharCnt > 1) then       -- Skip leading white spaces.
                                        EndOfWord := True;      
                                    end if;     
    
                                when cINI_COMMENT | CR | LF =>
                                    EndOfLine := True;

                                when cEQUAL_CHAR => 
                                    if (CharCnt > 1) then       -- Equal sign is used as word seperator.
                                        EndOfWord := True;      
                                    end if;

                                    if (LastChar = cINVERT_CHAR) then
                                        CommandOp := "ne";
                                    else
                                        CommandOp := "eq";
                                    end if;

                                when cINVERT_CHAR => NULL;

                                when others => 
                                    WordBuffer(CharCnt) := LowerCase(CurChar);
                                    CharCnt := CharCnt + 1;
                            end case;

                        else
                            -- Print a message
                            if (MessageAct = True) then
                                if ((CurChar = ')') and (CharTrap = False)) then
                                    MessageAct := False;
                                    EndOfLine  := True;
                                else
                                    if (CharTrap = False) then
                                        StringBuffer(StrCharCnt) := CurChar;
                                        StrCharCnt := StrCharCnt + 1;
                                    else
                                        StringBuffer(StrCharCnt-1) := CurChar;
                                    end if;
    
                                    if (CurChar = cTRAP_CHAR) then
                                        CharTrap := True;
                                    else
                                        CharTrap := False;
                                    end if;
                                end if;
                            elsif (CurChar = '(') then
                                MessageAct   := True;
                                StringBuffer := (others => ' ');
                            elsif ((CurChar = CR) or (CurChar = LF)) then
                                FileErrCode := cSYSBUSSTIM_ERRNUM_1;
                            end if;
                        end if;
            
                        -- Analyze WordBuffer if valid characters have been detected.
                        if (((EndOfLine = True) or (EndOfWord = True)) and ((CharCnt > 1) or (StrCharCnt > 1))) then
                            if (WordCnt = 0) then
                                -- The first key word on a line is the command key word.
                                case WordBuffer is
                                    when cCMDKEY_SYSWRITE    => CmdKeyName := SysWrite;
                                                                CmdIniData.WrCycle := True;
                                    when cCMDKEY_SYSREAD     => CmdKeyName := SysRead; 
                                                                CmdIniData.WrCycle := False;
                                                                CmdIniData.Verify  := False;
                                    when cCMDKEY_SYSVERIFY   => CmdKeyName := SysVerify;
                                                                CmdIniData.WrCycle := False;
                                                                CmdIniData.Verify  := True;
                                    when cCMDKEY_SIMWAIT     => CmdKeyName  := SimWait;
                                    when cCMDKEY_EVENTSIG    => CmdKeyName  := DebugSig;
                                    when cCMDKEY_EVENTBUS    => CmdKeyName  := DebugBus;
                                    when cCMDKEY_IMGBUSRESET => ImgBusReset := True;
                                    when cCMDKEY_STARTIMGBUS => StartImgTx  := True;
                                    when cCMDKEY_STOPIMGBUS  => StopImgTx   := True;
                                    when cCMDKEY_ABORTIMGBUS => AbortImgTx  := True;
                                    when cCMDKEY_PRINTMSG    => CmdKeyName  := PrintMsg;
                                    when others => FileErrCode := cSYSBUSSTIM_ERRNUM_2;
                                end case;

                            elsif (WordCnt = 1) then
                                case CmdKeyName is
                                    when SysWrite | SysRead | SysVerify =>
                                        -- The second key word contains the address offset as decimal or hexadecimal number.
                                        if (IsNumber(WordBuffer) = True) then
                                            CmdIniData.CmdAddr := Strg2Vec(WordBuffer, cCMD_ADDR_WIDTH); 
                                            -- Determine if the current SystemBus access is to a memory location.
                                            if (CmdIniData.CmdAddr >= to_bitvector(std_logic_vector(to_unsigned(cDDR3_MEMORY_ADDR, cCMD_ADDR_WIDTH)))) then
                                                CmdIniData.DdrCycle := True;        -- DDR3-RAM access cycle
                                                CmdIniData.ModCycle := False;       -- No Custom Module access cycle
                                            elsif ((CmdIniData.CmdAddr >= to_bitvector(std_logic_vector(to_unsigned(cCUSTOM_START_ADDR, cCMD_ADDR_WIDTH)))) and
                                                   (CmdIniData.CmdAddr <= to_bitvector(std_logic_vector(to_unsigned(cCUSTOM_END_ADDR, cCMD_ADDR_WIDTH))))) then
                                                CmdIniData.DdrCycle := False;       -- No DDR3-RAM access cycle
                                                CmdIniData.ModCycle := True;        -- Custom Module access cycle
                                            else
                                                PrintLog("[SysBusStimulus] ERROR - Detected invalid SystemBus cycle at address: " & WordBuffer, LogError);
                                                CmdIniData.DdrCycle := False;       -- No DDR3-RAM access cycle
                                                CmdIniData.ModCycle := False;       -- No Custom Module access cycle
                                            end if;

                                            if (CmdKeyName = SysRead) then
                                                CmdIniData.Valid := True; 
                                            end if;
                                        else
                                            FileErrCode := cSYSBUSSTIM_ERRNUM_3;
                                        end if;

                                    when SimWait =>
                                        -- The second key word contains the wait period.
                                        if (IsNumber(WordBuffer) = True) then
                                            WaitCyc := Strg2Int(WordBuffer);
                                        else
                                            FileErrCode := cSYSBUSSTIM_ERRNUM_4;
                                        end if;

                                    when DebugSig | DebugBus =>
                                        -- The second key word contains the high-order DebugSignal index.
                                        if (IsNumber(WordBuffer) = True) then
                                            DbgIdxHi := Strg2Int(WordBuffer); 

                                            if (DbgIdxHi > cDBG_PORT_WIDTH-1) then
                                                FileErrCode := cSYSBUSSTIM_ERRNUM_5;
                                            end if;
                                        else
                                            FileErrCode := cSYSBUSSTIM_ERRNUM_6;
                                        end if;

                                    when PrintMsg =>
                                        PrintNow(RemoveBlanks(StringBuffer));

                                    when others => FileErrCode := cSYSBUSSTIM_ERRNUM_7;
                                end case;

                            elsif (WordCnt = 2) then
                                case CmdKeyName is
                                    when SysWrite | SysVerify =>
                                        -- The third key word contains register or memory data as decimal or hexadecimal number.
                                        if (IsNumber(WordBuffer) = True) then
                                            CmdIniData.CmdData(0) := Strg2Vec(WordBuffer, cCMD_DATA_WIDTH);
                                            CmdIniData.DataCnt := 1;

                                            if (CmdKeyName = SysWrite) then
                                                CmdIniData.Valid := True; 
                                            end if;
                                        else
                                            FileErrCode := cSYSBUSSTIM_ERRNUM_8;
                                        end if;

                                    when SimWait =>
                                        -- The second key word contains the wait period (in us).
                                        case WordBuffer(1 to 2) is
                                            when "ns" => 
                                                PrintLog ("[SysBusStimulus] Start " & integer'image(WaitCyc) & " ns wait cycle", LogWait);
                                                wait for (WaitCyc * 1 ns);
                                            when "us" => 
                                                PrintLog ("[SysBusStimulus] Start " & integer'image(WaitCyc) & " us wait cycle", LogWait);
                                                wait for (WaitCyc * 1 us);
                                            when "ms" => 
                                                PrintLog ("[SysBusStimulus] Start " & integer'image(WaitCyc) & " ms wait cycle", LogWait);
                                                wait for (WaitCyc * 1 ms);
                                            when "se" => 
                                                PrintLog ("[SysBusStimulus] Start " & integer'image(WaitCyc) & " sec wait cycle", LogWait);
                                                wait for (WaitCyc * 1 sec);
                                            when others => FileErrCode := cSYSBUSSTIM_ERRNUM_9;
                                        end case;

                                        PrintLog ("[SysBusStimulus] Wait cycle completed", LogWait);

                                    when DebugSig =>
                                        case WordBuffer is
                                            when cCMDKEY_RISEEDGE =>
                                                PrintLog ("Waiting for rising edge on DebugSignal(" & integer'image(DbgIdxHi) & ")", LogDbgWait);
                                                wait until (DebugPortxTI(DbgIdxHi) = '1');
                                                PrintLog ("Rising edge on DebugSignal(" & integer'image(DbgIdxHi) & ") detected", LogDbgWait);
                                            when cCMDKEY_FALLEDGE =>
                                                PrintLog ("Waiting for falling edge on DebugSignal(" & integer'image(DbgIdxHi) & ")", LogDbgWait);
                                                wait until (DebugPortxTI(DbgIdxHi) = '0');
                                                PrintLog ("Falling edge on DebugSignal(" & integer'image(DbgIdxHi) & ") detected", LogDbgWait);
                                            when cCMDKEY_HIGHLEVEL =>
                                                PrintLog ("Waiting for high level on DebugSignal(" & integer'image(DbgIdxHi) & ")", LogDbgWait);
                                                if (DebugPortxTI(DbgIdxHi) = '0') then
                                                    wait until (DebugPortxTI(DbgIdxHi) = '1');
                                                end if;
                                                PrintLog ("High level on DebugSignal(" & integer'image(DbgIdxHi) & ") detected", LogDbgWait);
                                            when cCMDKEY_LOWLEVEL =>
                                                PrintLog ("Waiting for low level on DebugSignal(" & integer'image(DbgIdxHi) & ")", LogDbgWait);
                                                if (DebugPortxTI(DbgIdxHi) = '1') then
                                                    wait until (DebugPortxTI(DbgIdxHi) = '0');
                                                end if;
                                                PrintLog ("Low level on DebugSignal(" & integer'image(DbgIdxHi) & ") detected", LogDbgWait);
                                            when others => FileErrCode := cSYSBUSSTIM_ERRNUM_10;
                                        end case;

                                    when DebugBus =>
                                        if (IsNumber(WordBuffer) = True) then
                                            DbgIdxLo := Strg2Int(WordBuffer); 

                                            if ((DbgIdxLo > cDBG_PORT_WIDTH-1) or (DbgIdxLo > DbgIdxHi)) then
                                                FileErrCode := cSYSBUSSTIM_ERRNUM_11;
                                            end if;
                                        else
                                            FileErrCode := cSYSBUSSTIM_ERRNUM_12;
                                        end if;

                                    when others => FileErrCode := cSYSBUSSTIM_ERRNUM_13;
                                end case;

                            elsif (WordCnt = 3) then
                                case CmdKeyName is
                                    when SysWrite | SysVerify =>
                                        -- The third key word contains register or memory data (a decimal or hexadecimal number).
                                        if (IsNumber(WordBuffer) = True) then
                                            if (CmdKeyName = SysWrite) then
                                                CmdIniData.CmdData(1) := Strg2Vec(WordBuffer, cCMD_DATA_WIDTH);
                                                CmdIniData.DataCnt := 2;
                                            else
                                                CmdIniData.CmdMask := Strg2Vec(WordBuffer, cCMD_DATA_WIDTH);
                                            end if;

                                            CmdIniData.Valid := True; 
                                        else
                                            FileErrCode := cSYSBUSSTIM_ERRNUM_14;
                                        end if;

                                    when DebugBus =>
                                        case WordBuffer is
                                            when cCMDKEY_EQUALOP =>
                                                EventOp := "eq";
                                            when cCMDKEY_NOTEQOP =>
                                                EventOp := "ne";
                                            when others => FileErrCode := cSYSBUSSTIM_ERRNUM_15;
                                        end case;

                                    when others => FileErrCode := cSYSBUSSTIM_ERRNUM_16;
                                end case;

                            elsif (WordCnt = 4) then
                                case CmdKeyName is
                                    when SysWrite =>
                                        -- The fourth key word contains register or memory data (a decimal or hexadecimal number).
                                        if (CmdKeyName = SysWrite) then
                                            if (IsNumber(WordBuffer) = True) then
                                                CmdIniData.CmdData(CmdIniData.DataCnt) := Strg2Vec(WordBuffer, cCMD_DATA_WIDTH);
                                                CmdIniData.DataCnt := CmdIniData.DataCnt + 1;
                                                CmdIniData.Valid   := True; 
                                            else
                                                FileErrCode := cSYSBUSSTIM_ERRNUM_17;
                                            end if;
                                        else
                                            FileErrCode := cSYSBUSSTIM_ERRNUM_18;
                                        end if;

                                    when DebugBus =>
                                        if (IsNumber(WordBuffer) = True) then
                                            DebugBusVal := Strg2Int(WordBuffer);

                                            if (EventOp = "eq") then
                                                PrintLog ("Waiting for DebugBus(" & integer'image(DbgIdxHi) & ".." & integer'image(DbgIdxLo) & ") equal to " & integer'image(DebugBusVal), LogDbgWait);
                                                if (DebugPortxTI(DbgIdxHi downto DbgIdxLo) /= std_logic_vector(to_unsigned(DebugBusVal, (DbgIdxHi-DbgIdxLo)))) then
                                                    wait until (DebugPortxTI(DbgIdxHi downto DbgIdxLo) = std_logic_vector(to_unsigned(DebugBusVal, (DbgIdxHi-DbgIdxLo))));
                                                end if;
                                                PrintLog ("DebugBus(" & integer'image(DbgIdxHi) & ".." & integer'image(DbgIdxLo) & ") equal to " & integer'image(DebugBusVal) & " detected", LogDbgWait);

                                            elsif (EventOp = "ne") then
                                                PrintLog ("Waiting for DebugBus(" & integer'image(DbgIdxHi) & ".." & integer'image(DbgIdxLo) & ") not equal to " & integer'image(DebugBusVal), LogDbgWait);
                                                if (DebugPortxTI(DbgIdxHi downto DbgIdxLo) = std_logic_vector(to_unsigned(DebugBusVal, (DbgIdxHi-DbgIdxLo)))) then
                                                    wait until (DebugPortxTI(DbgIdxHi downto DbgIdxLo) /= std_logic_vector(to_unsigned(DebugBusVal, (DbgIdxHi-DbgIdxLo))));
                                                end if;
                                                PrintLog ("DebugBus(" & integer'image(DbgIdxHi) & ".." & integer'image(DbgIdxLo) & ") not equal to " & integer'image(DebugBusVal) & " detected", LogDbgWait);
                                            end if;
                                        else
                                            FileErrCode := cSYSBUSSTIM_ERRNUM_19;
                                        end if;

                                    when others => FileErrCode := cSYSBUSSTIM_ERRNUM_20;
                                end case;

                            else
                                -- The fifth and subsequent arguments contain register or memory data (a decimal or hexadecimal number).
                                if (CmdKeyName = SysWrite) then
                                    if (IsNumber(WordBuffer) = True) then
                                        CmdIniData.CmdData(CmdIniData.DataCnt) := Strg2Vec(WordBuffer, cCMD_DATA_WIDTH);
                                        CmdIniData.DataCnt := CmdIniData.DataCnt + 1;
                                        CmdIniData.Valid   := True; 
                                    else
                                        FileErrCode := cSYSBUSSTIM_ERRNUM_21;
                                    end if;
                                else
                                    FileErrCode := cSYSBUSSTIM_ERRNUM_22;
                                end if;
                            end if;
    
                            CharCnt    := 1;
                            EndOfWord  := False;
                            WordBuffer := (others => ' ');
                            WordCnt    := WordCnt + 1;
                        end if;
            
                        if ((EndOfLine = True) or (FileErrCode /= cSIM_ERROR_DEFAULT)) then
                            exit SimCmdLineLoop;
                        end if;
                    end loop SimCmdLineLoop;
            
                    -- Print error message and abort simulation
                    if (FileErrCode /= cSIM_ERROR_DEFAULT) then
                        PrintLog("[SysBusStimulus] SystemBus read error in file " & RemoveBlanks(SimParaxTI.CommandFile) & ", line " & integer'image(LineNumber) & ": " & RemoveBlanks(FileErrCode), LogError);
                        AbortSim;
                    end if;

                    -- Perform an ImageBus module reset. Valid for 2 clock cycles.
                    if (ImgBusReset = True) then
                        PrintLog ("[SysBusStimulus] Executing ImageBus Reset", LogFlowCtl);
                        ImgRstxR <= '1';
                        wait until rising_edge(SysClkxCI);
                        wait until rising_edge(SysClkxCI);
                        ImgRstxR <= '0';
                    end if;

                    -- Signal to the Image Stimulus Module to start an ImageBus frame transfer
                    if (StartImgTx = True) then
                        PrintLog ("[SysBusStimulus] ImageBus Start of Frame", LogFlowCtl);
                        StartImgBusxM <= True;
                        wait until rising_edge(SysClkxCI);
                        StartImgBusxM <= False;
                    end if;

                    -- Signal to the Image Stimulus Module to stop an ImageBus frame transfer
                    if (StopImgTx = True) then
                        PrintLog ("[SysBusStimulus] ImageBus Stop of Frame", LogFlowCtl);
                        StopImgBusxM <= True;
                        wait until rising_edge(SysClkxCI);
                        StopImgBusxM <= False;
                    end if;

                    -- Signal to the Image Stimulus Module to abort an ImageBus frame transfer
                    if (AbortImgTx = True) then
                        PrintLog ("[SysBusStimulus] Executing ImageBus Frame Abort", LogFlowCtl);
                        AbortImgBusxM <= True;
                        wait until rising_edge(SysClkxCI);
                        AbortImgBusxM <= False;
                    end if;
                    if (CmdIniData.Valid = True) then
                        if (CmdIniData.WrCycle = True) then
                            -- Perform SystemBus write cycles of CmdIniData.
                            Proc_SysBus_Write (CmdIniData, SysClkxCI, SysAckxMI, SysErrxMI, 
                                               SysModReqxM, SysDdrReqxM, SysWrEnxM, SysAddrxA, SysWrDataxD);
                        else
                            -- Perform SystemBus read or verify cycles of CmdIniData.
                            Proc_SysBus_Read (CmdIniData, SysClkxCI, SysAckxMI, SysErrxMI, 
                                              SysModReqxM, SysDdrReqxM, SysWrEnxM, SysAddrxA, SysRdDataxDI);
                        end if;
                    end if;
                end if;
            end if;
        end loop ReadFileLoop;
    
        file_close(SimCmdFile);
        PrintLog ("Exiting SysBusStimulus", LogReturn);
        wait;
    end process;

    -- Assign internally used output signals.
    SysModReqxMO   <= SysModReqxM;  
    SysDdrReqxMO   <= SysDdrReqxM;  
    SysWrEnxMO     <= SysWrEnxM; 
    SysAddrxAO     <= SysAddrxA; 
    SysWrDataxDO   <= SysWrDataxD;

    ImgRstxRO      <= SysRstxRI or ImgRstxR;
    StartImgBusxMO <= StartImgBusxM;
    StopImgBusxMO  <= StopImgBusxM;
    AbortImgBusxMO <= AbortImgBusxM;
end;

