//------------------------------------------------------------------------------
// File Name         :  $RCSfile:  $
// Author            :  $Author:  $
// Revision          :  $Revision:  $
// Last Modification :  $Date:  $
//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2013 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : canny_b12
// Submodules : canny_b12_regif
//              gauss_b12
//              h3b12x3
//              h3b14x3
//              h5b12
//              nomax_b12
//              rstbuf2
//              sobel_b12
//              v3b12k4
//              v3b14k4
//              v5b12k4
//
// Purpose    : Canny Edge Detector 12bit
//
// Creator    : nuessi
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module canny_b12 (
    // system signals
    clk,
    sys_rst_n,
    img_rst_n,

    // config port
    cfg_maddr_i,
    cfg_mcmd_i,
    cfg_mdata_i,
    cfg_mrespaccept_i,
    cfg_scmdaccept_o,
    cfg_sdata_o,
    cfg_sresp_o,

    // data input
    din_evt_i,
    din_fe_i,
    din_fs_i,
    din_le_i,
    din_ls_i,
    din_pel_i,

    // data output
    dout_evt_o,
    dout_fe_o,
    dout_fs_o,
    dout_le_o,
    dout_ls_o,
    dout_pel_o,

    // processing control
    proc_en_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           sys_rst_n;            // asynchronous system reset, active low
  input           img_rst_n;            // asynchronous image bus reset, active low

  //--------------------------------------------------------
  // config port
  //--------------------------------------------------------
  input  [16:2]   cfg_maddr_i;          // register address
  input  [1:0]    cfg_mcmd_i;           // 00:IDEL
                                        // 01:WR
                                        // 10:RD
                                        // 11:not implemented
  input  [31:0]   cfg_mdata_i;          // write data
  input           cfg_mrespaccept_i;    // read data acknowledge
  output          cfg_scmdaccept_o;     // command acknowledge
  output [31:0]   cfg_sdata_o;          // read data
  output          cfg_sresp_o;          // 0:NULL, 1:DVA

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input           din_evt_i;            // transaction event
  input           din_fe_i;             // frame end
  input           din_fs_i;             // frame start
  input           din_le_i;             // line end
  input           din_ls_i;             // line start
  input  [11:0]   din_pel_i;            // one pel

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output          dout_evt_o;           // transaction event
  output          dout_fe_o;            // frame end
  output          dout_fs_o;            // frame start
  output          dout_le_o;            // line end
  output          dout_ls_o;            // line start
  output [11:0]   dout_pel_o;           // pel

  //--------------------------------------------------------
  // processing control
  //--------------------------------------------------------
  input           proc_en_i;            // enable

  //----------------------------------------------------------------------------
  // wire declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // buffered reset
  //--------------------------------------------------------
  wire            buf_clk_s;            // system clock
  wire            sys_rst_n_s;          // asynchron low active reset
  wire            img_rst_n_s;          // asynchron low active reset

  //--------------------------------------------------------
  // data input gauss horz
  //--------------------------------------------------------
  wire            gauss_h_din_evt_s;    // transaction event
  wire            gauss_h_din_fe_s;     // frame end
  wire            gauss_h_din_fs_s;     // frame start
  wire            gauss_h_din_le_s;     // line end
  wire            gauss_h_din_ls_s;     // line start
  wire   [11:0]   gauss_h_din_pel0_s;   // one pel x=-2
  wire   [11:0]   gauss_h_din_pel1_s;   // one pel x=-1
  wire   [11:0]   gauss_h_din_pel2_s;   // one pel x=0
  wire   [11:0]   gauss_h_din_pel3_s;   // one pel x=1
  wire   [11:0]   gauss_h_din_pel4_s;   // one pel x=2

  //--------------------------------------------------------
  // data output gauss horz
  //--------------------------------------------------------
  wire            gauss_h_dout_evt_s;   // transaction event
  wire            gauss_h_dout_fe_s;    // frame end
  wire            gauss_h_dout_fs_s;    // frame start
  wire            gauss_h_dout_le_s;    // line end
  wire            gauss_h_dout_ls_s;    // line start
  wire   [11:0]   gauss_h_dout_pel_s;   // one pel

  //--------------------------------------------------------
  // data input gauss vert
  //--------------------------------------------------------
  wire            gauss_v_din_evt_s;    // transaction event
  wire            gauss_v_din_fe_s;     // frame end
  wire            gauss_v_din_fs_s;     // frame start
  wire            gauss_v_din_le_s;     // line end
  wire            gauss_v_din_ls_s;     // line start
  wire   [11:0]   gauss_v_din_pel0_s;   // one pel y=-2
  wire   [11:0]   gauss_v_din_pel1_s;   // one pel y=-1
  wire   [11:0]   gauss_v_din_pel2_s;   // one pel y=0
  wire   [11:0]   gauss_v_din_pel3_s;   // one pel y=1
  wire   [11:0]   gauss_v_din_pel4_s;   // one pel y=2

  //--------------------------------------------------------
  // data output gauss vert
  //--------------------------------------------------------
  wire            gauss_v_dout_evt_s;   // transaction event
  wire            gauss_v_dout_fe_s;    // frame end
  wire            gauss_v_dout_fs_s;    // frame start
  wire            gauss_v_dout_le_s;    // line end
  wire            gauss_v_dout_ls_s;    // line start
  wire   [11:0]   gauss_v_dout_pel_s;   // one pel

  //--------------------------------------------------------
  // sobel data input
  //--------------------------------------------------------
  wire            sobel_din_pre_evt_s;  // transaction event
  wire            sobel_din_pre_fe_s;   // frame end
  wire            sobel_din_pre_fs_s;   // frame start
  wire            sobel_din_pre_le_s;   // line end
  wire            sobel_din_pre_ls_s;   // line start
  wire   [11:0]   sobel_din_pre_pel0_s; // one pel y=-1
  wire   [11:0]   sobel_din_pre_pel1_s; // one pel y=0
  wire   [11:0]   sobel_din_pre_pel2_s; // one pel y=1

  //--------------------------------------------------------
  // data input sobel
  //--------------------------------------------------------
  wire            sobel_din_evt_s;      // transaction event
  wire            sobel_din_fe_s;       // frame end
  wire            sobel_din_fs_s;       // frame start
  wire            sobel_din_le_s;       // line end
  wire            sobel_din_ls_s;       // line start
  wire   [11:0]   sobel_din_pel00_s;    // one pel -1,-1
  wire   [11:0]   sobel_din_pel01_s;    // one pel  0,-1
  wire   [11:0]   sobel_din_pel02_s;    // one pel  1,-1
  wire   [11:0]   sobel_din_pel10_s;    // one pel -1, 0
  wire   [11:0]   sobel_din_pel11_s;    // one pel  0, 0
  wire   [11:0]   sobel_din_pel12_s;    // one pel  1, 0
  wire   [11:0]   sobel_din_pel20_s;    // one pel -1, 1
  wire   [11:0]   sobel_din_pel21_s;    // one pel  0, 1
  wire   [11:0]   sobel_din_pel22_s;    // one pel  1, 1

  //--------------------------------------------------------
  // data output gauss horz
  //--------------------------------------------------------
  wire            sobel_dout_evt_s;     // transaction event
  wire            sobel_dout_fe_s;      // frame end
  wire            sobel_dout_fs_s;      // frame start
  wire            sobel_dout_le_s;      // line end
  wire            sobel_dout_ls_s;      // line start
  wire   [13:0]   sobel_dout_pel_s;     // [13:12] dir
                                        // [11:0]  pel

  //--------------------------------------------------------
  // data input nomax
  //--------------------------------------------------------
  wire            nomax_din_pre_evt_s;  // transaction event
  wire            nomax_din_pre_fe_s;   // frame end
  wire            nomax_din_pre_fs_s;   // frame start
  wire            nomax_din_pre_le_s;   // line end
  wire            nomax_din_pre_ls_s;   // line start
  wire   [13:0]   nomax_din_pre_pel0_s; // one pel y=-1
  wire   [13:0]   nomax_din_pre_pel1_s; // one pel y=0
  wire   [13:0]   nomax_din_pre_pel2_s; // one pel y=1

  //--------------------------------------------------------
  // data input nomax
  //--------------------------------------------------------
  wire            nomax_din_evt_s;      // transaction event
  wire            nomax_din_fe_s;       // frame end
  wire            nomax_din_fs_s;       // frame start
  wire            nomax_din_le_s;       // line end
  wire            nomax_din_ls_s;       // line start
  wire   [13:0]   nomax_din_pel00_s;    // one pel -1,-1
  wire   [13:0]   nomax_din_pel01_s;    // one pel  0,-1
  wire   [13:0]   nomax_din_pel02_s;    // one pel  1,-1
  wire   [13:0]   nomax_din_pel10_s;    // one pel -1, 0
  wire   [13:0]   nomax_din_pel11_s;    // dir, one pel  0, 0
  wire   [13:0]   nomax_din_pel12_s;    // one pel  1, 0
  wire   [13:0]   nomax_din_pel20_s;    // one pel -1, 1
  wire   [13:0]   nomax_din_pel21_s;    // one pel  0, 1
  wire   [13:0]   nomax_din_pel22_s;    // one pel  1, 1

  //--------------------------------------------------------
  // parameter
  //--------------------------------------------------------
  wire            param_gauss_byp_h_s;  // bypass horizontal gaussian
  wire            param_gauss_byp_v_s;  // bypass vertical gaussian
  wire            param_gauss_mode_h_s; // 0: 3tab
                                        // 1: 5tab
  wire            param_gauss_mode_v_s; // 0: 3tab
                                        // 1: 5tab
  wire            param_nomax_byp_s;    // bypass no max elimination
  wire            param_nomax_mode_s;   // 0: just edge
                                        // 1: egde with weight
  wire            param_sobel_byp_s;    // bypass sobel
  wire   [11:0]   param_thres_s;        // edge threshold
  
  
  wire            init=1'd0;            // unused signal 

  //--------------------------------------------------------
  // instantiation of module -- GAUSS_B12 --
  //--------------------------------------------------------
  gauss_b12 u_gauss_h (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( gauss_h_din_evt_s            ),
    .din_fe_i                     ( gauss_h_din_fe_s             ),
    .din_fs_i                     ( gauss_h_din_fs_s             ),
    .din_le_i                     ( gauss_h_din_le_s             ),
    .din_ls_i                     ( gauss_h_din_ls_s             ),
    .din_pel0_i                   ( gauss_h_din_pel0_s           ),
    .din_pel1_i                   ( gauss_h_din_pel1_s           ),
    .din_pel2_i                   ( gauss_h_din_pel2_s           ),
    .din_pel3_i                   ( gauss_h_din_pel3_s           ),
    .din_pel4_i                   ( gauss_h_din_pel4_s           ),
    .dout_evt_o                   ( gauss_h_dout_evt_s           ),
    .dout_fe_o                    ( gauss_h_dout_fe_s            ),
    .dout_fs_o                    ( gauss_h_dout_fs_s            ),
    .dout_le_o                    ( gauss_h_dout_le_s            ),
    .dout_ls_o                    ( gauss_h_dout_ls_s            ),
    .dout_pel_o                   ( gauss_h_dout_pel_s           ),
    .param_gauss_byp_i            ( param_gauss_byp_h_s          ),
    .param_gauss_mode_i           ( param_gauss_mode_h_s         )
  );

  //--------------------------------------------------------
  // instantiation of module -- GAUSS_B12 --
  //--------------------------------------------------------
  gauss_b12 u_gauss_v (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( gauss_v_din_evt_s            ),
    .din_fe_i                     ( gauss_v_din_fe_s             ),
    .din_fs_i                     ( gauss_v_din_fs_s             ),
    .din_le_i                     ( gauss_v_din_le_s             ),
    .din_ls_i                     ( gauss_v_din_ls_s             ),
    .din_pel0_i                   ( gauss_v_din_pel0_s           ),
    .din_pel1_i                   ( gauss_v_din_pel1_s           ),
    .din_pel2_i                   ( gauss_v_din_pel2_s           ),
    .din_pel3_i                   ( gauss_v_din_pel3_s           ),
    .din_pel4_i                   ( gauss_v_din_pel4_s           ),
    .dout_evt_o                   ( gauss_v_dout_evt_s           ),
    .dout_fe_o                    ( gauss_v_dout_fe_s            ),
    .dout_fs_o                    ( gauss_v_dout_fs_s            ),
    .dout_le_o                    ( gauss_v_dout_le_s            ),
    .dout_ls_o                    ( gauss_v_dout_ls_s            ),
    .dout_pel_o                   ( gauss_v_dout_pel_s           ),
    .param_gauss_byp_i            ( param_gauss_byp_v_s          ),
    .param_gauss_mode_i           ( param_gauss_mode_v_s         )
  );

  //--------------------------------------------------------
  // instantiation of module -- H3B12X3 --
  //--------------------------------------------------------
  h3b12x3 u_h3b12x3 (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( sobel_din_pre_evt_s          ),
    .din_fe_i                     ( sobel_din_pre_fe_s           ),
    .din_fs_i                     ( sobel_din_pre_fs_s           ),
    .din_le_i                     ( sobel_din_pre_le_s           ),
    .din_ls_i                     ( sobel_din_pre_ls_s           ),
    .din_pel0_i                   ( sobel_din_pre_pel0_s         ),
    .din_pel1_i                   ( sobel_din_pre_pel1_s         ),
    .din_pel2_i                   ( sobel_din_pre_pel2_s         ),
    .dout_evt_o                   ( sobel_din_evt_s              ),
    .dout_fe_o                    ( sobel_din_fe_s               ),
    .dout_fs_o                    ( sobel_din_fs_s               ),
    .dout_le_o                    ( sobel_din_le_s               ),
    .dout_ls_o                    ( sobel_din_ls_s               ),
    .dout_pel00_o                 ( sobel_din_pel00_s            ),
    .dout_pel01_o                 ( sobel_din_pel01_s            ),
    .dout_pel02_o                 ( sobel_din_pel02_s            ),
    .dout_pel10_o                 ( sobel_din_pel10_s            ),
    .dout_pel11_o                 ( sobel_din_pel11_s            ),
    .dout_pel12_o                 ( sobel_din_pel12_s            ),
    .dout_pel20_o                 ( sobel_din_pel20_s            ),
    .dout_pel21_o                 ( sobel_din_pel21_s            ),
    .dout_pel22_o                 ( sobel_din_pel22_s            )
  );

  //--------------------------------------------------------
  // instantiation of module -- H3B14X3 --
  //--------------------------------------------------------
  h3b14x3 u_h3b14x3 (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( nomax_din_pre_evt_s          ),
    .din_fe_i                     ( nomax_din_pre_fe_s           ),
    .din_fs_i                     ( nomax_din_pre_fs_s           ),
    .din_le_i                     ( nomax_din_pre_le_s           ),
    .din_ls_i                     ( nomax_din_pre_ls_s           ),
    .din_pel0_i                   ( nomax_din_pre_pel0_s         ),
    .din_pel1_i                   ( nomax_din_pre_pel1_s         ),
    .din_pel2_i                   ( nomax_din_pre_pel2_s         ),
    .dout_evt_o                   ( nomax_din_evt_s              ),
    .dout_fe_o                    ( nomax_din_fe_s               ),
    .dout_fs_o                    ( nomax_din_fs_s               ),
    .dout_le_o                    ( nomax_din_le_s               ),
    .dout_ls_o                    ( nomax_din_ls_s               ),
    .dout_pel00_o                 ( nomax_din_pel00_s            ),
    .dout_pel01_o                 ( nomax_din_pel01_s            ),
    .dout_pel02_o                 ( nomax_din_pel02_s            ),
    .dout_pel10_o                 ( nomax_din_pel10_s            ),
    .dout_pel11_o                 ( nomax_din_pel11_s            ),
    .dout_pel12_o                 ( nomax_din_pel12_s            ),
    .dout_pel20_o                 ( nomax_din_pel20_s            ),
    .dout_pel21_o                 ( nomax_din_pel21_s            ),
    .dout_pel22_o                 ( nomax_din_pel22_s            )
  );

  //--------------------------------------------------------
  // instantiation of module -- H5B12 --
  //--------------------------------------------------------
  h5b12 u_h5b12 (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( din_evt_i                    ),
    .din_fe_i                     ( din_fe_i                     ),
    .din_fs_i                     ( din_fs_i                     ),
    .din_le_i                     ( din_le_i                     ),
    .din_ls_i                     ( din_ls_i                     ),
    .din_pel_i                    ( din_pel_i                    ),
    .dout_evt_o                   ( gauss_h_din_evt_s            ),
    .dout_fe_o                    ( gauss_h_din_fe_s             ),
    .dout_fs_o                    ( gauss_h_din_fs_s             ),
    .dout_le_o                    ( gauss_h_din_le_s             ),
    .dout_ls_o                    ( gauss_h_din_ls_s             ),
    .dout_pel0_o                  ( gauss_h_din_pel0_s           ),
    .dout_pel1_o                  ( gauss_h_din_pel1_s           ),
    .dout_pel2_o                  ( gauss_h_din_pel2_s           ),
    .dout_pel3_o                  ( gauss_h_din_pel3_s           ),
    .dout_pel4_o                  ( gauss_h_din_pel4_s           )
  );

  //--------------------------------------------------------
  // instantiation of module -- NOMAX_B12 --
  //--------------------------------------------------------
  nomax_b12 u_nomax_b12 (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( nomax_din_evt_s              ),
    .din_fe_i                     ( nomax_din_fe_s               ),
    .din_fs_i                     ( nomax_din_fs_s               ),
    .din_le_i                     ( nomax_din_le_s               ),
    .din_ls_i                     ( nomax_din_ls_s               ),
    .din_pel00_i                  ( nomax_din_pel00_s[11:0]      ),
    .din_pel01_i                  ( nomax_din_pel01_s[11:0]      ),
    .din_pel02_i                  ( nomax_din_pel02_s[11:0]      ),
    .din_pel10_i                  ( nomax_din_pel10_s[11:0]      ),
    .din_pel11_i                  ( nomax_din_pel11_s            ),
    .din_pel12_i                  ( nomax_din_pel12_s[11:0]      ),
    .din_pel20_i                  ( nomax_din_pel20_s[11:0]      ),
    .din_pel21_i                  ( nomax_din_pel21_s[11:0]      ),
    .din_pel22_i                  ( nomax_din_pel22_s[11:0]      ),
    .dout_evt_o                   ( dout_evt_o                   ),
    .dout_fe_o                    ( dout_fe_o                    ),
    .dout_fs_o                    ( dout_fs_o                    ),
    .dout_le_o                    ( dout_le_o                    ),
    .dout_ls_o                    ( dout_ls_o                    ),
    .dout_pel_o                   ( dout_pel_o                   ),
    .param_nomax_byp_i            ( param_nomax_byp_s            ),
    .param_nomax_mode_i           ( param_nomax_mode_s           ),
    .param_thres_i                ( param_thres_s                )
  );

  //--------------------------------------------------------
  // instantiation of module -- CANNY_B12_REGIF --
  //--------------------------------------------------------
  canny_b12_regif u_regif (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( sys_rst_n_s                  ),
    .init_i                       ( init                         ),
    .cfg_maddr_i                  ( cfg_maddr_i                  ),
    .cfg_mcmd_i                   ( cfg_mcmd_i                   ),
    .cfg_mdata_i                  ( cfg_mdata_i                  ),
    .cfg_mrespaccept_i            ( cfg_mrespaccept_i            ),
    .cfg_scmdaccept_o             ( cfg_scmdaccept_o             ),
    .cfg_sdata_o                  ( cfg_sdata_o                  ),
    .cfg_sresp_o                  ( cfg_sresp_o                  ),
    .gauss_byp_h_o                ( param_gauss_byp_h_s          ),
    .gauss_byp_v_o                ( param_gauss_byp_v_s          ),
    .gauss_mode_h_o               ( param_gauss_mode_h_s         ),
    .gauss_mode_v_o               ( param_gauss_mode_v_s         ),
    .nomax_byp_o                  ( param_nomax_byp_s            ),
    .nomax_mode_o                 ( param_nomax_mode_s           ),
    .sobel_byp_o                  ( param_sobel_byp_s            ),
    .thres_o                      ( param_thres_s                )
  );

  //--------------------------------------------------------
  // instantiation of module -- RSTBUF --
  //--------------------------------------------------------
  rstbuf2 u_rstbuf2 (
    .sys_clk                      ( clk                          ),
    .sys_rst_n                    ( sys_rst_n                    ),
    .img_rst_n                    ( img_rst_n                    ),
    .sys_int_clk                  ( buf_clk_s                    ),
    .sys_int_rst_n                ( sys_rst_n_s                  ),
    .img_int_rst_n                ( img_rst_n_s                  )
  );

  //--------------------------------------------------------
  // instantiation of module -- SOBEL_B12 --
  //--------------------------------------------------------
  sobel_b12 u_sobel_b12 (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( sobel_din_evt_s              ),
    .din_fe_i                     ( sobel_din_fe_s               ),
    .din_fs_i                     ( sobel_din_fs_s               ),
    .din_le_i                     ( sobel_din_le_s               ),
    .din_ls_i                     ( sobel_din_ls_s               ),
    .din_pel00_i                  ( sobel_din_pel00_s            ),
    .din_pel01_i                  ( sobel_din_pel01_s            ),
    .din_pel02_i                  ( sobel_din_pel02_s            ),
    .din_pel10_i                  ( sobel_din_pel10_s            ),
    .din_pel11_i                  ( sobel_din_pel11_s            ),
    .din_pel12_i                  ( sobel_din_pel12_s            ),
    .din_pel20_i                  ( sobel_din_pel20_s            ),
    .din_pel21_i                  ( sobel_din_pel21_s            ),
    .din_pel22_i                  ( sobel_din_pel22_s            ),
    .dout_evt_o                   ( sobel_dout_evt_s             ),
    .dout_fe_o                    ( sobel_dout_fe_s              ),
    .dout_fs_o                    ( sobel_dout_fs_s              ),
    .dout_le_o                    ( sobel_dout_le_s              ),
    .dout_ls_o                    ( sobel_dout_ls_s              ),
    .dout_pel_o                   ( sobel_dout_pel_s             ),
    .param_sobel_byp_i            ( param_sobel_byp_s            )
  );

  //--------------------------------------------------------
  // instantiation of module -- V3B12K4 --
  //--------------------------------------------------------
  v3b12k4 u_v3b12k4 (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( gauss_v_dout_evt_s           ),
    .din_fe_i                     ( gauss_v_dout_fe_s            ),
    .din_fs_i                     ( gauss_v_dout_fs_s            ),
    .din_le_i                     ( gauss_v_dout_le_s            ),
    .din_ls_i                     ( gauss_v_dout_ls_s            ),
    .din_pel_i                    ( gauss_v_dout_pel_s           ),
    .dout_evt_o                   ( sobel_din_pre_evt_s          ),
    .dout_fe_o                    ( sobel_din_pre_fe_s           ),
    .dout_fs_o                    ( sobel_din_pre_fs_s           ),
    .dout_le_o                    ( sobel_din_pre_le_s           ),
    .dout_ls_o                    ( sobel_din_pre_ls_s           ),
    .dout_pel0_o                  ( sobel_din_pre_pel0_s         ),
    .dout_pel1_o                  ( sobel_din_pre_pel1_s         ),
    .dout_pel2_o                  ( sobel_din_pre_pel2_s         ),
    .proc_en_i                    ( proc_en_i                    )
  );

  //--------------------------------------------------------
  // instantiation of module -- V3B14K4 --
  //--------------------------------------------------------
  v3b14k4 u_v3b14k4 (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( sobel_dout_evt_s             ),
    .din_fe_i                     ( sobel_dout_fe_s              ),
    .din_fs_i                     ( sobel_dout_fs_s              ),
    .din_le_i                     ( sobel_dout_le_s              ),
    .din_ls_i                     ( sobel_dout_ls_s              ),
    .din_pel_i                    ( sobel_dout_pel_s             ),
    .dout_evt_o                   ( nomax_din_pre_evt_s          ),
    .dout_fe_o                    ( nomax_din_pre_fe_s           ),
    .dout_fs_o                    ( nomax_din_pre_fs_s           ),
    .dout_le_o                    ( nomax_din_pre_le_s           ),
    .dout_ls_o                    ( nomax_din_pre_ls_s           ),
    .dout_pel0_o                  ( nomax_din_pre_pel0_s         ),
    .dout_pel1_o                  ( nomax_din_pre_pel1_s         ),
    .dout_pel2_o                  ( nomax_din_pre_pel2_s         ),
    .proc_en_i                    ( proc_en_i                    )
  );

  //--------------------------------------------------------
  // instantiation of module -- V5B12K4 --
  //--------------------------------------------------------
  v5b12k4 u_v5b12k4 (
    .clk                          ( buf_clk_s                    ),
    .rst_n                        ( img_rst_n_s                  ),
    .din_evt_i                    ( gauss_h_dout_evt_s           ),
    .din_fe_i                     ( gauss_h_dout_fe_s            ),
    .din_fs_i                     ( gauss_h_dout_fs_s            ),
    .din_le_i                     ( gauss_h_dout_le_s            ),
    .din_ls_i                     ( gauss_h_dout_ls_s            ),
    .din_pel_i                    ( gauss_h_dout_pel_s           ),
    .dout_evt_o                   ( gauss_v_din_evt_s            ),
    .dout_fe_o                    ( gauss_v_din_fe_s             ),
    .dout_fs_o                    ( gauss_v_din_fs_s             ),
    .dout_le_o                    ( gauss_v_din_le_s             ),
    .dout_ls_o                    ( gauss_v_din_ls_s             ),
    .dout_pel0_o                  ( gauss_v_din_pel0_s           ),
    .dout_pel1_o                  ( gauss_v_din_pel1_s           ),
    .dout_pel2_o                  ( gauss_v_din_pel2_s           ),
    .dout_pel3_o                  ( gauss_v_din_pel3_s           ),
    .dout_pel4_o                  ( gauss_v_din_pel4_s           ),
    .proc_en_i                    ( proc_en_i                    )
  );

endmodule
