//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : canny_b12_regif
// Submodules : non
//
// Purpose    :   Register Interface
//                  - OCP register profile interface
//                  - 32 bit interface
//
// Creator    : ird2v.pl
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module canny_b12_regif (
    // global signals
    clk,
    rst_n,

    // synchron reset
    init_i,

    // config bus
    cfg_maddr_i,
    cfg_mcmd_i,
    cfg_mdata_i,
    cfg_mrespaccept_i,
    cfg_scmdaccept_o,
    cfg_sdata_o,
    cfg_sresp_o,

    // parameter interface
    gauss_byp_h_o,
    gauss_byp_v_o,
    gauss_mode_h_o,
    gauss_mode_v_o,
    nomax_byp_o,
    nomax_mode_o,
    sobel_byp_o,
    thres_o
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // global signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           rst_n;                // asynchron low active reset

  //--------------------------------------------------------
  // synchron reset
  //--------------------------------------------------------
  input           init_i;               // synchron high active

  //--------------------------------------------------------
  // config bus
  //--------------------------------------------------------
  input  [16:2]   cfg_maddr_i;          // register address
  input  [1:0]    cfg_mcmd_i;           // 00:IDEL
                                        // 01:WR
                                        // 10:RD
                                        // 11:not implemented
  input  [31:0]   cfg_mdata_i;          // write data
  input           cfg_mrespaccept_i;    // read data acknowledge
  output          cfg_scmdaccept_o;     // command acknowledge
  output [31:0]   cfg_sdata_o;          // read data
  output          cfg_sresp_o;          // 0:NULL, 1:DVA

  //--------------------------------------------------------
  // parameter interface
  //--------------------------------------------------------
  output          gauss_byp_h_o;        // bypass horizontal gaussian
  output          gauss_byp_v_o;        // bypass vertical gaussian
  output          gauss_mode_h_o;       // 0: 3tap
                                        // 1: 5tab
  output          gauss_mode_v_o;       // 0: 3tap
                                        // 1: 5tab
  output          nomax_byp_o;          // bypass no max elimination
  output          nomax_mode_o;         // 0: just edge
                                        // 1: edge with weight
  output          sobel_byp_o;          // bypass sobel
  output [11:0]   thres_o;              // edge threshold

  //-----------------------------------------------------
  // Signals -- address decoder --
  //-----------------------------------------------------
  // combinatorial
  reg             ad_canny_id_sel;      // address selector 0
  reg             ad_canny_id_evt;      // related access event
  reg             ad_canny_cfg_sel;     // address selector 1
  reg             ad_canny_cfg_evt;     // related access event
  reg             ad_canny_thres_sel;   // address selector 2
  reg             ad_canny_thres_evt;   // related access event

  //-----------------------------------------------------
  // Signals -- register buffer --
  //-----------------------------------------------------
  // FF
  reg             nomax_mode_o;         // field register
  reg             gauss_mode_v_o;       // field register
  reg             gauss_mode_h_o;       // field register
  reg             nomax_byp_o;          // field register
  reg             sobel_byp_o;          // field register
  reg             gauss_byp_v_o;        // field register
  reg             gauss_byp_h_o;        // field register
  reg    [11:0]   thres_o;              // field register

  //-----------------------------------------------------
  // Signals -- read data generation --
  //-----------------------------------------------------
  // combinatorial
  wire   [31:0]   obi_sdata;
  wire   [31:0]   rdg_canny_id;
  wire   [31:0]   rdg_canny_cfg;
  wire   [31:0]   rdg_canny_thres;

  //-----------------------------------------------------
  // Signals -- FHS decouble buffer ibo=FWBW(ibi) --
  //-----------------------------------------------------
  // FF
  reg    [14:0]   ibo_fwbwint_maddr;
  reg    [31:0]   ibo_fwbwint_mdata;
  reg             ibo_fwbwint_rw;
  reg    [14:0]   ibo_fwbwbuf_maddr;
  reg    [31:0]   ibo_fwbwbuf_mdata;
  reg             ibo_fwbwbuf_rw;
  reg             ibo_fwbwint_val;
  reg             ibo_fwbwint_ack;      // buffered ack
  reg             ibo_fwbwbuf_val;
  // combinatorial
  wire   [14:0]   ibo_maddr;
  wire   [31:0]   ibo_mdata;
  wire            ibo_rw;
  wire            ibi_ack;
  wire            ibo_val;

  //-----------------------------------------------------
  // Signals -- input buffer --
  //-----------------------------------------------------
  // combinatorial
  wire            ibi_val;
  wire            ibi_rw;               // 0:read, 1:write
  wire   [14:0]   ibi_maddr;
  wire   [31:0]   ibi_mdata;

  //-----------------------------------------------------
  // Signals -- FHS forward buffer obo=FW(obi) --
  //-----------------------------------------------------
  // FF
  reg    [31:0]   obo_sdata;
  reg             obo_val;
  // combinatorial
  wire            obi_ack;

  //-----------------------------------------------------
  // Signals -- output buffer --
  //-----------------------------------------------------
  // combinatorial
  wire            obo_ack;

  //-----------------------------------------------------
  // Signals -- ack generation --
  //-----------------------------------------------------
  // combinatorial
  wire            ag_read_val;
  wire            ag_read_ack;
  wire            ag_write_val;
  wire            ag_write_ack;
  wire            ag_write_evt;
  wire            obi_val;
  wire            ibo_ack;

  //-----------------------------------------------------
  // Async Process -- address decoder --
  //-----------------------------------------------------
  always @(*) begin
    ad_canny_id_sel    = (ibo_maddr==15'h0);
    ad_canny_cfg_sel   = (ibo_maddr==15'h1);
    ad_canny_thres_sel = (ibo_maddr==15'h2);
    ad_canny_id_evt    = ag_write_evt & ad_canny_id_sel;
    ad_canny_cfg_evt   = ag_write_evt & ad_canny_cfg_sel;
    ad_canny_thres_evt = ag_write_evt & ad_canny_thres_sel;
  end
  //-----------------------------------------------------
  // Sync Process  -- register buffer --
  //      initialized
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      nomax_mode_o   <= 1'b0;
      gauss_mode_v_o <= 1'b1;
      gauss_mode_h_o <= 1'b1;
      nomax_byp_o    <= 1'b0;
      sobel_byp_o    <= 1'b0;
      gauss_byp_v_o  <= 1'b0;
      gauss_byp_h_o  <= 1'b0;
      thres_o        <= 12'b000100000000;
    end
    else if (init_i) begin
      nomax_mode_o   <= 1'b0;
      gauss_mode_v_o <= 1'b1;
      gauss_mode_h_o <= 1'b1;
      nomax_byp_o    <= 1'b0;
      sobel_byp_o    <= 1'b0;
      gauss_byp_v_o  <= 1'b0;
      gauss_byp_h_o  <= 1'b0;
      thres_o        <= 12'b000100000000;
    end
    else begin
      //--------------------------------------
      // canny_cfg
      //--------------------------------------
      if (ad_canny_cfg_evt) nomax_mode_o   <= ibo_mdata[6];
      if (ad_canny_cfg_evt) gauss_mode_v_o <= ibo_mdata[5];
      if (ad_canny_cfg_evt) gauss_mode_h_o <= ibo_mdata[4];
      if (ad_canny_cfg_evt) nomax_byp_o    <= ibo_mdata[3];
      if (ad_canny_cfg_evt) sobel_byp_o    <= ibo_mdata[2];
      if (ad_canny_cfg_evt) gauss_byp_v_o  <= ibo_mdata[1];
      if (ad_canny_cfg_evt) gauss_byp_h_o  <= ibo_mdata[0];
      //--------------------------------------
      // canny_thres
      //--------------------------------------
      if (ad_canny_thres_evt) thres_o <= ibo_mdata[11:0];
    end
  end
  //--------------------------------------------------------
  // Combinatorial -- read data generation --
  //--------------------------------------------------------
  assign obi_sdata
    = ( rdg_canny_id    & {32{ad_canny_id_sel}}    )
    | ( rdg_canny_cfg   & {32{ad_canny_cfg_sel}}   )
    | ( rdg_canny_thres & {32{ad_canny_thres_sel}} );
  assign rdg_canny_id    = {
    32'h636e6e79
  };
  assign rdg_canny_cfg   = {
    25'b0000000000000000000000000,
    nomax_mode_o,
    gauss_mode_v_o,
    gauss_mode_h_o,
    nomax_byp_o,
    sobel_byp_o,
    gauss_byp_v_o,
    gauss_byp_h_o
  };
  assign rdg_canny_thres = {
    20'h00000,
    thres_o
  };
  //-----------------------------------------------------
  // Sync Process  -- FHS decouble buffer ibo=FWBW(ibi) --
  //      input puffer
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      ibo_fwbwint_val <= 1'b0;
      ibo_fwbwint_ack <= 1'b0;
      ibo_fwbwbuf_val <= 1'b0;
    end
    else begin
      if (ibi_ack) ibo_fwbwint_val <= ibi_val;
      ibo_fwbwbuf_val <= ibo_val;
      ibo_fwbwint_ack <= !ibo_val | ibo_ack;
    end
  end
  //-----------------------------------------------------
  // Sync Process  -- FHS decouble buffer ibo=FWBW(ibi) --
  //      data path
  //-----------------------------------------------------
  always @(posedge clk) begin
    if (ibi_ack) begin
      ibo_fwbwint_maddr <= ibi_maddr;
      ibo_fwbwint_mdata <= ibi_mdata;
      ibo_fwbwint_rw    <= ibi_rw;
    end
    ibo_fwbwbuf_maddr <= ibo_maddr;
    ibo_fwbwbuf_mdata <= ibo_mdata;
    ibo_fwbwbuf_rw    <= ibo_rw;
  end

  // bypass mux
  assign ibo_val   = ibo_fwbwint_ack ? ibo_fwbwint_val : ibo_fwbwbuf_val;
  assign ibo_maddr = ibo_fwbwint_ack ? ibo_fwbwint_maddr : ibo_fwbwbuf_maddr;
  assign ibo_mdata = ibo_fwbwint_ack ? ibo_fwbwint_mdata : ibo_fwbwbuf_mdata;
  assign ibo_rw    = ibo_fwbwint_ack ? ibo_fwbwint_rw : ibo_fwbwbuf_rw;

  // acknowledge generation FW
  assign  ibi_ack = !ibo_fwbwint_val | ibo_fwbwint_ack;

  //-----------------------------------------------------
  // Assignment  -- input buffer --
  //-----------------------------------------------------
  assign ibi_val          = (cfg_mcmd_i!=2'b00);
  assign ibi_rw           = (cfg_mcmd_i==2'b01);
  assign ibi_maddr        = cfg_maddr_i;
  assign ibi_mdata        = cfg_mdata_i;
  assign cfg_scmdaccept_o = ibi_ack;
  //-----------------------------------------------------
  // Sync Process  -- FHS forward buffer obo=FW(obi) --
  //      output puffer
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      obo_val <= 1'b0;
    end
    else begin
      if (obi_ack) obo_val <= obi_val;
    end
  end
  //-----------------------------------------------------
  // Sync Process  -- FHS forward buffer obo=FW(obi) --
  //      data path
  //-----------------------------------------------------
  always @(posedge clk) begin
    if (obi_ack & obi_val) begin
      obo_sdata <= obi_sdata;
    end
  end

  // combinatorial acknowledge generation
  assign obi_ack = !obo_val | obo_ack;

  //-----------------------------------------------------
  // Assignment  -- output buffer --
  //-----------------------------------------------------
  assign cfg_sresp_o = obo_val;
  assign obo_ack     = cfg_mrespaccept_i;
  assign cfg_sdata_o = obo_sdata;
  //-----------------------------------------------------
  // Combinatorial -- ack generation --
  //-----------------------------------------------------
  assign ag_read_val  = ibo_val & !ibo_rw;
  assign ag_write_val = ibo_val & ibo_rw;
  assign obi_val      = ag_read_val;
  assign ag_read_ack  = (obi_val & obi_ack);
  assign ag_write_ack = ag_write_val;
  assign ibo_ack      = ag_read_ack | ag_write_ack;
  assign ag_write_evt = ag_write_val;               // each cycle an write

endmodule
