//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : cmif2sif
// Submodules : cmif2sif_rstbuf
//              lvpp2pic
//              lvsb2ocp
//              pic2lvpp
//
// Purpose    : Custom Module to Sensor Interface
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module cmif2sif (
    // system signals
    clk,
    sys_rst_n,
    img_rst_n,

    // Module Infrastructure Signals
    ImgRstxRI,
    SysClkxCI,
    SysRstxRI,

    // config OCP
    cfg_maddr_o,
    cfg_mcmd_o,
    cfg_mdata_o,
    cfg_mrespaccept_o,
    cfg_scmdaccept_i,
    cfg_sdata_i,
    cfg_sresp_i,

    // ImageBus Receive Port
    ImgRxPixDataxDI,
    ImgRxPixRdyxMO,
    ImgRxPixStatxSI,
    ImgRxPixValxMI,

    // ImageBus Transmit Port
    ImgTxPixDataxDO,
    ImgTxPixRdyxMI,
    ImgTxPixStatxSO,
    ImgTxPixValxMO,

    // SystemBus Control Interface
    SysAckxMO,
    SysAddrxAI,
    SysErrxMO,
    SysRdDataxDO,
    SysReqxMI,
    SysWrDataxDI,
    SysWrEnxMI,

    // data input
    din_b_i,
    din_evt_i,
    din_fe_i,
    din_fs_i,
    din_g_i,
    din_le_i,
    din_ls_i,
    din_r_i,

    // data output
    dout_b_o,
    dout_evt_o,
    dout_fe_o,
    dout_fs_o,
    dout_g_o,
    dout_le_o,
    dout_ls_o,
    dout_r_o
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  output          clk;                  // system clock
  output          sys_rst_n;            // asynchronous system reset, active low
  output          img_rst_n;            // asynchronous image bus reset, active low

  //--------------------------------------------------------
  // Module Infrastructure Signals
  //--------------------------------------------------------
  input           ImgRstxRI;            // ImagePipeline Reset
  input           SysClkxCI;            // Custom Module Clock
  input           SysRstxRI;            // Custom Module Reset

  //--------------------------------------------------------
  // config OCP
  //--------------------------------------------------------
  output [16:2]   cfg_maddr_o;          // address
  output [1:0]    cfg_mcmd_o;           // command, 0:NOP, 1:write, 2:read
  output [31:0]   cfg_mdata_o;          // write data
  output          cfg_mrespaccept_o;    // read response accept
  input           cfg_scmdaccept_i;     // command accept
  input  [31:0]   cfg_sdata_i;          // read data
  input           cfg_sresp_i;          // read response

  //--------------------------------------------------------
  // ImageBus Receive Port
  //--------------------------------------------------------
  input  [35:0]   ImgRxPixDataxDI;      // packed data
  output          ImgRxPixRdyxMO;       // receive Pixel Ready - ignore if
                                        // cIMG_HANDSHAKE_EN = False
  input  [2:0]    ImgRxPixStatxSI;      // 000 No-Status (NONE)
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-signle-Line-Frame
                                        // (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)
  input           ImgRxPixValxMI;       // transaction valid

  //--------------------------------------------------------
  // ImageBus Transmit Port
  //--------------------------------------------------------
  output [35:0]   ImgTxPixDataxDO;      // packed data
  input           ImgTxPixRdyxMI;       // ImageBus Transmit Pixel Ready -
                                        // ignore if cIMG_HANDSHAKE_EN = False
  output [2:0]    ImgTxPixStatxSO;      // 000 No-Status (NONE)
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-single-Line-Frame
                                        // (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)
  output          ImgTxPixValxMO;       // transaction valid

  //--------------------------------------------------------
  // SystemBus Control Interface
  //--------------------------------------------------------
  output          SysAckxMO;            // indicates successful cycle
                                        // termination
  input  [16:0]   SysAddrxAI;           // address input array
  output          SysErrxMO;            // indicates abnormal cycle termination
  output [31:0]   SysRdDataxDO;         // read data
  input           SysReqxMI;            // cycle input indicating a valid bus
                                        // cycle
  input  [31:0]   SysWrDataxDI;         // write data
  input           SysWrEnxMI;           // write enable

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input  [11:0]   din_b_i;              // blue
  input           din_evt_i;            // transaction event, 1cc pulse
  input           din_fe_i;             // frame end
  input           din_fs_i;             // frame start
  input  [11:0]   din_g_i;              // green
  input           din_le_i;             // line end
  input           din_ls_i;             // line start
  input  [11:0]   din_r_i;              // red

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output [11:0]   dout_b_o;             // blue
  output          dout_evt_o;           // transaction event, 1cc pulse
  output          dout_fe_o;            // frame end
  output          dout_fs_o;            // frame start
  output [11:0]   dout_g_o;             // green
  output          dout_le_o;            // line end
  output          dout_ls_o;            // line start
  output [11:0]   dout_r_o;             // red


  assign ImgRxPixRdyxMO = 1'd1; // unused

  //--------------------------------------------------------
  // instantiation of module -- LVPP2PIC --
  //--------------------------------------------------------
  lvpp2pic u_lvpp2pic (
    .clk                         ( clk                         ),
    .rst_n                       ( img_rst_n                   ),
    .din_data_i                  ( ImgRxPixDataxDI             ),
    .din_evt_i                   ( ImgRxPixValxMI              ),
    .din_stat_i                  ( ImgRxPixStatxSI             ),
    .dout_b_o                    ( dout_b_o                    ),
    .dout_evt_o                  ( dout_evt_o                  ),
    .dout_fe_o                   ( dout_fe_o                   ),
    .dout_fs_o                   ( dout_fs_o                   ),
    .dout_g_o                    ( dout_g_o                    ),
    .dout_le_o                   ( dout_le_o                   ),
    .dout_ls_o                   ( dout_ls_o                   ),
    .dout_r_o                    ( dout_r_o                    )
  );

  //--------------------------------------------------------
  // instantiation of module -- LVSB2OCP --
  //--------------------------------------------------------
  lvsb2ocp u_lvsb2ocp (
    .clk                         ( clk                         ),
    .rst_n                       ( sys_rst_n                   ),
    .din_ack_o                   ( SysAckxMO                   ),
    .din_addr_i                  ( SysAddrxAI                  ),
    .din_err_o                   ( SysErrxMO                   ),
    .din_rdat_o                  ( SysRdDataxDO                ),
    .din_req_i                   ( SysReqxMI                   ),
    .din_wdat_i                  ( SysWrDataxDI                ),
    .din_we_i                    ( SysWrEnxMI                  ),
    .dout_maddr_o                ( cfg_maddr_o                 ),
    .dout_mcmd_o                 ( cfg_mcmd_o                  ),
    .dout_mdata_o                ( cfg_mdata_o                 ),
    .dout_mrespaccept_o          ( cfg_mrespaccept_o           ),
    .dout_scmdaccept_i           ( cfg_scmdaccept_i            ),
    .dout_sdata_i                ( cfg_sdata_i                 ),
    .dout_sresp_i                ( cfg_sresp_i                 )
  );

  //--------------------------------------------------------
  // instantiation of module -- PIC2LVPP --
  //--------------------------------------------------------
  pic2lvpp u_pic2lvpp (
    .clk                         ( clk                         ),
    .rst_n                       ( img_rst_n                   ),
    .din_b_i                     ( din_b_i                     ),
    .din_evt_i                   ( din_evt_i                   ),
    .din_fe_i                    ( din_fe_i                    ),
    .din_fs_i                    ( din_fs_i                    ),
    .din_g_i                     ( din_g_i                     ),
    .din_le_i                    ( din_le_i                    ),
    .din_ls_i                    ( din_ls_i                    ),
    .din_r_i                     ( din_r_i                     ),
    .dout_data_o                 ( ImgTxPixDataxDO             ),
    .dout_evt_o                  ( ImgTxPixValxMO              ),
    .dout_stat_o                 ( ImgTxPixStatxSO             )
  );

  //--------------------------------------------------------
  // instantiation of module -- RSTBUF --
  //--------------------------------------------------------
  cmif2sif_rstbuf u_cmif2sif_rstbuf (
    .ImgRstxRI                   ( ImgRstxRI                   ),
    .SysClkxCI                   ( SysClkxCI                   ),
    .SysRstxRI                   ( SysRstxRI                   ),
    .sys_clk                     ( clk                         ),
    .sys_rst_n                   ( sys_rst_n                   ),
    .img_rst_n                   ( img_rst_n                   )    
  );

endmodule
