//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : gauss_b12
// Submodules : non
//
// Purpose    : Gaussian filter
//
//               - selectable bypass / 3tab / 5tab
//               - 3tab: 1,2,1
//               - 5tab: 2,4,5,4,2
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module gauss_b12 (
    // system signals
    clk,
    rst_n,

    // data input
    din_evt_i,
    din_fe_i,
    din_fs_i,
    din_le_i,
    din_ls_i,
    din_pel0_i,
    din_pel1_i,
    din_pel2_i,
    din_pel3_i,
    din_pel4_i,

    // data output
    dout_evt_o,
    dout_fe_o,
    dout_fs_o,
    dout_le_o,
    dout_ls_o,
    dout_pel_o,

    // parameter
    param_gauss_byp_i,
    param_gauss_mode_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           rst_n;                // asynchron low active reset

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input           din_evt_i;            // transaction event
  input           din_fe_i;             // frame end
  input           din_fs_i;             // frame start
  input           din_le_i;             // line end
  input           din_ls_i;             // line start
  input  [11:0]   din_pel0_i;           // pel 0
  input  [11:0]   din_pel1_i;           // pel 1
  input  [11:0]   din_pel2_i;           // pel 2
  input  [11:0]   din_pel3_i;           // pel 3
  input  [11:0]   din_pel4_i;           // pel 4

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output          dout_evt_o;           // transaction event
  output          dout_fe_o;            // frame end
  output          dout_fs_o;            // frame start
  output          dout_le_o;            // line end
  output          dout_ls_o;            // line start
  output [11:0]   dout_pel_o;           // pel

  //--------------------------------------------------------
  // parameter
  //--------------------------------------------------------
  input           param_gauss_byp_i;    // bypass
  input           param_gauss_mode_i;   // 0: 3tab
                                        // 1: 5tab

  //-----------------------------------------------------
  // Signals -- race buffer dout=buf(din) --
  //-----------------------------------------------------
  // FF
  reg             dout_d0_ls;
  reg             dout_d0_le;
  reg             dout_d0_fs;
  reg             dout_d0_fe;
  reg             dout_d0_evt;
  reg             dout_d1_ls;
  reg             dout_d1_le;
  reg             dout_d1_fs;
  reg             dout_d1_fe;
  reg             dout_d1_evt;
  reg             dout_ls_o;
  reg             dout_le_o;
  reg             dout_fs_o;
  reg             dout_fe_o;
  reg             dout_evt_o;


  //-----------------------------------------------------
  // Signals -- gauss --
  //-----------------------------------------------------
  // FF
  reg    [12:0]   g_d0_a0;
  reg    [12:0]   g_d0_a1;
  reg    [15:0]   g_d0_a2;
  reg    [11:0]   g_d0_m;
  reg    [16:3]   g_d1_g5;
  reg    [14:1]   g_d1_g3;
  reg    [11:0]   g_d1_m;
  reg    [11:0]   dout_pel_o;
  // combinatorial
  reg    [16:7]   g_d1_g5s;
  reg    [15:3]   g_d1_g5n;

  //-----------------------------------------------------
  // Sync Process  -- gauss --
  //-----------------------------------------------------
  always @(posedge clk) begin
    g_d0_a0 <= {1'd0, din_pel0_i}       + {1'd0, din_pel4_i};
    g_d0_a1 <= {1'd0, din_pel1_i}       + {1'd0, din_pel3_i};
    g_d0_a2 <= {1'd0, din_pel2_i, 2'd0} + {3'd0, din_pel2_i};
    g_d0_m  <= din_pel2_i;

    g_d1_g5 <= {1'd0, g_d0_a2[15:3] }
             + {2'd0, g_d0_a1[12:1] }
             + {3'd0, g_d0_a0[12:2] };

    g_d1_g3 <= {1'd0, g_d0_m }
             + {1'd0, g_d0_a1[12:1] };
    g_d1_m  <= g_d0_m;

    case({param_gauss_byp_i, param_gauss_mode_i})
      2'd0:    dout_pel_o <= g_d1_g3[13:2];
      2'd1:    dout_pel_o <= g_d1_g5n[15:4];
      default: dout_pel_o <= g_d1_m;
    endcase
  end

  //-----------------------------------------------------
  // Async Process -- gauss --
  //-----------------------------------------------------
  always @(*) begin
    g_d1_g5s = g_d1_g5[16:7] - {4'd0, g_d1_g5[16:11]};
    g_d1_g5n = g_d1_g5[15:3] - {3'd0, g_d1_g5s[16:7]};
  end

  //-----------------------------------------------------
  // Sync Process  -- race buffer dout=buf(din) --
  //      handshake generation
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      dout_d0_evt <= 1'b0;
      dout_d1_evt <= 1'b0;
      dout_evt_o  <= 1'b0;
    end
    else begin
      dout_d0_evt <= din_evt_i;
      dout_d1_evt <= dout_d0_evt;
      dout_evt_o  <= dout_d1_evt;
    end
  end
  //-----------------------------------------------------
  // Sync Process  -- race buffer dout=buf(din) --
  //      data path
  //-----------------------------------------------------
  always @(posedge clk) begin
    dout_d0_ls <= din_ls_i;
    dout_d0_le <= din_le_i;
    dout_d0_fs <= din_fs_i;
    dout_d0_fe <= din_fe_i;
    dout_d1_ls <= dout_d0_ls;
    dout_d1_le <= dout_d0_le;
    dout_d1_fs <= dout_d0_fs;
    dout_d1_fe <= dout_d0_fe;
    dout_ls_o  <= dout_d1_ls;
    dout_le_o  <= dout_d1_le;
    dout_fs_o  <= dout_d1_fs;
    dout_fe_o  <= dout_d1_fe;
  end
endmodule
