//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : h3b12x3
// Submodules : non
//
// Purpose    : 1D horizontal 5pel 12 bit converter
//
//               - fully back to back capable
//               - 3 pel min frame width
//               - 2cc delay
//
// Creator    : nuessi
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module h3b12x3 (
    // system signals
    clk,
    rst_n,

    // data input
    din_evt_i,
    din_fe_i,
    din_fs_i,
    din_le_i,
    din_ls_i,
    din_pel0_i,
    din_pel1_i,
    din_pel2_i,

    // data output
    dout_evt_o,
    dout_fe_o,
    dout_fs_o,
    dout_le_o,
    dout_ls_o,
    dout_pel00_o,
    dout_pel01_o,
    dout_pel02_o,
    dout_pel10_o,
    dout_pel11_o,
    dout_pel12_o,
    dout_pel20_o,
    dout_pel21_o,
    dout_pel22_o
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           rst_n;                // asynchron low active reset

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input           din_evt_i;            // transaction event
  input           din_fe_i;             // frame end
  input           din_fs_i;             // frame start
  input           din_le_i;             // line end
  input           din_ls_i;             // line start
  input  [11:0]   din_pel0_i;           // one pel y=-1
  input  [11:0]   din_pel1_i;           // one pel y=0
  input  [11:0]   din_pel2_i;           // one pel y=1

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output          dout_evt_o;           // transaction event
  output          dout_fe_o;            // frame end
  output          dout_fs_o;            // frame start
  output          dout_le_o;            // line end
  output          dout_ls_o;            // line start
  output [11:0]   dout_pel00_o;         // one pel -1,-1
  output [11:0]   dout_pel01_o;         // one pel  0,-1
  output [11:0]   dout_pel02_o;         // one pel  1,-1
  output [11:0]   dout_pel10_o;         // one pel -1, 0
  output [11:0]   dout_pel11_o;         // one pel  0, 0
  output [11:0]   dout_pel12_o;         // one pel  1, 0
  output [11:0]   dout_pel20_o;         // one pel -1, 1
  output [11:0]   dout_pel21_o;         // one pel  0, 1
  output [11:0]   dout_pel22_o;         // one pel  1, 1


  //-----------------------------------------------------
  // Signals -- converter --
  //-----------------------------------------------------
  // FF
  reg             dout_evt_o;
  reg             c_s0_le;
  reg    [11:0]   c_s0_pel0, c_s0_pel1, c_s0_pel2;
  reg             c_s0_ls;
  reg             c_fs, c_s0_fe, c_s1_fe;
  reg    [11:0]   dout_pel00_o, dout_pel10_o, dout_pel20_o;
  reg    [11:0]   dout_pel01_o, dout_pel11_o, dout_pel21_o;
  reg    [11:0]   dout_pel02_o, dout_pel12_o, dout_pel22_o;
  reg             dout_ls_o;
  reg             dout_fs_o;
  reg             dout_le_o;
  reg             dout_fe_o;
  // combinatorial
  reg             c_s0_upd0_evt;
  reg             c_s0_dout_evt;
  reg             c_s0_load_evt;

  //-----------------------------------------------------
  // Sync Process  -- converter --
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      dout_evt_o <= 1'd0;
      c_s0_le    <= 1'd0;
    end
    else begin
      dout_evt_o <= c_s0_dout_evt;
      if (din_evt_i & din_le_i) begin
        c_s0_le  <= 1'd1;
      end
      else begin
        c_s0_le  <= 1'd0;
      end
    end
  end

  //-----------------------------------------------------
  // Sync Process  -- converter --
  //-----------------------------------------------------
  always @(posedge clk) begin
    if (din_evt_i) begin
      c_s0_pel0 <= din_pel0_i;
      c_s0_pel1 <= din_pel1_i;
      c_s0_pel2 <= din_pel2_i;
      c_s0_ls   <= din_ls_i;
    end

    if      (din_fs_i & din_ls_i & din_evt_i) c_fs <= 1'd1;
    else if (c_s0_load_evt)                   c_fs <= 1'd0;

    c_s0_fe <= din_fe_i & din_le_i;

    if (c_s0_load_evt) begin
      dout_pel00_o <= c_s0_pel0;
      dout_pel01_o <= c_s0_pel1;
      dout_pel02_o <= c_s0_pel2;
      dout_pel10_o <= c_s0_pel0;
      dout_pel11_o <= c_s0_pel1;
      dout_pel12_o <= c_s0_pel2;
      dout_ls_o    <= 1'd1;
      dout_fs_o    <= c_fs;
    end
    else if (c_s0_dout_evt) begin
      dout_pel00_o <= dout_pel10_o;
      dout_pel01_o <= dout_pel11_o;
      dout_pel02_o <= dout_pel12_o;
      dout_pel10_o <= dout_pel20_o;
      dout_pel11_o <= dout_pel21_o;
      dout_pel12_o <= dout_pel22_o;
      dout_ls_o    <= 1'd0;
      dout_fs_o    <= 1'd0;
    end
    dout_le_o <= c_s0_le;
    dout_fe_o <= c_s0_fe;

    if (c_s0_upd0_evt) begin
      dout_pel20_o <= din_pel0_i;
      dout_pel21_o <= din_pel1_i;
      dout_pel22_o <= din_pel2_i;
    end
  end

  //-----------------------------------------------------
  // Async Process -- converter --
  //-----------------------------------------------------
  always @(*) begin
    c_s0_upd0_evt = din_evt_i & !din_ls_i;
    c_s0_dout_evt = c_s0_upd0_evt | c_s0_le;
    c_s0_load_evt = din_evt_i & c_s0_ls;
  end

endmodule
