//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : lvpp2pic
// Submodules : non
//
// Purpose    : LVPP to PIC
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module lvpp2pic (
    // system signals
    clk,
    rst_n,

    // data input
    din_data_i,
    din_evt_i,
    din_stat_i,

    // data output
    dout_b_o,
    dout_evt_o,
    dout_fe_o,
    dout_fs_o,
    dout_g_o,
    dout_le_o,
    dout_ls_o,
    dout_r_o
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           rst_n;                // asynchron low active reset

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input  [35:0]   din_data_i;           // packed data
  input           din_evt_i;            // transaction valid
  input  [2:0]    din_stat_i;           // 000 No-Status (NONE)
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-signle-Line-Frame
                                        // (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output [11:0]   dout_b_o;             // blue
  output          dout_evt_o;           // transaction event, 1cc pulse
  output          dout_fe_o;            // frame end
  output          dout_fs_o;            // frame start
  output [11:0]   dout_g_o;             // green
  output          dout_le_o;            // line end
  output          dout_ls_o;            // line start
  output [11:0]   dout_r_o;             // red


  localparam NOP  = 3'd0,
             SOF  = 3'd1,
             SOSLF= 3'd2,
             SOD  = 3'd3,
             SOL  = 3'd4,
             SOLL = 3'd5,
             EOL  = 3'd6,
             ERR  = 3'd7;

  //-----------------------------------------------------
  // Signals -- buf --
  //-----------------------------------------------------
  // FF
  reg             dout_evt_o;
  reg             b_skip;
  reg             dout_fs_o, dout_fe_o, dout_ls_o, dout_le_o;
  reg    [11:0]   dout_r_o, dout_g_o, dout_b_o;
  // combinatorial
  reg             b_evt;
  reg             b_soslf;
  reg             b_sof;
  reg             b_err;
  reg             b_soll;
  reg             b_eol;
  reg             b_start;

  //-----------------------------------------------------
  // Sync Process  -- buf --
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      dout_evt_o <= 1'd0;
      b_skip     <= 1'd0;
    end
    else begin
      dout_evt_o <= b_evt;
      if      (b_err) b_skip <= 1'd1;
      else if (b_sof) b_skip <= 1'd0;
    end
  end

  //-----------------------------------------------------
  // Sync Process  -- buf --
  //-----------------------------------------------------
  always @(posedge clk) begin
    if (b_evt) begin
      if      (b_sof)   dout_fs_o <= 1'd1;
      else if (b_start) dout_fs_o <= 1'd0;

      if      (b_soll)  dout_fe_o <= 1'd1;
      else if (b_start) dout_fe_o <= 1'd0;

      dout_ls_o <= b_start;
      dout_le_o <= b_eol;

      dout_r_o <= din_data_i[35:24];
      dout_g_o <= din_data_i[23:12];
      dout_b_o <= din_data_i[11: 0];
    end
  end

  //-----------------------------------------------------
  // Async Process -- buf --
  //-----------------------------------------------------
  always @(*) begin
    b_soslf = (din_stat_i == SOSLF);
    b_sof   = (din_stat_i == SOF)
            | b_soslf
            ;
    b_err   = (din_stat_i == ERR);
    b_soll  = (din_stat_i == SOLL)
            | b_soslf
            ;
    b_eol   = (din_stat_i == EOL);
    b_start = (din_stat_i == SOL)
            | b_soll
            | b_sof
            ;

    b_evt   = din_evt_i & (!b_skip || b_sof) & !b_err;
  end

endmodule
