//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : lvsb2ocp
// Submodules : non
//
// Purpose    : LVSB to OCP converter
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module lvsb2ocp (
    // system signals
    clk,
    rst_n,

    // data input
    din_ack_o,
    din_addr_i,
    din_err_o,
    din_rdat_o,
    din_req_i,
    din_wdat_i,
    din_we_i,

    // data output
    dout_maddr_o,
    dout_mcmd_o,
    dout_mdata_o,
    dout_mrespaccept_o,
    dout_scmdaccept_i,
    dout_sdata_i,
    dout_sresp_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           rst_n;                // asynchron low active reset

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  output          din_ack_o;            // indicates successful cycle
                                        // termination
  input  [16:0]   din_addr_i;           // address input array
  output          din_err_o;            // indicates abnormal cycle termination
  output [31:0]   din_rdat_o;           // read data
  input           din_req_i;            // cycle input indicating a valid bus
                                        // cycle
  input  [31:0]   din_wdat_i;           // write data
  input           din_we_i;             // write enable

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output [16:2]   dout_maddr_o;         // address
  output [1:0]    dout_mcmd_o;          // command, 0:NOP, 1:write, 2:read
  output [31:0]   dout_mdata_o;         // write data
  output          dout_mrespaccept_o;   // read response accept
  input           dout_scmdaccept_i;    // command accept
  input  [31:0]   dout_sdata_i;         // read data
  input           dout_sresp_i;         // read response


  assign din_err_o          = 1'd0; // unused
  assign dout_mrespaccept_o = 1'd1; // unused

  //-----------------------------------------------------
  // Signals -- converter --
  //-----------------------------------------------------
  // FF
  reg             c_rreq_wait;
  reg             c_rres_wait;
  reg    [31:0]   c_sdata;
  // combinatorial
  reg    [1:0]    dout_mcmd_o;
  reg    [16:2]   dout_maddr_o;
  reg    [31:0]   dout_mdata_o;
  reg    [31:0]   din_rdat_o;
  reg             din_ack_o;

  //-----------------------------------------------------
  // Sync Process  -- converter --
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      c_rres_wait <= 1'd0;
      c_rreq_wait <= 1'd0;
    end
    else begin
      if      (din_ack_o)                   c_rres_wait <= 1'd0;
      else if (dout_sresp_i & !c_rreq_wait) c_rres_wait <= 1'd1;

      if      (dout_sresp_i) begin
        c_rreq_wait <= 1'd0;
      end
      else if (dout_mcmd_o[1] & dout_scmdaccept_i & !c_rres_wait) begin
        c_rreq_wait <= 1'd1;
      end
    end
  end

  //-----------------------------------------------------
  // Sync Process  -- converter --
  //-----------------------------------------------------
  always @(posedge clk) begin
    if (dout_sresp_i) c_sdata <= dout_sdata_i;
  end

  //-----------------------------------------------------
  // Async Process -- converter --
  //-----------------------------------------------------
  always @(*) begin
    dout_mcmd_o  = {din_req_i & !din_we_i & !c_rreq_wait, din_req_i & din_we_i};
    dout_maddr_o = din_addr_i[16:2];
    dout_mdata_o = din_wdat_i;
    din_rdat_o   = dout_sresp_i ? dout_sdata_i : c_sdata;
    din_ack_o    = (din_req_i & din_we_i & dout_scmdaccept_i)
                 | (c_rreq_wait & dout_sresp_i )
                 | (c_rres_wait & dout_scmdaccept_i )
                 | (dout_sresp_i & dout_scmdaccept_i )
                 ;
  end


endmodule
