//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : nomax_b12
// Submodules : non
//
// Purpose    : No Maximum Eimination
//
//                - 2cc delay
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module nomax_b12 (
    // system signals
    clk,
    rst_n,

    // data input
    din_evt_i,
    din_fe_i,
    din_fs_i,
    din_le_i,
    din_ls_i,
    din_pel00_i,
    din_pel01_i,
    din_pel02_i,
    din_pel10_i,
    din_pel11_i,
    din_pel12_i,
    din_pel20_i,
    din_pel21_i,
    din_pel22_i,

    // data output
    dout_evt_o,
    dout_fe_o,
    dout_fs_o,
    dout_le_o,
    dout_ls_o,
    dout_pel_o,

    // parameter
    param_nomax_byp_i,
    param_nomax_mode_i,
    param_thres_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           rst_n;                // asynchron low active reset

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input           din_evt_i;            // transaction event
  input           din_fe_i;             // frame end
  input           din_fs_i;             // frame start
  input           din_le_i;             // line end
  input           din_ls_i;             // line start
  input  [11:0]   din_pel00_i;          // one pel -1,-1
  input  [11:0]   din_pel01_i;          // one pel  0,-1
  input  [11:0]   din_pel02_i;          // one pel  1,-1
  input  [11:0]   din_pel10_i;          // one pel -1, 0
  input  [13:0]   din_pel11_i;          // dir, one pel  0, 0
  input  [11:0]   din_pel12_i;          // one pel  1, 0
  input  [11:0]   din_pel20_i;          // one pel -1, 1
  input  [11:0]   din_pel21_i;          // one pel  0, 1
  input  [11:0]   din_pel22_i;          // one pel  1, 1

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output          dout_evt_o;           // transaction event
  output          dout_fe_o;            // frame end
  output          dout_fs_o;            // frame start
  output          dout_le_o;            // line end
  output          dout_ls_o;            // line start
  output [11:0]   dout_pel_o;           // pel

  //--------------------------------------------------------
  // parameter
  //--------------------------------------------------------
  input           param_nomax_byp_i;    // bypass
  input           param_nomax_mode_i;   // 0: just edge
                                        // 1: egde with weight
  input  [11:0]   param_thres_i;        // threshold

  //-----------------------------------------------------
  // Signals -- race buffer dout=buf(din) --
  //-----------------------------------------------------
  // FF
  reg             dout_d0_ls;
  reg             dout_d0_le;
  reg             dout_d0_fs;
  reg             dout_d0_fe;
  reg             dout_d0_evt;
  reg             dout_ls_o;
  reg             dout_le_o;
  reg             dout_fs_o;
  reg             dout_fe_o;
  reg             dout_evt_o;


  //-----------------------------------------------------
  // Signals -- nomax --
  //-----------------------------------------------------
  // FF
  reg    [11:0]   n_s0_a, n_s0_b, n_s0_c;
  reg    [11:0]   n_s1_pel;
  // combinatorial
  reg             n_s0_thres_en, n_s0_a_en, n_s0_c_en;
  reg             n_s0_en;

  //-----------------------------------------------------
  // Sync Process  -- nomax --
  //-----------------------------------------------------
  always @(posedge clk) begin
    case( din_pel11_i[13:12] )
      2'd0: n_s0_a <= din_pel01_i;  // horiz
      2'd1: n_s0_a <= din_pel02_i;  // diag
      2'd2: n_s0_a <= din_pel10_i;  // vert
      2'd3: n_s0_a <= din_pel00_i;  // diag
    endcase
    n_s0_b <= din_pel11_i[11:0];
    case( din_pel11_i[13:12] )
      2'd0: n_s0_c <= din_pel21_i;  // horiz
      2'd1: n_s0_c <= din_pel20_i;  // diag
      2'd2: n_s0_c <= din_pel12_i;  // vert
      2'd3: n_s0_c <= din_pel22_i;  // diag
    endcase

    if (param_nomax_byp_i | (param_nomax_mode_i & n_s0_en)) begin
      n_s1_pel <= n_s0_b;
    end
    else begin
      n_s1_pel <= {12{n_s0_en}};
    end
  end

  //-----------------------------------------------------
  // Async Process -- nomax --
  //-----------------------------------------------------
  always @(*) begin
    n_s0_thres_en = n_s0_b >  param_thres_i;
    n_s0_a_en     = n_s0_a <  n_s0_b;
    n_s0_c_en     = n_s0_b >= n_s0_c;
    n_s0_en       = n_s0_thres_en & n_s0_a_en & n_s0_c_en;
  end

  assign dout_pel_o = n_s1_pel;

  //-----------------------------------------------------
  // Sync Process  -- race buffer dout=buf(din) --
  //      handshake generation
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      dout_d0_evt <= 1'b0;
      dout_evt_o  <= 1'b0;
    end
    else begin
      dout_d0_evt <= din_evt_i;
      dout_evt_o  <= dout_d0_evt;
    end
  end
  //-----------------------------------------------------
  // Sync Process  -- race buffer dout=buf(din) --
  //      data path
  //-----------------------------------------------------
  always @(posedge clk) begin
    dout_d0_ls <= din_ls_i;
    dout_d0_le <= din_le_i;
    dout_d0_fs <= din_fs_i;
    dout_d0_fe <= din_fe_i;
    dout_ls_o  <= dout_d0_ls;
    dout_le_o  <= dout_d0_le;
    dout_fs_o  <= dout_d0_fs;
    dout_fe_o  <= dout_d0_fe;
  end
endmodule
