//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : pic2lvpp
// Submodules : non
//
// Purpose    : PIC to LVPP converter
//
//               - early frame end pules
//                     ____________
//               fe __|            |_
//                     _
//               ls __| |____________
//                                _
//               le _____________| |_
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module pic2lvpp (
    // system signals
    clk,
    rst_n,

    // data input
    din_b_i,
    din_evt_i,
    din_fe_i,
    din_fs_i,
    din_g_i,
    din_le_i,
    din_ls_i,
    din_r_i,

    // data output
    dout_data_o,
    dout_evt_o,
    dout_stat_o
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           rst_n;                // asynchron low active reset

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input  [11:0]   din_b_i;              // blue
  input           din_evt_i;            // transaction event, 1cc pulse
  input           din_fe_i;             // frame end
  input           din_fs_i;             // frame start
  input  [11:0]   din_g_i;              // green
  input           din_le_i;             // line end
  input           din_ls_i;             // line start
  input  [11:0]   din_r_i;              // red

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output [35:0]   dout_data_o;          // packed data
  output          dout_evt_o;           // transaction valid
  output [2:0]    dout_stat_o;          // 000 No-Status (NONE)
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-single-Line-Frame
                                        // (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)


  localparam NOP  = 3'd0,
             SOF  = 3'd1,
             SOSLF= 3'd2,
             SOD  = 3'd3,
             SOL  = 3'd4,
             SOLL = 3'd5,
             EOL  = 3'd6,
             ERR  = 3'd7;

  //-----------------------------------------------------
  // Signals -- buf --
  //-----------------------------------------------------
  // FF
  reg             dout_evt_o;
  reg    [2:0]    dout_stat_o;
  reg    [35:0]   dout_data_o;

  //-----------------------------------------------------
  // Sync Process  -- buf --
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      dout_evt_o <= 1'd0;
    end
    else begin
      dout_evt_o <= din_evt_i;
    end
  end

  //-----------------------------------------------------
  // Sync Process  -- buf --
  //-----------------------------------------------------
  always @(posedge clk) begin
    if (din_evt_i) begin
      dout_data_o <= {din_r_i, din_g_i, din_b_i};
      casez( {din_fs_i, din_fe_i, din_ls_i, din_le_i} )
        default: dout_stat_o <= NOP;
        4'b1010: dout_stat_o <= SOF;
        4'b1110: dout_stat_o <= SOSLF;
        4'b0010: dout_stat_o <= SOL;
        4'b??01: dout_stat_o <= EOL;
        4'b0110: dout_stat_o <= SOLL;
        4'b??11: dout_stat_o <= ERR;
      endcase
    end
  end

endmodule
