//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : siftop
// Submodules : canny_b12
//              px_null32
//
// Purpose    : sensor interface Custom Module top level
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module siftop (
    // system signal
    clk,
    sys_rst_n,
    img_rst_n,

    // config OCP
    cfg_maddr_i,
    cfg_mcmd_i,
    cfg_mdata_i,
    cfg_mrespaccept_i,
    cfg_scmdaccept_o,
    cfg_sdata_o,
    cfg_sresp_o,

    // data input
    din_b_i,
    din_evt_i,
    din_fe_i,
    din_fs_i,
    din_g_i,
    din_le_i,
    din_ls_i,
    din_r_i,

    // data output
    dout_b_o,
    dout_evt_o,
    dout_fe_o,
    dout_fs_o,
    dout_g_o,
    dout_le_o,
    dout_ls_o,
    dout_r_o,

    // process enable
    proc_en_i,

    // command
    px_cmd_addr_o,
    px_cmd_bl_o,
    px_cmd_en_o,
    px_cmd_full_i,
    px_cmd_instr_o,

    // read
    px_rd_data_i,
    px_rd_empty_i,
    px_rd_en_o,

    // write
    px_wr_data_o,
    px_wr_en_o,
    px_wr_full_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signal
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           sys_rst_n;            // asynchronous system reset, active low
  input           img_rst_n;            // asynchronous image bus reset, active low

  //--------------------------------------------------------
  // config OCP
  //--------------------------------------------------------
  input  [16:2]   cfg_maddr_i;          // register address
  input  [1:0]    cfg_mcmd_i;           // 00:IDEL
                                        // 01:WR
                                        // 10:RD
                                        // 11:not implemented
  input  [31:0]   cfg_mdata_i;          // write data
  input           cfg_mrespaccept_i;    // read data acknowledge
  output          cfg_scmdaccept_o;     // command acknowledge
  output [31:0]   cfg_sdata_o;          // read data
  output          cfg_sresp_o;          // 0:NULL, 1:DVA

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input  [11:0]   din_b_i;              // blue
  input           din_evt_i;            // transaction event
  input           din_fe_i;             // frame end
  input           din_fs_i;             // frame start
  input  [11:0]   din_g_i;              // green
  input           din_le_i;             // line end
  input           din_ls_i;             // line start
  input  [11:0]   din_r_i;              // one pel

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output [11:0]   dout_b_o;             // blue
  output          dout_evt_o;           // transaction event
  output          dout_fe_o;            // frame end
  output          dout_fs_o;            // frame start
  output [11:0]   dout_g_o;             // green
  output          dout_le_o;            // line end
  output          dout_ls_o;            // line start
  output [11:0]   dout_r_o;             // pel

  //--------------------------------------------------------
  // process enable
  //--------------------------------------------------------
  input           proc_en_i;            // enable

  //--------------------------------------------------------
  // command
  //--------------------------------------------------------
  output [29:0]   px_cmd_addr_o;        // Byte start address for current
                                        // transaction. Addresses must be
                                        // aligned to port size:
                                        // 32-bit ports: Lower two bits must be
                                        //  0s.
  output [5:0]    px_cmd_bl_o;          // Burst length in number of user words
                                        //  for the current transaction. Burst
                                        // length is encoded as 0 to 63,
                                        // representing 1 to 64 user words [for
                                        //  example, b00011 is a burst length 4
                                        //  transaction]. The user word width
                                        // equals the port width [for example,
                                        // a burst length of 3 on a 64-bit port
                                        //  transfers 3 x 64-bit user words =
                                        // 192 bits total].
  output          px_cmd_en_o;          // This active-High signal is the
                                        // write-enable signal for the Command
                                        // FIFO. This signal is covered in more
                                        //  detail in Chapter 4.
  input           px_cmd_full_i;        // This active-High output is the full
                                        // flag for the Command
                                        // FIFO. It indicates the FIFO cannot
                                        // accept any more
                                        // commands and blocks writes to the
                                        // Command FIFO.
  output [2:0]    px_cmd_instr_o;       // Command code for the current
                                        // instruction. Bit 0 represents the
                                        // READ/WRITE select, Bit 1 is Auto
                                        // Precharge enable, and Bit 2
                                        // represents Refresh, which always
                                        // takes priority:
                                        // Write: 3b000
                                        // Read: 3b001
                                        // This signal is covered in more
                                        // detail in Chapter 4.

  //--------------------------------------------------------
  // read
  //--------------------------------------------------------
  input  [31:0]   px_rd_data_i;         // Read Data value returning from
                                        // memory. This signal is driven by the
                                        //  output of the Read Data FIFO into
                                        // FPGA logic. PX_SIZE can be 32, 64,
                                        // or 128 bits, depending on the port
                                        // configuration.
  input           px_rd_empty_i;        // This active-High signal is the empty
                                        //  flag for the Read Data FIFO. It
                                        // indicates there is no valid data in
                                        // the FIFO.
  output          px_rd_en_o;           // This active-High signal is the read
                                        // enable for the Read Data FIFO. Read
                                        // Data is clocked out of the FIFO on
                                        // the rising edge of pX_rd_clk when
                                        // pX_rd_en = 1 and pX_rd_empty = 0.

  //--------------------------------------------------------
  // write
  //--------------------------------------------------------
  output [31:0]   px_wr_data_o;         // Write Data value to be loaded into
                                        // Write Data FIFO and sent to memory.
                                        // PX_SIZE can be 32, 64, or 128 bits,
                                        // depending on port configuration.
  output          px_wr_en_o;           // This active-High signal is the write
                                        //  enable for the Write Data FIFO. It
                                        // indicates that the value on
                                        // pX_wr_data is valid for
                                        // loading into the FIFO. Data is
                                        // loaded on the rising edge of
                                        // pX_wr_clk when pX_wr_en = 1 and
                                        // pX_wr_full = 0.
  input           px_wr_full_i;         // This active-High signal is the full
                                        // flag for the Write Data FIFO. When
                                        // this signal is high, it prevents
                                        // data from being loade into the FIFO.


  assign dout_b_o = dout_r_o;
  assign dout_g_o = dout_r_o;

  wire px_sys_clk;                      // unused

  //--------------------------------------------------------
  // instantiation of module -- CANNY_B12 --
  //--------------------------------------------------------
  canny_b12 u_canny_b12 (
    .clk                  ( clk                  ),
    .sys_rst_n            ( sys_rst_n            ),
    .img_rst_n            ( img_rst_n            ),
    .cfg_maddr_i          ( cfg_maddr_i          ),
    .cfg_mcmd_i           ( cfg_mcmd_i           ),
    .cfg_mdata_i          ( cfg_mdata_i          ),
    .cfg_mrespaccept_i    ( cfg_mrespaccept_i    ),
    .cfg_scmdaccept_o     ( cfg_scmdaccept_o     ),
    .cfg_sdata_o          ( cfg_sdata_o          ),
    .cfg_sresp_o          ( cfg_sresp_o          ),
    .din_evt_i            ( din_evt_i            ),
    .din_fe_i             ( din_fe_i             ),
    .din_fs_i             ( din_fs_i             ),
    .din_le_i             ( din_le_i             ),
    .din_ls_i             ( din_ls_i             ),
    .din_pel_i            ( din_r_i              ),
    .dout_evt_o           ( dout_evt_o           ),
    .dout_fe_o            ( dout_fe_o            ),
    .dout_fs_o            ( dout_fs_o            ),
    .dout_le_o            ( dout_le_o            ),
    .dout_ls_o            ( dout_ls_o            ),
    .dout_pel_o           ( dout_r_o             ),
    .proc_en_i            ( proc_en_i            )
  );

  //--------------------------------------------------------
  // instantiation of module -- PX_NULL32 --
  //--------------------------------------------------------
  px_null32 u_px_null32 (
    .px_cmd_addr_o        ( px_cmd_addr_o        ),
    .px_cmd_bl_o          ( px_cmd_bl_o          ),
    .px_cmd_en_o          ( px_cmd_en_o          ),
    .px_cmd_full_i        ( px_cmd_full_i        ),
    .px_cmd_instr_o       ( px_cmd_instr_o       ),
    .px_rd_data_i         ( px_rd_data_i         ),
    .px_rd_empty_i        ( px_rd_empty_i        ),
    .px_rd_en_o           ( px_rd_en_o           ),
    .px_sys_clk           ( px_sys_clk           ),
    .px_wr_data_o         ( px_wr_data_o         ),
    .px_wr_en_o           ( px_wr_en_o           ),
    .px_wr_full_i         ( px_wr_full_i         )
  );

endmodule
