//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : sobel_b12
// Submodules : non
//
// Purpose    : Sobel filter
//
//                 (x)       (y)
//               -1 0 1   -1 -2 -1
//               -2 0 2    0  0  0
//               -1 0 1    1  2  1
//
//               sobel = abs(x) + abs(y) / 2
//
//               - 4cc delay
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module sobel_b12 (
    // system signals
    clk,
    rst_n,

    // data input
    din_evt_i,
    din_fe_i,
    din_fs_i,
    din_le_i,
    din_ls_i,
    din_pel00_i,
    din_pel01_i,
    din_pel02_i,
    din_pel10_i,
    din_pel11_i,
    din_pel12_i,
    din_pel20_i,
    din_pel21_i,
    din_pel22_i,

    // data output
    dout_evt_o,
    dout_fe_o,
    dout_fs_o,
    dout_le_o,
    dout_ls_o,
    dout_pel_o,

    // parameter
    param_sobel_byp_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           rst_n;                // asynchron low active reset

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input           din_evt_i;            // transaction event
  input           din_fe_i;             // frame end
  input           din_fs_i;             // frame start
  input           din_le_i;             // line end
  input           din_ls_i;             // line start
  input  [11:0]   din_pel00_i;          // one pel -1,-1
  input  [11:0]   din_pel01_i;          // one pel  0,-1
  input  [11:0]   din_pel02_i;          // one pel  1,-1
  input  [11:0]   din_pel10_i;          // one pel -1, 0
  input  [11:0]   din_pel11_i;          // one pel  0, 0
  input  [11:0]   din_pel12_i;          // one pel  1, 0
  input  [11:0]   din_pel20_i;          // one pel -1, 1
  input  [11:0]   din_pel21_i;          // one pel  0, 1
  input  [11:0]   din_pel22_i;          // one pel  1, 1

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output          dout_evt_o;           // transaction event
  output          dout_fe_o;            // frame end
  output          dout_fs_o;            // frame start
  output          dout_le_o;            // line end
  output          dout_ls_o;            // line start
  output [13:0]   dout_pel_o;           // [13:12] dir
                                        // [11:0]  pel

  //--------------------------------------------------------
  // parameter
  //--------------------------------------------------------
  input           param_sobel_byp_i;    // bypass

  //-----------------------------------------------------
  // Signals -- race buffer dout=buf(din) --
  //-----------------------------------------------------
  // FF
  reg             dout_d0_ls;
  reg             dout_d0_le;
  reg             dout_d0_fs;
  reg             dout_d0_fe;
  reg             dout_d0_evt;
  reg             dout_d1_ls;
  reg             dout_d1_le;
  reg             dout_d1_fs;
  reg             dout_d1_fe;
  reg             dout_d1_evt;
  reg             dout_d2_ls;
  reg             dout_d2_le;
  reg             dout_d2_fs;
  reg             dout_d2_fe;
  reg             dout_d2_evt;
  reg             dout_ls_o;
  reg             dout_le_o;
  reg             dout_fs_o;
  reg             dout_fe_o;
  reg             dout_evt_o;


  //-----------------------------------------------------
  // Signals -- sobel --
  //-----------------------------------------------------
  // FF
  reg    [12:0]   s_s0_a, s_s0_b, s_s0_c, s_s0_d, s_s0_e;
  reg    [12:0]   s_s1_a;
  reg    [14:0]   s_s1_b, s_s1_d;
  reg    [10:0]   s_s2_x,  s_s2_y;
  reg             s_s2_sx, s_s2_sy;
  reg    [11:0]   s_s0_pel11, s_s1_pel11, s_s2_pel11, s_s3_pel11;
  reg    [11:0]   s_s3_s;
  reg    [1:0]    s_s3_d;
  // combinatorial
  reg    [14:0]   s_s1_x,  s_s1_y;
  reg             s_s1_sx, s_s1_sy;
  reg             s_s2_d2, s_s2_d0;

  //-----------------------------------------------------
  // Sync Process  -- sobel --
  //-----------------------------------------------------
  always @(posedge clk) begin
    s_s0_a <= din_pel20_i - din_pel02_i;
    s_s0_b <= din_pel21_i - din_pel01_i;
    s_s0_c <= din_pel22_i - din_pel00_i;
    s_s0_d <= din_pel10_i - din_pel12_i;
    s_s0_e <= din_pel00_i - din_pel22_i;
    s_s1_a <= s_s0_a;
    s_s1_b <= { s_s0_b[12], s_s0_b, 1'd0 } + { {2{s_s0_c[12]}}, s_s0_c};
    s_s1_d <= { s_s0_d[12], s_s0_d, 1'd0 } + { {2{s_s0_e[12]}}, s_s0_e};
    s_s2_x <= s_s1_x[13:3] ^ {11{s_s1_sx}};
    s_s2_y <= s_s1_y[13:3] ^ {11{s_s1_sy}};
    s_s2_sx<= s_s1_sx;
    s_s2_sy<= s_s1_sy;

    s_s0_pel11 <= din_pel11_i;
    s_s1_pel11 <= s_s0_pel11;
    s_s2_pel11 <= s_s1_pel11;
    if (param_sobel_byp_i) begin
      s_s3_s <= s_s2_pel11;
      s_s3_d <= 2'd0;
    end
    else begin
      s_s3_s <= {1'd0, s_s2_x} + {1'd0, s_s2_y};
      s_s3_d <= s_s2_d2
              ? 2'd2
              : s_s2_d0
                ? 2'd0
                : { (s_s2_sx!=s_s2_sy), 1'd1};
    end
  end

  assign dout_pel_o = { s_s3_d, s_s3_s };

  //-----------------------------------------------------
  // Async Process -- sobel --
  //-----------------------------------------------------
  always @(*) begin
    s_s1_x  = (s_s1_b + { {2{s_s1_a[12]}}, s_s1_a});
    s_s1_y  = (s_s1_d + { {2{s_s1_a[12]}}, s_s1_a});
    s_s1_sx = s_s1_x[14];
    s_s1_sy = s_s1_y[14];
  end
  always @(*) begin
    s_s2_d2 = {1'd0, s_s2_y} > {s_s2_x, 1'd0};
    s_s2_d0 = {1'd0, s_s2_x} > {s_s2_y, 1'd0};
  end

  //-----------------------------------------------------
  // Sync Process  -- race buffer dout=buf(din) --
  //      handshake generation
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      dout_d0_evt <= 1'b0;
      dout_d1_evt <= 1'b0;
      dout_d2_evt <= 1'b0;
      dout_evt_o  <= 1'b0;
    end
    else begin
      dout_d0_evt <= din_evt_i;
      dout_d1_evt <= dout_d0_evt;
      dout_d2_evt <= dout_d1_evt;
      dout_evt_o  <= dout_d2_evt;
    end
  end
  //-----------------------------------------------------
  // Sync Process  -- race buffer dout=buf(din) --
  //      data path
  //-----------------------------------------------------
  always @(posedge clk) begin
    dout_d0_ls <= din_ls_i;
    dout_d0_le <= din_le_i;
    dout_d0_fs <= din_fs_i;
    dout_d0_fe <= din_fe_i;
    dout_d1_ls <= dout_d0_ls;
    dout_d1_le <= dout_d0_le;
    dout_d1_fs <= dout_d0_fs;
    dout_d1_fe <= dout_d0_fe;
    dout_d2_ls <= dout_d1_ls;
    dout_d2_le <= dout_d1_le;
    dout_d2_fs <= dout_d1_fs;
    dout_d2_fe <= dout_d1_fe;
    dout_ls_o  <= dout_d2_ls;
    dout_le_o  <= dout_d2_le;
    dout_fs_o  <= dout_d2_fs;
    dout_fe_o  <= dout_d2_fe;
  end
endmodule
