//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : v3b14k4
// Submodules : non
//
// Purpose    : 1D horizontal 5pel 12 bit converter
//
//               - fully back to back capable
//               - 2x3 pel min frame size
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module v3b14k4 (
    // system signals
    clk,
    rst_n,

    // data input
    din_evt_i,
    din_fe_i,
    din_fs_i,
    din_le_i,
    din_ls_i,
    din_pel_i,

    // data output
    dout_evt_o,
    dout_fe_o,
    dout_fs_o,
    dout_le_o,
    dout_ls_o,
    dout_pel0_o,
    dout_pel1_o,
    dout_pel2_o,

    // processing control
    proc_en_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  input           clk;                  // system clock
  input           rst_n;                // asynchron low active reset

  //--------------------------------------------------------
  // data input
  //--------------------------------------------------------
  input           din_evt_i;            // transaction event
  input           din_fe_i;             // frame end
  input           din_fs_i;             // frame start
  input           din_le_i;             // line end
  input           din_ls_i;             // line start
  input  [13:0]   din_pel_i;            // one pel

  //--------------------------------------------------------
  // data output
  //--------------------------------------------------------
  output          dout_evt_o;           // transaction event
  output          dout_fe_o;            // frame end
  output          dout_fs_o;            // frame start
  output          dout_le_o;            // line end
  output          dout_ls_o;            // line start
  output [13:0]   dout_pel0_o;          // one pel y=-1
  output [13:0]   dout_pel1_o;          // one pel y=0
  output [13:0]   dout_pel2_o;          // one pel y=1

  //--------------------------------------------------------
  // processing control
  //--------------------------------------------------------
  input           proc_en_i;            // enable


  //-----------------------------------------------------
  // Signals -- converter --
  //-----------------------------------------------------
  // FF
  reg             c_s0_ievt;
  reg             c_s0_iypos;
  reg    [1:0]    c_s0_istate;
  reg    [1:0]    c_s0_ostate;
  reg             c_s0_le_ievt;
  reg             c_s1_evt, c_s2_evt, c_s3_evt;

  reg             c_s0_fe_ievt;
  reg    [11:0]   c_s0_iaddr, c_s1_iaddr;
  reg    [13:0]   c_s0_idata, c_s1_idata, c_s2_idata;
  reg    [11:0]   c_s0_oaddr;
  reg             c_s1_ostart_evt, c_s2_ostart_evt;
  reg             c_s1_le_evt, c_s2_le_evt;
  reg             c_s1_obody, c_s2_obody;
  reg             c_s2_oypos0, c_s2_oypos1;
  reg    [11:0]   c_s0_xsize;
  reg             c_s1_ls, c_s2_ls, c_s3_ls;
  reg             c_s1_le, c_s2_le, c_s3_le;
  reg             c_s1_fs, c_s2_fs, c_s3_fs;
  reg             c_s1_fe, c_s2_fe, c_s3_fe;
  reg    [13:0]   c_s3_pel0, c_s3_pel1, c_s3_pel2;
  // combinatorial
  reg    [13:0]   c_s2_pel0, c_s2_pel1;
  reg             c_ls_ievt;
  reg             c_le_ievt;
  reg             c_s0_orunout;
  reg             c_s0_evt;
  reg             c_s0_le_evt;
  reg             c_s0_ls;
  reg    [1:0]    c_s1_en;
  reg    [1:0]    c_s0_istate_next;
  reg             c_s0_1st_iline;
  reg             c_s0_2nd_iline;

  reg             c_s0_ostart_evt;
  reg    [1:0]    c_s0_ostate_next;
  reg             c_s0_obody;
  reg             c_s0_olast;

  //-----------------------------------------------------
  // Signals -- memory --
  //-----------------------------------------------------
  // FF
  reg    [11:0]   m_s1_raddr0;
  reg    [11:0]   m_s1_raddr1;
  reg    [13:0]   m_mem0[4095:0];
  reg    [13:0]   m_mem1[4095:0];
  reg    [13:0]   m_s2_rdata0;
  reg    [13:0]   m_s2_rdata1;


  //-----------------------------------------------------
  // Sync Process  -- converter --
  //-----------------------------------------------------
  always @(posedge clk or negedge rst_n) begin
    if (!rst_n) begin
      c_s0_ievt    <= 1'd0;
      c_s0_iypos   <= 1'd0;
      c_s0_istate  <= 2'd0;
      c_s0_ostate  <= 2'd0;
      c_s0_le_ievt <= 1'd0;
      c_s1_evt     <= 1'd0;
      c_s2_evt     <= 1'd0;
      c_s3_evt     <= 1'd0;
    end
    else begin
      c_s0_ievt    <= din_evt_i;
      c_s0_iypos   <= c_s0_iypos ^ c_s0_le_ievt;
      c_s0_istate  <= c_s0_fe_ievt    ? 2'd0 : c_s0_istate_next;
      c_s0_ostate  <= c_s0_ostart_evt ? 2'd1 : c_s0_ostate_next;
      c_s0_le_ievt <= c_le_ievt;
      c_s1_evt     <= c_s0_evt;
      c_s2_evt     <= c_s1_evt;
      c_s3_evt     <= c_s2_evt;
    end
  end

  //-----------------------------------------------------
  // Sync Process  -- converter --
  //-----------------------------------------------------
  always @(posedge clk) begin
    c_s0_fe_ievt <= din_evt_i & din_le_i & din_fe_i;
    c_s0_iaddr  <= c_ls_ievt ? 12'd0 : (c_s0_iaddr + {11'd0, c_s0_ievt});
    c_s1_iaddr  <= c_s0_iaddr;
    c_s0_idata  <= din_pel_i;
    c_s1_idata  <= c_s0_idata;
    c_s2_idata  <= c_s1_idata;

    c_s0_oaddr  <= (c_s0_ostart_evt | c_s0_le_evt)
                 ? 12'd0 : (c_s0_oaddr + {11'd0, c_s0_evt});


    c_s1_ostart_evt <= c_s0_ostart_evt;
    c_s2_ostart_evt <= c_s1_ostart_evt;
    c_s1_le_evt     <= c_s0_le_evt;
    c_s2_le_evt     <= c_s1_le_evt;
    c_s1_obody      <= c_s0_obody;
    c_s2_obody      <= c_s1_obody;
    if (c_s2_ostart_evt) begin
      c_s2_oypos0 <= c_s0_iypos;
      c_s2_oypos1 <= c_s0_iypos;
    end
    else if (c_s2_le_evt) begin
      c_s2_oypos0 <= c_s2_oypos1;
      c_s2_oypos1 <= c_s2_oypos1 ^ c_s2_obody;
    end


    if (c_s0_1st_iline & c_s0_le_ievt) c_s0_xsize <= c_s0_iaddr;

    c_s1_en     <= {
      ( c_s0_iypos) & c_s0_ievt,
      (!c_s0_iypos) & c_s0_ievt
    };

    c_s1_ls <= c_s0_ls;
    c_s2_ls <= c_s1_ls;
    c_s3_ls <= c_s2_ls;
    c_s1_le <= c_s0_le_evt;
    c_s2_le <= c_s1_le;
    c_s3_le <= c_s2_le;

    c_s1_fs <= c_s0_2nd_iline /*& c_s0_ls*/;
    c_s2_fs <= c_s1_fs;
    c_s3_fs <= c_s2_fs;
    c_s1_fe <= c_s0_olast /*& c_s0_le_evt*/;
    c_s2_fe <= c_s1_fe;
    c_s3_fe <= c_s2_fe;

    c_s3_pel0 <= c_s2_pel0;
    c_s3_pel1 <= c_s2_pel1;
    c_s3_pel2 <= c_s2_obody ? c_s2_idata : c_s2_pel1;
  end

  //-----------------------------------------------------
  // Async Process -- converter --
  //-----------------------------------------------------
  always @(*) begin
    c_s2_pel0 = c_s2_oypos0 ? m_s2_rdata0 : m_s2_rdata1;
    c_s2_pel1 = c_s2_oypos1 ? m_s2_rdata0 : m_s2_rdata1;
  end

  //-----------------------------------------------------
  // Async Process -- converter --
  //-----------------------------------------------------
  always @(*) begin
    c_ls_ievt = din_evt_i & din_ls_i;
    c_le_ievt = din_evt_i & din_le_i;
  end

  //-----------------------------------------------------
  // Async Process -- converter --
  //-----------------------------------------------------
  always @(*) begin
    c_s0_evt     = c_s0_ievt & c_s0_obody | (c_s0_orunout & (proc_en_i | c_s0_ievt));
    c_s0_le_evt  = (c_s0_oaddr == c_s0_xsize) & c_s0_evt;
    c_s0_ls      = (c_s0_oaddr == 12'd0);
  end

  //-----------------------------------------------------
  // Async Process -- converter --
  //
  // input FSM
  //-----------------------------------------------------
  always @(*) begin

    //------------------------------
    // default assignment
    //------------------------------
    c_s0_istate_next = c_s0_istate;
    c_s0_1st_iline   = 1'd0;
    c_s0_2nd_iline   = 1'd0;

    //------------------------------
    // FSM
    //------------------------------
    case( c_s0_istate )
      default: begin
        c_s0_1st_iline = 1'd1;
        if (c_s0_le_ievt) c_s0_istate_next = 2'd1;
      end
      2'd1: begin
        c_s0_2nd_iline = 1'd1;
        if (c_s0_le_ievt) c_s0_istate_next = 2'd2;
      end
      2'd2: begin
        // nothing
      end
    endcase
  end

  //-----------------------------------------------------
  // Async Process -- converter --
  //
  // output FSM
  //-----------------------------------------------------
  always @(*) begin

    c_s0_ostart_evt = c_s0_le_ievt & c_s0_1st_iline;

    //------------------------------
    // default assignment
    //------------------------------
    c_s0_ostate_next = c_s0_ostate;

    //------------------------------
    // FSM
    //------------------------------
    case( c_s0_ostate )
      default: begin
        // nothing
      end
      2'd1: begin
        if (c_s0_fe_ievt) c_s0_ostate_next = 2'd2;
      end
      2'd2: begin
        if (c_s0_le_evt) c_s0_ostate_next = 2'd0;
      end
    endcase
  end
  always @(*) begin
    c_s0_orunout = c_s0_ostate[1];
    c_s0_obody   = c_s0_ostate[0];
    c_s0_olast   = c_s0_ostate[1];
  end

  //-----------------------------------------------------
  // Sync Process  -- memory --
  //-----------------------------------------------------
  always @(posedge clk) begin
    if (c_s1_en[0]) m_mem0[c_s1_iaddr] <= c_s1_idata;
    m_s1_raddr0 <= c_s0_oaddr;
    m_s2_rdata0 <= m_mem0[m_s1_raddr0];

    if (c_s1_en[1]) m_mem1[c_s1_iaddr] <= c_s1_idata;
    m_s1_raddr1 <= c_s0_oaddr;
    m_s2_rdata1 <= m_mem1[m_s1_raddr1];
  end

  assign dout_evt_o  = c_s3_evt;
  assign dout_fe_o   = c_s3_fe;
  assign dout_fs_o   = c_s3_fs;
  assign dout_le_o   = c_s3_le;
  assign dout_ls_o   = c_s3_ls;
  assign dout_pel0_o = c_s3_pel0;
  assign dout_pel1_o = c_s3_pel1;
  assign dout_pel2_o = c_s3_pel2;

endmodule
