-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Top Level Design Module
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-10  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity CustomModule is
    port (
        -- Module Infrastructure Signals
        SysClkxCI        : in  std_logic;                       -- Custom Module Clock
        SysRstxRI        : in  std_logic;                       -- Custom Module Reset
        ImgRstxRI        : in  std_logic;                       -- ImageBus Reset

        -- SystemBus Control Interface
        SysReqxMI        : in  std_logic;                       -- SystemBus Cycle Request
        SysAckxMO        : out std_logic;                       -- SystemBus Cycle Acknowledge
        SysErrxMO        : out std_logic;                       -- SystemBus Cycle Error
        SysWrEnxMI       : in  std_logic;                       -- SystemBus Write Enable
        SysAddrxAI       : in  std_logic_vector(16 downto 0);   -- SystemBus Address
        SysWrDataxDI     : in  std_logic_vector(31 downto 0);   -- SystemBus Write Data
        SysRdDataxDO     : out std_logic_vector(31 downto 0);   -- SystemBus Read Data

        -- Custom Module GPIO
        CustIoCtrlxMO    : out std_logic_vector(5 downto 0);    -- GPIO Output Control
        CustIoDataOutxDO : out std_logic_vector(5 downto 0);    -- GPIO Output Data
        CustIoDataInxDI  : in  std_logic_vector(5 downto 0);    -- GPIO Input Data

        -- ImageBus Receive Port
        ImgRxPixValxMI   : in  std_logic;                       -- ImageBus Receive Pixel Valid
        ImgRxPixRdyxMO   : out std_logic;                       -- ImageBus Receive Pixel Ready
        ImgRxPixStatxSI  : in  std_logic_vector(2 downto 0);    -- ImageBus Receive Pixel Status
        ImgRxPixDataxDI  : in  std_logic_vector(35 downto 0);   -- ImageBus Receive Pixel Data

        -- ImageBus Transmit Port
        ImgTxPixValxMO   : out std_logic;                       -- ImageBus Transmit Pixel Valid
        ImgTxPixRdyxMI   : in  std_logic;                       -- ImageBus Transmit Pixel Ready
        ImgTxPixStatxSO  : out std_logic_vector(2 downto 0);    -- ImageBus Transmit Pixel Status
        ImgTxPixDataxDO  : out std_logic_vector(35 downto 0);   -- ImageBus Transmit Pixel Data

        -- MemoryBus Command Port
        px_cmd_en_o      : out std_logic;                       -- MemoryBus Command Valid
        px_cmd_full_i    : in  std_logic;                       -- MemoryBus Command Buffer Full
        px_cmd_instr_o   : out std_logic_vector(2 downto 0);    -- MemoryBus Command Instruction
        px_cmd_bl_o      : out std_logic_vector(5 downto 0);    -- MemoryBus Command Burst Length
        px_cmd_addr_o    : out std_logic_vector(29 downto 0);   -- MemoryBus Command Address

        -- MemoryBus Read Port
        px_rd_en_o       : out std_logic;                       -- MemoryBus Read Data Valid
        px_rd_empty_i    : in  std_logic;                       -- MemoryBus Read Data Buffer Empty
        px_rd_data_i     : in  std_logic_vector(31 downto 0);   -- MemoryBus Read Data Bus

        -- MemoryBus Write Port
        px_wr_en_o       : out std_logic;                       -- MemoryBus Write Data Valid
        px_wr_full_i     : in  std_logic;                       -- MemoryBus Write Data Buffer Full
        px_wr_data_o     : out std_logic_vector(31 downto 0));  -- MemoryBus Write Data Bus
end CustomModule;

architecture structural of CustomModule is

begin

    -- Connect ImageBus input port to output port
    ImgTxPixValxMO  <= ImgRxPixValxMI;
    ImgTxPixStatxSO <= ImgRxPixStatxSI;
    ImgTxPixDataxDO <= ImgRxPixDataxDI;
    ImgRxPixRdyxMO  <= ImgTxPixRdyxMI;

    -- Define unused output signals
    SysAckxMO    <= SysReqxMI;
    SysErrxMO    <= '0';
    SysRdDataxDO <= (others => '0');

    CustIoCtrlxMO    <= (others => '0');
    CustIoDataOutxDO <= (others => '0');

    px_cmd_en_o    <= '0';
    px_cmd_instr_o <= (others => '0');
    px_cmd_bl_o    <= (others => '0');
    px_cmd_addr_o  <= (others => '0');

    px_rd_en_o     <= '0';

    px_wr_en_o     <= '0';
    px_wr_data_o   <= (others => '0');

end structural;