-- ------------------------------------------------------------------------------
-- File Name         :  $RCSfile:  $
-- Author            :  $Author:  $
-- Revision          :  $Revision:  $
-- Last Modification :  $Date:  $
-- ------------------------------------------------------------------------------
--                                                 __________
--                                   _            / ________ \ 
--                                  | |          / /    ___ \ \ 
--                                  | |         / /    |__ \ \ \ 
--                                  | |        / /     __ \ \ \ \ 
--          ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
--         / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
--        | |      | | | |  ____| | | |            \ \____/ / / /
--        | |      | | | | |_____/  | |             \______/ / /
--        | |      | | | |________  | |________         ____/ /
--        |_|      |_|  \_________|  \_________|       |_____/
--
--                                   NEW ELECTRONIC TECHNOLOGY
--
-- ------------------------------------------------------------------------------
--
-- Copyright (C) 2014 NET GmbH All Rights Reserved.
--
-- This document, and its contents are the property of NET GmbH.
-- NET GmbH retains all rights to: this document, the document contents,
-- and the design techniques expressed by the document contents.
-- Neither this document, nor any of the information, nor any of the design
-- techniques expressed within may be, either in whole or in part: used,
-- reproduced, copied, stored, or conveyed to a third party by any means
-- without the express written consent of an authorized representative of
-- NET GmbH.
--
-- ------------------------------------------------------------------------------
-- Module     : diffpic_cm_gpio_proc
-- Submodules : non
--
-- Purpose    : difference picture
--              The frame data are written to DDR memory and the data from the
--              last frame are read from memory.
--              (Byte/Pixel write + Byte/Pixel read)
--              The data output is the scaled pixel difference of both frames.
--              The module sum the pixel difference for each frame, if the 
--              value is higher then param_motion_thres_i than CustIoDataOutxDO(1)=1
--              else CustIoDataOutxDO(0)=1. When CustIoDataInxDI is low the motion
--              control is disabled. 
--              Sensor data must be monochrome.
--
-- Creator    : a.kramer
-- ------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library work;
use work.diffpic_cm_gpio_pkg.all;


entity diffpic_cm_gpio_proc_bw_e is

  port(
    -- Module Infrastructure Signals
    SysClkxCI:            in std_logic;
    SysRstxRI:            in std_logic;
    -- ImageBus Receive Port
    ImgRxPixDataxDI:      in std_logic_vector(35 downto 0);
    ImgRxPixRdyxMO:       out std_logic;
    ImgRxPixStatxSI:      in std_logic_vector(2 downto 0);
    ImgRxPixValxMI:       in std_logic;
    -- ImageBus Transmit Port
    ImgTxPixDataxDO:      out std_logic_vector(35 downto 0);
    ImgTxPixRdyxMI:       in std_logic;
    ImgTxPixStatxSO:      out std_logic_vector(2 downto 0);
    ImgTxPixValxMO:       out std_logic;
    -- Custom Module GPIO
    CustIoCtrlxMO:        out std_logic_vector(NUM_CM_GPIO_OUT-1 downto 0);
    CustIoDataOutxDO:     out std_logic_vector(NUM_CM_GPIO_OUT-1 downto 0);
    CustIoDataInxDI:      in  std_logic_vector(NUM_CM_GPIO_IN-1 downto 0);
    -- MCP command
    px_cmd_addr_o:        out std_logic_vector(29 downto 0);
    px_cmd_bl_o:          out std_logic_vector(5 downto 0);
    px_cmd_en_o:          out std_logic;
    px_cmd_full_i:        in std_logic;
    px_cmd_instr_o:       out std_logic_vector(2 downto 0);
    -- MCB read
    px_rd_data_i:         in std_logic_vector(31 downto 0);
    px_rd_empty_i:        in std_logic;
    px_rd_en_o:           out std_logic;
    -- MCP write
    px_wr_data_o:         out std_logic_vector(31 downto 0);
    px_wr_en_o:           out std_logic;
    px_wr_full_i:         in std_logic;
    -- Parameter
    param_mem_ptr_i:      in std_logic_vector(29 downto 0);
    param_mem_size_i:     in std_logic_vector(29 downto 0);
    param_mem_rdy_i:      in std_logic;
    param_diffpic_en_i:   in std_logic;
    param_gray_offs_i:    in std_logic_vector(7 downto 0);
    param_motion_thres_i: in std_logic_vector(31 downto 0)
  );
end diffpic_cm_gpio_proc_bw_e;


architecture diffpic_cm_gpio_proc_bw_a of diffpic_cm_gpio_proc_bw_e is
  
  type state_typ is (idle, waitcmd, wr, wr_last, rd_first, rd);
  
  signal p_state:              state_typ;
  signal p_next_state:         state_typ;
  
  signal p_wr_count:           unsigned(5 downto 0);
  signal p_wr_bl:              unsigned(4 downto 0);
  signal p_wr_count_byte:      unsigned(1 downto 0);
  signal p_wr_count_msb:       std_logic;
  signal p_wr_buf_evt:         std_logic;
  signal p_start_evt:          std_logic;
  signal p_lastline:           std_logic;
  signal p_lastpix:            std_logic;
  signal p_lastpix_s0_evt:     std_logic;
  signal p_lastpix_s1_evt:     std_logic;
  signal p_lastpix_s2_evt:     std_logic;
  signal p_lastpix_s1:         std_logic;
  signal p_lastpix_s0_buf:     std_logic;
  signal p_start_read:         std_logic;
  signal p_start_read_buf:     std_logic;
  signal p_start_write:        std_logic;
  signal p_start_write_buf:    std_logic;
  signal p_wr_evt:             std_logic;
  signal p_wr_last_evt:        std_logic;
  signal p_rd_first_evt:       std_logic;
  signal p_rd_evt:             std_logic;
  signal p_rd_end:             std_logic;
  signal p_rd_count_byte:      unsigned(1 downto 0);
  signal px_rd_en_s:           std_logic;
  signal param_gray_offs_s:    signed(8 downto 0);
  signal p_diffpic_en:         std_logic;
  signal p_do_rd_empty:        std_logic;
  
  signal p_memin_y:            std_logic_vector(7 downto 0);
  signal p_memout_y:           signed(8 downto 0);
  
  signal p_proc_y_s0:          signed(8 downto 0);
  signal p_proc_val_s0:        std_logic;
  signal p_proc_state_s0:      std_logic_vector(2 downto 0);

  signal p_proc_y_s1:          signed(8 downto 0);
  signal p_proc_val_s1:        std_logic;
  signal p_proc_state_s1:      std_logic_vector(2 downto 0);
  signal p_proc_y_sig_s2:      std_logic;

  signal p_dout_y:             std_logic_vector(8 downto 0);
  signal p_dout_val:           std_logic;
  signal p_dout_state:         std_logic_vector(2 downto 0);
  
  signal p_cmd_addr:           std_logic_vector(27 downto 0);
  signal p_wr_addr:            unsigned(27 downto 0);
  signal p_rd_addr:            unsigned(27 downto 0);
  signal p_max_addr:           unsigned(27 downto 0);
  signal p_wr_addr_max:        unsigned(27 downto 0);
  signal p_addr_overrun:       std_logic;
  signal p_last_addr:          std_logic_vector(27 downto 0);
  signal p_last_bl:            std_logic_vector(5 downto 0);
  
  signal p_motion_sum:         signed(31 downto 0);
  signal p_motion_val:         std_logic_vector(31 downto 0);
    

begin

  status_sig: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      p_lastline <= '0';
      p_lastpix_s0_evt <= '0';
      p_lastpix_s1_evt <= '0';
      p_lastpix_s2_evt <= '0';
      p_lastpix_s0_buf <= '0';
      p_start_read_buf <= '0';
      p_start_write_buf <= '0';
      p_max_addr <= (others => '0');
      
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if ((ImgRxPixStatxSI=SOLL or ImgRxPixStatxSI=SOSLF) and ImgRxPixValxMI='1' and p_state/=idle) then
        p_lastline <= '1';
      elsif (p_lastpix='1') then
        p_lastline <= '0';
      end if;
      
      p_lastpix_s0_evt <= p_lastpix;
      p_lastpix_s1_evt <= p_lastpix_s0_evt;
      p_lastpix_s2_evt <= p_lastpix_s1_evt;
      if (p_lastpix_s0_evt='1') then
        p_lastpix_s0_buf <= '1';
      elsif (p_wr_last_evt='1') then
        p_lastpix_s0_buf <= '0';
      end if;
      
      if (p_diffpic_en='1' or p_state/=idle) then
        p_start_read_buf <= p_start_read;
        p_start_write_buf <= p_start_write;
      else
        p_start_read_buf <= '0';
        p_start_write_buf <= '0';
      end if;
      
      p_max_addr <= unsigned(param_mem_ptr_i(29 downto 2)) + unsigned(param_mem_size_i(29 downto 2));
    end if;
  end process;
  
  p_diffpic_en <= param_diffpic_en_i and param_mem_rdy_i;
  
  p_start_read <= '1' when ((ImgRxPixStatxSI=SOF or ImgRxPixStatxSI=SOSLF) and ImgRxPixValxMI='1' and p_start_evt='0' and p_diffpic_en='1') else
                  p_start_read_buf;
                  
  p_start_write <= '1' when (p_start_evt='1' and p_diffpic_en='1') else
                   p_start_write_buf;
  
  p_lastpix <=  '1' when (ImgRxPixStatxSI=EOL and ImgRxPixValxMI='1' and p_lastline='1' and p_state/=idle) else
                '0';

  p_lastpix_s1 <= p_lastpix_s0_evt or p_lastpix_s0_buf;
                  

  comb: process(p_state, ImgRxPixStatxSI, ImgRxPixValxMI, px_cmd_full_i, p_lastpix_s1, p_wr_buf_evt, p_start_read, p_diffpic_en, p_rd_end, px_rd_empty_i)
  begin
    p_next_state <= p_state;
    p_start_evt  <= '0';
    p_wr_evt  <= '0';
    p_wr_last_evt  <= '0';
    p_rd_first_evt  <= '0';
    p_rd_evt  <= '0';
    p_do_rd_empty  <= '0';
    
    case p_state is 
      when idle =>
        if ((ImgRxPixStatxSI=SOF or ImgRxPixStatxSI=SOSLF) and ImgRxPixValxMI='1' and p_diffpic_en='1') then
          p_next_state <= waitcmd;
          p_start_evt  <= '1';
        end if;
          
      when waitcmd =>
        if (p_lastpix_s1='1') then
          p_next_state <= wr_last;
        elsif (p_wr_buf_evt='1') then
          p_next_state <= wr;        
        end if;
        
      when wr =>
        if (px_cmd_full_i='0') then
          p_wr_evt  <= '1';
          if (p_start_read='1' and p_rd_end='0') then
            p_next_state <= rd;
          else
            p_next_state <= waitcmd;
          end if;
        end if;
        
      when wr_last =>         
        if (px_cmd_full_i='0') then
          p_wr_last_evt  <= '1';
          if (p_diffpic_en='1') then
            p_next_state <= rd_first;
          else
            p_next_state <= idle;
          end if;
        end if;
        
      when rd_first =>
        if (px_cmd_full_i='0') then
          if (px_rd_empty_i='1') then
            p_rd_first_evt  <= '1';
            p_next_state <= waitcmd;
          else
            p_do_rd_empty <= '1';      -- read mcb-read-fifo empty
          end if;
        end if;
        
      when rd =>
        if (px_cmd_full_i='0') then
          p_rd_evt  <= '1';
          p_next_state <= waitcmd;
        end if;
        
    end case;
  end process;
          
    
  reg: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      p_state <= idle;	  
    elsif (SysClkxCI'event and SysClkxCI='1') then
      p_state <= p_next_state; 
    end if;
  end process;
  
  outreg_cmd_en: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      px_cmd_en_o  <= '0';
    elsif (SysClkxCI'event and SysClkxCI='1') then     
      if (p_wr_evt='1' or p_wr_last_evt='1' or p_rd_first_evt='1' or p_rd_evt='1') then
        px_cmd_en_o  <= '1';
      else
        px_cmd_en_o  <= '0';
      end if;      
    end if;
  end process;
  
  outreg: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then     
      
      if (p_wr_evt='1' or p_wr_last_evt='1') then
        if (p_addr_overrun = '0') then    -- write only on a valid address
          p_cmd_addr <= std_logic_vector(p_wr_addr);
        end if;
        px_cmd_instr_o  <= "000";
      elsif (p_rd_first_evt='1' or p_rd_evt='1') then
        p_cmd_addr <= std_logic_vector(p_rd_addr);
        px_cmd_instr_o  <= "001";
      end if;
      
      if (p_wr_last_evt='1' or p_state=idle) then
        p_wr_addr <= unsigned(param_mem_ptr_i(29 downto 2));
      elsif (p_wr_evt='1') then
        p_wr_addr  <= p_wr_addr+"100000";
      end if;
      
      if (p_wr_last_evt='1' or p_state=idle) then
        p_rd_addr <= unsigned(param_mem_ptr_i(29 downto 2));
      elsif (p_rd_first_evt='1') then
        p_rd_addr  <= p_rd_addr+"1000000";
      elsif (p_rd_evt='1') then
        p_rd_addr  <= p_rd_addr+"100000";
      end if;      
        
      if (p_wr_evt='1') then  
        px_cmd_bl_o  <= "011111";       -- Burstlength: 32
      elsif (p_wr_last_evt='1') then
        px_cmd_bl_o  <= '0' & std_logic_vector(p_wr_bl);
      elsif (p_rd_first_evt='1') then
        px_cmd_bl_o  <= "111111";       -- Burstlength: 64 (at the first read)
      elsif (p_rd_evt='1') then
        if (std_logic_vector(p_rd_addr)=p_last_addr) then
          px_cmd_bl_o  <= p_last_bl;
        else
          px_cmd_bl_o  <= "011111";
        end if;      
      end if;             
        
      if (p_wr_last_evt='1') then       -- memory last address and burstlength for read
        p_last_addr <= std_logic_vector(p_wr_addr);
        p_last_bl  <= '0' & std_logic_vector(p_wr_bl);
      end if;        
                
      if (p_rd_evt='1' and std_logic_vector(p_rd_addr)=p_last_addr) then
        p_rd_end <= '1';          
      elsif (p_wr_last_evt='1') then
        p_rd_end <= '0';
      end if;   
    end if;
  end process;
  
  p_wr_addr_max <= p_wr_addr + p_wr_bl;    -- compute the max write-address
  p_addr_overrun <= '0' when (p_wr_addr_max < p_max_addr) else
                    '1';                   -- detect a address overrun
   
  px_cmd_addr_o <= p_cmd_addr & "00";
    
    
  p_memin_y <= ImgRxPixDataxDI(35 downto 28);
 
  
  wr_buf: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      p_wr_count_byte <= "00";
      p_wr_count <= "000000";
      p_wr_count_msb <= '0';
      px_wr_en_o <= '0';
      px_wr_data_o <= (others => '0');
      p_wr_bl <= "00000";
      
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (ImgRxPixValxMI='1' and p_start_write='1') then
        p_wr_count_byte <= p_wr_count_byte+1;
        
        case p_wr_count_byte is
          when "00" => 
            px_wr_data_o(31 downto 24) <= p_memin_y;
            
          when "01" => 
            px_wr_data_o(23 downto 16) <= p_memin_y;
          
          when "10" => 
            px_wr_data_o(15 downto 8) <= p_memin_y;
            
          when others => 
            px_wr_data_o(7 downto 0) <= p_memin_y;        
        end case;
        
      elsif (p_wr_last_evt='1') then
        p_wr_count_byte <= "00";
      end if;
      
      if (ImgRxPixValxMI='1' and p_start_write='1' and (p_wr_count_byte="11" or p_wr_last_evt='1')) then
        px_wr_en_o <= '1';
      else
        px_wr_en_o <= '0';
      end if;
      
      if (ImgRxPixValxMI='1' and p_wr_count_byte="11" and p_start_write='1') then
        if (p_wr_last_evt='1') then
          p_wr_count <= "000001";
        else
          p_wr_count <= p_wr_count+1;
        end if;
      elsif (p_wr_last_evt='1') then
        p_wr_count <= "000000";
      end if;
      
      if (ImgRxPixValxMI='1' and p_wr_count_byte="11" and p_start_write='1') then
        p_wr_bl <= p_wr_count(4 downto 0);            -- burstlength = count-1
      elsif (p_wr_last_evt='1' or p_wr_evt='1') then
        p_wr_bl <= "00000";
      end if;
      
      if (p_wr_last_evt='1') then
        p_wr_count_msb <= '0';
      else
        p_wr_count_msb <= p_wr_count(5);
      end if;
         
    end if;
  end process;
  
  p_wr_buf_evt <= '1' when (p_wr_count_msb/=p_wr_count(5)) else
                  '0';
  
  
  rd_buf: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      p_rd_count_byte <= "00";
      p_memout_y <= (others => '0');
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (px_rd_empty_i='0' and ImgRxPixValxMI='1' and p_start_read='1') then
        p_rd_count_byte <= p_rd_count_byte+1;
        
        case p_rd_count_byte is
          when "00" => 
            p_memout_y <= signed('0' & px_rd_data_i(31 downto 24));
            
          when "01" => 
            p_memout_y <= signed('0' & px_rd_data_i(23 downto 16));
          
          when "10" => 
            p_memout_y <= signed('0' & px_rd_data_i(15 downto 8));
            
          when others => 
            p_memout_y <= signed('0' & px_rd_data_i(7 downto 0));        
        end case;
        
      elsif (p_wr_last_evt='1') then
        p_rd_count_byte <= "00";
      end if;    
    end if;
  end process;
  
  px_rd_en_s <= '1' when (px_rd_empty_i='0' and ImgRxPixValxMI='1' and p_start_read='1' and p_rd_count_byte="11") or (p_do_rd_empty='1') else
                '0';
  px_rd_en_o <= px_rd_en_s;
  
    
  param_gray_offs_s <= signed('0' & param_gray_offs_i);
  
  proc: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      if (ImgRxPixValxMI='1') then
        p_proc_y_s0 <= signed('0' & p_memin_y);
      end if;      
      p_proc_state_s0 <= ImgRxPixStatxSI;
      
      if (p_proc_val_s0='1') then
        if (p_start_read='1') then
          p_proc_y_s1 <= (p_memout_y - p_proc_y_s0);
        else
          p_proc_y_s1 <= p_proc_y_s0;
        end if;
      end if;      
      p_proc_state_s1 <= p_proc_state_s0;
      
      if (p_proc_val_s1='1') then
        if (p_start_read='1') then
          p_dout_y <= std_logic_vector((p_proc_y_s1(8)&p_proc_y_s1(8 downto 1))+param_gray_offs_s);
        else
          p_dout_y <= std_logic_vector(p_proc_y_s1);
        end if;
      end if;           
      p_dout_state <= p_proc_state_s1;
      p_proc_y_sig_s2 <= p_proc_y_s1(8);
      
      if (p_dout_y(8)='0' or p_start_read='0') then
        ImgTxPixDataxDO <= p_dout_y(7 downto 0) & "0000" & p_dout_y(7 downto 0) & "0000" & p_dout_y(7 downto 0) & "0000";
      elsif (p_proc_y_sig_s2='0') then
        ImgTxPixDataxDO <= "111111110000" & "111111110000" & "111111110000";   --upper limit
      else
        ImgTxPixDataxDO <= "000000000000" & "000000000000" & "000000000000";   --lower limit
      end if;
      ImgTxPixStatxSO <= p_dout_state;
    end if;
  end process;
  
  proc_val: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      p_proc_val_s0 <= '0';
      p_proc_val_s1 <= '0';
      p_dout_val <= '0';
      ImgTxPixValxMO <= '0';
    elsif (SysClkxCI'event and SysClkxCI='1') then   
      p_proc_val_s0  <= ImgRxPixValxMI;
      p_proc_val_s1  <= p_proc_val_s0;         
      p_dout_val     <= p_proc_val_s1;
      ImgTxPixValxMO <= p_dout_val;       
    end if;
  end process;
  
  ImgRxPixRdyxMO <= ImgTxPixRdyxMI;
  
  
  motion_ctl: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      p_motion_sum <= (others => '0');
      p_motion_val <= (others => '0');
      CustIoDataOutxDO(1 downto 0) <= "00";
    
    elsif (SysClkxCI'event and SysClkxCI='1') then
      
      if (p_lastpix_s2_evt='1') then
        p_motion_sum <= (others => '0');
        p_motion_val <= std_logic_vector(p_motion_sum);
      elsif (p_proc_val_s1='1' and p_start_read='1') then
        p_motion_sum <= p_motion_sum + abs(p_proc_y_s1); 
      end if;
      
      if (CustIoDataInxDI(0)='1') then       -- CustIoDataInxDI is aktive low
        if (p_motion_val > param_motion_thres_i) then
          CustIoDataOutxDO(1 downto 0) <= "10";
        else
          CustIoDataOutxDO(1 downto 0) <= "01";
        end if;
      else
        CustIoDataOutxDO(1 downto 0) <= "00";
      end if;      
    end if;
  end process;
  
  cust_gpio_dout_g: if NUM_CM_GPIO_OUT>2 generate
    CustIoDataOutxDO(NUM_CM_GPIO_OUT-1 downto 2) <= (others => '0');
  end generate;
  
  gpio_ctl: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      CustIoCtrlxMO <= (others => '0');
    elsif (SysClkxCI'event and SysClkxCI='1') then
      CustIoCtrlxMO <= CM_GPIO_OUT_CTRL;
    end if;
  end process;


end diffpic_cm_gpio_proc_bw_a;