-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Controller for OpenCamera LUT design
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-20  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;

use work.CustomPkg.all;
use work.LutPkg.all;

entity LutCtrl is
    port (
        -- Module Infrastructure Signals
        SysClkxCI        : in  std_logic;                                       -- Custom Module Clock 100 MHz
        SysRstxRI        : in  std_logic;                                       -- Custom Module Reset

        -- SystemBus Control/Status Port
        LutCtrlRegxMI    : in  tLUT_CTRL_REG;                                   -- SystemBus LUT Control Registers
        LutStatRegxMO    : out tLUT_STAT_REG;                                   -- SystemBus LUT Status Registers

        -- SystemBus Memory Access Port
        SysMemReqxMI     : in  std_logic;                                       -- SystemBus Memory Cycle Request
        SysMemAckxMO     : out std_logic;                                       -- SystemBus Memory Cycle Acknowledge
        SysMemErrxMO     : out std_logic;                                       -- SystemBus Memory Cycle Error
        SysMemWrEnxMI    : in  std_logic;                                       -- SystemBus Memory Write Enable
        SysMemAddrxAI    : in  std_logic_vector(cSYS_ADDR_WIDTH-1 downto 0);    -- SystemBus Memory Address
        SysMemWrDataxDI  : in  std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Memory Write Data
        SysMemRdDataxDO  : out std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Memory Read Data

        -- ImageBus Receive Port
        ImgRxPixValxMI   : in  std_logic;                                       -- ImageBus Receive Pixel Valid
        ImgRxPixRdyxMO   : out std_logic;                                       -- ImageBus Receive Pixel Ready
        ImgRxPixStatxSI  : in  std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);    -- ImageBus Receive Pixel Status
        ImgRxPixDataxDI  : in  std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);    -- ImageBus Receive Pixel Data

        -- ImageBus Transmit Port
        ImgTxPixValxMO   : out std_logic;                                       -- ImageBus Transmit Pixel Valid
        ImgTxPixRdyxMI   : in  std_logic;                                       -- ImageBus Transmit Pixel Ready
        ImgTxPixStatxSO  : out std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Status
        ImgTxPixDataxDO  : out std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Data

        -- Custom Module GPIO
        CustIoCtrlxMO    : out std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Control
        CustIoDataOutxDO : out std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Data
        CustIoDataInxDI  : in  std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Input Data

        -- Debug Logic Port
        DbgLogicPortxTO  : out std_logic_vector(cDEBUG_PORT_WIDTH-1 downto 0)); -- Custom Module Debug Signals
end LutCtrl;

architecture rtl of LutCtrl is

    ---------------------------------------------------------------------------
    -- Component Declarations
    ---------------------------------------------------------------------------
    component LutRAM is
    generic (
        gLUT_DATA_WIDTH : integer   := 8;              -- LUT data/address width
        gLUT8_07_INIT   : tRAM64X32 := cDEFAULT_INIT;  -- LUT data bits 0..7 initialization (for 8-bit LUT)
        gLUT12_03_INIT  : tRAM64X32 := cDEFAULT_INIT;  -- LUT data bits 0..3 initialization (for 12-bit LUT)
        gLUT12_47_INIT  : tRAM64X32 := cDEFAULT_INIT;  -- LUT data bits 4..7 initialization (for 12-bit LUT)
        gLUT12_811_INIT : tRAM64X32 := cDEFAULT_INIT); -- LUT data bits 8..11 initialization (for 12-bit LUT)
    port (
        -- SystemBus Port
        SysClkxCI    : in  std_logic;
        SysRstxRI    : in  std_logic;
        SysEnxMI     : in  std_logic;
        SysWrEnxMI   : in  std_logic;
        SysAddrxAI   : in  std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);
        SysWrDataxDI : in  std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);
        SysRdDataxDO : out std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);

        -- ImageBus Port
        ImgClkxCI    : in  std_logic;
        ImgRstxRI    : in  std_logic;
        ImgWrDataxDI : in  std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);
        ImgRdDataxDO : out std_logic_vector(gLUT_DATA_WIDTH-1 downto 0));
    end component;

    ---------------------------------------------------------------------------
    -- Constant and Type Definitions
    ---------------------------------------------------------------------------
    type tIMG_PIPE_ARRAY is array (cIMG_PIPE_DELAY-1 downto 0) of std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);
    type tLUT_DATA_ARRAY is array (0 to 2) of std_logic_vector(cLUT_DATA_WIDTH-1 downto 0);

    ---------------------------------------------------------------------------
    -- Signal Declarations
    ---------------------------------------------------------------------------
    -- SystemBus signals
    signal SysMemReqxM          : std_logic_vector(2 downto 0);                         -- SystemBus Memory Cycle Request
    signal SysMemAckxM          : std_logic_vector(cSYS_PIPE_DELAY-1 downto 0);         -- SystemBus Memory Cycle Acknowledge
    signal SysMemRdDataxD       : std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);         -- SystemBus Memory Read Data
    signal SysMemLutDataxD      : tLUT_DATA_ARRAY;
    signal SysMemWrDataxD       : tLUT_DATA_ARRAY;

    -- ImageBus signals
    signal ImgTxValDelayxM      : std_logic_vector(cIMG_PIPE_DELAY-1 downto 0);
    signal ImgTxStatDelayxM     : tIMG_PIPE_ARRAY;
    signal ImgRxLutDataxD       : tLUT_DATA_ARRAY;
    signal ImgTxLutDataxD       : tLUT_DATA_ARRAY;

    signal ImgRxPixRdyxM        : std_logic;                                            -- ImageBus Receive Pixel Ready
    signal ImgTxPixValxM        : std_logic;                                            -- ImageBus Transmit Pixel Valid
    signal ImgTxPixStatxS       : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);         -- ImageBus Transmit Pixel Status
    signal ImgTxPixDataxD       : std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);         -- ImageBus Transmit Pixel Data

    -- GPIO signals
    signal GpioClkOutxM         : std_logic;
    signal GpioClkCntxM         : std_logic_vector(23 downto 0);
    signal CustIoCtrlxM         : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);         -- GPIO Output Control
    signal CustIoDataOutxD      : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);         -- GPIO Output Data

begin

    ----------------------------------------------------------------------------
    -- ImageBus Pipeline Delay and SystemBus Acknowledge
    ----------------------------------------------------------------------------
    ImgTxProc: process (SysClkxCI, SysRstxRI)
        variable LoBit, HiBit: natural;
    begin
        if (SysRstxRI = '1') then
            SysMemAckxM      <= (others => '0');
            SysMemRdDataxD   <= (others => '0');

            ImgTxValDelayxM  <= (others => '0');
            ImgTxStatDelayxM <= (others => (others => '0'));
            ImgRxLutDataxD   <= (others => (others => '0'));

            ImgTxPixValxM    <= '0';
            ImgTxPixStatxS   <= (others => '0');
            ImgTxPixDataxD   <= (others => '0');
        elsif rising_edge(SysClkxCI) then
            -- Generate the SystemBus acknowledge signal
            if (SysMemReqxMI = '1') then
                if (SysMemWrEnxMI = '1') then
                    -- SystemBus acknowledge write delay
                    SysMemAckxM <= (others => '1');
                else
                    -- SystemBus acknowledge read delay
                    SysMemAckxM <= SysMemAckxM(cSYS_PIPE_DELAY-2 downto 0) & SysMemReqxMI;
                end if;
            else
                SysMemAckxM <= (others => '0');
            end if;

            -- Assign the LUT BlockRAM read data to the SystemBus. On the SystemBus
            -- LUT monochrome data is located on data bit 7..0 (8-bit) or 11..0 (12-bit).
            -- RGB data is located at 23..0 (RGB 8-bit) or 23..0 (GB 12-bit) and 11..0 (R, 12-bit).
            SysMemRdDataxD <= (others => '0');
            if (SysMemAddrxAI(cLUT_DATA_WIDTH+3 downto cLUT_DATA_WIDTH+2) = "00") then
                if (cLUT_DATA_WIDTH = 8) then
                    SysMemRdDataxD(23 downto 0) <= SysMemLutDataxD(2) & SysMemLutDataxD(1) & SysMemLutDataxD(0);
                else
                    SysMemRdDataxD(23 downto 0) <= SysMemLutDataxD(1) & SysMemLutDataxD(0);
                end if;
            else
                SysMemRdDataxD(cLUT_DATA_WIDTH-1 downto 0) <= SysMemLutDataxD(2);
            end if;

            ----------------------------------------------------------------------------

            -- ImageBus Valid/Status pipeline delay.
            ImgTxValDelayxM  <= ImgTxValDelayxM(cIMG_PIPE_DELAY-2 downto 0)  & ImgRxPixValxMI;
            ImgTxStatDelayxM <= ImgTxStatDelayxM(cIMG_PIPE_DELAY-2 downto 0) & ImgRxPixStatxSI;

            -- Assign LUT input and output pixel data to/from the ImageBus data.
            -- 8-bit pixel data is located on ImageBus(11..4)(Blue), ImageBus(23..16)(Green) and ImageBus(35..28)(Red, Mono)
            -- 12-bit pixel data is located on ImageBus(11..0)(Blue), ImageBus(23..12)(Green) and ImageBus(35..24)(Red, Mono)
            ImgTxPixDataxD <= (others => '0');
            for i in 0 to 2 loop
                HiBit := (i*12)+11;
                LoBit := HiBit-cLUT_DATA_WIDTH+1;
                ImgRxLutDataxD(i) <= ImgRxPixDataxDI(HiBit downto LoBit);
                ImgTxPixDataxD(HiBit downto LoBit) <= ImgTxLutDataxD(i);
            end loop;

            -- Assign ImageBus TX Valid and Status from pipeline delay.
            ImgTxPixValxM  <= ImgTxValDelayxM(cIMG_PIPE_DELAY-1);
            ImgTxPixStatxS <= ImgTxStatDelayxM(cIMG_PIPE_DELAY-1);
        end if;
    end process;

    -- ImageBus handshake not implemented
    ImgRxPixRdyxM  <= '1';
    ImgRxPixRdyxMO <= ImgRxPixRdyxM;

    -- Assign ImageBus output signals
    ImgTxPixValxMO  <= ImgTxPixValxM;
    ImgTxPixStatxSO <= ImgTxPixStatxS;
    ImgTxPixDataxDO <= ImgTxPixDataxD;

    -- Assign SystemBus output signals
    SysMemAckxMO    <= SysMemAckxM(cSYS_PIPE_DELAY-1);
    SysMemErrxMO    <= '0';
    SysMemRdDataxDO <= SysMemRdDataxD;

    ----------------------------------------------------------------------------
    -- 8-bit LUT Instantiation
    ----------------------------------------------------------------------------

    -- 8-bit LUT SystemBus Address Mapping:
    -- +--------------+-----+------------+----------+----------+
    -- |  Address Bit | A16 | A15 .. A10 | A9 .. A2 | A1 .. A0 |
    -- +--------------+-----+------------+----------+----------+
    -- | RGB/Mono LUT | '1' |  "000000"  | LUT-Addr |   "00"   |
    -- +--------------+-----+------------+----------+----------+

    Lut8Gen: if (cLUT_DATA_WIDTH = 8) generate
        SysMemReqxM <= (others => '1') when ((SysMemReqxMI = '1') and (SysMemAddrxAI(cSYS_ADDR_WIDTH-1 downto cSYS_ADDR_WIDTH-7) = "1000000")) else (others => '0');
        SysMemWrDataxD(0) <= SysMemWrDataxDI(7 downto 0);       -- Blue LUT
        SysMemWrDataxD(1) <= SysMemWrDataxDI(15 downto 8);      -- Green LUT
        SysMemWrDataxD(2) <= SysMemWrDataxDI(23 downto 16);     -- Red (Mono) LUT

        -- Instantiate 3x 8-bit data BlockRAMs
        Lut8Loop: for i in 0 to 2 generate
            Lut8Inst: LutRAM
            generic map (
                gLUT_DATA_WIDTH => cLUT_DATA_WIDTH,
                gLUT8_07_INIT   => cLUT8_07_IDENT)
                -- Alternatively use:
                -- gLUT8_07_INIT   => cLUT8_07_INVERSE)
            port map (
                -- SystemBus Port
                SysClkxCI    => SysClkxCI,
                SysRstxRI    => SysRstxRI,
                SysEnxMI     => SysMemReqxM(i),
                SysWrEnxMI   => SysMemWrEnxMI,
                SysAddrxAI   => SysMemAddrxAI(cLUT_DATA_WIDTH+1 downto 2),
                SysWrDataxDI => SysMemWrDataxD(i),
                SysRdDataxDO => SysMemLutDataxD(i),

                -- ImageBus Port
                ImgClkxCI    => SysClkxCI,
                ImgRstxRI    => SysRstxRI,
                ImgWrDataxDI => ImgRxLutDataxD(i),
                ImgRdDataxDO => ImgTxLutDataxD(i));
        end generate;
    end generate;

    ----------------------------------------------------------------------------
    -- 12-bit LUT Instantiation
    ----------------------------------------------------------------------------

    -- 12-bit LUT SystemBus Address Mapping:
    -- +----------------+-----|------------+-----------+----------+
    -- |   Address Bit  | A16 | A15 .. A14 | A13 .. A2 | A1 .. A0 |
    -- +----------------+-----+------------+-----------+----------+
    -- |    Mono LUT    | '1' |    "00"    | LUT-Addr  |   "00"   |
    -- +----------------+-----+------------+-----------+----------+
    -- | Blue/Green LUT | '1' |    "00"    | LUT-Addr  |   "00"   |
    -- |     Red LUT    | '1' |    "01"    | LUT-Addr  |   "00"   |
    -- +----------------+-----+------------+-----------+----------+

    Lut12Gen: if (cLUT_DATA_WIDTH = 12) generate
        SysMemReqxM(0) <= '1' when ((SysMemReqxMI = '1') and (SysMemAddrxAI(cSYS_ADDR_WIDTH-1 downto cSYS_ADDR_WIDTH-3) = "100") and (SysMemAddrxAI(cLUT_DATA_WIDTH+3 downto cLUT_DATA_WIDTH+2) = "00")) else '0';  -- Blue LUT
        SysMemReqxM(1) <= '1' when ((SysMemReqxMI = '1') and (SysMemAddrxAI(cSYS_ADDR_WIDTH-1 downto cSYS_ADDR_WIDTH-3) = "100") and (SysMemAddrxAI(cLUT_DATA_WIDTH+3 downto cLUT_DATA_WIDTH+2) = "00")) else '0';  -- Green LUT
        SysMemReqxM(2) <= '1' when ((SysMemReqxMI = '1') and (SysMemAddrxAI(cSYS_ADDR_WIDTH-1 downto cSYS_ADDR_WIDTH-3) = "100") and (SysMemAddrxAI(cLUT_DATA_WIDTH+3 downto cLUT_DATA_WIDTH+2) = "01")) else '0';  -- Red (Mono) LUT

        SysMemWrDataxD(0) <= SysMemWrDataxDI(11 downto 0);  -- Blue LUT
        SysMemWrDataxD(1) <= SysMemWrDataxDI(23 downto 12); -- Green LUT
        SysMemWrDataxD(2) <= SysMemWrDataxDI(11 downto 0);  -- Red (Mono) LUT

        -- Instantiate 3x 12-bit data BlockRAMs
        Lut12Loop: for i in 0 to 2 generate
            Lut12Inst: LutRAM
            generic map (
                gLUT_DATA_WIDTH => cLUT_DATA_WIDTH,
                gLUT12_03_INIT  => cLUT12_03_IDENT,
                gLUT12_47_INIT  => cLUT12_47_IDENT,
                gLUT12_811_INIT => cLUT12_811_IDENT)
                -- Alternatively use:
                -- gLUT12_03_INIT  => cLUT12_03_INVERSE,
                -- gLUT12_47_INIT  => cLUT12_47_INVERSE,
                -- gLUT12_811_INIT => cLUT12_811_INVERSE)
            port map (
                -- SystemBus Port
                SysClkxCI    => SysClkxCI,
                SysRstxRI    => SysRstxRI,
                SysEnxMI     => SysMemReqxM(i),
                SysWrEnxMI   => SysMemWrEnxMI,
                SysAddrxAI   => SysMemAddrxAI(cLUT_DATA_WIDTH+1 downto 2),
                SysWrDataxDI => SysMemWrDataxD(i),
                SysRdDataxDO => SysMemLutDataxD(i),

                -- ImageBus Port
                ImgClkxCI    => SysClkxCI,
                ImgRstxRI    => SysRstxRI,
                ImgWrDataxDI => ImgRxLutDataxD(i),
                ImgRdDataxDO => ImgTxLutDataxD(i));
        end generate;
    end generate;

    ----------------------------------------------------------------------------
    -- GPIO Controller
    ----------------------------------------------------------------------------
    -- Toggle GPIO pins at variable speed (for testing only)
    -- GPIO Map: Bit(0..3) => Opto(0..3), Bit(4..5) => TTL(0..1)
    --
    -- CustIoCtrlxMO(x): '0' => Associated GPIO pin is controlled by System GPIO Handler
    --                   '1' => Associated GPIO pin is controlled by CustIoDataOutxDO(x)
    --
    -- CustIoDataOutxDO(x): '0' => Opto: Photodiode OFF; TTL: Low Level
    --                      '1' => Opto: Photodiode ON;  TTL: High Level
    ----------------------------------------------------------------------------

    -- GPIO clock divider to toggle GPIO outputs at programmable intervals.
    GpioCtrlProc: process (SysClkxCI, SysRstxRI)
    begin
        if (SysRstxRI = '1') then
            GpioClkOutxM <= '0';
            GpioClkCntxM <= (others => '0');
        elsif rising_edge(SysClkxCI) then
            if (GpioClkCntxM = LutCtrlRegxMI.GpioDivide) then
                GpioClkOutxM <= not(GpioClkOutxM);
                GpioClkCntxM <= (others => '0');
            else
                -- Increment the GPIO clock divider counter
                GpioClkCntxM <= GpioClkCntxM + '1';
            end if;
        end if;
    end process;

    -- Generate GPIO output signals
    CustGpioGen: for i in 0 to cSYS_GPIO_WIDTH-1 generate
        CustIoCtrlxM(i)    <= '0' when (cCUSTOM_GPIO_ENABLE(i) = FALSE) else LutCtrlRegxMI.GpioCtrl(i);
        CustIoDataOutxD(i) <= '0' when (cCUSTOM_GPIO_ENABLE(i) = FALSE) else LutCtrlRegxMI.GpioData(i) when (LutCtrlRegxMI.GpioClkSel(i) = '0') else GpioClkOutxM;
    end generate;

    -- Assign GPIO output signals
    CustIoCtrlxMO    <= CustIoCtrlxM;
    CustIoDataOutxDo <= CustIoDataOutxD;

    -- Read the GPIO input pins via a SystemBus status register
    LutStatRegxMO.GpioStat <= CustIoDataInxDI;

    ----------------------------------------------------------------------------
    -- Debug Logic
    ----------------------------------------------------------------------------
    NoDbgGen: if (cDEBUG_ENABLE = False) generate
        DbgLogicPortxTO <= (others => '0');
    end generate;

    DbgGen: if (cDEBUG_ENABLE = True) generate
        signal DbgLogicPortxT : std_logic_vector(255 downto 0);
    begin
        DbgProc: process (SysClkxCI)
            variable i: natural;
        begin
            if rising_edge(SysClkxCI) then
                i := 0;
                DbgLogicPortxT(i)                <= SysMemReqxMI;
                DbgLogicPortxT(i+1)              <= SysMemAckxM(cSYS_PIPE_DELAY-1);
                DbgLogicPortxT(i+2)              <= SysMemWrEnxMI;
                DbgLogicPortxT(i+19 downto i+3)  <= SysMemAddrxAI;
                DbgLogicPortxT(i+51 downto i+20) <= SysMemWrDataxDI;
                DbgLogicPortxT(i+83 downto i+52) <= SysMemRdDataxD;
                DbgLogicPortxT(i+86 downto i+84) <= SysMemReqxM;

                i := 87;
                DbgLogicPortxT(i)                <= ImgRxPixValxMI;
                DbgLogicPortxT(i+1)              <= ImgRxPixRdyxM;
                DbgLogicPortxT(i+4 downto i+2)   <= ImgRxPixStatxSI;
                DbgLogicPortxT(i+40 downto i+5)  <= ImgRxPixDataxDI;

                i := 128;
                DbgLogicPortxT(i)                <= ImgTxPixValxM;
                DbgLogicPortxT(i+1)              <= ImgTxPixRdyxMI;
                DbgLogicPortxT(i+4 downto i+2)   <= ImgTxPixStatxS;
                DbgLogicPortxT(i+40 downto i+5)  <= ImgTxPixDataxD;

                i := 169;
                DbgLogicPortxT(i+5 downto i)     <= CustIoCtrlxM;
                DbgLogicPortxT(i+11 downto i+6)  <= CustIoDataOutxD;
                DbgLogicPortxT(i+17 downto i+12) <= CustIoDataInxDI;

                i := 187;
                DbgLogicPortxT(i)                <= GpioClkOutxM;
                DbgLogicPortxT(i+24 downto i+1)  <= GpioClkCntxM;

                i := 212;
                DbgLogicPortxT(255 downto i) <= (others => '0');
            end if;
        end process;

        DbgLogicPortxTO <= DbgLogicPortxT(cDEBUG_PORT_WIDTH-1 downto 0);
    end generate;

end rtl;
