-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Top Level for OpenCamera LUT design
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-20  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

use work.CustSimPkg.all;
use work.CustomPkg.all;
use work.LutPkg.all;

------------  Do not modify CustomModule entity declaration  ------------
entity CustomModule is
    port (
        -- Module Infrastructure Signals
        SysClkxCI        : in  std_logic;                                       -- Custom Module Clock
        SysRstxRI        : in  std_logic;                                       -- Custom Module Reset
        ImgRstxRI        : in  std_logic;                                       -- ImageBus Reset

        -- SystemBus Control Interface
        SysReqxMI        : in  std_logic;                                       -- SystemBus Cycle Request
        SysAckxMO        : out std_logic;                                       -- SystemBus Cycle Acknowledge
        SysErrxMO        : out std_logic;                                       -- SystemBus Cycle Error
        SysWrEnxMI       : in  std_logic;                                       -- SystemBus Write Enable
        SysAddrxAI       : in  std_logic_vector(cSYS_ADDR_WIDTH-1 downto 0);    -- SystemBus Address
        SysWrDataxDI     : in  std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Write Data
        SysRdDataxDO     : out std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Read Data

        -- Custom Module GPIO
        CustIoCtrlxMO    : out std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Control
        CustIoDataOutxDO : out std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Data
        CustIoDataInxDI  : in  std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Input Data

        -- ImageBus Receive Port
        ImgRxPixValxMI   : in  std_logic;                                       -- ImageBus Receive Pixel Valid
        ImgRxPixRdyxMO   : out std_logic;                                       -- ImageBus Receive Pixel Ready
        ImgRxPixStatxSI  : in  std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);    -- ImageBus Receive Pixel Status
        ImgRxPixDataxDI  : in  std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);    -- ImageBus Receive Pixel Data

        -- ImageBus Transmit Port
        ImgTxPixValxMO   : out std_logic;                                       -- ImageBus Transmit Pixel Valid
        ImgTxPixRdyxMI   : in  std_logic;                                       -- ImageBus Transmit Pixel Ready
        ImgTxPixStatxSO  : out std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Status
        ImgTxPixDataxDO  : out std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Data

        -- MemoryBus Command Port
        px_cmd_en_o      : out std_logic;                                       -- MemoryBus Command Valid
        px_cmd_full_i    : in  std_logic;                                       -- MemoryBus Command Buffer Full
        px_cmd_instr_o   : out std_logic_vector(cMEM_CMD_WIDTH-1 downto 0);     -- MemoryBus Command Instruction
        px_cmd_bl_o      : out std_logic_vector(cMEM_BLEN_WIDTH-1 downto 0);    -- MemoryBus Command Burst Length
        px_cmd_addr_o    : out std_logic_vector(cMEM_ADDR_WIDTH-1 downto 0);    -- MemoryBus Command Address

        -- MemoryBus Read Port
        px_rd_en_o       : out std_logic;                                       -- MemoryBus Read Data Valid
        px_rd_empty_i    : in  std_logic;                                       -- MemoryBus Read Data Buffer Empty
        px_rd_data_i     : in  std_logic_vector(cMEM_DATA_WIDTH-1 downto 0);    -- MemoryBus Read Data Bus

        -- MemoryBus Write Port
        px_wr_en_o       : out std_logic;                                       -- MemoryBus Write Data Valid
        px_wr_full_i     : in  std_logic;                                       -- MemoryBus Write Data Buffer Full
        px_wr_data_o     : out std_logic_vector(cMEM_DATA_WIDTH-1 downto 0));   -- MemoryBus Write Data Bus
end CustomModule;
------------  Do not modify CustomModule entity declaration  ------------

architecture structural of CustomModule is

    ---------------------------------------------------------------------------
    -- Component Declarations
    ---------------------------------------------------------------------------
    component LutCtrl is
    port (
        -- Module Infrastructure Signals
        SysClkxCI        : in  std_logic;                                       -- Custom Module Clock 100 MHz
        SysRstxRI        : in  std_logic;                                       -- Custom Module Reset
    
        -- SystemBus Control/Status Port    
        LutCtrlRegxMI    : in  tLUT_CTRL_REG;                                   -- SystemBus LUT Control Registers
        LutStatRegxMO    : out tLUT_STAT_REG;                                   -- SystemBus LUT Status Registers
    
        -- SystemBus Memory Access Port 
        SysMemReqxMI     : in  std_logic;                                       -- SystemBus Memory Cycle Request
        SysMemAckxMO     : out std_logic;                                       -- SystemBus Memory Cycle Acknowledge
        SysMemErrxMO     : out std_logic;                                       -- SystemBus Memory Cycle Error
        SysMemWrEnxMI    : in  std_logic;                                       -- SystemBus Memory Write Enable
        SysMemAddrxAI    : in  std_logic_vector(cSYS_ADDR_WIDTH-1 downto 0);    -- SystemBus Memory Address
        SysMemWrDataxDI  : in  std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Memory Write Data
        SysMemRdDataxDO  : out std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Memory Read Data
    
        -- ImageBus Receive Port    
        ImgRxPixValxMI   : in  std_logic;                                       -- ImageBus Receive Pixel Valid
        ImgRxPixRdyxMO   : out std_logic;                                       -- ImageBus Receive Pixel Ready
        ImgRxPixStatxSI  : in  std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);    -- ImageBus Receive Pixel Status
        ImgRxPixDataxDI  : in  std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);    -- ImageBus Receive Pixel Data
    
        -- ImageBus Transmit Port   
        ImgTxPixValxMO   : out std_logic;                                       -- ImageBus Transmit Pixel Valid
        ImgTxPixRdyxMI   : in  std_logic;                                       -- ImageBus Transmit Pixel Ready
        ImgTxPixStatxSO  : out std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Status
        ImgTxPixDataxDO  : out std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Data
    
        -- Custom Module GPIO   
        CustIoCtrlxMO    : out std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Control
        CustIoDataOutxDO : out std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Data
        CustIoDataInxDI  : in  std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Input Data

        -- Debug Logic Port
        DbgLogicPortxTO  : out std_logic_vector(cDEBUG_PORT_WIDTH-1 downto 0)); -- Custom Module Debug Signals
    end component;

    component LutSysBusIf is
    port (
        -- Custom Module Infrastructure Signals
        SysClkxCI         : in  std_logic;                                      -- Custom Module Clock 100 MHz
        SysRstxRI         : in  std_logic;                                      -- Custom Module Reset

        -- SystemBus Control Interface
        SysReqxMI         : in  std_logic;                                      -- SystemBus Cycle Request
        SysAckxMO         : out std_logic;                                      -- SystemBus Cycle Acknowledge
        SysErrxMO         : out std_logic;                                      -- SystemBus Cycle Error
        SysWrEnxMI        : in  std_logic;                                      -- SystemBus Write Enable
        SysAddrxAI        : in  std_logic_vector(cSYS_ADDR_WIDTH-1 downto 0);   -- SystemBus Address
        SysWrDataxDI      : in  std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);   -- SystemBus Write Data
        SysRdDataxDO      : out std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);   -- SystemBus Read Data

        -- SystemBus Control/Status Port
        LutCtrlRegxMO     : out tLUT_CTRL_REG;                                  -- SystemBus LUT Control Registers
        LutStatRegxMI     : in  tLUT_STAT_REG;                                  -- SystemBus LUT Status Registers

        -- SystemBus Memory Access Port
        SysMemReqxMO      : out std_logic;                                      -- SystemBus Memory Cycle Request
        SysMemAckxMI      : in  std_logic;                                      -- SystemBus Memory Cycle Acknowledge
        SysMemErrxMI      : in  std_logic;                                      -- SystemBus Memory Cycle Error
        SysMemRdDataxDI   : in  std_logic_vector(cSYS_DATA_WIDTH-1 downto 0));  -- SystemBus Memory Read Data
    end component;

    -- Custom Debug Module
    component DebugModule is
    generic (
        gLOGIC_ENABLE     : boolean := False;
        gCHIPSCOPE_ENABLE : boolean := True;
        gCSP_ICON_ENABLE  : boolean := True;
        gDEBUG_PORT_WIDTH : integer := 256);
    port (
        DbgClkxCI         : in  std_logic;                                          -- Debug Clock
        DbgLogicPortxTI   : in  std_logic_vector(gDEBUG_PORT_WIDTH-1 downto 0));    -- Debug Port
    end component;

    ---------------------------------------------------------------------------
    -- Signal Declarations
    ---------------------------------------------------------------------------
    -- SystemBus Control/Status Port
    signal LutCtrlRegxM      : tLUT_CTRL_REG;                                   -- SystemBus LUT Control Registers
    signal LutStatRegxM      : tLUT_STAT_REG;                                   -- SystemBus LUT Status Registers

    -- SystemBus Memory Access Port
    signal SysMemReqxM       : std_logic;                                       -- SystemBus Memory Cycle Request
    signal SysMemAckxM       : std_logic;                                       -- SystemBus Memory Cycle Acknowledge
    signal SysMemErrxM       : std_logic;                                       -- SystemBus Memory Cycle Error
    signal SysMemRdDataxD    : std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Memory Read Data

    -- ImageBus Receive Port
    signal ImgTxPixValxM     : std_logic;                                       -- ImageBus Transmit Pixel Valid
    signal ImgTxPixStatxS    : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Status
    signal ImgTxPixDataxD    : std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Data

    -- Custom Module GPIO
    signal CustIoCtrlxM      : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Control
    signal CustIoDataOutxD   : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Data

    -- Debug Logic Port
    signal DbgLogicPortxT    : std_logic_vector(cDEBUG_PORT_WIDTH-1 downto 0);  -- Custom Module Debug Signals

begin

    ---------------------------------------------------------------------------
    -- LUT SystemBus Interface
    ---------------------------------------------------------------------------
    CustSysBusInst: LutSysBusIf
    port map (
        -- Custom Module Infrastructure Signals
        SysClkxCI         => SysClkxCI,
        SysRstxRI         => SysRstxRI,

        -- SystemBus Control Interface
        SysReqxMI         => SysReqxMI,
        SysAckxMO         => SysAckxMO,
        SysErrxMO         => SysErrxMO,
        SysWrEnxMI        => SysWrEnxMI,
        SysAddrxAI        => SysAddrxAI,
        SysWrDataxDI      => SysWrDataxDI,
        SysRdDataxDO      => SysRdDataxDO,

        -- SystemBus Control/Status Port
        LutCtrlRegxMO     => LutCtrlRegxM,
        LutStatRegxMI     => LutStatRegxM,

        -- SystemBus Memory Access Port
        SysMemReqxMO      => SysMemReqxM,
        SysMemAckxMI      => SysMemAckxM,
        SysMemErrxMI      => SysMemErrxM,
        SysMemRdDataxDI   => SysMemRdDataxD);

    ---------------------------------------------------------------------------
    -- LUT Controller
    ---------------------------------------------------------------------------
    CustLutInst: LutCtrl
    port map (
        -- Module Infrastructure Signals
        SysClkxCI         => SysClkxCI,
        SysRstxRI         => SysRstxRI,

        -- SystemBus Control/Status Port
        LutCtrlRegxMI     => LutCtrlRegxM,
        LutStatRegxMO     => LutStatRegxM,

        -- SystemBus Memory Access Port
        SysMemReqxMI      => SysMemReqxM,
        SysMemAckxMO      => SysMemAckxM,
        SysMemErrxMO      => SysMemErrxM,
        SysMemWrEnxMI     => SysWrEnxMI,
        SysMemAddrxAI     => SysAddrxAI,
        SysMemWrDataxDI   => SysWrDataxDI,
        SysMemRdDataxDO   => SysMemRdDataxD,

        -- ImageBus Receive Port
        ImgRxPixValxMI    => ImgRxPixValxMI,
        ImgRxPixRdyxMO    => ImgRxPixRdyxMO,
        ImgRxPixStatxSI   => ImgRxPixStatxSI,
        ImgRxPixDataxDI   => ImgRxPixDataxDI,

        -- ImageBus Transmit Port
        ImgTxPixValxMO    => ImgTxPixValxM,
        ImgTxPixRdyxMI    => ImgTxPixRdyxMI,
        ImgTxPixStatxSO   => ImgTxPixStatxS,
        ImgTxPixDataxDO   => ImgTxPixDataxD,

        -- Custom Module GPIO
        CustIoCtrlxMO     => CustIoCtrlxM,
        CustIoDataOutxDO  => CustIoDataOutxD,
        CustIoDataInxDI   => CustIoDataInxDI,

        -- Debug Logic Port
        DbgLogicPortxTO   => DbgLogicPortxT);

    -- Assign LUT output signals
    CustIoCtrlxMO    <= CustIoCtrlxM;
    CustIoDataOutxDO <= CustIoDataOutxD;

    ImgTxPixValxMO  <= ImgTxPixValxM;
    ImgTxPixStatxSO <= ImgTxPixStatxS;
    ImgTxPixDataxDO <= ImgTxPixDataxD;

    ----------------------------------------------------------------------------
    -- MemoryBus Assignments (not used in LutCtrl)
    ----------------------------------------------------------------------------
    px_cmd_en_o    <= '0';
    px_cmd_instr_o <= (others => '0');
    px_cmd_bl_o    <= (others => '0');
    px_cmd_addr_o  <= (others => '0');

    px_rd_en_o     <= '0';
    px_wr_en_o     <= '0';
    px_wr_data_o   <= (others => '0');

    ---------------------------------------------------------------------------
    -- Debug Module Instantiation
    ---------------------------------------------------------------------------
    DbgGen: if ((cDEBUG_ENABLE = True) and (cCUST_SIM_CONFIG = False)) generate
        DbgInst: DebugModule
        generic map (
            gLOGIC_ENABLE     => cLOGIC_ENABLE,
            gCHIPSCOPE_ENABLE => cCHIPSCOPE_ENABLE,
            gCSP_ICON_ENABLE  => cCSP_ICON_ENABLE,
            gDEBUG_PORT_WIDTH => cDEBUG_PORT_WIDTH)
        port map (
            DbgClkxCI         => SysClkxCI,
            DbgLogicPortxTI   => DbgLogicPortxT);
    end generate;

end structural;
