-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Package file for OpenCamera LUT design
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-20  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.CustomPkg.all;
use work.LutVerPkg.all;

package LutPkg is

    ----------------------------------------------------------------------------
    -- ImageBus Definitions
    ----------------------------------------------------------------------------
    constant cIMG_PIPE_DELAY    : integer := 3;         -- Length of ImageBus Pipeline Delay
    constant cLUT_DATA_WIDTH    : integer := 8;         -- Width of ImageBus Pixel Data [8, 12]

    ----------------------------------------------------------------------------
    -- SystemBus Definitions
    ----------------------------------------------------------------------------
    constant cSYS_PIPE_DELAY    : integer := 3;                     -- Length of SystemBus Pipeline Delay
    constant cSYSMEN_ENABLE     : boolean := True;                  -- Enable SystemBus Memory Access
    constant cSYSMEM_START_ADDR : std_logic_vector(2 downto 0) := "100";    -- SystemBus Memory Start Address [eq. SysAddrxAI(16 downto 14)]

    -- LUT control register
    type tLUT_CTRL_REG is record
        GpioCtrl   : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);  -- GPIO Output Enable
        GpioData   : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);  -- GPIO Output Data
        GpioClkSel : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);  -- GPIO Output Clock Select
        GpioDivide : std_logic_vector(23 downto 0);
    end record;

    constant cLUT_CTRL_DEFAULT : tLUT_CTRL_REG := (
        GpioCtrl   => (others => '0'),
        GpioData   => (others => '0'),
        GpioClkSel => (others => '0'),
        GpioDivide => (others => '0'));

    -- LUT status register
    type tLUT_STAT_REG is record
        GpioStat : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Input Status
    end record;

    ----------------------------------------------------------------------------
    -- Version and Debug Definitions
    ----------------------------------------------------------------------------
    constant cVERSION_NUMBER   : integer := cCUSTOM_VERSION_NUM;        -- Set Custom Module version number
    constant cDEBUG_ENABLE     : boolean := cCUSTOM_DEBUG_ENABLE;       -- Enable/disable Custom Module debug mode
    constant cLOGIC_ENABLE     : boolean := cCUSTOM_ANALYZER_ENABLE;    -- Enable/disable Vivado Logic Analyzer
    constant cCHIPSCOPE_ENABLE : boolean := cCUSTOM_CHIPSCOPE_ENABLE;   -- Enable/disable ChipScopePro Analyzer
    constant cCSP_ICON_ENABLE  : boolean := cCUSTOM_CSP_ICON_ENABLE;    -- Enable/disable ICON module (JTAG mode only)
    constant cDEBUG_PORT_WIDTH : integer := cCUSTOM_DEBUG_PORT_WIDTH;   -- Width of ILA debug port

    ----------------------------------------------------------------------------
    -- GPIO Definitions
    ----------------------------------------------------------------------------
    -- For each GPIO output (OptoOut0..3, TTLOut0..1) define if the Custom Module is driving
    -- the selected signal via the CustIoDataOutxDO(x) pin (cCUSTOM_GPIO_ENABLE(i) = True).
    -- If (cCUSTOM_GPIO_ENABLE(i) = False) the associated CustIoCtrlxMO(i) signal is de-asserted
    -- and the Custom Module does not have control of the GPIO output pin on the FPGA.
    type tCUSTOM_GPIO_ARRAY is array (0 to cSYS_GPIO_WIDTH-1) of boolean;

    -- Enable individual GPIO signals in Custom Module:   Opto(0), Opto(1), Opto(2), Opto(3), TTL(0), TTL(1)
    constant cCUSTOM_GPIO_ENABLE : tCUSTOM_GPIO_ARRAY := (FALSE,   FALSE,   TRUE,    TRUE,    FALSE,  TRUE);

    ---------------------------------------------------------------------------
    -- LUT Initialization
    ---------------------------------------------------------------------------
    type tRAM64X32 is array (0 to 63) of bit_vector(255 downto 0);

    -- Default LUT initialization
    constant cDEFAULT_INIT : tRAM64X32 := (  -- 32 BYTES Entries
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_00
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_01
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_02
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_03
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_04
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_05
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_06
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_07
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_08
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_09
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_0A
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_0B
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_0C
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_0D
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_0E
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_0F
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_10
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_11
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_12
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_13
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_14
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_15
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_16
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_17
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_18
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_19
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_1A
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_1B
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_1C
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_1D
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_1E
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_1F
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_20
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_21
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_22
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_23
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_24
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_25
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_26
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_27
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_28
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_29
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_2A
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_2B
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_2C
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_2D
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_2E
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_2F
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_30
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_31
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_32
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_33
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_34
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_35
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_36
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_37
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_38
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_39
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_3A
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_3B
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_3C
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_3D
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_3E
        X"0000000000000000000000000000000000000000000000000000000000000000");   -- INIT_3F

    -- 8-bit Identity LUT initialization, data bits 7..0
    constant cLUT8_07_IDENT : tRAM64X32 := (  -- 32 BYTES Entries
        X"1F1E1D1C1B1A191817161514131211100F0E0D0C0B0A09080706050403020100",    -- INIT_00
        X"3F3E3D3C3B3A393837363534333231302F2E2D2C2B2A29282726252423222120",    -- INIT_01
        X"5F5E5D5C5B5A595857565554535251504F4E4D4C4B4A49484746454443424140",    -- INIT_02
        X"7F7E7D7C7B7A797877767574737271706F6E6D6C6B6A69686766656463626160",    -- INIT_03
        X"9F9E9D9C9B9A999897969594939291908F8E8D8C8B8A89888786858483828180",    -- INIT_04
        X"BFBEBDBCBBBAB9B8B7B6B5B4B3B2B1B0AFAEADACABAAA9A8A7A6A5A4A3A2A1A0",    -- INIT_05
        X"DFDEDDDCDBDAD9D8D7D6D5D4D3D2D1D0CFCECDCCCBCAC9C8C7C6C5C4C3C2C1C0",    -- INIT_06
        X"FFFEFDFCFBFAF9F8F7F6F5F4F3F2F1F0EFEEEDECEBEAE9E8E7E6E5E4E3E2E1E0",    -- INIT_07
        X"1F1E1D1C1B1A191817161514131211100F0E0D0C0B0A09080706050403020100",    -- INIT_08
        X"3F3E3D3C3B3A393837363534333231302F2E2D2C2B2A29282726252423222120",    -- INIT_09
        X"5F5E5D5C5B5A595857565554535251504F4E4D4C4B4A49484746454443424140",    -- INIT_0A
        X"7F7E7D7C7B7A797877767574737271706F6E6D6C6B6A69686766656463626160",    -- INIT_0B
        X"9F9E9D9C9B9A999897969594939291908F8E8D8C8B8A89888786858483828180",    -- INIT_0C
        X"BFBEBDBCBBBAB9B8B7B6B5B4B3B2B1B0AFAEADACABAAA9A8A7A6A5A4A3A2A1A0",    -- INIT_0D
        X"DFDEDDDCDBDAD9D8D7D6D5D4D3D2D1D0CFCECDCCCBCAC9C8C7C6C5C4C3C2C1C0",    -- INIT_0E
        X"FFFEFDFCFBFAF9F8F7F6F5F4F3F2F1F0EFEEEDECEBEAE9E8E7E6E5E4E3E2E1E0",    -- INIT_0F
        X"1F1E1D1C1B1A191817161514131211100F0E0D0C0B0A09080706050403020100",    -- INIT_10
        X"3F3E3D3C3B3A393837363534333231302F2E2D2C2B2A29282726252423222120",    -- INIT_11
        X"5F5E5D5C5B5A595857565554535251504F4E4D4C4B4A49484746454443424140",    -- INIT_12
        X"7F7E7D7C7B7A797877767574737271706F6E6D6C6B6A69686766656463626160",    -- INIT_13
        X"9F9E9D9C9B9A999897969594939291908F8E8D8C8B8A89888786858483828180",    -- INIT_14
        X"BFBEBDBCBBBAB9B8B7B6B5B4B3B2B1B0AFAEADACABAAA9A8A7A6A5A4A3A2A1A0",    -- INIT_15
        X"DFDEDDDCDBDAD9D8D7D6D5D4D3D2D1D0CFCECDCCCBCAC9C8C7C6C5C4C3C2C1C0",    -- INIT_16
        X"FFFEFDFCFBFAF9F8F7F6F5F4F3F2F1F0EFEEEDECEBEAE9E8E7E6E5E4E3E2E1E0",    -- INIT_17
        X"1F1E1D1C1B1A191817161514131211100F0E0D0C0B0A09080706050403020100",    -- INIT_18
        X"3F3E3D3C3B3A393837363534333231302F2E2D2C2B2A29282726252423222120",    -- INIT_19
        X"5F5E5D5C5B5A595857565554535251504F4E4D4C4B4A49484746454443424140",    -- INIT_1A
        X"7F7E7D7C7B7A797877767574737271706F6E6D6C6B6A69686766656463626160",    -- INIT_1B
        X"9F9E9D9C9B9A999897969594939291908F8E8D8C8B8A89888786858483828180",    -- INIT_1C
        X"BFBEBDBCBBBAB9B8B7B6B5B4B3B2B1B0AFAEADACABAAA9A8A7A6A5A4A3A2A1A0",    -- INIT_1D
        X"DFDEDDDCDBDAD9D8D7D6D5D4D3D2D1D0CFCECDCCCBCAC9C8C7C6C5C4C3C2C1C0",    -- INIT_1E
        X"FFFEFDFCFBFAF9F8F7F6F5F4F3F2F1F0EFEEEDECEBEAE9E8E7E6E5E4E3E2E1E0",    -- INIT_1F
        X"1F1E1D1C1B1A191817161514131211100F0E0D0C0B0A09080706050403020100",    -- INIT_20
        X"3F3E3D3C3B3A393837363534333231302F2E2D2C2B2A29282726252423222120",    -- INIT_21
        X"5F5E5D5C5B5A595857565554535251504F4E4D4C4B4A49484746454443424140",    -- INIT_22
        X"7F7E7D7C7B7A797877767574737271706F6E6D6C6B6A69686766656463626160",    -- INIT_23
        X"9F9E9D9C9B9A999897969594939291908F8E8D8C8B8A89888786858483828180",    -- INIT_24
        X"BFBEBDBCBBBAB9B8B7B6B5B4B3B2B1B0AFAEADACABAAA9A8A7A6A5A4A3A2A1A0",    -- INIT_25
        X"DFDEDDDCDBDAD9D8D7D6D5D4D3D2D1D0CFCECDCCCBCAC9C8C7C6C5C4C3C2C1C0",    -- INIT_26
        X"FFFEFDFCFBFAF9F8F7F6F5F4F3F2F1F0EFEEEDECEBEAE9E8E7E6E5E4E3E2E1E0",    -- INIT_27
        X"1F1E1D1C1B1A191817161514131211100F0E0D0C0B0A09080706050403020100",    -- INIT_28
        X"3F3E3D3C3B3A393837363534333231302F2E2D2C2B2A29282726252423222120",    -- INIT_29
        X"5F5E5D5C5B5A595857565554535251504F4E4D4C4B4A49484746454443424140",    -- INIT_2A
        X"7F7E7D7C7B7A797877767574737271706F6E6D6C6B6A69686766656463626160",    -- INIT_2B
        X"9F9E9D9C9B9A999897969594939291908F8E8D8C8B8A89888786858483828180",    -- INIT_2C
        X"BFBEBDBCBBBAB9B8B7B6B5B4B3B2B1B0AFAEADACABAAA9A8A7A6A5A4A3A2A1A0",    -- INIT_2D
        X"DFDEDDDCDBDAD9D8D7D6D5D4D3D2D1D0CFCECDCCCBCAC9C8C7C6C5C4C3C2C1C0",    -- INIT_2E
        X"FFFEFDFCFBFAF9F8F7F6F5F4F3F2F1F0EFEEEDECEBEAE9E8E7E6E5E4E3E2E1E0",    -- INIT_2F
        X"1F1E1D1C1B1A191817161514131211100F0E0D0C0B0A09080706050403020100",    -- INIT_30
        X"3F3E3D3C3B3A393837363534333231302F2E2D2C2B2A29282726252423222120",    -- INIT_31
        X"5F5E5D5C5B5A595857565554535251504F4E4D4C4B4A49484746454443424140",    -- INIT_32
        X"7F7E7D7C7B7A797877767574737271706F6E6D6C6B6A69686766656463626160",    -- INIT_33
        X"9F9E9D9C9B9A999897969594939291908F8E8D8C8B8A89888786858483828180",    -- INIT_34
        X"BFBEBDBCBBBAB9B8B7B6B5B4B3B2B1B0AFAEADACABAAA9A8A7A6A5A4A3A2A1A0",    -- INIT_35
        X"DFDEDDDCDBDAD9D8D7D6D5D4D3D2D1D0CFCECDCCCBCAC9C8C7C6C5C4C3C2C1C0",    -- INIT_36
        X"FFFEFDFCFBFAF9F8F7F6F5F4F3F2F1F0EFEEEDECEBEAE9E8E7E6E5E4E3E2E1E0",    -- INIT_37
        X"1F1E1D1C1B1A191817161514131211100F0E0D0C0B0A09080706050403020100",    -- INIT_38
        X"3F3E3D3C3B3A393837363534333231302F2E2D2C2B2A29282726252423222120",    -- INIT_39
        X"5F5E5D5C5B5A595857565554535251504F4E4D4C4B4A49484746454443424140",    -- INIT_3A
        X"7F7E7D7C7B7A797877767574737271706F6E6D6C6B6A69686766656463626160",    -- INIT_3B
        X"9F9E9D9C9B9A999897969594939291908F8E8D8C8B8A89888786858483828180",    -- INIT_3C
        X"BFBEBDBCBBBAB9B8B7B6B5B4B3B2B1B0AFAEADACABAAA9A8A7A6A5A4A3A2A1A0",    -- INIT_3D
        X"DFDEDDDCDBDAD9D8D7D6D5D4D3D2D1D0CFCECDCCCBCAC9C8C7C6C5C4C3C2C1C0",    -- INIT_3E
        X"FFFEFDFCFBFAF9F8F7F6F5F4F3F2F1F0EFEEEDECEBEAE9E8E7E6E5E4E3E2E1E0");   -- INIT_3F

    -- 8-bit Inverse LUT initialization, data bits 7..0
    constant cLUT8_07_INVERSE : tRAM64X32 := (  -- 32 BYTES Entries
        X"E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEFF0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF",    -- INIT_00
        X"C0C1C2C3C4C5C6C7C8C9CACBCCCDCECFD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF",    -- INIT_01
        X"A0A1A2A3A4A5A6A7A8A9AAABACADAEAFB0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF",    -- INIT_02
        X"808182838485868788898A8B8C8D8E8F909192939495969798999A9B9C9D9E9F",    -- INIT_03
        X"606162636465666768696A6B6C6D6E6F707172737475767778797A7B7C7D7E7F",    -- INIT_04
        X"404142434445464748494A4B4C4D4E4F505152535455565758595A5B5C5D5E5F",    -- INIT_05
        X"202122232425262728292A2B2C2D2E2F303132333435363738393A3B3C3D3E3F",    -- INIT_06
        X"000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F",    -- INIT_07
        X"E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEFF0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF",    -- INIT_08
        X"C0C1C2C3C4C5C6C7C8C9CACBCCCDCECFD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF",    -- INIT_09
        X"A0A1A2A3A4A5A6A7A8A9AAABACADAEAFB0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF",    -- INIT_0A
        X"808182838485868788898A8B8C8D8E8F909192939495969798999A9B9C9D9E9F",    -- INIT_0B
        X"606162636465666768696A6B6C6D6E6F707172737475767778797A7B7C7D7E7F",    -- INIT_0C
        X"404142434445464748494A4B4C4D4E4F505152535455565758595A5B5C5D5E5F",    -- INIT_0D
        X"202122232425262728292A2B2C2D2E2F303132333435363738393A3B3C3D3E3F",    -- INIT_0E
        X"000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F",    -- INIT_0F
        X"E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEFF0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF",    -- INIT_10
        X"C0C1C2C3C4C5C6C7C8C9CACBCCCDCECFD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF",    -- INIT_11
        X"A0A1A2A3A4A5A6A7A8A9AAABACADAEAFB0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF",    -- INIT_12
        X"808182838485868788898A8B8C8D8E8F909192939495969798999A9B9C9D9E9F",    -- INIT_13
        X"606162636465666768696A6B6C6D6E6F707172737475767778797A7B7C7D7E7F",    -- INIT_14
        X"404142434445464748494A4B4C4D4E4F505152535455565758595A5B5C5D5E5F",    -- INIT_15
        X"202122232425262728292A2B2C2D2E2F303132333435363738393A3B3C3D3E3F",    -- INIT_16
        X"000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F",    -- INIT_17
        X"E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEFF0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF",    -- INIT_18
        X"C0C1C2C3C4C5C6C7C8C9CACBCCCDCECFD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF",    -- INIT_19
        X"A0A1A2A3A4A5A6A7A8A9AAABACADAEAFB0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF",    -- INIT_1A
        X"808182838485868788898A8B8C8D8E8F909192939495969798999A9B9C9D9E9F",    -- INIT_1B
        X"606162636465666768696A6B6C6D6E6F707172737475767778797A7B7C7D7E7F",    -- INIT_1C
        X"404142434445464748494A4B4C4D4E4F505152535455565758595A5B5C5D5E5F",    -- INIT_1D
        X"202122232425262728292A2B2C2D2E2F303132333435363738393A3B3C3D3E3F",    -- INIT_1E
        X"000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F",    -- INIT_1F
        X"E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEFF0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF",    -- INIT_20
        X"C0C1C2C3C4C5C6C7C8C9CACBCCCDCECFD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF",    -- INIT_21
        X"A0A1A2A3A4A5A6A7A8A9AAABACADAEAFB0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF",    -- INIT_22
        X"808182838485868788898A8B8C8D8E8F909192939495969798999A9B9C9D9E9F",    -- INIT_23
        X"606162636465666768696A6B6C6D6E6F707172737475767778797A7B7C7D7E7F",    -- INIT_24
        X"404142434445464748494A4B4C4D4E4F505152535455565758595A5B5C5D5E5F",    -- INIT_25
        X"202122232425262728292A2B2C2D2E2F303132333435363738393A3B3C3D3E3F",    -- INIT_26
        X"000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F",    -- INIT_27
        X"E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEFF0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF",    -- INIT_28
        X"C0C1C2C3C4C5C6C7C8C9CACBCCCDCECFD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF",    -- INIT_29
        X"A0A1A2A3A4A5A6A7A8A9AAABACADAEAFB0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF",    -- INIT_2A
        X"808182838485868788898A8B8C8D8E8F909192939495969798999A9B9C9D9E9F",    -- INIT_2B
        X"606162636465666768696A6B6C6D6E6F707172737475767778797A7B7C7D7E7F",    -- INIT_2C
        X"404142434445464748494A4B4C4D4E4F505152535455565758595A5B5C5D5E5F",    -- INIT_2D
        X"202122232425262728292A2B2C2D2E2F303132333435363738393A3B3C3D3E3F",    -- INIT_2E
        X"000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F",    -- INIT_2F
        X"E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEFF0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF",    -- INIT_30
        X"C0C1C2C3C4C5C6C7C8C9CACBCCCDCECFD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF",    -- INIT_31
        X"A0A1A2A3A4A5A6A7A8A9AAABACADAEAFB0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF",    -- INIT_32
        X"808182838485868788898A8B8C8D8E8F909192939495969798999A9B9C9D9E9F",    -- INIT_33
        X"606162636465666768696A6B6C6D6E6F707172737475767778797A7B7C7D7E7F",    -- INIT_34
        X"404142434445464748494A4B4C4D4E4F505152535455565758595A5B5C5D5E5F",    -- INIT_35
        X"202122232425262728292A2B2C2D2E2F303132333435363738393A3B3C3D3E3F",    -- INIT_36
        X"000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F",    -- INIT_37
        X"E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEFF0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF",    -- INIT_38
        X"C0C1C2C3C4C5C6C7C8C9CACBCCCDCECFD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF",    -- INIT_39
        X"A0A1A2A3A4A5A6A7A8A9AAABACADAEAFB0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF",    -- INIT_3A
        X"808182838485868788898A8B8C8D8E8F909192939495969798999A9B9C9D9E9F",    -- INIT_3B
        X"606162636465666768696A6B6C6D6E6F707172737475767778797A7B7C7D7E7F",    -- INIT_3C
        X"404142434445464748494A4B4C4D4E4F505152535455565758595A5B5C5D5E5F",    -- INIT_3D
        X"202122232425262728292A2B2C2D2E2F303132333435363738393A3B3C3D3E3F",    -- INIT_3E
        X"000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F");   -- INIT_3F

    -- 12-bit Identity LUT initialization, data bits 3..0
    constant cLUT12_03_IDENT : tRAM64X32 := (  -- 32 BYTES Entries
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_00
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_01
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_02
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_03
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_04
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_05
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_06
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_07
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_08
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_09
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_0A
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_0B
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_0C
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_0D
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_0E
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_0F
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_10
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_11
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_12
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_13
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_14
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_15
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_16
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_17
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_18
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_19
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_1A
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_1B
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_1C
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_1D
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_1E
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_1F
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_20
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_21
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_22
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_23
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_24
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_25
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_26
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_27
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_28
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_29
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_2A
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_2B
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_2C
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_2D
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_2E
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_2F
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_30
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_31
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_32
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_33
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_34
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_35
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_36
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_37
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_38
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_39
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_3A
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_3B
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_3C
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_3D
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210",    -- INIT_3E
        X"FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210FEDCBA9876543210");   -- INIT_3F

    -- 12-bit Identity LUT initialization, data bits 7..4
    constant cLUT12_47_IDENT : tRAM64X32 := (  -- 32 BYTES Entries
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_00
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_01
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_02
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_03
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_04
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_05
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_06
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_07
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_08
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_09
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_0A
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_0B
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_0C
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_0D
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_0E
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_0F
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_10
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_11
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_12
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_13
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_14
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_15
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_16
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_17
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_18
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_19
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_1A
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_1B
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_1C
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_1D
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_1E
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_1F
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_20
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_21
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_22
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_23
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_24
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_25
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_26
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_27
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_28
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_29
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_2A
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_2B
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_2C
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_2D
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_2E
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_2F
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_30
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_31
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_32
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_33
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_34
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_35
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_36
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_37
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_38
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_39
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_3A
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC",    -- INIT_3B
        X"3333333333333333222222222222222211111111111111110000000000000000",    -- INIT_3C
        X"7777777777777777666666666666666655555555555555554444444444444444",    -- INIT_3D
        X"BBBBBBBBBBBBBBBBAAAAAAAAAAAAAAAA99999999999999998888888888888888",    -- INIT_3E
        X"FFFFFFFFFFFFFFFFEEEEEEEEEEEEEEEEDDDDDDDDDDDDDDDDCCCCCCCCCCCCCCCC");   -- INIT_3F

    -- 12-bit Identity LUT initialization, data bits 11..8
    constant cLUT12_811_IDENT : tRAM64X32 := (  -- 32 BYTES Entries
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_00
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_01
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_02
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_03
        X"1111111111111111111111111111111111111111111111111111111111111111",    -- INIT_04
        X"1111111111111111111111111111111111111111111111111111111111111111",    -- INIT_05
        X"1111111111111111111111111111111111111111111111111111111111111111",    -- INIT_06
        X"1111111111111111111111111111111111111111111111111111111111111111",    -- INIT_07
        X"2222222222222222222222222222222222222222222222222222222222222222",    -- INIT_08
        X"2222222222222222222222222222222222222222222222222222222222222222",    -- INIT_09
        X"2222222222222222222222222222222222222222222222222222222222222222",    -- INIT_0A
        X"2222222222222222222222222222222222222222222222222222222222222222",    -- INIT_0B
        X"3333333333333333333333333333333333333333333333333333333333333333",    -- INIT_0C
        X"3333333333333333333333333333333333333333333333333333333333333333",    -- INIT_0D
        X"3333333333333333333333333333333333333333333333333333333333333333",    -- INIT_0E
        X"3333333333333333333333333333333333333333333333333333333333333333",    -- INIT_0F
        X"4444444444444444444444444444444444444444444444444444444444444444",    -- INIT_10
        X"4444444444444444444444444444444444444444444444444444444444444444",    -- INIT_11
        X"4444444444444444444444444444444444444444444444444444444444444444",    -- INIT_12
        X"4444444444444444444444444444444444444444444444444444444444444444",    -- INIT_13
        X"5555555555555555555555555555555555555555555555555555555555555555",    -- INIT_14
        X"5555555555555555555555555555555555555555555555555555555555555555",    -- INIT_15
        X"5555555555555555555555555555555555555555555555555555555555555555",    -- INIT_16
        X"5555555555555555555555555555555555555555555555555555555555555555",    -- INIT_17
        X"6666666666666666666666666666666666666666666666666666666666666666",    -- INIT_18
        X"6666666666666666666666666666666666666666666666666666666666666666",    -- INIT_19
        X"6666666666666666666666666666666666666666666666666666666666666666",    -- INIT_1A
        X"6666666666666666666666666666666666666666666666666666666666666666",    -- INIT_1B
        X"7777777777777777777777777777777777777777777777777777777777777777",    -- INIT_1C
        X"7777777777777777777777777777777777777777777777777777777777777777",    -- INIT_1D
        X"7777777777777777777777777777777777777777777777777777777777777777",    -- INIT_1E
        X"7777777777777777777777777777777777777777777777777777777777777777",    -- INIT_1F
        X"8888888888888888888888888888888888888888888888888888888888888888",    -- INIT_20
        X"8888888888888888888888888888888888888888888888888888888888888888",    -- INIT_21
        X"8888888888888888888888888888888888888888888888888888888888888888",    -- INIT_22
        X"8888888888888888888888888888888888888888888888888888888888888888",    -- INIT_23
        X"9999999999999999999999999999999999999999999999999999999999999999",    -- INIT_24
        X"9999999999999999999999999999999999999999999999999999999999999999",    -- INIT_25
        X"9999999999999999999999999999999999999999999999999999999999999999",    -- INIT_26
        X"9999999999999999999999999999999999999999999999999999999999999999",    -- INIT_27
        X"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",    -- INIT_28
        X"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",    -- INIT_29
        X"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",    -- INIT_2A
        X"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",    -- INIT_2B
        X"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",    -- INIT_2C
        X"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",    -- INIT_2D
        X"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",    -- INIT_2E
        X"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",    -- INIT_2F
        X"CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",    -- INIT_30
        X"CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",    -- INIT_31
        X"CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",    -- INIT_32
        X"CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",    -- INIT_33
        X"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",    -- INIT_34
        X"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",    -- INIT_35
        X"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",    -- INIT_36
        X"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",    -- INIT_37
        X"EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE",    -- INIT_38
        X"EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE",    -- INIT_39
        X"EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE",    -- INIT_3A
        X"EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE",    -- INIT_3B
        X"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF",    -- INIT_3C
        X"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF",    -- INIT_3D
        X"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF",    -- INIT_3E
        X"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF");   -- INIT_3F


    -- 12-bit Inverse LUT initialization, data bits 3..0
    constant cLUT12_03_INVERSE : tRAM64X32 := (  -- 32 BYTES Entries
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_00
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_01
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_02
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_03
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_04
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_05
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_06
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_07
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_08
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_09
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_0A
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_0B
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_0C
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_0D
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_0E
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_0F
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_10
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_11
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_12
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_13
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_14
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_15
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_16
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_17
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_18
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_19
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_1A
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_1B
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_1C
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_1D
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_1E
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_1F
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_20
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_21
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_22
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_23
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_24
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_25
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_26
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_27
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_28
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_29
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_2A
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_2B
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_2C
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_2D
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_2E
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_2F
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_30
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_31
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_32
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_33
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_34
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_35
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_36
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_37
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_38
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_39
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_3A
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_3B
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_3C
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_3D
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF",    -- INIT_3E
        X"0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF");   -- INIT_3F

    -- 12-bit Inverse LUT initialization, data bits 7..4
    constant cLUT12_47_INVERSE : tRAM64X32 := (  -- 32 BYTES Entries
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_00
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_01
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_02
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_03
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_04
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_05
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_06
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_07
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_08
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_09
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_0A
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_0B
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_0C
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_0D
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_0E
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_0F
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_10
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_11
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_12
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_13
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_14
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_15
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_16
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_17
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_18
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_19
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_1A
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_1B
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_1C
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_1D
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_1E
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_1F
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_20
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_21
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_22
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_23
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_24
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_25
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_26
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_27
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_28
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_29
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_2A
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_2B
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_2C
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_2D
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_2E
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_2F
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_30
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_31
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_32
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_33
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_34
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_35
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_36
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_37
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_38
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_39
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_3A
        X"0000000000000000111111111111111122222222222222223333333333333333",    -- INIT_3B
        X"CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF",    -- INIT_3C
        X"88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB",    -- INIT_3D
        X"4444444444444444555555555555555566666666666666667777777777777777",    -- INIT_3E
        X"0000000000000000111111111111111122222222222222223333333333333333");   -- INIT_3F

    -- 12-bit Inverse LUT initialization, data bits 11..8
    constant cLUT12_811_INVERSE : tRAM64X32 := (  -- 32 BYTES Entries
        X"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF",    -- INIT_00
        X"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF",    -- INIT_01
        X"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF",    -- INIT_02
        X"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF",    -- INIT_03
        X"EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE",    -- INIT_04
        X"EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE",    -- INIT_05
        X"EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE",    -- INIT_06
        X"EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE",    -- INIT_07
        X"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",    -- INIT_08
        X"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",    -- INIT_09
        X"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",    -- INIT_0A
        X"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD",    -- INIT_0B
        X"CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",    -- INIT_0C
        X"CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",    -- INIT_0D
        X"CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",    -- INIT_0E
        X"CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",    -- INIT_0F
        X"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",    -- INIT_10
        X"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",    -- INIT_11
        X"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",    -- INIT_12
        X"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",    -- INIT_13
        X"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",    -- INIT_14
        X"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",    -- INIT_15
        X"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",    -- INIT_16
        X"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",    -- INIT_17
        X"9999999999999999999999999999999999999999999999999999999999999999",    -- INIT_18
        X"9999999999999999999999999999999999999999999999999999999999999999",    -- INIT_19
        X"9999999999999999999999999999999999999999999999999999999999999999",    -- INIT_1A
        X"9999999999999999999999999999999999999999999999999999999999999999",    -- INIT_1B
        X"8888888888888888888888888888888888888888888888888888888888888888",    -- INIT_1C
        X"8888888888888888888888888888888888888888888888888888888888888888",    -- INIT_1D
        X"8888888888888888888888888888888888888888888888888888888888888888",    -- INIT_1E
        X"8888888888888888888888888888888888888888888888888888888888888888",    -- INIT_1F
        X"7777777777777777777777777777777777777777777777777777777777777777",    -- INIT_20
        X"7777777777777777777777777777777777777777777777777777777777777777",    -- INIT_21
        X"7777777777777777777777777777777777777777777777777777777777777777",    -- INIT_22
        X"7777777777777777777777777777777777777777777777777777777777777777",    -- INIT_23
        X"6666666666666666666666666666666666666666666666666666666666666666",    -- INIT_24
        X"6666666666666666666666666666666666666666666666666666666666666666",    -- INIT_25
        X"6666666666666666666666666666666666666666666666666666666666666666",    -- INIT_26
        X"6666666666666666666666666666666666666666666666666666666666666666",    -- INIT_27
        X"5555555555555555555555555555555555555555555555555555555555555555",    -- INIT_28
        X"5555555555555555555555555555555555555555555555555555555555555555",    -- INIT_29
        X"5555555555555555555555555555555555555555555555555555555555555555",    -- INIT_2A
        X"5555555555555555555555555555555555555555555555555555555555555555",    -- INIT_2B
        X"4444444444444444444444444444444444444444444444444444444444444444",    -- INIT_2C
        X"4444444444444444444444444444444444444444444444444444444444444444",    -- INIT_2D
        X"4444444444444444444444444444444444444444444444444444444444444444",    -- INIT_2E
        X"4444444444444444444444444444444444444444444444444444444444444444",    -- INIT_2F
        X"3333333333333333333333333333333333333333333333333333333333333333",    -- INIT_30
        X"3333333333333333333333333333333333333333333333333333333333333333",    -- INIT_31
        X"3333333333333333333333333333333333333333333333333333333333333333",    -- INIT_32
        X"3333333333333333333333333333333333333333333333333333333333333333",    -- INIT_33
        X"2222222222222222222222222222222222222222222222222222222222222222",    -- INIT_34
        X"2222222222222222222222222222222222222222222222222222222222222222",    -- INIT_35
        X"2222222222222222222222222222222222222222222222222222222222222222",    -- INIT_36
        X"2222222222222222222222222222222222222222222222222222222222222222",    -- INIT_37
        X"1111111111111111111111111111111111111111111111111111111111111111",    -- INIT_38
        X"1111111111111111111111111111111111111111111111111111111111111111",    -- INIT_39
        X"1111111111111111111111111111111111111111111111111111111111111111",    -- INIT_3A
        X"1111111111111111111111111111111111111111111111111111111111111111",    -- INIT_3B
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_3C
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_3D
        X"0000000000000000000000000000000000000000000000000000000000000000",    -- INIT_3E
        X"0000000000000000000000000000000000000000000000000000000000000000");   -- INIT_3F

end LutPkg;
