-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : BlockRAM for OpenCamera LUT design
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-20  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library unisim;
use unisim.vcomponents.all;
library unimacro;
use unimacro.vcomponents.all;

use work.LutPkg.all;

entity LutRAM is
    generic (
        gLUT_DATA_WIDTH : integer   := 8;              -- LUT data/address width
        gLUT8_07_INIT   : tRAM64X32 := cDEFAULT_INIT;  -- LUT data bits 0..7 initialization (for 8-bit LUT)
        gLUT12_03_INIT  : tRAM64X32 := cDEFAULT_INIT;  -- LUT data bits 0..3 initialization (for 12-bit LUT)
        gLUT12_47_INIT  : tRAM64X32 := cDEFAULT_INIT;  -- LUT data bits 4..7 initialization (for 12-bit LUT)
        gLUT12_811_INIT : tRAM64X32 := cDEFAULT_INIT); -- LUT data bits 8..11 initialization (for 12-bit LUT)
    port (
        -- SystemBus Port
        SysClkxCI    : in  std_logic;
        SysRstxRI    : in  std_logic;
        SysEnxMI     : in  std_logic;
        SysWrEnxMI   : in  std_logic;
        SysAddrxAI   : in  std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);
        SysWrDataxDI : in  std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);
        SysRdDataxDO : out std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);

        -- ImageBus Port
        ImgClkxCI    : in  std_logic;
        ImgRstxRI    : in  std_logic;
        ImgWrDataxDI : in  std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);
        ImgRdDataxDO : out std_logic_vector(gLUT_DATA_WIDTH-1 downto 0));
end LutRAM;

architecture structural of LutRAM is

    -- +----------------------------------------------------------------------+
    -- |            7-Series BRAM_TDP_MACRO (True Dual Port RAM)              |
    -- +----------------+-----------+-----------+---------------+-------------+
    -- | DATA_WIDTH_A/B | BRAM_SIZE | RAM Depth | ADDRA/B Width | WEA/B Width |
    -- +----------------+-----------+-----------+---------------+-------------+
    -- |      19-36     |   "36Kb"  |   1024    |     10-bit    |    4-bit    |
    -- |      10-18     |   "36Kb"  |   2048    |     11-bit    |    2-bit    |
    -- |       5-9      |   "36Kb"  |   4096    |     12-bit    |    1-bit    |
    -- |       3-4      |   "36Kb"  |   8192    |     13-bit    |    1-bit    |
    -- |        2       |   "36Kb"  |   16384   |     14-bit    |    1-bit    |
    -- |        1       |   "36Kb"  |   32768   |     15-bit    |    1-bit    |
    -- +----------------+-----------+-----------+---------------+-------------+
    -- |      10-18     |   "18Kb"  |   1024    |     10-bit    |    2-bit    |
    -- |       5-9      |   "18Kb"  |   2048    |     11-bit    |    1-bit    |
    -- |       3-4      |   "18Kb"  |   4096    |     12-bit    |    1-bit    |
    -- |        2       |   "18Kb"  |   8192    |     13-bit    |    1-bit    |
    -- |        1       |   "18Kb"  |   16384   |     14-bit    |    1-bit    |
    -- +----------------+-----------+-----------+---------------+-------------+

    component BRAM_TDP_MACRO is
    generic (
        BRAM_SIZE           : string     := "18Kb";
        DEVICE              : string     := "VIRTEX5";
        INIT_FILE           : string     := "NONE";
        SIM_COLLISION_CHECK : string     := "ALL";

        DOA_REG             : integer    := 0;
        INIT_A              : bit_vector := X"000000000";
        SRVAL_A             : bit_vector := X"000000000";
        WRITE_MODE_A        : string     := "WRITE_FIRST";
        WRITE_WIDTH_A       : integer    := 1;
        READ_WIDTH_A        : integer    := 1;

        DOB_REG             : integer    := 0;
        INIT_B              : bit_vector := X"000000000";
        SRVAL_B             : bit_vector := X"000000000";
        WRITE_MODE_B        : string     := "WRITE_FIRST";
        WRITE_WIDTH_B       : integer    := 1;
        READ_WIDTH_B        : integer    := 1;

        INIT_00  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_01  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_02  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_03  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_04  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_05  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_06  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_07  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_08  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_09  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_0A  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_0B  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_0C  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_0D  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_0E  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_0F  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_10  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_11  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_12  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_13  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_14  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_15  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_16  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_17  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_18  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_19  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_1A  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_1B  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_1C  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_1D  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_1E  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_1F  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_20  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_21  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_22  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_23  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_24  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_25  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_26  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_27  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_28  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_29  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_2A  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_2B  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_2C  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_2D  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_2E  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_2F  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_30  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_31  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_32  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_33  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_34  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_35  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_36  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_37  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_38  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_39  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_3A  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_3B  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_3C  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_3D  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_3E  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000";
        INIT_3F  : bit_vector := X"0000000000000000000000000000000000000000000000000000000000000000");
    port (
        CLKA   : in std_ulogic;
        RSTA   : in std_ulogic;
        REGCEA : in std_ulogic;
        ENA    : in std_ulogic;
        WEA    : in std_logic_vector;
        ADDRA  : in std_logic_vector;
        DIA    : in std_logic_vector(WRITE_WIDTH_A-1 downto 0);
        DOA    : out std_logic_vector(READ_WIDTH_A-1 downto 0);

        CLKB   : in std_ulogic;
        RSTB   : in std_ulogic;
        REGCEB : in std_ulogic;
        ENB    : in std_ulogic;
        WEB    : in std_logic_vector;
        ADDRB  : in std_logic_vector;
        DIB    : in std_logic_vector(WRITE_WIDTH_B-1 downto 0);
        DOB    : out std_logic_vector(READ_WIDTH_B-1 downto 0));
    end component;

    ---------------------------------------------------------------------------
    -- Determine the BlockRAM address bus width
    ---------------------------------------------------------------------------
    function GetRamAddrWidth (constant DataWidth : integer) return integer is
        variable vActAddrWidth : integer := 0;
    begin
        if (DataWidth = 8) then
            vActAddrWidth := 11;
        elsif (DataWidth = 12) then
            vActAddrWidth := 12;
        else
            vActAddrWidth := DataWidth;
            report "[Custom Module] ERROR - LUT BlockRAM address width not supported."
            severity failure;
        end if;

        return vActAddrWidth;
    end;

    ---------------------------------------------------------------------------
    -- Constant Definitions
    ---------------------------------------------------------------------------
    constant cLUT_ADDR_WIDTH : integer := GetRamAddrWidth(gLUT_DATA_WIDTH);

    ---------------------------------------------------------------------------
    -- Signal Declarations
    ---------------------------------------------------------------------------
    signal SysWrEnxM   : std_logic_vector(0 downto 0);
    signal SysAddrxA   : std_logic_vector(cLUT_ADDR_WIDTH-1 downto 0);
    signal SysWrDataxD : std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);
    signal SysRdDataxD : std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);

    signal ImgWrEnxM   : std_logic_vector(0 downto 0);
    signal ImgAddrxA   : std_logic_vector(cLUT_ADDR_WIDTH-1 downto 0);
    signal ImgRdDataxD : std_logic_vector(gLUT_DATA_WIDTH-1 downto 0);

begin
    -- Assign LUT address and data ports to fixed 12-bit signals
    LutAddrProc: process(SysAddrxAI, SysWrDataxDI, ImgWrDataxDI, SysRdDataxD, ImgRdDataxD)
    begin
        SysAddrxA   <= (others => '0');
        SysWrDataxD <= (others => '0');
        ImgAddrxA   <= (others => '0');
        SysAddrxA(gLUT_DATA_WIDTH-1 downto 0)   <= SysAddrxAI;
        SysWrDataxD(gLUT_DATA_WIDTH-1 downto 0) <= SysWrDataxDI;
        ImgAddrxA(gLUT_DATA_WIDTH-1 downto 0)   <= ImgWrDataxDI;

        SysRdDataxDO <= SysRdDataxD(gLUT_DATA_WIDTH-1 downto 0);
        ImgRdDataxDO <= ImgRdDataxD(gLUT_DATA_WIDTH-1 downto 0);
    end process;

    -- Convert the SystemBus Write-Enable signal to std_logic_vector.
    ImgWrEnxM    <= (others => '0');
    SysWrEnxM(0) <= SysWrEnxMI;

    -- 8-bit LUT instantiation
    Lut8RamGen: if (gLUT_DATA_WIDTH = 8) generate
        -- Instantiate LUT RAM on bit slice 0..7
        Lut03RamInst : BRAM_TDP_MACRO
        generic map (
            BRAM_SIZE           => "18Kb",              -- Target BRAM, "18Kb" or "36Kb"
            DEVICE              => "7SERIES",           -- Target Device: "VIRTEX5", "VIRTEX6", "7SERIES"
            INIT_FILE           => "NONE",
            SIM_COLLISION_CHECK => "NONE",              -- Collision check enable "ALL", "WARNING_ONLY", "GENERATE_X_ONLY" or "NONE"
    
            DOA_REG             => 1,                   -- Optional port A output register (0 or 1)
            INIT_A              => x"000000000",        -- Initial values on A output port
            SRVAL_A             => x"000000000",        -- Set/Reset value for A port output
            WRITE_MODE_A        => "WRITE_FIRST",       -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
            WRITE_WIDTH_A       => 8,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
            READ_WIDTH_A        => 8,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
    
            DOB_REG             => 1,                   -- Optional port B output register (0 or 1)
            INIT_B              => x"000000000",        -- Initial values on B output port
            SRVAL_B             => x"000000000",        -- Set/Reset value for B port output
            WRITE_MODE_B        => "WRITE_FIRST",       -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
            WRITE_WIDTH_B       => 8,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
            READ_WIDTH_B        => 8,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
    
            -- The following INIT_xx declarations specify the initial contents of the RAM
            INIT_00 => gLUT8_07_INIT(0),
            INIT_01 => gLUT8_07_INIT(1),
            INIT_02 => gLUT8_07_INIT(2),
            INIT_03 => gLUT8_07_INIT(3),
            INIT_04 => gLUT8_07_INIT(4),
            INIT_05 => gLUT8_07_INIT(5),
            INIT_06 => gLUT8_07_INIT(6),
            INIT_07 => gLUT8_07_INIT(7),
            INIT_08 => gLUT8_07_INIT(8),
            INIT_09 => gLUT8_07_INIT(9),
            INIT_0A => gLUT8_07_INIT(10),
            INIT_0B => gLUT8_07_INIT(11),
            INIT_0C => gLUT8_07_INIT(12),
            INIT_0D => gLUT8_07_INIT(13),
            INIT_0E => gLUT8_07_INIT(14),
            INIT_0F => gLUT8_07_INIT(15),
            INIT_10 => gLUT8_07_INIT(16),
            INIT_11 => gLUT8_07_INIT(17),
            INIT_12 => gLUT8_07_INIT(18),
            INIT_13 => gLUT8_07_INIT(19),
            INIT_14 => gLUT8_07_INIT(20),
            INIT_15 => gLUT8_07_INIT(21),
            INIT_16 => gLUT8_07_INIT(22),
            INIT_17 => gLUT8_07_INIT(23),
            INIT_18 => gLUT8_07_INIT(24),
            INIT_19 => gLUT8_07_INIT(25),
            INIT_1A => gLUT8_07_INIT(26),
            INIT_1B => gLUT8_07_INIT(27),
            INIT_1C => gLUT8_07_INIT(28),
            INIT_1D => gLUT8_07_INIT(29),
            INIT_1E => gLUT8_07_INIT(30),
            INIT_1F => gLUT8_07_INIT(31),
            INIT_20 => gLUT8_07_INIT(32),
            INIT_21 => gLUT8_07_INIT(33),
            INIT_22 => gLUT8_07_INIT(34),
            INIT_23 => gLUT8_07_INIT(35),
            INIT_24 => gLUT8_07_INIT(36),
            INIT_25 => gLUT8_07_INIT(37),
            INIT_26 => gLUT8_07_INIT(38),
            INIT_27 => gLUT8_07_INIT(39),
            INIT_28 => gLUT8_07_INIT(40),
            INIT_29 => gLUT8_07_INIT(41),
            INIT_2A => gLUT8_07_INIT(42),
            INIT_2B => gLUT8_07_INIT(43),
            INIT_2C => gLUT8_07_INIT(44),
            INIT_2D => gLUT8_07_INIT(45),
            INIT_2E => gLUT8_07_INIT(46),
            INIT_2F => gLUT8_07_INIT(47),
            INIT_30 => gLUT8_07_INIT(48),
            INIT_31 => gLUT8_07_INIT(49),
            INIT_32 => gLUT8_07_INIT(50),
            INIT_33 => gLUT8_07_INIT(51),
            INIT_34 => gLUT8_07_INIT(52),
            INIT_35 => gLUT8_07_INIT(53),
            INIT_36 => gLUT8_07_INIT(54),
            INIT_37 => gLUT8_07_INIT(55),
            INIT_38 => gLUT8_07_INIT(56),
            INIT_39 => gLUT8_07_INIT(57),
            INIT_3A => gLUT8_07_INIT(58),
            INIT_3B => gLUT8_07_INIT(59),
            INIT_3C => gLUT8_07_INIT(60),
            INIT_3D => gLUT8_07_INIT(61),
            INIT_3E => gLUT8_07_INIT(62),
            INIT_3F => gLUT8_07_INIT(63))
        port map (
            CLKA        => SysClkxCI,                           -- 1-bit input port-A clock
            RSTA        => SysRstxRI,                           -- 1-bit input port-A reset
            REGCEA      => '1',                                 -- 1-bit input port-A output register enable
            ENA         => SysEnxMI,                            -- 1-bit input port-A enable
            WEA         => SysWrEnxM,                           -- Input port-A write enable, width defined by Port A depth
            ADDRA       => SysAddrxA,                           -- Input port-A address, width defined by Port A depth
            DIA         => SysWrDataxD(7 downto 0),             -- Input port-A data, width defined by WRITE_WIDTH_A parameter
            DOA         => SysRdDataxD(7 downto 0),             -- Output port-A data, width defined by READ_WIDTH_A parameter
    
            CLKB        => ImgClkxCI,                           -- 1-bit input port-B clock
            RSTB        => ImgRstxRI,                           -- 1-bit input port-B reset
            REGCEB      => '1',                                 -- 1-bit input port-B output register enable
            ENB         => '1',                                 -- 1-bit input port-B enable
            WEB         => ImgWrEnxM,                           -- Input port-B write enable, width defined by Port B depth
            ADDRB       => ImgAddrxA,                           -- Input port-B address, width defined by Port B depth
            DIB         => (others => '0'),                     -- Input port-B data, width defined by WRITE_WIDTH_B parameter
            DOB         => ImgRdDataxD(7 downto 0));            -- Output port-B data, width defined by READ_WIDTH_B parameter
    end generate;

    -- 12-bit LUT instantiation
    Lut12RamGen: if (gLUT_DATA_WIDTH = 12) generate
        -- Instantiate LUT RAM on bit slice 0..3
        Lut03RamInst : BRAM_TDP_MACRO
        generic map (
            BRAM_SIZE           => "18Kb",              -- Target BRAM, "18Kb" or "36Kb"
            DEVICE              => "7SERIES",           -- Target Device: "VIRTEX5", "VIRTEX6", "7SERIES"
            INIT_FILE           => "NONE",
            SIM_COLLISION_CHECK => "NONE",              -- Collision check enable "ALL", "WARNING_ONLY", "GENERATE_X_ONLY" or "NONE"
    
            DOA_REG             => 1,                   -- Optional port A output register (0 or 1)
            INIT_A              => x"000000000",        -- Initial values on A output port
            SRVAL_A             => x"000000000",        -- Set/Reset value for A port output
            WRITE_MODE_A        => "WRITE_FIRST",       -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
            WRITE_WIDTH_A       => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
            READ_WIDTH_A        => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
    
            DOB_REG             => 1,                   -- Optional port B output register (0 or 1)
            INIT_B              => x"000000000",        -- Initial values on B output port
            SRVAL_B             => x"000000000",        -- Set/Reset value for B port output
            WRITE_MODE_B        => "WRITE_FIRST",       -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
            WRITE_WIDTH_B       => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
            READ_WIDTH_B        => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
    
            -- The following INIT_xx declarations specify the initial contents of the RAM
            INIT_00 => gLUT12_03_INIT(0),
            INIT_01 => gLUT12_03_INIT(1),
            INIT_02 => gLUT12_03_INIT(2),
            INIT_03 => gLUT12_03_INIT(3),
            INIT_04 => gLUT12_03_INIT(4),
            INIT_05 => gLUT12_03_INIT(5),
            INIT_06 => gLUT12_03_INIT(6),
            INIT_07 => gLUT12_03_INIT(7),
            INIT_08 => gLUT12_03_INIT(8),
            INIT_09 => gLUT12_03_INIT(9),
            INIT_0A => gLUT12_03_INIT(10),
            INIT_0B => gLUT12_03_INIT(11),
            INIT_0C => gLUT12_03_INIT(12),
            INIT_0D => gLUT12_03_INIT(13),
            INIT_0E => gLUT12_03_INIT(14),
            INIT_0F => gLUT12_03_INIT(15),
            INIT_10 => gLUT12_03_INIT(16),
            INIT_11 => gLUT12_03_INIT(17),
            INIT_12 => gLUT12_03_INIT(18),
            INIT_13 => gLUT12_03_INIT(19),
            INIT_14 => gLUT12_03_INIT(20),
            INIT_15 => gLUT12_03_INIT(21),
            INIT_16 => gLUT12_03_INIT(22),
            INIT_17 => gLUT12_03_INIT(23),
            INIT_18 => gLUT12_03_INIT(24),
            INIT_19 => gLUT12_03_INIT(25),
            INIT_1A => gLUT12_03_INIT(26),
            INIT_1B => gLUT12_03_INIT(27),
            INIT_1C => gLUT12_03_INIT(28),
            INIT_1D => gLUT12_03_INIT(29),
            INIT_1E => gLUT12_03_INIT(30),
            INIT_1F => gLUT12_03_INIT(31),
            INIT_20 => gLUT12_03_INIT(32),
            INIT_21 => gLUT12_03_INIT(33),
            INIT_22 => gLUT12_03_INIT(34),
            INIT_23 => gLUT12_03_INIT(35),
            INIT_24 => gLUT12_03_INIT(36),
            INIT_25 => gLUT12_03_INIT(37),
            INIT_26 => gLUT12_03_INIT(38),
            INIT_27 => gLUT12_03_INIT(39),
            INIT_28 => gLUT12_03_INIT(40),
            INIT_29 => gLUT12_03_INIT(41),
            INIT_2A => gLUT12_03_INIT(42),
            INIT_2B => gLUT12_03_INIT(43),
            INIT_2C => gLUT12_03_INIT(44),
            INIT_2D => gLUT12_03_INIT(45),
            INIT_2E => gLUT12_03_INIT(46),
            INIT_2F => gLUT12_03_INIT(47),
            INIT_30 => gLUT12_03_INIT(48),
            INIT_31 => gLUT12_03_INIT(49),
            INIT_32 => gLUT12_03_INIT(50),
            INIT_33 => gLUT12_03_INIT(51),
            INIT_34 => gLUT12_03_INIT(52),
            INIT_35 => gLUT12_03_INIT(53),
            INIT_36 => gLUT12_03_INIT(54),
            INIT_37 => gLUT12_03_INIT(55),
            INIT_38 => gLUT12_03_INIT(56),
            INIT_39 => gLUT12_03_INIT(57),
            INIT_3A => gLUT12_03_INIT(58),
            INIT_3B => gLUT12_03_INIT(59),
            INIT_3C => gLUT12_03_INIT(60),
            INIT_3D => gLUT12_03_INIT(61),
            INIT_3E => gLUT12_03_INIT(62),
            INIT_3F => gLUT12_03_INIT(63))
        port map (
            CLKA        => SysClkxCI,                           -- 1-bit input port-A clock
            RSTA        => SysRstxRI,                           -- 1-bit input port-A reset
            REGCEA      => '1',                                 -- 1-bit input port-A output register enable
            ENA         => SysEnxMI,                            -- 1-bit input port-A enable
            WEA         => SysWrEnxM,                           -- Input port-A write enable, width defined by Port A depth
            ADDRA       => SysAddrxA,                           -- Input port-A address, width defined by Port A depth
            DIA         => SysWrDataxD(3 downto 0),             -- Input port-A data, width defined by WRITE_WIDTH_A parameter
            DOA         => SysRdDataxD(3 downto 0),             -- Output port-A data, width defined by READ_WIDTH_A parameter
    
            CLKB        => ImgClkxCI,                           -- 1-bit input port-B clock
            RSTB        => ImgRstxRI,                           -- 1-bit input port-B reset
            REGCEB      => '1',                                 -- 1-bit input port-B output register enable
            ENB         => '1',                                 -- 1-bit input port-B enable
            WEB         => ImgWrEnxM,                           -- Input port-B write enable, width defined by Port B depth
            ADDRB       => ImgAddrxA,                           -- Input port-B address, width defined by Port B depth
            DIB         => (others => '0'),                     -- Input port-B data, width defined by WRITE_WIDTH_B parameter
            DOB         => ImgRdDataxD(3 downto 0));            -- Output port-B data, width defined by READ_WIDTH_B parameter
    
        -- Instantiate LUT RAM on bit slice 4..7
        Lut47RamInst : BRAM_TDP_MACRO
        generic map (
            BRAM_SIZE           => "18Kb",              -- Target BRAM, "18Kb" or "36Kb"
            DEVICE              => "7SERIES",           -- Target Device: "VIRTEX5", "VIRTEX6", "7SERIES"
            INIT_FILE           => "NONE",
            SIM_COLLISION_CHECK => "NONE",              -- Collision check enable "ALL", "WARNING_ONLY", "GENERATE_X_ONLY" or "NONE"
    
            DOA_REG             => 1,                   -- Optional port A output register (0 or 1)
            INIT_A              => x"000000000",        -- Initial values on A output port
            SRVAL_A             => x"000000000",        -- Set/Reset value for A port output
            WRITE_MODE_A        => "WRITE_FIRST",       -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
            WRITE_WIDTH_A       => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
            READ_WIDTH_A        => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
    
            DOB_REG             => 1,                   -- Optional port B output register (0 or 1)
            INIT_B              => x"000000000",        -- Initial values on B output port
            SRVAL_B             => x"000000000",        -- Set/Reset value for B port output
            WRITE_MODE_B        => "WRITE_FIRST",       -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
            WRITE_WIDTH_B       => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
            READ_WIDTH_B        => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
    
            -- The following INIT_xx declarations specify the initial contents of the RAM
            INIT_00 => gLUT12_47_INIT(0),
            INIT_01 => gLUT12_47_INIT(1),
            INIT_02 => gLUT12_47_INIT(2),
            INIT_03 => gLUT12_47_INIT(3),
            INIT_04 => gLUT12_47_INIT(4),
            INIT_05 => gLUT12_47_INIT(5),
            INIT_06 => gLUT12_47_INIT(6),
            INIT_07 => gLUT12_47_INIT(7),
            INIT_08 => gLUT12_47_INIT(8),
            INIT_09 => gLUT12_47_INIT(9),
            INIT_0A => gLUT12_47_INIT(10),
            INIT_0B => gLUT12_47_INIT(11),
            INIT_0C => gLUT12_47_INIT(12),
            INIT_0D => gLUT12_47_INIT(13),
            INIT_0E => gLUT12_47_INIT(14),
            INIT_0F => gLUT12_47_INIT(15),
            INIT_10 => gLUT12_47_INIT(16),
            INIT_11 => gLUT12_47_INIT(17),
            INIT_12 => gLUT12_47_INIT(18),
            INIT_13 => gLUT12_47_INIT(19),
            INIT_14 => gLUT12_47_INIT(20),
            INIT_15 => gLUT12_47_INIT(21),
            INIT_16 => gLUT12_47_INIT(22),
            INIT_17 => gLUT12_47_INIT(23),
            INIT_18 => gLUT12_47_INIT(24),
            INIT_19 => gLUT12_47_INIT(25),
            INIT_1A => gLUT12_47_INIT(26),
            INIT_1B => gLUT12_47_INIT(27),
            INIT_1C => gLUT12_47_INIT(28),
            INIT_1D => gLUT12_47_INIT(29),
            INIT_1E => gLUT12_47_INIT(30),
            INIT_1F => gLUT12_47_INIT(31),
            INIT_20 => gLUT12_47_INIT(32),
            INIT_21 => gLUT12_47_INIT(33),
            INIT_22 => gLUT12_47_INIT(34),
            INIT_23 => gLUT12_47_INIT(35),
            INIT_24 => gLUT12_47_INIT(36),
            INIT_25 => gLUT12_47_INIT(37),
            INIT_26 => gLUT12_47_INIT(38),
            INIT_27 => gLUT12_47_INIT(39),
            INIT_28 => gLUT12_47_INIT(40),
            INIT_29 => gLUT12_47_INIT(41),
            INIT_2A => gLUT12_47_INIT(42),
            INIT_2B => gLUT12_47_INIT(43),
            INIT_2C => gLUT12_47_INIT(44),
            INIT_2D => gLUT12_47_INIT(45),
            INIT_2E => gLUT12_47_INIT(46),
            INIT_2F => gLUT12_47_INIT(47),
            INIT_30 => gLUT12_47_INIT(48),
            INIT_31 => gLUT12_47_INIT(49),
            INIT_32 => gLUT12_47_INIT(50),
            INIT_33 => gLUT12_47_INIT(51),
            INIT_34 => gLUT12_47_INIT(52),
            INIT_35 => gLUT12_47_INIT(53),
            INIT_36 => gLUT12_47_INIT(54),
            INIT_37 => gLUT12_47_INIT(55),
            INIT_38 => gLUT12_47_INIT(56),
            INIT_39 => gLUT12_47_INIT(57),
            INIT_3A => gLUT12_47_INIT(58),
            INIT_3B => gLUT12_47_INIT(59),
            INIT_3C => gLUT12_47_INIT(60),
            INIT_3D => gLUT12_47_INIT(61),
            INIT_3E => gLUT12_47_INIT(62),
            INIT_3F => gLUT12_47_INIT(63))
        port map (
            CLKA        => SysClkxCI,                           -- 1-bit input port-A clock
            RSTA        => SysRstxRI,                           -- 1-bit input port-A reset
            REGCEA      => '1',                                 -- 1-bit input port-A output register enable
            ENA         => SysEnxMI,                            -- 1-bit input port-A enable
            WEA         => SysWrEnxM,                           -- Input port-A write enable, width defined by Port A depth
            ADDRA       => SysAddrxA,                           -- Input port-A address, width defined by Port A depth
            DIA         => SysWrDataxD(7 downto 4),             -- Input port-A data, width defined by WRITE_WIDTH_A parameter
            DOA         => SysRdDataxD(7 downto 4),             -- Output port-A data, width defined by READ_WIDTH_A parameter
    
            CLKB        => ImgClkxCI,                           -- 1-bit input port-B clock
            RSTB        => ImgRstxRI,                           -- 1-bit input port-B reset
            REGCEB      => '1',                                 -- 1-bit input port-B output register enable
            ENB         => '1',                                 -- 1-bit input port-B enable
            WEB         => ImgWrEnxM,                           -- Input port-B write enable, width defined by Port B depth
            ADDRB       => ImgAddrxA,                           -- Input port-B address, width defined by Port B depth
            DIB         => (others => '0'),                     -- Input port-B data, width defined by WRITE_WIDTH_B parameter
            DOB         => ImgRdDataxD(7 downto 4));            -- Output port-B data, width defined by READ_WIDTH_B parameter

        -- Instantiate LUT RAM on bit slice 8..11
        Lut811RamInst : BRAM_TDP_MACRO
        generic map (
            BRAM_SIZE           => "18Kb",              -- Target BRAM, "18Kb" or "36Kb"
            DEVICE              => "7SERIES",           -- Target Device: "VIRTEX5", "VIRTEX6", "7SERIES"
            INIT_FILE           => "NONE",
            SIM_COLLISION_CHECK => "NONE",              -- Collision check enable "ALL", "WARNING_ONLY", "GENERATE_X_ONLY" or "NONE"

            DOA_REG             => 1,                   -- Optional port A output register (0 or 1)
            INIT_A              => x"000000000",        -- Initial values on A output port
            SRVAL_A             => x"000000000",        -- Set/Reset value for A port output
            WRITE_MODE_A        => "WRITE_FIRST",       -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
            WRITE_WIDTH_A       => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
            READ_WIDTH_A        => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")

            DOB_REG             => 1,                   -- Optional port B output register (0 or 1)
            INIT_B              => x"000000000",        -- Initial values on B output port
            SRVAL_B             => x"000000000",        -- Set/Reset value for B port output
            WRITE_MODE_B        => "WRITE_FIRST",       -- "WRITE_FIRST", "READ_FIRST" or "NO_CHANGE"
            WRITE_WIDTH_B       => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")
            READ_WIDTH_B        => 4,                   -- Valid values are 1-36 (19-36 only valid when BRAM_SIZE="36Kb")

            -- The following INIT_xx declarations specify the initial contents of the RAM
            INIT_00 => gLUT12_811_INIT(0),
            INIT_01 => gLUT12_811_INIT(1),
            INIT_02 => gLUT12_811_INIT(2),
            INIT_03 => gLUT12_811_INIT(3),
            INIT_04 => gLUT12_811_INIT(4),
            INIT_05 => gLUT12_811_INIT(5),
            INIT_06 => gLUT12_811_INIT(6),
            INIT_07 => gLUT12_811_INIT(7),
            INIT_08 => gLUT12_811_INIT(8),
            INIT_09 => gLUT12_811_INIT(9),
            INIT_0A => gLUT12_811_INIT(10),
            INIT_0B => gLUT12_811_INIT(11),
            INIT_0C => gLUT12_811_INIT(12),
            INIT_0D => gLUT12_811_INIT(13),
            INIT_0E => gLUT12_811_INIT(14),
            INIT_0F => gLUT12_811_INIT(15),
            INIT_10 => gLUT12_811_INIT(16),
            INIT_11 => gLUT12_811_INIT(17),
            INIT_12 => gLUT12_811_INIT(18),
            INIT_13 => gLUT12_811_INIT(19),
            INIT_14 => gLUT12_811_INIT(20),
            INIT_15 => gLUT12_811_INIT(21),
            INIT_16 => gLUT12_811_INIT(22),
            INIT_17 => gLUT12_811_INIT(23),
            INIT_18 => gLUT12_811_INIT(24),
            INIT_19 => gLUT12_811_INIT(25),
            INIT_1A => gLUT12_811_INIT(26),
            INIT_1B => gLUT12_811_INIT(27),
            INIT_1C => gLUT12_811_INIT(28),
            INIT_1D => gLUT12_811_INIT(29),
            INIT_1E => gLUT12_811_INIT(30),
            INIT_1F => gLUT12_811_INIT(31),
            INIT_20 => gLUT12_811_INIT(32),
            INIT_21 => gLUT12_811_INIT(33),
            INIT_22 => gLUT12_811_INIT(34),
            INIT_23 => gLUT12_811_INIT(35),
            INIT_24 => gLUT12_811_INIT(36),
            INIT_25 => gLUT12_811_INIT(37),
            INIT_26 => gLUT12_811_INIT(38),
            INIT_27 => gLUT12_811_INIT(39),
            INIT_28 => gLUT12_811_INIT(40),
            INIT_29 => gLUT12_811_INIT(41),
            INIT_2A => gLUT12_811_INIT(42),
            INIT_2B => gLUT12_811_INIT(43),
            INIT_2C => gLUT12_811_INIT(44),
            INIT_2D => gLUT12_811_INIT(45),
            INIT_2E => gLUT12_811_INIT(46),
            INIT_2F => gLUT12_811_INIT(47),
            INIT_30 => gLUT12_811_INIT(48),
            INIT_31 => gLUT12_811_INIT(49),
            INIT_32 => gLUT12_811_INIT(50),
            INIT_33 => gLUT12_811_INIT(51),
            INIT_34 => gLUT12_811_INIT(52),
            INIT_35 => gLUT12_811_INIT(53),
            INIT_36 => gLUT12_811_INIT(54),
            INIT_37 => gLUT12_811_INIT(55),
            INIT_38 => gLUT12_811_INIT(56),
            INIT_39 => gLUT12_811_INIT(57),
            INIT_3A => gLUT12_811_INIT(58),
            INIT_3B => gLUT12_811_INIT(59),
            INIT_3C => gLUT12_811_INIT(60),
            INIT_3D => gLUT12_811_INIT(61),
            INIT_3E => gLUT12_811_INIT(62),
            INIT_3F => gLUT12_811_INIT(63))
        port map (
            CLKA        => SysClkxCI,                           -- 1-bit input port-A clock
            RSTA        => SysRstxRI,                           -- 1-bit input port-A reset
            REGCEA      => '1',                                 -- 1-bit input port-A output register enable
            ENA         => SysEnxMI,                            -- 1-bit input port-A enable
            WEA         => SysWrEnxM,                           -- Input port-A write enable, width defined by Port A depth
            ADDRA       => SysAddrxA,                           -- Input port-A address, width defined by Port A depth
            DIA         => SysWrDataxD(11 downto 8),            -- Input port-A data, width defined by WRITE_WIDTH_A parameter
            DOA         => SysRdDataxD(11 downto 8),            -- Output port-A data, width defined by READ_WIDTH_A parameter

            CLKB        => ImgClkxCI,                           -- 1-bit input port-B clock
            RSTB        => ImgRstxRI,                           -- 1-bit input port-B reset
            REGCEB      => '1',                                 -- 1-bit input port-B output register enable
            ENB         => '1',                                 -- 1-bit input port-B enable
            WEB         => ImgWrEnxM,                           -- Input port-B write enable, width defined by Port B depth
            ADDRB       => ImgAddrxA,                           -- Input port-B address, width defined by Port B depth
            DIB         => (others => '0'),                     -- Input port-B data, width defined by WRITE_WIDTH_B parameter
            DOB         => ImgRdDataxD(11 downto 8));           -- Output port-B data, width defined by READ_WIDTH_B parameter
    end generate;

end structural;

