-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Memory Test Controller
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-10  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;
library unimacro;
use unimacro.vcomponents.all;

use work.CustSimPkg.all;
use work.CustomPkg.all;
use work.MemTestPkg.all;

entity MemTestCtrl is
    port (
        -- Module Infrastructure Signals
        SysClkxCI        : in  std_logic;                                       -- Custom Module Clock
        SysRstxRI        : in  std_logic;                                       -- Custom Module Reset

        -- SystemBus Control/Status Port
        SysCtrlRegxMI    : in  tSYS_CTRL_REG;                                   -- SystemBus Control Registers
        SysStatRegxMO    : out tSYS_STAT_REG;                                   -- SystemBus Status Registers

        -- MemoryBus Command Port
        px_cmd_en_o      : out std_logic;                                       -- MemoryBus Command Valid
        px_cmd_full_i    : in  std_logic;                                       -- MemoryBus Command Buffer Full
        px_cmd_instr_o   : out std_logic_vector(cMEM_CMD_WIDTH-1 downto 0);     -- MemoryBus Command Instruction
        px_cmd_bl_o      : out std_logic_vector(cMEM_BLEN_WIDTH-1 downto 0);    -- MemoryBus Command Burst Length
        px_cmd_addr_o    : out std_logic_vector(cMEM_ADDR_WIDTH-1 downto 0);    -- MemoryBus Command Address

        -- MemoryBus Read Port
        px_rd_en_o       : out std_logic;                                       -- MemoryBus Read Data Valid
        px_rd_empty_i    : in  std_logic;                                       -- MemoryBus Read Data Buffer Empty
        px_rd_data_i     : in  std_logic_vector(cMEM_DATA_WIDTH-1 downto 0);    -- MemoryBus Read Data Bus

        -- MemoryBus Write Port
        px_wr_en_o       : out std_logic;                                       -- MemoryBus Write Data Valid
        px_wr_full_i     : in  std_logic;                                       -- MemoryBus Write Data Buffer Full
        px_wr_data_o     : out std_logic_vector(cMEM_DATA_WIDTH-1 downto 0);    -- MemoryBus Write Data Bus

        -- Debug Logic Port
        DbgLogicPortxTO  : out std_logic_vector(cDEBUG_PORT_WIDTH-1 downto 0)); -- Custom Module Debug Signals
end MemTestCtrl;

architecture rtl of MemTestCtrl is

    ---------------------------------------------------------------------------
    -- Constant and Type Definitions
    ---------------------------------------------------------------------------
    type tMEM_TEST_STATE is (MemIdle, MemCmd, MemWrite, MemRead, MemDone);

    ---------------------------------------------------------------------------
    -- Function Definitions
    ---------------------------------------------------------------------------
    -- Select an argument for synthesis or simulation.
    function SynSimSel(SynArg : integer; SimArg : integer) return integer is
        variable Result : integer;
    begin
        if (cCUST_SIM_CONFIG = True) then
            Result := SimArg;
        else
            Result := SynArg;
        end if;

        return Result;
    end;

    ---------------------------------------------------------------------------
    -- Signal Declarations
    ---------------------------------------------------------------------------
    signal MemTestStatexM   : tMEM_TEST_STATE;                                      -- Memory Test Controller FSM

    -- SystemBus Control Interface
    signal SysStatRegxM     : tSYS_STAT_REG;                                        -- SystemBus Status Registers
    signal MemCycleEndxM    : std_logic;                                            -- Memory Test cycle end
    signal MemDataErrorxM   : std_logic;                                            -- Memory Test data comparison error
    signal MemTestErrorxM   : std_logic;                                            -- Memory Test data comparison error flag
    signal MemBlenCntxM     : std_logic_vector(cMEM_BLEN_WIDTH-1 downto 0);         -- Memory Test burst length counter
    signal MemAddrOffsetxM  : std_logic_vector(cMEM_BLEN_WIDTH downto 0);           -- Memory Test burst length terminal value

    signal MemTestStartxM   : std_logic;                                            -- Memory Test cycle start
    signal MemCmdReadxM     : std_logic;                                            -- Memory Test read command transfer active
    signal MemDataReadxM    : std_logic;                                            -- Memory Test read data transfer active
    signal MemReadLastxM    : std_logic;                                            -- Memory Test last requested read data transfer
    signal MemRdDataCntxM   : std_logic_vector(cMEM_READ_CNT_WIDTH-1 downto 0);     -- Memory Test read data transfer counter

    -- MemoryBus Command Port
    signal MemCmdEnxM       : std_logic;                                            -- MemoryBus Command Valid
    signal MemCmdInstrxM    : std_logic_vector(cMEM_CMD_WIDTH-1 downto 0);          -- MemoryBus Command Instruction
    signal MemCmdBLenxM     : std_logic_vector(cMEM_BLEN_WIDTH-1 downto 0);         -- MemoryBus Command Burst Length
    signal MemCmdAddrxA     : std_logic_vector(cMEM_ADDR_WIDTH-3 downto 0);         -- MemoryBus Command Address
    signal MemNextAddrxA    : std_logic_vector(cMEM_ADDR_WIDTH-3 downto 0);         -- MemoryBus Command Next Burst Address

    -- MemoryBus Read Port
    signal MemRdEnxM        : std_logic;                                            -- MemoryBus Read Data Valid

    -- MemoryBus Write Port
    signal MemWrEnxM        : std_logic;                                            -- MemoryBus Write Data Valid
    signal MemWrDataxD      : std_logic_vector(cMEM_DATA_WIDTH-1 downto 0);         -- MemoryBus Write Data Bus
    signal MemExpDataxD     : std_logic_vector(cMEM_DATA_WIDTH-1 downto 0);         -- MemoryBus Expected Data Bus

    -- Bandwidth Analyzer (BWA) signals
    signal MemBwaClkCycZeroxM : std_logic;                                          -- BWA Cycle Counter Zero Indicatior
    signal MemBwaCmdCycRegxM  : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);    -- BWA Command Count Register
    signal MemBwaWrCycRegxM   : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);    -- BWA Write Data Register
    signal MemBwaRdCycRegxM   : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);    -- BWA Read Data Register
    signal MemBwaCmdCycCntxM  : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);    -- BWA Command Count Counter
    signal MemBwaWrCycCntxM   : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);    -- BWA Write Data Counter
    signal MemBwaRdCycCntxM   : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);    -- BWA Read Data Counter
    signal MemBwaClkCycCntxM  : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);    -- BWA Clock Cycle Counter

begin

    ----------------------------------------------------------------------------
    -- Memory Test Controller
    ----------------------------------------------------------------------------
    MemTstProc: process (SysClkxCI)
    begin
        if rising_edge(SysClkxCI) then
            if (SysRstxRI = '1') then
                MemCmdEnxM      <= '0';
                MemCmdInstrxM   <= (others => '0');
                MemCmdBLenxM    <= (others => '0');
                MemCmdAddrxA    <= (others => '0');
                MemBlenCntxM    <= (others => '0');
                MemAddrOffsetxM <= (others => '0');

                MemWrEnxM       <= '0';
                MemWrDataxD     <= (others => '0');

                MemTestStartxM  <= '0';
                MemTestStatexM  <= MemIdle;
            else
                -- Memory Test Controller FSM
                case MemTestStatexM is
                    when MemIdle =>
                        -- In Idle mode, de-assert the MCB Command, Read and Write Interfaces.
                        MemCmdEnxM     <= '0';
                        MemWrEnxM      <= '0';
                        MemTestStartxM <= '0';
                        MemBlenCntxM   <= (others => '0');
                        MemWrDataxD    <= (others => '0');

                        -- The Burst Length register determines how many 32-bit DWord transfers are performed
                        -- per burst transfer. MemBurstLen contains a value of "Number-of-DWord-Transfers - 1",
                        -- i.e. a value of 63 requests 64 DWords transfers.
                        -- PLEASE NOTE: The address range defined by MemStartAddr and MemEndAddr must be an
                        -- even multiple of the burst length, i.e. the last data transfer of the last burst
                        -- transfer of a read or write cycle must end at MemEndAddr.
                        MemCmdBLenxM <= SysCtrlRegxMI.MemBurstLen;

                        -- The Start Address register identifies the first 32-bit DWord
                        -- of the Memory Test. Only 32-bit data transfers are allowed, hence
                        -- the 2 LSB address bits MemStartAddr(1..0) must always be "00".
                        MemCmdAddrxA <= SysCtrlRegxMI.MemStartAddr(cMEM_ADDR_WIDTH-1 downto 2);

                        -- Select the first data word to be written to the DDR3-RAM.
                        if (SysCtrlRegxMI.MemDataMode = cMEM_DATA_ADDR_MODE) then
                            MemWrDataxD(cMEM_ADDR_WIDTH-1 downto 0) <= SysCtrlRegxMI.MemStartAddr(cMEM_ADDR_WIDTH-1 downto 2) & "00";
                        elsif (SysCtrlRegxMI.MemDataMode = cMEM_DATA_TOGGLE_BIT) then
                            MemWrDataxD <= x"A5A55A5A";
                        end if;

                        -- Check the requested Memory Test mode at the start of the test.
                        -- The first burst cycle is a Write Operation for Write-Only or Write-Read Modes.
                        -- Otherwise (i.e. if Read-Only Mode is selected) perform a Read Operation.
                        -- A Write Operation consists of the data transfer to the MCB, followed
                        -- by a Write Command. For a Read Operation the Read Command is issued first
                        -- followed by the read data transfer from the MCB to the Custom Module.
                        if (SysCtrlRegxMI.MemTestEnable = '1') then
                            -- Indicate the start of a memory test cycle
                            MemTestStartxM <= '1';

                            if (SysCtrlRegxMI.MemTestMode = cMEM_TEST_READ_ONLY) then
                                MemCmdEnxM     <= '1';
                                MemCmdInstrxM  <= cMEM_READ_COMMAND;
                                MemTestStatexM <= MemCmd;
                            else
                                MemWrEnxM      <= '1';
                                MemTestStatexM <= MemWrite;
                            end if;
                        end if;

                    when MemCmd =>
                        MemTestStartxM <= '0';

                        -- Perform an MCB Command transfer.
                        if ((MemCmdEnxM = '1') and (px_cmd_full_i = '0')) then
                            MemCmdEnxM <= '0';

                            -- A memory read command has been lodged.
                            if (MemCmdInstrxM = cMEM_READ_COMMAND) then
                                -- Perform successive read operations in Read-Only Mode or if
                                if (SysCtrlRegxMI.MemTestEnable = '0') then
                                    MemTestStatexM <= MemIdle;
                                elsif (MemCycleEndxM = '0') then
                                    -- The end of the memory address range has NOT been reached. When Read-Only Mode is selected
                                    -- or if Write-Read Interleave Mode is switched off issue another Read Command, otherwise
                                    -- (i.e. Write-Read Mode in Interleave Mode) perform the next write burst data transfer.
                                    if ((SysCtrlRegxMI.MemTestIntLv = '0') or (SysCtrlRegxMI.MemTestMode = cMEM_TEST_READ_ONLY)) then
                                        MemCmdEnxM <= '1';
                                    else
                                        -- Write-Read Mode in Interleave Mode is selected but the end of a write
                                        -- cycle has not been reached. Perform a write burst transfer.
                                        MemWrEnxM      <= '1';
                                        MemBlenCntxM   <= (others => '0');
                                        MemTestStatexM <= MemWrite;
                                    end if;
                                else
                                    -- The end of the memory address range has been reached. When Read-Only Mode is selected
                                    MemTestStatexM <= MemRead;
                                end if;

                                -- Update the MCB Command Address register for the next burst transfer.
                                MemCmdAddrxA <= MemNextAddrxA;

                            elsif (SysCtrlRegxMI.MemTestMode = cMEM_TEST_WRITE_READ) then
                                -- A Write Command has been issued. In Write-Read Mode perform a Read Command from
                                -- the same burst start address as the just completed write burst if Write-Read
                                -- Interleave Mode is selected or the end of the Write cycle has been reached.
                                -- If the MemTestEnable control register is de-asserted abort the memory test.
                                if (SysCtrlRegxMI.MemTestEnable = '0') then
                                    MemTestStatexM <= MemIdle;
                                elsif (SysCtrlRegxMI.MemTestIntLv = '1') then
                                    MemCmdEnxM    <= '1';
                                    MemCmdInstrxM <= cMEM_READ_COMMAND;
                                elsif (MemCycleEndxM = '1') then
                                    -- Write-Read Mode in Non-Interleave Mode is selected and the end of a write
                                    -- cycle has been reached. Start a new read cycle of the entire address range.
                                    MemCmdEnxM    <= '1';
                                    MemCmdInstrxM <= cMEM_READ_COMMAND;
                                    MemCmdAddrxA  <= SysCtrlRegxMI.MemStartAddr(cMEM_ADDR_WIDTH-1 downto 2);
                                else
                                    -- Write-Read Mode in Non-Interleave Mode is selected but the end of a write
                                    -- cycle has not been reached. Perform another write burst transfer.
                                    MemWrEnxM      <= '1';
                                    MemCmdAddrxA   <= MemNextAddrxA;
                                    MemBlenCntxM   <= (others => '0');
                                    MemTestStatexM <= MemWrite;
                                end if;
                            else
                                -- Write-Only Mode is selected. Perform the next burst write transfer. Update the MCB
                                -- Command Address register with the start address of the next burst. Check if the end
                                -- of the memory address range has been reached. If Single Mode is selected stop all
                                -- further burst transfers, otherwise repeat the same memory test cycle.
                                if (MemCycleEndxM = '0') then
                                    -- If the MemTestEnable control register is de-asserted abort the memory test.
                                    if (SysCtrlRegxMI.MemTestEnable = '0') then
                                        MemTestStatexM <= MemIdle;
                                    else
                                        MemWrEnxM      <= '1';
                                        MemCmdAddrxA   <= MemNextAddrxA;
                                        MemBlenCntxM   <= (others => '0');
                                        MemTestStatexM <= MemWrite;
                                    end if;
                                elsif (SysCtrlRegxMI.MemTestSglCyc = '1') then
                                    -- Memory Test has completed.
                                    MemTestStatexM <= MemDone;
                                else
                                    -- Repeat Memory Test cycle
                                    MemTestStatexM <= MemIdle;
                                end if;
                            end if;
                        end if;

                    when MemWrite =>
                        MemTestStartxM <= '0';

                        -- Transfer a burst of write data to the MCB memory. Once the burst data
                        -- transfer is finished (max. 64 DWords) issue the corresponding write command.
                        if ((MemWrEnxM = '1') and (px_wr_full_i = '0')) then
                            -- Increment the burst data counter with every transmitted DWord.
                            MemBlenCntxM <= MemBlenCntxM + '1';

                            -- The burst write transfer has finished when the data counter
                            -- reaches terminal value. Issue the Write Command next and
                            -- update the burst start address for the next transfer.
                            if (MemBlenCntxM = SysCtrlRegxMI.MemBurstLen) then
                                MemWrEnxM      <= '0';
                                MemCmdEnxM     <= '1';
                                MemCmdInstrxM  <= cMEM_WRITE_COMMAND;
                                MemTestStatexM <= MemCmd;
                            end if;

                            -- Update the write data with every transferred DWord.
                            if (SysCtrlRegxMI.MemDataMode = cMEM_DATA_RAMP_MODE) then
                                MemWrDataxD <= MemWrDataxD + '1';
                            elsif (SysCtrlRegxMI.MemDataMode = cMEM_DATA_ADDR_MODE) then
                                MemWrDataxD <= MemWrDataxD + "100";
                            elsif (SysCtrlRegxMI.MemDataMode = cMEM_DATA_TOGGLE_BIT) then
                                MemWrDataxD <= not(MemWrDataxD);
                            end if;
                        end if;

                    when MemRead =>
                        -- Wait until all read data transfers which have been requested from the MCB are complete.
                        -- Read transfers are finished when the read data counter reaches 0.
                        if (MemRdEnxM = '0') then
                            -- If the MemTestEnable control register is de-asserted abort the memory test.
                            if (SysCtrlRegxMI.MemTestEnable = '0') then
                                MemTestStatexM <= MemIdle;
                            elsif (SysCtrlRegxMI.MemTestSglCyc = '1') then
                                -- The end of the memory address range has been reached. If Single Mode is selected
                                -- stop all further burst transfers, otherwise repeat the same memory test cycle.
                                MemTestStatexM <= MemDone;
                            else
                                MemTestStatexM <= MemIdle;
                            end if;
                        end if;

                    when MemDone =>
                        -- The Memory Test is finished. Wait for the MemTestEnable control register to be de-asserted by the Host.
                        if (SysCtrlRegxMI.MemTestEnable = '0') then
                            MemTestStatexM <= MemIdle;
                        end if;

                    when others => NULL;
                end case;

                -- Calculate the address offset to the start of the next burst.
                MemAddrOffsetxM <= ('0' & SysCtrlRegxMI.MemBurstLen) + '1';
            end if;
        end if;
    end process;

    -- Calculate the start address for the next burst transfer.
    MemNextAddrxA <= MemCmdAddrxA + MemAddrOffsetxM;

    -- Determine the end of a memory test cycle by comparing the start address
    -- of the next burst with the End Address register. If the burst address falls
    -- outside the MemEndAddr range the current burst cycle has completed.
    MemCycleEndxM <= '1' when (MemNextAddrxA > SysCtrlRegxMI.MemEndAddr(cMEM_ADDR_WIDTH-1 downto 2)) else '0';

    ----------------------------------------------------------------------------
    -- Memory Test Read Data Comparator
    ----------------------------------------------------------------------------
    MemCmpProc: process (SysClkxCI)
    begin
        if rising_edge(SysClkxCI) then
            if (SysRstxRI = '1') then
                MemRdEnxM      <= '0';
                MemDataErrorxM <= '0';
                MemTestErrorxM <= '0';
                MemExpDataxD   <= (others => '0');
                MemRdDataCntxM <= (others => '0');
            else
                -- Enable memory read data transfers when the number of outstanding read requests is greater than 0.
                if (MemReadLastxM = '1') then
                    MemRdEnxM <= '0';
                elsif (MemRdDataCntxM > cMEM_READ_DATA_ZERO) then
                    MemRdEnxM <= '1';
                end if;

                -- Monitor the number of requested (i.e. outstanding) memory read data transfers.
                if (MemCmdReadxM = '1') then
                    -- A memory read command is issued. Increment the read data counter by the requested
                    -- burst length (if no read data transfer is pending at the same time) or by the
                    -- requested burst length - 1 (if a read data transfer is pending).
                    if (MemDataReadxM = '0') then
                        MemRdDataCntxM <= MemRdDataCntxM + MemAddrOffsetxM;
                    else
                        MemRdDataCntxM <= MemRdDataCntxM + ('0' & SysCtrlRegxMI.MemBurstLen);
                    end if;
                elsif (MemDataReadxM = '1') then
                    -- No memory read command is issued at the moment but a read data
                    -- transfer is pending. Decrement the read data counter by 1.
                    MemRdDataCntxM <= MemRdDataCntxM - '1';
                end if;

                -- Update the expected read data register with every transferred DWord. When a new
                -- Memory Test cycle is started update MemExpDataxD with the first expected data word.
                if (MemTestStartxM = '1') then
                    MemExpDataxD <= MemWrDataxD;
                elsif (MemDataReadxM = '1') then
                    if (SysCtrlRegxMI.MemDataMode = cMEM_DATA_RAMP_MODE) then
                        MemExpDataxD <= MemExpDataxD + '1';
                    elsif (SysCtrlRegxMI.MemDataMode = cMEM_DATA_ADDR_MODE) then
                        MemExpDataxD <= MemExpDataxD + "100";
                    elsif (SysCtrlRegxMI.MemDataMode = cMEM_DATA_TOGGLE_BIT) then
                        MemExpDataxD <= not(MemExpDataxD);
                    end if;
                end if;

                -- Compare the read data with the expected data and flag an error if
                -- a mismatch occurs and set the Memory Test Data Error status bit.
                if (SysCtrlRegxMI.MemTestEnable = '0') then
                    MemDataErrorxM <= '0';
                    MemTestErrorxM <= '0';
                elsif ((MemDataReadxM = '1') and (px_rd_data_i /= MemExpDataxD)) then
                    MemDataErrorxM <= '1';
                    MemTestErrorxM <= '1';
                else
                    MemDataErrorxM <= '0';
                end if;
            end if;
        end if;
    end process;

    -- Indicate that a memory read command is issued to MCB.
    MemCmdReadxM <= '1' when ((MemCmdEnxM = '1') and (px_cmd_full_i = '0') and (MemCmdInstrxM = cMEM_READ_COMMAND)) else '0';

    -- Indicate that memory read data is received from MCB.
    MemDataReadxM <= '1' when ((MemRdEnxM = '1') and (px_rd_empty_i = '0')) else '0';

    -- Indicate the transfer of the last requested read data.
    MemReadLastxM <= '1' when ((MemDataReadxM = '1') and (MemCmdReadxM = '0') and (MemRdDataCntxM = cMEM_READ_DATA_ONE)) else '0';

    ----------------------------------------------------------------------------
    -- Memory Test Bandwidth Analyzer
    ----------------------------------------------------------------------------
    MemBwaGen: if (cMEM_BWA_ENABLE = True) generate
        MemBwaProc: process (SysClkxCI)
        begin
            if rising_edge(SysClkxCI) then
                if ((SysRstxRI = '1') or (SysCtrlRegxMI.MemTestEnable = '0')) then
                    MemBwaClkCycZeroxM <= '0';
                    MemBwaCmdCycRegxM  <= (others => '0');
                    MemBwaWrCycRegxM   <= (others => '0');
                    MemBwaRdCycRegxM   <= (others => '0');
                    MemBwaCmdCycCntxM  <= (others => '0');
                    MemBwaWrCycCntxM   <= (others => '0');
                    MemBwaRdCycCntxM   <= (others => '0');
                    MemBwaClkCycCntxM  <= std_logic_vector(to_unsigned(SynSimSel(cSYS_CLOCK_FREQ-1, cMEM_BWA_SIM_TERM-1), cMEM_BWA_CNT_WIDTH));
                elsif (MemBwaClkCycZeroxM = '1') then
                    MemBwaClkCycZeroxM <= '0';

                    if ((SysCtrlRegxMI.MemBwaMaxMode = '0') or (MemBwaCmdCycCntxM >= MemBwaCmdCycRegxM)) then
                        MemBwaCmdCycRegxM <= MemBwaCmdCycCntxM;
                    end if;

                    if ((SysCtrlRegxMI.MemBwaMaxMode = '0') or (MemBwaWrCycCntxM >= MemBwaWrCycRegxM)) then
                        MemBwaWrCycRegxM <= MemBwaWrCycCntxM;
                    end if;

                    if ((SysCtrlRegxMI.MemBwaMaxMode = '0') or (MemBwaRdCycCntxM >= MemBwaRdCycRegxM)) then
                        MemBwaRdCycRegxM <= MemBwaRdCycCntxM;
                    end if;

                    MemBwaCmdCycCntxM <= (others => '0');
                    MemBwaWrCycCntxM  <= (others => '0');
                    MemBwaRdCycCntxM  <= (others => '0');
                    MemBwaClkCycCntxM <= std_logic_vector(to_unsigned(SynSimSel(cSYS_CLOCK_FREQ-1, cMEM_BWA_SIM_TERM-1), cMEM_BWA_CNT_WIDTH));
                else
                    -- Decrement the 1-second clock counter
                    MemBwaClkCycCntxM <= MemBwaClkCycCntxM - '1';

                    -- Indicate the end of a 1 second interval
                    if (MemBwaClkCycCntxM = std_logic_vector(to_unsigned(1, cMEM_BWA_CNT_WIDTH))) then
                        MemBwaClkCycZeroxM <= '1';
                    end if;

                    -- Count the number of command cycles to the MCB
                    if ((MemCmdEnxM = '1') and (px_cmd_full_i = '0')) then
                        MemBwaCmdCycCntxM <= MemBwaCmdCycCntxM + '1';
                    end if;

                    -- Count the number of data write cycles to the MCB
                    if ((MemWrEnxM = '1') and (px_wr_full_i = '0')) then
                        MemBwaWrCycCntxM <= MemBwaWrCycCntxM + '1';
                    end if;

                    -- Count the number of data read cycles from the MCB
                    if ((MemRdEnxM = '1') and (px_rd_empty_i = '0')) then
                        MemBwaRdCycCntxM <= MemBwaRdCycCntxM + '1';
                    end if;
                end if;
            end if;
        end process;
    end generate;

    NoMemBwaGen: if (cMEM_BWA_ENABLE = False) generate
        MemBwaClkCycZeroxM <= '0';
        MemBwaCmdCycRegxM  <= (others => '0');
        MemBwaWrCycRegxM   <= (others => '0');
        MemBwaRdCycRegxM   <= (others => '0');
        MemBwaCmdCycCntxM  <= (others => '0');
        MemBwaWrCycCntxM   <= (others => '0');
        MemBwaRdCycCntxM   <= (others => '0');
        MemBwaClkCycCntxM  <= (others => '0');
    end generate;

    ----------------------------------------------------------------------------
    -- SystemBus Control and Status Signal Assignments
    ----------------------------------------------------------------------------
    SysStatRegxM.MemTestDone  <= '1' when (((MemTestStatexM = MemIdle) or (MemTestStatexM = MemDone)) and (MemRdEnxM = '0')) else '0';
    SysStatRegxM.MemTestError <= MemTestErrorxM;
    SysStatRegxM.MemBwaCmdCnt <= MemBwaCmdCycRegxM;
    SysStatRegxM.MemBwaWrCnt  <= MemBwaWrCycRegxM;
    SysStatRegxM.MemBwaRdCnt  <= MemBwaRdCycRegxM;
    SysStatRegxMO <= SysStatRegxM;

    ----------------------------------------------------------------------------
    -- MemoryBus Signal Assignments
    ----------------------------------------------------------------------------
    px_cmd_en_o    <= MemCmdEnxM;
    px_cmd_instr_o <= MemCmdInstrxM;
    px_cmd_bl_o    <= MemCmdBLenxM;
    px_cmd_addr_o  <= MemCmdAddrxA & "00";      -- The 2 LSB address bits MemCmdAddrxA(1..0) must always be "00".

    px_rd_en_o     <= MemRdEnxM;
    px_wr_en_o     <= MemWrEnxM;
    px_wr_data_o   <= MemWrDataxD;

    ----------------------------------------------------------------------------
    -- Debug Logic
    ----------------------------------------------------------------------------
    NoDbgGen: if (cDEBUG_ENABLE = False) generate
        DbgLogicPortxTO <= (others => '0');
    end generate;

    DbgGen: if (cDEBUG_ENABLE = True) generate
        signal MemTestStateDecodexM : std_logic_vector(2 downto 0);
        signal DbgLogicPortxT       : std_logic_vector(255 downto 0);
    begin
        MemTestStateDecodexM <= std_logic_vector(to_unsigned(tMEM_TEST_STATE'pos(MemTestStatexM), MemTestStateDecodexM'length));

        DbgProc: process (SysClkxCI)
            variable i: natural;
        begin
            if rising_edge(SysClkxCI) then
                i := 0;
                DbgLogicPortxT(i)                <= MemCmdEnxM;              -- px_cmd_en_o
                DbgLogicPortxT(i+1)              <= px_cmd_full_i;           -- px_cmd_full_i
                DbgLogicPortxT(i+4 downto i+2)   <= MemCmdInstrxM;           -- px_cmd_instr_o (2 downto 0);
                DbgLogicPortxT(i+10 downto i+5)  <= MemCmdBLenxM;            -- px_cmd_bl_o    (5 downto 0)
                DbgLogicPortxT(i+40 downto i+11) <= MemCmdAddrxA & "00";     -- px_cmd_addr_o  (29 downto 0)

                i := 41;
                DbgLogicPortxT(i)                <= MemRdEnxM;               -- px_rd_en_o
                DbgLogicPortxT(i+1)              <= px_rd_empty_i;           -- px_rd_empty_i
                DbgLogicPortxT(i+33 downto i+2)  <= px_rd_data_i;            -- px_rd_data_i  (31 downto 0);

                i := 75;
                DbgLogicPortxT(i)                <= MemWrEnxM;               -- px_wr_en_o
                DbgLogicPortxT(i+1)              <= px_wr_full_i;            -- px_wr_full_i
                DbgLogicPortxT(i+33 downto i+2)  <= MemWrDataxD;             -- px_wr_data_o (31 downto 0);

                i := 109;
                DbgLogicPortxT(i+2 downto i)     <= MemTestStateDecodexM;

                i := 112;
                DbgLogicPortxT(i)                <= MemCycleEndxM;
                DbgLogicPortxT(i+1)              <= MemDataErrorxM;
                DbgLogicPortxT(i+2)              <= MemTestErrorxM;
                DbgLogicPortxT(i+8 downto i+3)   <= MemBlenCntxM;
                DbgLogicPortxT(i+15 downto i+9)  <= MemAddrOffsetxM;
                DbgLogicPortxT(i+45 downto i+16) <= MemNextAddrxA & "00";
                DbgLogicPortxT(i+77 downto i+46) <= MemExpDataxD;

                i := 190;
                DbgLogicPortxT(i)                <= MemTestStartxM;
                DbgLogicPortxT(i+1)              <= MemCmdReadxM;
                DbgLogicPortxT(i+2)              <= MemDataReadxM;
                DbgLogicPortxT(i+3)              <= MemReadLastxM;
                DbgLogicPortxT(i+19 downto i+4)  <= MemRdDataCntxM;

                i := 210;
                DbgLogicPortxT(i)                <= MemBwaClkCycZeroxM;
                --DbgLogicPortxT(i+28 downto i+1)  <= MemBwaCmdCycCntxM;
                DbgLogicPortxT(i+28 downto i+1)  <= MemBwaWrCycCntxM;
                --DbgLogicPortxT(i+28 downto i+1)  <= MemBwaRdCycCntxM;

                i := 239;
                DbgLogicPortxT(255 downto i) <= (others => '0');
            end if;
        end process;

        DbgLogicPortxTO <= DbgLogicPortxT(cDEBUG_PORT_WIDTH-1 downto 0);
    end generate;

end rtl;
