-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Memory Test Package
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-09-12  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.CustomPkg.all;
use work.MemTestVerPkg.all;

package MemTestPkg is
    ----------------------------------------------------------------------------
    -- SystemBus Definitions
    ----------------------------------------------------------------------------
    constant cSYS_CLOCK_FREQ      : integer := 150000000;                   -- Custom Module Clock Frequency

    constant cMEM_BWA_ENABLE      : boolean := True;                        -- Enable Memory Test Bandwidth Analyzer
    constant cMEM_MODE_REG_WIDTH  : integer := 2;                           -- Memory Test Mode register width
    constant cMEM_BWA_CNT_WIDTH   : integer := 28;                          -- Width of Bandwidth Analyzer Counters/Registers
    constant cMEM_BWA_SIM_TERM    : integer := 1000000;                     -- Bandwidth Analyzer interval terminal value for simulation
    constant cMEM_READ_CNT_WIDTH  : integer := 16;                          -- Width of Memory read data transfer counter

    constant cMEM_TEST_WRITE_READ : std_logic_vector(cMEM_MODE_REG_WIDTH-1 downto 0) := "00";   -- Memory Test Write-Read Mode
    constant cMEM_TEST_READ_ONLY  : std_logic_vector(cMEM_MODE_REG_WIDTH-1 downto 0) := "01";   -- Memory Test Read-Only Mode
    constant cMEM_TEST_WRITE_ONLY : std_logic_vector(cMEM_MODE_REG_WIDTH-1 downto 0) := "10";   -- Memory Test Write-Only Mode

    constant cMEM_DATA_RAMP_MODE  : std_logic_vector(cMEM_MODE_REG_WIDTH-1 downto 0) := "00";   -- Memory Test Ramp Data Pattern
    constant cMEM_DATA_ADDR_MODE  : std_logic_vector(cMEM_MODE_REG_WIDTH-1 downto 0) := "01";   -- Memory Test Ramp Address Pattern
    constant cMEM_DATA_TOGGLE_BIT : std_logic_vector(cMEM_MODE_REG_WIDTH-1 downto 0) := "10";   -- Memory Test Toggle Data Pattern

    constant cMEM_READ_DATA_ZERO  : std_logic_vector(cMEM_READ_CNT_WIDTH-1 downto 0) := std_logic_vector(to_unsigned(0, cMEM_READ_CNT_WIDTH));   -- Memory read data transfer counter zero
    constant cMEM_READ_DATA_ONE   : std_logic_vector(cMEM_READ_CNT_WIDTH-1 downto 0) := std_logic_vector(to_unsigned(1, cMEM_READ_CNT_WIDTH));   -- Memory read data transfer counter one

    -- Memory Test control register
    type tSYS_CTRL_REG is record
        MemTestEnable : std_logic;                                          -- Memory Test Enable
        MemTestSglCyc : std_logic;                                          -- Memory Test Single Cycle Mode
        MemTestIntLv  : std_logic;                                          -- Memory Test Read/Write Interleave Mode
        MemBwaMaxMode : std_logic;                                          -- Memory Test BWA Maximum Count Mode
        MemTestMode   : std_logic_vector(cMEM_MODE_REG_WIDTH-1 downto 0);   -- Memory Test Mode Select [0x0: WRITE-READ, 0x1: READ-ONLY, 0x2: WRITE_ONLY]
        MemDataMode   : std_logic_vector(cMEM_MODE_REG_WIDTH-1 downto 0);   -- Memory Test Data Mode Select [0x0: RAMP, 0x1: ADDR, 0x2: TOGGLE]
        MemBurstLen   : std_logic_vector(cMEM_BLEN_WIDTH-1 downto 0);       -- Memory Test Burst Length [Number-of-DWord-Transfers - 1]
        MemStartAddr  : std_logic_vector(cMEM_ADDR_WIDTH-1 downto 0);       -- Memory Test Start Address
        MemEndAddr    : std_logic_vector(cMEM_ADDR_WIDTH-1 downto 0);       -- Memory Test End Address
    end record;

    constant cSYS_CTRL_DEFAULT : tSYS_CTRL_REG := (
        MemTestEnable => '0',
        MemTestSglCyc => '0',
        MemTestIntLv  => '0',
        MemBwaMaxMode => '0',
        MemTestMode   => (others => '0'),
        MemDataMode   => (others => '0'),
        MemBurstLen   => (others => '0'),
        MemStartAddr  => (others => '0'),
        MemEndAddr    => (others => '0'));

    -- Memory Test status register
    type tSYS_STAT_REG is record
        MemTestDone  : std_logic;                                           -- Memory Test Done status
        MemTestError : std_logic;                                           -- Memory Test data comparison error
        MemBwaCmdCnt : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);     -- Bandwidth Analyzer Command Count Register
        MemBwaWrCnt  : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);     -- Bandwidth Analyzer Write Data Count Register
        MemBwaRdCnt  : std_logic_vector(cMEM_BWA_CNT_WIDTH-1 downto 0);     -- Bandwidth Analyzer Read Data Count Register
    end record;

    ----------------------------------------------------------------------------
    -- Version and Debug Definitions
    ----------------------------------------------------------------------------
    constant cVERSION_NUMBER   : integer := cCUSTOM_VERSION_NUM;        -- Set Custom Module version number
    constant cDEBUG_ENABLE     : boolean := cCUSTOM_DEBUG_ENABLE;       -- Enable/disable Custom Module debug mode
    constant cLOGIC_ENABLE     : boolean := cCUSTOM_ANALYZER_ENABLE;    -- Enable/disable Vivado Logic Analyzer
    constant cCHIPSCOPE_ENABLE : boolean := cCUSTOM_CHIPSCOPE_ENABLE;   -- Enable/disable ChipScopePro Analyzer
    constant cCSP_ICON_ENABLE  : boolean := cCUSTOM_CSP_ICON_ENABLE;    -- Enable/disable ICON module (JTAG mode only)
    constant cDEBUG_PORT_WIDTH : integer := cCUSTOM_DEBUG_PORT_WIDTH;   -- Width of ILA debug port

end MemTestPkg;
