-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module 
-- Title      : Memory Test SystemBus Interface
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-10  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

use work.CustomPkg.all;
use work.MemTestPkg.all;

entity MemTestSysBusIf is
    port (
        -- Module Infrastructure Signals
        SysClkxCI        : in  std_logic;                                       -- Custom Module Clock
        SysRstxRI        : in  std_logic;                                       -- Custom Module Reset

        -- SystemBus Control Interface
        SysReqxMI        : in  std_logic;                                       -- SystemBus Cycle Request
        SysAckxMO        : out std_logic;                                       -- SystemBus Cycle Acknowledge
        SysErrxMO        : out std_logic;                                       -- SystemBus Cycle Error
        SysWrEnxMI       : in  std_logic;                                       -- SystemBus Write Enable
        SysAddrxAI       : in  std_logic_vector(cSYS_ADDR_WIDTH-1 downto 0);    -- SystemBus Address
        SysWrDataxDI     : in  std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Write Data
        SysRdDataxDO     : out std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Read Data

        -- SystemBus Control/Status Registers
        SysCtrlRegxMO    : out tSYS_CTRL_REG;                                   -- SystemBus Control Registers
        SysStatRegxMI    : in  tSYS_STAT_REG);                                  -- SystemBus Status Registers
end MemTestSysBusIf;

architecture rtl of MemTestSysBusIf is

    ---------------------------------------------------------------------------
    -- Signal Declarations
    ---------------------------------------------------------------------------
    signal SysAckxM      : std_logic;
    signal SysRdDataxD   : std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);
    signal SysCtrlRegxM  : tSYS_CTRL_REG;

begin

    ---------------------------------------------------------------------------
    -- SystemBus Interface and Control/Status Registers
    ---------------------------------------------------------------------------
    SysBusProc: process (SysClkxCI, SysRstxRI)
    begin
        if (SysRstxRI = '1') then
            SysAckxM     <= '0';
            SysRdDataxD  <= (others => '0');
            SysCtrlRegxM <= cSYS_CTRL_DEFAULT;
        elsif rising_edge(SysClkxCI) then
            if (SysReqxMI = '1') then
                -- Reset SystemBus read data bus by default, i.e. unused data bits appear as '0'.
                SysRdDataxD <= (others => '0');

                case SysAddrxAI(4 downto 2) is
                    when "000" =>  -- 0x00
                        -- Required SystemBus status registers at address 0.
                        -- Registers must be present to identify the Custom Module - DO NOT MODIFY
                        SysRdDataxD(9 downto 0)   <= std_logic_vector(to_unsigned(cCUSTOM_DESIGN_ID, 10));
                        SysRdDataxD(20 downto 10) <= std_logic_vector(to_unsigned(cVERSION_NUMBER, 11));
                        SysRdDataxD(31 downto 21) <= std_logic_vector(to_unsigned(cMEMTEST_DESIGN_ID, 11));

                    when "001" =>  -- 0x04
                        if (SysWrEnxMI = '1') then
                            -- Masked control register: Bit0..15 = Data, Bit 16..31 = Mask
                            if (SysWrDataxDI(16) = '1') then
                                SysCtrlRegxM.MemTestEnable <= SysWrDataxDI(0);
                            end if;

                            if (SysWrDataxDI(17) = '1') then
                                SysCtrlRegxM.MemTestSglCyc <= SysWrDataxDI(1);
                            end if;

                            if (SysWrDataxDI(18) = '1') then
                                SysCtrlRegxM.MemTestIntLv <= SysWrDataxDI(2);
                            end if;

                            if (SysWrDataxDI(19) = '1') then
                                SysCtrlRegxM.MemBwaMaxMode <= SysWrDataxDI(3);
                            end if;
                        end if;

                        -- SystemBus read data
                        SysRdDataxD(0) <= SysCtrlRegxM.MemTestEnable;
                        SysRdDataxD(1) <= SysCtrlRegxM.MemTestSglCyc;
                        SysRdDataxD(2) <= SysCtrlRegxM.MemTestIntLv;
                        SysRdDataxD(3) <= SysCtrlRegxM.MemBwaMaxMode;

                        -- Memory Test Status Register
                        SysRdDataxD(8) <= SysStatRegxMI.MemTestDone;
                        SysRdDataxD(9) <= SysStatRegxMI.MemTestError;

                    when "010" =>  -- 0x08
                        if (SysWrEnxMI = '1') then
                            -- Non-masked control register. Memory Test Burst Length.
                            SysCtrlRegxM.MemBurstLen <= SysWrDataxDI(cMEM_BLEN_WIDTH-1 downto 0);
                            SysCtrlRegxM.MemTestMode <= SysWrDataxDI(cMEM_MODE_REG_WIDTH+7 downto 8);
                            SysCtrlRegxM.MemDataMode <= SysWrDataxDI(cMEM_MODE_REG_WIDTH+11 downto 12);
                        end if;

                        -- SystemBus read data
                        SysRdDataxD(cMEM_BLEN_WIDTH-1 downto 0)       <= SysCtrlRegxM.MemBurstLen;
                        SysRdDataxD(cMEM_MODE_REG_WIDTH+7 downto 8)   <= SysCtrlRegxM.MemTestMode;
                        SysRdDataxD(cMEM_MODE_REG_WIDTH+11 downto 12) <= SysCtrlRegxM.MemDataMode;

                    when "011" =>  -- 0x0C
                        if (SysWrEnxMI = '1') then
                            -- Non-masked control register. Memory Test Start Address.
                            SysCtrlRegxM.MemStartAddr <= SysWrDataxDI(cMEM_ADDR_WIDTH-1 downto 0);
                        end if;

                        -- SystemBus read data
                        SysRdDataxD(cMEM_ADDR_WIDTH-1 downto 0) <= SysCtrlRegxM.MemStartAddr;

                    when "100" =>  -- 0x10
                        if (SysWrEnxMI = '1') then
                            -- Non-masked control register. Memory Test End Address.
                            SysCtrlRegxM.MemEndAddr <= SysWrDataxDI(cMEM_ADDR_WIDTH-1 downto 0);
                        end if;

                        -- SystemBus read data
                        SysRdDataxD(cMEM_ADDR_WIDTH-1 downto 0) <= SysCtrlRegxM.MemEndAddr;

                    when "101" =>  -- 0x14
                        -- Memory Test Bandwidth Analyzer Command Counter Status.
                        SysRdDataxD(cMEM_BWA_CNT_WIDTH-1 downto 0) <= SysStatRegxMI.MemBwaCmdCnt;

                    when "110" =>  -- 0x18
                        -- Memory Test Bandwidth Analyzer Write Data Counter Status.
                        SysRdDataxD(cMEM_BWA_CNT_WIDTH-1 downto 0) <= SysStatRegxMI.MemBwaWrCnt;

                    when "111" =>  -- 0x1C
                        -- Memory Test Bandwidth Analyzer Read Data Counter Status.
                        SysRdDataxD(cMEM_BWA_CNT_WIDTH-1 downto 0) <= SysStatRegxMI.MemBwaRdCnt;

                    when others => NULL;
                end case;
            end if;

            -- Generate the SystemBus cycle acknowledge.
            SysAckxM <= SysReqxMI;
        end if;
    end process;

    -- Assign internally used output ports
    SysAckxMO     <= SysAckxM;
    SysErrxMO     <= '0';
    SysRdDataxDO  <= SysRdDataxD;
    SysCtrlRegxMO <= SysCtrlRegxM;

end rtl;
