-- ------------------------------------------------------------------------------
-- File Name         :  $RCSfile:  $
-- Author            :  $Author:  $
-- Revision          :  $Revision:  $
-- Last Modification :  $Date:  $
-- ------------------------------------------------------------------------------
--                                                 __________
--                                   _            / ________ \ 
--                                  | |          / /    ___ \ \ 
--                                  | |         / /    |__ \ \ \ 
--                                  | |        / /     __ \ \ \ \ 
--          ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
--         / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
--        | |      | | | |  ____| | | |            \ \____/ / / /
--        | |      | | | | |_____/  | |             \______/ / /
--        | |      | | | |________  | |________         ____/ /
--        |_|      |_|  \_________|  \_________|       |_____/
--
--                                   NEW ELECTRONIC TECHNOLOGY
--
-- ------------------------------------------------------------------------------
--
-- Copyright (C) 2016 NET GmbH All Rights Reserved.
--
-- This document, and its contents are the property of NET GmbH.
-- NET GmbH retains all rights to: this document, the document contents,
-- and the design techniques expressed by the document contents.
-- Neither this document, nor any of the information, nor any of the design
-- techniques expressed within may be, either in whole or in part: used,
-- reproduced, copied, stored, or conveyed to a third party by any means
-- without the express written consent of an authorized representative of
-- NET GmbH.
--
-- ------------------------------------------------------------------------------
-- Module     : scale_d_cm
-- Submodules : 
--
-- Purpose    : scale_d_cm cut a roi out of the input image and scale it down. The
--              scaling factor is variable an can be set by the register interface.
--              It is not necessary to stop the acquisition when you change the 
--              roi-values or the scaling factor, because the new values will be active
--              when the input_evt-Bit will be set and the current image is finished.         
--
--                     _________      _________ 
--                    |         |    |         |
--      ImgRxPix--+-->| roi_gen |--->|  proc   |--->ImgTxPix
--                |   |_________|    |_________|
--                |       /.\            /.\
--                |        .              .
--                |        ................ param
--                |                       .
--                | ImgRxPixStat       ___.____
--                +------------------>| shadow |
--                  _______           |  reg   |
--                 |       |     ....>|________|
--      SysBus --->| regif |......
--                 |_______| param_sr         
--                                    
-- Interface Description:
--    ----------------------------------------------------------------
--    -- Module Infrastructure Signals
--    ----------------------------------------------------------------
--    ImgRstxRI:             ImagePipeline Reset
--    SysClkxCI:             Custom Module Clock
--    SysRstxRI:             Custom Module Reset
--    ----------------------------------------------------------------
--    -- ImageBus Receive Port
--    ----------------------------------------------------------------
--    ImgRxPixDataxDI:       packed data
--    ImgRxPixRdyxMO:        receive Pixel Ready - ignore if
--                           cIMG_HANDSHAKE_EN = False
--    ImgRxPixStatxSI:       000 No-Status (NONE)
--                           001 Start-of-Frame (SOF)
--                           010 Start-of-single-Line-Frame (SOSLF)
--                           011 Start-of-Data (SOD)
--                           100 Start-of-Line (SOL)
--                           101 Start-of-Last-Line (SOLL)
--                           110 End-of-Line/Data (EOL)
--                           111 Data-Error (ERR)
--    ImgRxPixValxMI:        transaction valid
--    ----------------------------------------------------------------
--    -- ImageBus Transmit Port
--    ----------------------------------------------------------------
--    ImgTxPixDataxDO:       packed data
--    ImgTxPixRdyxMI:        ImageBus Transmit Pixel Ready -
--                           ignore if cIMG_HANDSHAKE_EN = False
--    ImgTxPixStatxSO:       000 No-Status (NONE)
--                           001 Start-of-Frame (SOF)
--                           010 Start-of-single-Line-Frame (SOSLF)
--                           011 Start-of-Data (SOD)
--                           100 Start-of-Line (SOL)
--                           101 Start-of-Last-Line (SOLL)
--                           110 End-of-Line/Data (EOL)
--                           111 Data-Error (ERR)
--    ImgTxPixValxMO:        transaction valid
--    ----------------------------------------------------------------
--    -- SystemBus Control Interface
--    ----------------------------------------------------------------
--    SysAckxMO:             indicates successful cycle
--                           termination
--    SysAddrxAI:            address input array
--    SysErrxMO:             indicates abnormal cycle termination
--    SysRdDataxDO:          read data
--    SysReqxMI:             cycle input indicating a valid bus cycle
--    SysWrDataxDI:          write data
--    SysWrEnxMI:            write enable
--    ----------------------------------------------------------------
--    -- Custom Module GPIO
--    ----------------------------------------------------------------
--    CustIoCtrlxMO:         GPIO Output Control
--                           00: CustIoDataOutxDO(1) disable,
--                               CustIoDataOutxDO(0) disable
--                           01: CustIoDataOutxDO(1) disable,
--                               CustIoDataOutxDO(0) enable
--                           10: CustIoDataOutxDO(1) enable,
--                               CustIoDataOutxDO(0) disable
--                           11: CustIoDataOutxDO(1) enable,
--                               CustIoDataOutxDO(0) enable
--    CustIoDataOutxDO:      GPIO Output Data
--    CustIoDataInxDI:       GPIO Input Data (active Low)
--    ----------------------------------------------------------------
--    -- MCP command
--    ----------------------------------------------------------------
--    px_cmd_addr_o:         Byte start address for current
--                           transaction. Addresses must be
--                           aligned to port size:
--                           32-bit ports: Lower two bits must be
--                           0s.
--    px_cmd_bl_o:           Burst length in number of user words
--                           for the current transaction. Burst
--                           length is encoded as 0 to 63,
--                           representing 1 to 64 user words [for
--                           example, b00011 is a burst length 4
--                           transaction]. The user word width
--                           equals the port width [for example,
--                           a burst length of 3 on a 64-bit port
--                           transfers 3 x 64-bit user words =
--                           192 bits total].
--    px_cmd_en_o:           This active-High signal is the
--                           write-enable signal for the Command
--                           FIFO. This signal is covered in more
--                           detail in Chapter 4.
--    px_cmd_full_i:         This active-High output is the full
--                           flag for the Command FIFO. It
--                           indicates the FIFO cannot accept any
--                           more commands and blocks writes to
--                           the Command FIFO.
--    px_cmd_instr_o:        Command code for the current
--                           instruction. Bit 0 represents the
--                           READ/WRITE select, Bit 1 is Auto
--                           Precharge enable, and Bit 2
--                           represents Refresh, which always
--                           takes priority:
--                           Write: 3b000
--                           Read: 3b001
--                           This signal is covered in more
--                           detail in Chapter 4.
--    ----------------------------------------------------------------
--    -- MCB read          
--    ----------------------------------------------------------------
--    px_rd_data_i:          Read Data value returning from
--                           memory. This signal is driven by the
--                           output of the Read Data FIFO into
--                           FPGA logic. PX_SIZE can be 32, 64,
--                           or 128 bits, depending on the port
--                           configuration.
--    px_rd_empty_i:         This active-High signal is the empty
--                           flag for the Read Data FIFO. It
--                           indicates there is no valid data in
--                           the FIFO.
--    px_rd_en_o:            This active-High signal is the read
--                           enable for the Read Data FIFO. Read
--                           Data is clocked out of the FIFO on
--                           the rising edge of pX_rd_clk when
--                           pX_rd_en = 1 and pX_rd_empty = 0.
--    ----------------------------------------------------------------
--    -- MCP write          
--    ----------------------------------------------------------------
--    px_wr_data_o:          Write Data value to be loaded into
--                           Write Data FIFO and sent to memory.
--                           PX_SIZE can be 32, 64, or 128 bits,
--                           depending on port configuration.
--    px_wr_en_o:            This active-High signal is the write
--                           enable for the Write Data FIFO. It
--                           indicates that the value on
--                           pX_wr_data is valid for loading into
--                           the FIFO. Data is loaded on the
--                           rising edge of pX_wr_clk when
--                           pX_wr_en = 1 and pX_wr_full = 0.
--    px_wr_full_i:          This active-High signal is the full
--                           flag for the Write Data FIFO. When
--                           this signal is high, it prevents
--                           data from being loade into the FIFO.
--
--
-- Creator    : a.kramer
-- ------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library work;
use work.scale_d_cm_pkg.all;


entity CustomModule is	
  port(
    -- Module Infrastructure Signals
    ImgRstxRI:            in std_logic;
    SysClkxCI:            in std_logic;
    SysRstxRI:            in std_logic;
    -- ImageBus Receive Port
    ImgRxPixDataxDI:      in std_logic_vector(35 downto 0);
    ImgRxPixRdyxMO:       out std_logic;
    ImgRxPixStatxSI:      in std_logic_vector(2 downto 0);
    ImgRxPixValxMI:       in std_logic;
    -- ImageBus Transmit Port
    ImgTxPixDataxDO:      out std_logic_vector(35 downto 0);
    ImgTxPixRdyxMI:       in std_logic;
    ImgTxPixStatxSO:      out std_logic_vector(2 downto 0);
    ImgTxPixValxMO:       out std_logic;
    -- SystemBus Control Interface
    SysAckxMO:            out std_logic;
    SysAddrxAI:           in std_logic_vector(16 downto 0);
    SysErrxMO:            out std_logic;
    SysRdDataxDO:         out std_logic_vector(31 downto 0);
    SysReqxMI:            in std_logic;
    SysWrDataxDI:         in std_logic_vector(31 downto 0);
    SysWrEnxMI:           in std_logic;
    -- Custom Module GPIO
    CustIoCtrlxMO:        out std_logic_vector(NUM_CM_GPIO_OUT-1 downto 0);
    CustIoDataOutxDO:     out std_logic_vector(NUM_CM_GPIO_OUT-1 downto 0);
    CustIoDataInxDI:      in  std_logic_vector(NUM_CM_GPIO_IN-1 downto 0);
    -- MCP command
    px_cmd_addr_o:        out std_logic_vector(29 downto 0);
    px_cmd_bl_o:          out std_logic_vector(5 downto 0);
    px_cmd_en_o:          out std_logic;
    px_cmd_full_i:        in std_logic;
    px_cmd_instr_o:       out std_logic_vector(2 downto 0);
    -- MCB read
    px_rd_data_i:         in std_logic_vector(31 downto 0);
    px_rd_empty_i:        in std_logic;
    px_rd_en_o:           out std_logic;
    -- MCP write
    px_wr_data_o:         out std_logic_vector(31 downto 0);
    px_wr_en_o:           out std_logic;
    px_wr_full_i:         in std_logic
    );  
end CustomModule;


architecture hdr_cm_a of CustomModule is
  
  component scale_d_cm_roi_gen_e
    port(SysClkxCI         : in  std_logic;
         SysRstxRI         : in  std_logic;
         ImgRxPixDataxDI   : in  std_logic_vector(35 downto 0);
         ImgRxPixRdyxMO    : out std_logic;
         ImgRxPixStatxSI   : in  std_logic_vector(2 downto 0);
         ImgRxPixValxMI    : in  std_logic;
         ImgTxPixDataxDO   : out std_logic_vector(35 downto 0);
         ImgTxPixRdyxMI    : in  std_logic;
         ImgTxPixStatxSO   : out std_logic_vector(2 downto 0);
         ImgTxPixValxMO    : out std_logic;
         param_picx_i      : in  std_logic_vector(MAX_LLENGTH - 1 downto 0);
         param_picy_i      : in  std_logic_vector(MAX_LNUM - 1 downto 0);
         param_picx_offs_i : in  std_logic_vector(MAX_LLENGTH - 1 downto 0);
         param_picy_offs_i : in  std_logic_vector(MAX_LNUM - 1 downto 0);
         param_bypass_en_i : in  std_logic);
  end component scale_d_cm_roi_gen_e;
  
  component scale_d_cm_proc_e
    port(SysClkxCI          : in  std_logic;
         SysRstxRI          : in  std_logic;
         ImgRxPixDataxDI    : in  std_logic_vector(35 downto 0);
         ImgRxPixRdyxMO     : out std_logic;
         ImgRxPixStatxSI    : in  std_logic_vector(2 downto 0);
         ImgRxPixValxMI     : in  std_logic;
         ImgTxPixDataxDO    : out std_logic_vector(35 downto 0);
         ImgTxPixRdyxMI     : in  std_logic;
         ImgTxPixStatxSO    : out std_logic_vector(2 downto 0);
         ImgTxPixValxMO     : out std_logic;
         param_picx_i       : in  std_logic_vector(MAX_LLENGTH - 1 downto 0);
         param_picy_i       : in  std_logic_vector(MAX_LNUM - 1 downto 0);
         param_scale_fact_i : in  std_logic_vector(SCALE_INT + SCALE_FRAC - 1 downto 0);
         param_bypass_en_i  : in  std_logic);
  end component scale_d_cm_proc_e;
  
  component scale_d_cm_regif_e
    port(SysClkxCI          : in  std_logic;
         SysRstxRI          : in  std_logic;
         SysAckxMO          : out std_logic;
         SysAddrxAI         : in  std_logic_vector(16 downto 0);
         SysErrxMO          : out std_logic;
         SysRdDataxDO       : out std_logic_vector(31 downto 0);
         SysReqxMI          : in  std_logic;
         SysWrDataxDI       : in  std_logic_vector(31 downto 0);
         SysWrEnxMI         : in  std_logic;
         param_mem_ptr_o    : out std_logic_vector(29 downto 0);
         param_mem_size_o   : out std_logic_vector(29 downto 0);
         param_mem_rdy_o    : out std_logic;
         param_picx_o       : out std_logic_vector(MAX_LLENGTH - 1 downto 0);
         param_picy_o       : out std_logic_vector(MAX_LNUM - 1 downto 0);
         param_picx_offs_o  : out std_logic_vector(MAX_LLENGTH - 1 downto 0);
         param_picy_offs_o  : out std_logic_vector(MAX_LNUM - 1 downto 0);
         param_scale_fact_o : out std_logic_vector(SCALE_INT + SCALE_FRAC - 1 downto 0);
         param_input_evt_o  : out std_logic;
         param_bypass_en_o  : out std_logic);
  end component scale_d_cm_regif_e;
  
  component scale_d_cm_shadow_reg_e
    port(SysClkxCI             : in  std_logic;
         SysRstxRI             : in  std_logic;
         ImgRxPixStatxSI       : in  std_logic_vector(2 downto 0);
         ImgRxPixValxMI        : in  std_logic;
         param_sr_picx_i       : in  std_logic_vector(MAX_LLENGTH - 1 downto 0);
         param_sr_picy_i       : in  std_logic_vector(MAX_LNUM - 1 downto 0);
         param_sr_picx_offs_i  : in  std_logic_vector(MAX_LLENGTH - 1 downto 0);
         param_sr_picy_offs_i  : in  std_logic_vector(MAX_LNUM - 1 downto 0);
         param_sr_scale_fact_i : in  std_logic_vector(SCALE_INT + SCALE_FRAC - 1 downto 0);
         param_sr_input_evt_i  : in  std_logic;
         param_sr_bypass_en_i  : in  std_logic;
         param_picx_o          : out std_logic_vector(MAX_LLENGTH - 1 downto 0);
         param_picy_o          : out std_logic_vector(MAX_LNUM - 1 downto 0);
         param_picx_offs_o     : out std_logic_vector(MAX_LLENGTH - 1 downto 0);
         param_picy_offs_o     : out std_logic_vector(MAX_LNUM - 1 downto 0);
         param_scale_fact_o    : out std_logic_vector(SCALE_INT + SCALE_FRAC - 1 downto 0);
         param_bypass_en_o     : out std_logic);
  end component scale_d_cm_shadow_reg_e;
  
  
  signal ImgRoiPixDataxD       : std_logic_vector(35 downto 0);
  signal ImgRoiPixRdyxM        : std_logic;
  signal ImgRoiPixStatxS       : std_logic_vector(2 downto 0);
  signal ImgRoiPixValxM        : std_logic;
  
  signal param_sr_picx_s       : std_logic_vector(MAX_LLENGTH - 1 downto 0);
  signal param_sr_picy_s       : std_logic_vector(MAX_LNUM - 1 downto 0);
  signal param_sr_picx_offs_s  : std_logic_vector(MAX_LLENGTH - 1 downto 0);
  signal param_sr_picy_offs_s  : std_logic_vector(MAX_LNUM - 1 downto 0);
  signal param_sr_scale_fact_s : std_logic_vector(SCALE_INT + SCALE_FRAC - 1 downto 0);
  signal param_sr_input_evt_s  : std_logic;
  signal param_sr_bypass_en_s  : std_logic;
  
  signal param_picx_s          : std_logic_vector(MAX_LLENGTH - 1 downto 0);
  signal param_picy_s          : std_logic_vector(MAX_LNUM - 1 downto 0);
  signal param_picx_offs_s     : std_logic_vector(MAX_LLENGTH - 1 downto 0);
  signal param_picy_offs_s     : std_logic_vector(MAX_LNUM - 1 downto 0);
  signal param_scale_fact_s    : std_logic_vector(SCALE_INT + SCALE_FRAC - 1 downto 0);
  signal param_bypass_en_s     : std_logic;
  
begin
  
  -- gpios not unsed
  CustIoCtrlxMO    <= CM_GPIO_OUT_CTRL;
  CustIoDataOutxDO <= (others=>'0');

  -- mcb (ddr3) not unsed  
  px_cmd_addr_o  <= (others=>'0');
  px_cmd_bl_o    <= (others=>'0');
  px_cmd_en_o    <= '0';
  px_cmd_instr_o <= (others=>'0');
  px_rd_en_o     <= '0';
  px_wr_data_o   <= (others=>'0');
  px_wr_en_o     <= '0';
  
  
  ROI_GEN: component scale_d_cm_roi_gen_e
    port map(
      SysClkxCI         => SysClkxCI,
      SysRstxRI         => SysRstxRI,
      ImgRxPixDataxDI   => ImgRxPixDataxDI,
      ImgRxPixRdyxMO    => ImgRxPixRdyxMO,
      ImgRxPixStatxSI   => ImgRxPixStatxSI,
      ImgRxPixValxMI    => ImgRxPixValxMI,
      ImgTxPixDataxDO   => ImgRoiPixDataxD,
      ImgTxPixRdyxMI    => ImgRoiPixRdyxM,
      ImgTxPixStatxSO   => ImgRoiPixStatxS,
      ImgTxPixValxMO    => ImgRoiPixValxM,
      param_picx_i      => param_picx_s,
      param_picy_i      => param_picy_s,
      param_picx_offs_i => param_picx_offs_s,
      param_picy_offs_i => param_picy_offs_s,
      param_bypass_en_i => param_bypass_en_s
    );
  
  
  PROC: component scale_d_cm_proc_e
    port map(
      SysClkxCI          => SysClkxCI,
      SysRstxRI          => SysRstxRI,
      ImgRxPixDataxDI    => ImgRoiPixDataxD,
      ImgRxPixRdyxMO     => ImgRoiPixRdyxM,
      ImgRxPixStatxSI    => ImgRoiPixStatxS,
      ImgRxPixValxMI     => ImgRoiPixValxM,
      ImgTxPixDataxDO    => ImgTxPixDataxDO,
      ImgTxPixRdyxMI     => ImgTxPixRdyxMI,
      ImgTxPixStatxSO    => ImgTxPixStatxSO,
      ImgTxPixValxMO     => ImgTxPixValxMO,
      param_picx_i       => param_picx_s,
      param_picy_i       => param_picy_s,
      param_scale_fact_i => param_scale_fact_s,
      param_bypass_en_i  => param_bypass_en_s
    );


  REGIF: component scale_d_cm_regif_e
    port map(
      SysClkxCI          => SysClkxCI,
      SysRstxRI          => SysRstxRI,
      SysAckxMO          => SysAckxMO,
      SysAddrxAI         => SysAddrxAI,
      SysErrxMO          => SysErrxMO,
      SysRdDataxDO       => SysRdDataxDO,
      SysReqxMI          => SysReqxMI,
      SysWrDataxDI       => SysWrDataxDI,
      SysWrEnxMI         => SysWrEnxMI,
      param_mem_ptr_o    => open,
      param_mem_size_o   => open,
      param_mem_rdy_o    => open,
      param_picx_o       => param_sr_picx_s,
      param_picy_o       => param_sr_picy_s,
      param_picx_offs_o  => param_sr_picx_offs_s,
      param_picy_offs_o  => param_sr_picy_offs_s,
      param_scale_fact_o => param_sr_scale_fact_s,
      param_input_evt_o  => param_sr_input_evt_s,
      param_bypass_en_o  => param_sr_bypass_en_s
    );


  SHAD_REG: component scale_d_cm_shadow_reg_e
    port map(
      SysClkxCI             => SysClkxCI,
      SysRstxRI             => SysRstxRI,
      ImgRxPixStatxSI       => ImgRxPixStatxSI,
      ImgRxPixValxMI        => ImgRxPixValxMI,
      param_sr_picx_i       => param_sr_picx_s,
      param_sr_picy_i       => param_sr_picy_s,
      param_sr_picx_offs_i  => param_sr_picx_offs_s,
      param_sr_picy_offs_i  => param_sr_picy_offs_s,
      param_sr_scale_fact_i => param_sr_scale_fact_s,
      param_sr_input_evt_i  => param_sr_input_evt_s,
      param_sr_bypass_en_i  => param_sr_bypass_en_s,
      param_picx_o          => param_picx_s,
      param_picy_o          => param_picy_s,
      param_picx_offs_o     => param_picx_offs_s,
      param_picy_offs_o     => param_picy_offs_s,
      param_scale_fact_o    => param_scale_fact_s,
      param_bypass_en_o     => param_bypass_en_s
    );

	
end hdr_cm_a;