-- ------------------------------------------------------------------------------
-- File Name         :  $RCSfile:  $
-- Author            :  $Author:  $
-- Revision          :  $Revision:  $
-- Last Modification :  $Date:  $
-- ------------------------------------------------------------------------------
--                                                 __________
--                                   _            / ________ \ 
--                                  | |          / /    ___ \ \ 
--                                  | |         / /    |__ \ \ \ 
--                                  | |        / /     __ \ \ \ \ 
--          ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
--         / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
--        | |      | | | |  ____| | | |            \ \____/ / / /
--        | |      | | | | |_____/  | |             \______/ / /
--        | |      | | | |________  | |________         ____/ /
--        |_|      |_|  \_________|  \_________|       |_____/
--
--                                   NEW ELECTRONIC TECHNOLOGY
--
-- ------------------------------------------------------------------------------
--
-- Copyright (C) 2016 NET GmbH All Rights Reserved.
--
-- This document, and its contents are the property of NET GmbH.
-- NET GmbH retains all rights to: this document, the document contents,
-- and the design techniques expressed by the document contents.
-- Neither this document, nor any of the information, nor any of the design
-- techniques expressed within may be, either in whole or in part: used,
-- reproduced, copied, stored, or conveyed to a third party by any means
-- without the express written consent of an authorized representative of
-- NET GmbH.
--
-- ------------------------------------------------------------------------------
-- Module     : scale_d_cm_proc 
--
-- Purpose    : scale down the input image.
--              - linear interpolation between two pixel
--              - skip unused pixel
--
--                           _______                   _______ 
--       rgb -+------------>|       |--+------------->|       |---> rgb
--       in   |   _____     |       |  |   ______     |       |     out
--            |  |     |    | hipol |  |  | line |    | vipol |
--            +->| reg |--->|       |  +->| mem  |--->|       |
--               |_____|    |_______|     |______|    |_______|
--
--
-- Creator    : a.kramer
-- ------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library work;
use work.scale_d_cm_pkg.all;


entity scale_d_cm_proc_e is	
  port(
    -- Module Infrastructure Signals
    SysClkxCI:            in std_logic;
    SysRstxRI:            in std_logic;
    -- ImageBus Receive Port
    ImgRxPixDataxDI:      in std_logic_vector(35 downto 0);
    ImgRxPixRdyxMO:       out std_logic;
    ImgRxPixStatxSI:      in std_logic_vector(2 downto 0);
    ImgRxPixValxMI:       in std_logic;
    -- ImageBus Transmit Port
    ImgTxPixDataxDO:      out std_logic_vector(35 downto 0);
    ImgTxPixRdyxMI:       in std_logic;
    ImgTxPixStatxSO:      out std_logic_vector(2 downto 0);
    ImgTxPixValxMO:       out std_logic;
    -- Parameter
    param_picx_i:         in std_logic_vector(MAX_LLENGTH-1 downto 0);           -- roi picture width
    param_picy_i:         in std_logic_vector(MAX_LNUM-1 downto 0);              -- roi picture height
    param_scale_fact_i:   in std_logic_vector(SCALE_INT+SCALE_FRAC-1 downto 0);  -- scale down factor
    param_bypass_en_i:    in std_logic
    );  
end scale_d_cm_proc_e;


architecture scale_d_cm_proc_a of scale_d_cm_proc_e is
  
  type bram     is array (0 to (2**MAX_LLENGTH)-1)  of std_logic_vector(11 downto 0);
  type stat_v   is array (7 downto 0) of std_logic_vector(2 downto 0);
  
  signal line_mem_r:             bram;
  signal line_mem_g:             bram;
  signal line_mem_b:             bram;
  
  signal proc_val_s0:            std_logic;
  signal proc_out_x_val_s0:      std_logic;
  signal proc_out_x_val_s1:      std_logic;
  signal proc_out_x_val_s2:      std_logic;
  signal proc_out_x_val_s3:      std_logic;
  signal proc_out_x_val_s4:      std_logic;
  
  signal proc_out_val_s4:        std_logic;
  signal proc_out_val_s5:        std_logic;
  signal proc_out_val_s6:        std_logic;
  signal proc_out_val_s7:        std_logic;
  
  signal proc_stat_v:            stat_v;
  signal proc_ls_s:              std_logic;
  signal proc_ls_v:              std_logic_vector(2 downto 0);
  
  signal proc_r_x_now_s0:        std_logic_vector(11 downto 0);
  signal proc_r_x_prev_s0:       std_logic_vector(11 downto 0);
  signal proc_g_x_now_s0:        std_logic_vector(11 downto 0);
  signal proc_g_x_prev_s0:       std_logic_vector(11 downto 0);
  signal proc_b_x_now_s0:        std_logic_vector(11 downto 0);
  signal proc_b_x_prev_s0:       std_logic_vector(11 downto 0);
  
  signal proc_r_x_now_s1:        unsigned(11 downto 0);
  signal proc_r_x_prev_s1:       unsigned(11 downto 0);
  signal proc_g_x_now_s1:        unsigned(11 downto 0);
  signal proc_g_x_prev_s1:       unsigned(11 downto 0);
  signal proc_b_x_now_s1:        unsigned(11 downto 0);
  signal proc_b_x_prev_s1:       unsigned(11 downto 0);
  
  signal proc_r_y_now_s4:        unsigned(11 downto 0);
  signal proc_r_y_prev_s4:       unsigned(11 downto 0); 
  signal proc_g_y_now_s4:        unsigned(11 downto 0);
  signal proc_g_y_prev_s4:       unsigned(11 downto 0);
  signal proc_b_y_now_s4:        unsigned(11 downto 0);
  signal proc_b_y_prev_s4:       unsigned(11 downto 0);
  
  signal proc_r_y_now_s5:        unsigned(11 downto 0);
  signal proc_r_y_prev_s5:       unsigned(11 downto 0);
  signal proc_g_y_now_s5:        unsigned(11 downto 0);
  signal proc_g_y_prev_s5:       unsigned(11 downto 0);
  signal proc_b_y_now_s5:        unsigned(11 downto 0);
  signal proc_b_y_prev_s5:       unsigned(11 downto 0);
  
  signal ipol_x_now_s1:          unsigned(SCALE_FRAC downto 0);
  signal ipol_x_prev_s1:         unsigned(SCALE_FRAC downto 0);
  
  signal proc_r_x_ipol_now_s2:   unsigned(12+SCALE_FRAC downto 0);
  signal proc_r_x_ipol_prev_s2:  unsigned(12+SCALE_FRAC downto 0);
  signal proc_g_x_ipol_now_s2:   unsigned(12+SCALE_FRAC downto 0);
  signal proc_g_x_ipol_prev_s2:  unsigned(12+SCALE_FRAC downto 0);
  signal proc_b_x_ipol_now_s2:   unsigned(12+SCALE_FRAC downto 0);
  signal proc_b_x_ipol_prev_s2:  unsigned(12+SCALE_FRAC downto 0);
  
  signal proc_r_x_ipol_s2:       unsigned(12+SCALE_FRAC downto 0);
  signal proc_g_x_ipol_s2:       unsigned(12+SCALE_FRAC downto 0);
  signal proc_b_x_ipol_s2:       unsigned(12+SCALE_FRAC downto 0);
  
  signal proc_r_x_ipol_s3:       unsigned(12+SCALE_FRAC downto SCALE_FRAC+1);
  signal proc_g_x_ipol_s3:       unsigned(12+SCALE_FRAC downto SCALE_FRAC+1);
  signal proc_b_x_ipol_s3:       unsigned(12+SCALE_FRAC downto SCALE_FRAC+1);
  
  signal ipol_y_now_s5:          unsigned(SCALE_FRAC downto 0);
  signal ipol_y_prev_s5:         unsigned(SCALE_FRAC downto 0);
  
  signal proc_r_y_ipol_now_s6:   unsigned(12+SCALE_FRAC downto 0);
  signal proc_r_y_ipol_prev_s6:  unsigned(12+SCALE_FRAC downto 0);
  signal proc_g_y_ipol_now_s6:   unsigned(12+SCALE_FRAC downto 0);
  signal proc_g_y_ipol_prev_s6:  unsigned(12+SCALE_FRAC downto 0);
  signal proc_b_y_ipol_now_s6:   unsigned(12+SCALE_FRAC downto 0);
  signal proc_b_y_ipol_prev_s6:  unsigned(12+SCALE_FRAC downto 0);
  
  signal proc_r_y_ipol_s6:       unsigned(12+SCALE_FRAC downto 0);
  signal proc_g_y_ipol_s6:       unsigned(12+SCALE_FRAC downto 0);
  signal proc_b_y_ipol_s6:       unsigned(12+SCALE_FRAC downto 0);
  
  signal proc_r_y_ipol_s7:       std_logic_vector(12+SCALE_FRAC downto SCALE_FRAC+1);
  signal proc_g_y_ipol_s7:       std_logic_vector(12+SCALE_FRAC downto SCALE_FRAC+1);
  signal proc_b_y_ipol_s7:       std_logic_vector(12+SCALE_FRAC downto SCALE_FRAC+1);
  
  signal posx_cnt:               unsigned(MAX_LLENGTH+SCALE_FRAC-1 downto 0);
  signal posx_scale_cnt:         unsigned(MAX_LLENGTH+SCALE_FRAC-1 downto 0);
  signal posx_scale_cnt_next:    unsigned(MAX_LLENGTH+SCALE_FRAC-1 downto 0);
  signal posx_scale_cnt_next_s1: unsigned(MAX_LLENGTH+SCALE_FRAC-1 downto 0);
  signal posx_scale_cnt_next_ceil_s2: unsigned(MAX_LLENGTH+SCALE_FRAC-1 downto 0);
  signal posy_cnt:               unsigned(MAX_LNUM+SCALE_FRAC-1 downto 0);
  signal posy_scale_cnt:         unsigned(MAX_LNUM+SCALE_FRAC-1 downto 0);
  signal posy_scale_cnt_next:    unsigned(MAX_LNUM+SCALE_FRAC-1 downto 0);
  signal posy_scale_cnt_next_s6: unsigned(MAX_LNUM+SCALE_FRAC-1 downto 0);
  signal posy_scale_cnt_next_ceil_s7: unsigned(MAX_LNUM+SCALE_FRAC-1 downto 0);
  
  signal line_pos:               integer range 0 to (2**MAX_LLENGTH)-1;
  
  signal ImgRxPixDataxD:         std_logic_vector(35 downto 0);
  signal ImgRxPixStatxS:         std_logic_vector(2 downto 0);
  signal ImgRxPixValxM:          std_logic;
  
  signal ImgTxPixDataxD:         std_logic_vector(35 downto 0);
  signal ImgTxPixStatxS:         std_logic_vector(2 downto 0);
  signal ImgTxPixValxM:          std_logic;
  
  
begin
  
 
  proc_ls_s  <= '1' when (ImgRxPixValxM='1' and (ImgRxPixStatxS=SOF or ImgRxPixStatxS=SOL or ImgRxPixStatxS=SOLL)) else
                '0';

  
  ctl_reg: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      proc_ls_v <= (others => '0');
    elsif (SysClkxCI'event and SysClkxCI='1') then
      proc_ls_v(2 downto 0) <= proc_ls_v(1 downto 0) & proc_ls_s;
    end if;
  end process;


  x_buf: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      ImgRxPixDataxD <= ImgRxPixDataxDI;
      
      if (ImgRxPixValxM='1') then
        proc_r_x_now_s0  <= ImgRxPixDataxD(35 downto 24);
        proc_g_x_now_s0  <= ImgRxPixDataxD(23 downto 12);
        proc_b_x_now_s0  <= ImgRxPixDataxD(11 downto 0);
        if (ImgRxPixStatxS=SOF) then
          proc_r_x_prev_s0 <= (others => '0');
          proc_g_x_prev_s0 <= (others => '0');
          proc_b_x_prev_s0 <= (others => '0');
        else  
          proc_r_x_prev_s0 <= proc_r_x_now_s0;
          proc_g_x_prev_s0 <= proc_g_x_now_s0;
          proc_b_x_prev_s0 <= proc_b_x_now_s0;
        end if;
      end if;
      
      proc_r_x_now_s1  <= unsigned(proc_r_x_now_s0);
      proc_g_x_now_s1  <= unsigned(proc_g_x_now_s0);
      proc_b_x_now_s1  <= unsigned(proc_b_x_now_s0);
      proc_r_x_prev_s1 <= unsigned(proc_r_x_prev_s0);
      proc_g_x_prev_s1 <= unsigned(proc_g_x_prev_s0);
      proc_b_x_prev_s1 <= unsigned(proc_b_x_prev_s0);
    end if;
  end process;

  
  x_buf_evt: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      ImgRxPixValxM <= '0';
      proc_val_s0   <= '0';
    elsif (SysClkxCI'event and SysClkxCI='1') then
      ImgRxPixValxM <= ImgRxPixValxMI;
      proc_val_s0   <= ImgRxPixValxM;
    end if;
  end process;

  
  proc_out_x_val_s0 <= '1' when (posx_cnt>=posx_scale_cnt and proc_val_s0='1') else
                       '0';

  x_ipol_scale_gen: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      posx_cnt <= (others => '0');
      posx_scale_cnt <= (others => '0');
      posx_scale_cnt_next_s1 <= (others => '0');
      proc_out_x_val_s1 <= '0';
      proc_out_x_val_s2 <= '0';
      proc_out_x_val_s3 <= '0';
      proc_out_x_val_s4 <= '0';
    elsif (SysClkxCI'event and SysClkxCI='1') then
      proc_out_x_val_s1 <= proc_out_x_val_s0;
      proc_out_x_val_s2 <= proc_out_x_val_s1;
      proc_out_x_val_s3 <= proc_out_x_val_s2;
      proc_out_x_val_s4 <= proc_out_x_val_s3;
      
      if (proc_ls_s='1') then
        posx_cnt <= (others => '0');
      elsif (proc_val_s0='1') then
        posx_cnt <= posx_cnt + (2**SCALE_FRAC);
      end if;
      
      if (proc_ls_s='1') then
        posx_scale_cnt <= (others => '0');
      elsif (proc_out_x_val_s0='1') then
        posx_scale_cnt <= posx_scale_cnt_next;
      end if;
      
      posx_scale_cnt_next_s1 <= posx_scale_cnt_next;
      posx_scale_cnt_next_ceil_s2 <= posx_scale_cnt_next_s1 + ((2**SCALE_FRAC)-1);    -- round up
    end if;
  end process;
  
  posx_scale_cnt_next <= posx_scale_cnt + unsigned(param_scale_fact_i);
      
  x_ipol: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      if (to_integer(posx_scale_cnt(SCALE_FRAC-1 downto 0))=0) then
        ipol_x_now_s1  <= to_unsigned(2**SCALE_FRAC,SCALE_FRAC+1);
        ipol_x_prev_s1 <= (others => '0');
      else
        ipol_x_now_s1  <= '0' & posx_scale_cnt(SCALE_FRAC-1 downto 0);
        ipol_x_prev_s1 <= ('0' & not posx_scale_cnt(SCALE_FRAC-1 downto 0)) + 1;
      end if;
      
      proc_r_x_ipol_now_s2  <= proc_r_x_now_s1  * ipol_x_now_s1;
      proc_r_x_ipol_prev_s2 <= proc_r_x_prev_s1 * ipol_x_prev_s1;
      proc_g_x_ipol_now_s2  <= proc_g_x_now_s1  * ipol_x_now_s1;
      proc_g_x_ipol_prev_s2 <= proc_g_x_prev_s1 * ipol_x_prev_s1;
      proc_b_x_ipol_now_s2  <= proc_b_x_now_s1  * ipol_x_now_s1;
      proc_b_x_ipol_prev_s2 <= proc_b_x_prev_s1 * ipol_x_prev_s1;
      
      proc_r_x_ipol_s3 <= proc_r_x_ipol_s2(11+SCALE_FRAC downto SCALE_FRAC);
      proc_g_x_ipol_s3 <= proc_g_x_ipol_s2(11+SCALE_FRAC downto SCALE_FRAC);
      proc_b_x_ipol_s3 <= proc_b_x_ipol_s2(11+SCALE_FRAC downto SCALE_FRAC);
    end if;
  end process;
   
  proc_r_x_ipol_s2 <= proc_r_x_ipol_now_s2 + proc_r_x_ipol_prev_s2;
  proc_g_x_ipol_s2 <= proc_g_x_ipol_now_s2 + proc_g_x_ipol_prev_s2;
  proc_b_x_ipol_s2 <= proc_b_x_ipol_now_s2 + proc_b_x_ipol_prev_s2;
  
  
  
  linepos: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      line_pos <= 0;
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (proc_ls_v(2)='1') then
        line_pos <= 0;
      elsif (proc_out_x_val_s2='1') then
        line_pos <= line_pos + 1;
      end if; 
    end if;
  end process;
  
  linemem: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      if (proc_out_x_val_s3='1') then
        line_mem_r(line_pos) <= std_logic_vector(proc_r_x_ipol_s3);
        line_mem_g(line_pos) <= std_logic_vector(proc_g_x_ipol_s3);
        line_mem_b(line_pos) <= std_logic_vector(proc_b_x_ipol_s3);
      end if;
      proc_r_y_prev_s4 <= unsigned(line_mem_r(line_pos));
      proc_g_y_prev_s4 <= unsigned(line_mem_g(line_pos));
      proc_b_y_prev_s4 <= unsigned(line_mem_b(line_pos));
        
      proc_r_y_now_s4  <= proc_r_x_ipol_s3;
      proc_g_y_now_s4  <= proc_g_x_ipol_s3;
      proc_b_y_now_s4  <= proc_b_x_ipol_s3;


      if (posy_cnt=x"000") then
        proc_r_y_prev_s5 <= (others => '0');
        proc_g_y_prev_s5 <= (others => '0');
        proc_b_y_prev_s5 <= (others => '0');
      else
        proc_r_y_prev_s5 <= proc_r_y_prev_s4;
        proc_g_y_prev_s5 <= proc_g_y_prev_s4;
        proc_b_y_prev_s5 <= proc_b_y_prev_s4;
      end if;
      
      proc_r_y_now_s5  <= proc_r_y_now_s4;
      proc_g_y_now_s5  <= proc_g_y_now_s4;
      proc_b_y_now_s5  <= proc_b_y_now_s4;   
    end if;
  end process;
  
 

  proc_out_val_s4 <= '1' when (posy_cnt>=posy_scale_cnt and proc_out_x_val_s4='1') else
                     '0';
  
  y_ipol_scale_gen: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      posy_cnt <= (others => '0');
      posy_scale_cnt <= (others => '0');
      posy_scale_cnt_next_s6 <= (others => '0');
      proc_out_val_s5 <= '0';
      proc_out_val_s6 <= '0';
      proc_out_val_s7 <= '0';
    elsif (SysClkxCI'event and SysClkxCI='1') then
      proc_out_val_s5 <= proc_out_val_s4;
      proc_out_val_s6 <= proc_out_val_s5;
      proc_out_val_s7 <= proc_out_val_s6;

      if (proc_stat_v(4)=SOF) then
        posy_cnt <= (others => '0');
      elsif (proc_out_x_val_s4='1' and proc_stat_v(4)=EOL) then
        posy_cnt <= posy_cnt + (2**SCALE_FRAC);
      end if;
      
      if (proc_stat_v(4)=SOF) then
        posy_scale_cnt <= (others => '0');
      elsif (proc_out_val_s4='1' and proc_stat_v(4)=EOL) then
        posy_scale_cnt <= posy_scale_cnt_next;
      end if;
      
      posy_scale_cnt_next_s6 <= posy_scale_cnt_next;
      posy_scale_cnt_next_ceil_s7 <= posy_scale_cnt_next_s6 + ((2**SCALE_FRAC)-1);    -- round up
    end if;
  end process;    
      
  posy_scale_cnt_next <= posy_scale_cnt + unsigned(param_scale_fact_i);
  
  y_ipol: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      if (to_integer(posy_scale_cnt(SCALE_FRAC-1 downto 0))=0) then
        ipol_y_now_s5  <= to_unsigned(2**SCALE_FRAC,SCALE_FRAC+1);
        ipol_y_prev_s5 <= (others => '0');
      else
        ipol_y_now_s5  <= '0' & posy_scale_cnt(SCALE_FRAC-1 downto 0);
        ipol_y_prev_s5 <= ('0' & not posy_scale_cnt(SCALE_FRAC-1 downto 0)) + 1;
      end if;
      
      proc_r_y_ipol_now_s6  <= proc_r_y_now_s5  * ipol_y_now_s5;
      proc_r_y_ipol_prev_s6 <= proc_r_y_prev_s5 * ipol_y_prev_s5;
      proc_g_y_ipol_now_s6  <= proc_g_y_now_s5  * ipol_y_now_s5;
      proc_g_y_ipol_prev_s6 <= proc_g_y_prev_s5 * ipol_y_prev_s5;
      proc_b_y_ipol_now_s6  <= proc_b_y_now_s5  * ipol_y_now_s5;
      proc_b_y_ipol_prev_s6 <= proc_b_y_prev_s5 * ipol_y_prev_s5;
      
      proc_r_y_ipol_s7 <= std_logic_vector(proc_r_y_ipol_s6(11+SCALE_FRAC downto SCALE_FRAC));
      proc_g_y_ipol_s7 <= std_logic_vector(proc_g_y_ipol_s6(11+SCALE_FRAC downto SCALE_FRAC));
      proc_b_y_ipol_s7 <= std_logic_vector(proc_b_y_ipol_s6(11+SCALE_FRAC downto SCALE_FRAC));
    end if;
  end process;
  
  proc_r_y_ipol_s6 <= proc_r_y_ipol_now_s6 + proc_r_y_ipol_prev_s6;
  proc_g_y_ipol_s6 <= proc_g_y_ipol_now_s6 + proc_g_y_ipol_prev_s6;
  proc_b_y_ipol_s6 <= proc_b_y_ipol_now_s6 + proc_b_y_ipol_prev_s6;
  
  
  
  stat_reg: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      ImgRxPixStatxS <= ImgRxPixStatxSI;
      proc_stat_v(2 downto 0) <= proc_stat_v(1 downto 0) & ImgRxPixStatxS;
      
      if (proc_out_x_val_s2='0') then
        proc_stat_v(3) <= NOP;
      elsif (posx_scale_cnt_next_ceil_s2(MAX_LLENGTH+SCALE_FRAC-1 downto SCALE_FRAC)>=unsigned(param_picx_i)) then
        proc_stat_v(3) <= EOL;
      else 
        proc_stat_v(3) <=proc_stat_v(2);
      end if;
      
      proc_stat_v(4) <= proc_stat_v(3);
      
      if (proc_out_val_s4='1') then
        proc_stat_v(5) <= proc_stat_v(4);
      else
        proc_stat_v(5) <= NOP;
      end if;
      
      proc_stat_v(7 downto 6) <= proc_stat_v(6 downto 5);
      
      if (param_bypass_en_i='1') then
        ImgTxPixStatxS <= ImgRxPixStatxS;
      elsif (posy_scale_cnt_next_ceil_s7(MAX_LNUM+SCALE_FRAC-1 downto SCALE_FRAC)>=unsigned(param_picy_i) and proc_stat_v(7)=SOL) then
        ImgTxPixStatxS <= SOLL;
      else
        ImgTxPixStatxS <= proc_stat_v(7);
      end if;
    end if;
  end process;
  

  out_reg_val: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      ImgTxPixValxM <= '0';
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (param_bypass_en_i='1') then
        ImgTxPixValxM  <= ImgRxPixValxM;
      else
        ImgTxPixValxM  <= proc_out_val_s7;
      end if;
    end if;
  end process;
  
  out_reg_dat: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      if (param_bypass_en_i='1') then
        ImgTxPixDataxD <= ImgRxPixDataxD;
      else
        ImgTxPixDataxD <= proc_r_y_ipol_s7 & proc_g_y_ipol_s7 & proc_b_y_ipol_s7;
      end if;
    end if;
  end process;
  
  ImgTxPixValxMO  <= ImgTxPixValxM;
  ImgTxPixStatxSO <= ImgTxPixStatxS;
  ImgTxPixDataxDO <= ImgTxPixDataxD;
  ImgRxPixRdyxMO  <= ImgTxPixRdyxMI;
	
end scale_d_cm_proc_a;

