-- ------------------------------------------------------------------------------
-- File Name         :  $RCSfile:  $
-- Author            :  $Author:  $
-- Revision          :  $Revision:  $
-- Last Modification :  $Date:  $
-- ------------------------------------------------------------------------------
--                                                 __________
--                                   _            / ________ \ 
--                                  | |          / /    ___ \ \ 
--                                  | |         / /    |__ \ \ \ 
--                                  | |        / /     __ \ \ \ \ 
--          ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
--         / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
--        | |      | | | |  ____| | | |            \ \____/ / / /
--        | |      | | | | |_____/  | |             \______/ / /
--        | |      | | | |________  | |________         ____/ /
--        |_|      |_|  \_________|  \_________|       |_____/
--
--                                   NEW ELECTRONIC TECHNOLOGY
--
-- ------------------------------------------------------------------------------
--
-- Copyright (C) 2016 NET GmbH All Rights Reserved.
--
-- This document, and its contents are the property of NET GmbH.
-- NET GmbH retains all rights to: this document, the document contents,
-- and the design techniques expressed by the document contents.
-- Neither this document, nor any of the information, nor any of the design
-- techniques expressed within may be, either in whole or in part: used,
-- reproduced, copied, stored, or conveyed to a third party by any means
-- without the express written consent of an authorized representative of
-- NET GmbH.
--
-- ------------------------------------------------------------------------------
-- Module     : scale_d_cm_roi_gen
--
-- Purpose    : cut a roi out of the input image
--              roi size:     param_picx_i * param_picy_offs_i
--              roi position: param_picx_offs_i, param_picy_offs_i
--
-- Creator    : a.kramer
-- ------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library work;
use work.scale_d_cm_pkg.all;


entity scale_d_cm_roi_gen_e is	
  port(
    -- Module Infrastructure Signals
    SysClkxCI:            in std_logic;
    SysRstxRI:            in std_logic;
    -- ImageBus Receive Port
    ImgRxPixDataxDI:      in std_logic_vector(35 downto 0);
    ImgRxPixRdyxMO:       out std_logic;
    ImgRxPixStatxSI:      in std_logic_vector(2 downto 0);
    ImgRxPixValxMI:       in std_logic;
    -- ImageBus Transmit Port
    ImgTxPixDataxDO:      out std_logic_vector(35 downto 0);
    ImgTxPixRdyxMI:       in std_logic;
    ImgTxPixStatxSO:      out std_logic_vector(2 downto 0);
    ImgTxPixValxMO:       out std_logic;
    -- Parameter
    param_picx_i:         in std_logic_vector(MAX_LLENGTH-1 downto 0);           -- roi picture width
    param_picy_i:         in std_logic_vector(MAX_LNUM-1 downto 0);              -- roi picture height
    param_picx_offs_i:    in std_logic_vector(MAX_LLENGTH-1 downto 0);           -- roi picture x offset
    param_picy_offs_i:    in std_logic_vector(MAX_LNUM-1 downto 0);              -- roi picture y offset
    param_bypass_en_i:    in std_logic
    );  
end scale_d_cm_roi_gen_e;


architecture scale_d_cm_roi_gen_a of scale_d_cm_roi_gen_e is
  
  
  signal roi_gen_data_s0:         std_logic_vector(35 downto 0);
  signal roi_gen_stat_s0:         std_logic_vector(2 downto 0);
  signal roi_gen_val_s0:          std_logic;
  
  signal roi_gen_data_s1:         std_logic_vector(35 downto 0);
  signal roi_gen_val_s1:          std_logic;
  
  signal roi_gen_data_s2:         std_logic_vector(35 downto 0);
  signal roi_gen_val_s2:          std_logic;
  
  signal roi_gen_data_s3:         std_logic_vector(35 downto 0);
  signal roi_gen_stat_s3:         std_logic_vector(2 downto 0);
  signal roi_gen_val_s3:          std_logic;
  
  signal ImgTxPixDataxD:          std_logic_vector(35 downto 0);
  signal ImgTxPixStatxS:          std_logic_vector(2 downto 0);
  signal ImgTxPixValxM:           std_logic;
  
  signal roi_posx_min:            unsigned(MAX_LLENGTH-1 downto 0);
  signal roi_posx_max:            unsigned(MAX_LLENGTH-1 downto 0);
  signal roi_posx_max_m1:         unsigned(MAX_LLENGTH-1 downto 0);
  signal roi_posy_min:            unsigned(MAX_LNUM-1 downto 0);
  signal roi_posy_max:            unsigned(MAX_LNUM-1 downto 0);
  signal roi_posy_max_m1:         unsigned(MAX_LNUM-1 downto 0);
  
  signal posx_cnt_s1:             unsigned(MAX_LLENGTH-1 downto 0);
  signal posy_cnt_s1:             unsigned(MAX_LNUM-1 downto 0);
  
  signal roi_posx_en_s2:          std_logic;
  signal roi_posy_en_s2:          std_logic;
  
  signal roi_sof_s2:              std_logic;
  signal roi_sol_s2:              std_logic;
  signal roi_soll_s2:             std_logic;
  signal roi_eol_s2:              std_logic;

  
begin
  

  roi_pos: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      roi_posx_min    <= unsigned(param_picx_offs_i);
      roi_posx_max    <= unsigned(param_picx_offs_i) + unsigned(param_picx_i);
      roi_posx_max_m1 <= roi_posx_max - 1;
      roi_posy_min    <= unsigned(param_picy_offs_i);
      roi_posy_max    <= unsigned(param_picy_offs_i) + unsigned(param_picy_i);
      roi_posy_max_m1 <= roi_posy_max - 1;
    end if;
  end process;


  pos_cnt: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      posx_cnt_s1 <= (others => '0');
      posy_cnt_s1 <= (others => '0');
      roi_posx_en_s2 <= '0';
      roi_posy_en_s2 <= '0';
      roi_sof_s2     <= '0';
      roi_sol_s2     <= '0';
      roi_soll_s2    <= '0';
      roi_eol_s2     <= '0';
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (roi_gen_val_s0='1' and (roi_gen_stat_s0=SOF or roi_gen_stat_s0=SOL or roi_gen_stat_s0=SOLL)) then
        posx_cnt_s1  <= (others => '0');
      elsif (roi_gen_val_s0='1') then
        posx_cnt_s1  <= posx_cnt_s1 + 1;
      end if;
      
      if (roi_gen_val_s0='1' and roi_gen_stat_s0=SOF) then
        posy_cnt_s1  <= (others => '0');
      elsif (roi_gen_val_s0='1' and (roi_gen_stat_s0=SOL or roi_gen_stat_s0=SOLL)) then
        posy_cnt_s1  <= posy_cnt_s1 + 1;
      end if;
      
      if (posx_cnt_s1>=roi_posx_min and posx_cnt_s1<roi_posx_max) then
        roi_posx_en_s2 <= '1';
      else
        roi_posx_en_s2 <= '0';
      end if;
      
      if (posy_cnt_s1>=roi_posy_min and posy_cnt_s1<roi_posy_max) then
        roi_posy_en_s2 <= '1';
      else
        roi_posy_en_s2 <= '0';
      end if;
      
      if (posx_cnt_s1=roi_posx_min and posy_cnt_s1=roi_posy_min) then
        roi_sof_s2 <= '1';
      else
        roi_sof_s2 <= '0';
      end if;
      
      if (posx_cnt_s1=roi_posx_min) then
        roi_sol_s2 <= '1';
      else
        roi_sol_s2 <= '0';
      end if;
      
      if (posx_cnt_s1=roi_posx_min and posy_cnt_s1=roi_posy_max_m1) then
        roi_soll_s2 <= '1';
      else
        roi_soll_s2 <= '0';
      end if;
      
      if (posx_cnt_s1=roi_posx_max_m1) then
        roi_eol_s2 <= '1';
      else
        roi_eol_s2 <= '0';
      end if;
    end if;
  end process;
  
  
  buf_reg_val: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      roi_gen_val_s0  <= '0';
      roi_gen_val_s1  <= '0';
      roi_gen_val_s2  <= '0';
      roi_gen_val_s3  <= '0';
    elsif (SysClkxCI'event and SysClkxCI='1') then
      roi_gen_val_s0  <= ImgRxPixValxMI;
      roi_gen_val_s1  <= roi_gen_val_s0;
      roi_gen_val_s2  <= roi_gen_val_s1;
      if (roi_posx_en_s2='1' and roi_posy_en_s2='1' and roi_gen_val_s2='1') then
        roi_gen_val_s3  <= '1';
      else
        roi_gen_val_s3  <= '0';
      end if;    
    end if;
  end process;
  
  buf_reg: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      roi_gen_data_s0 <= ImgRxPixDataxDI;
      roi_gen_stat_s0 <= ImgRxPixStatxSI;

      roi_gen_data_s1 <= roi_gen_data_s0;

      roi_gen_data_s2 <= roi_gen_data_s1;
      
      roi_gen_data_s3 <= roi_gen_data_s2; 
      if (roi_posx_en_s2='1' and roi_posy_en_s2='1' and roi_gen_val_s2='1') then
        if (roi_sof_s2='1') then
          roi_gen_stat_s3 <= SOF;
        elsif (roi_soll_s2='1') then
          roi_gen_stat_s3 <= SOLL;
        elsif (roi_sol_s2='1') then
          roi_gen_stat_s3 <= SOL;
        elsif (roi_eol_s2='1') then
          roi_gen_stat_s3 <= EOL;
        else
          roi_gen_stat_s3 <= NOP;
        end if;
      else
        roi_gen_stat_s3 <= NOP;
      end if;           
    end if;
  end process;


  out_reg_val: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      ImgTxPixValxM <= '0';
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (param_bypass_en_i='1') then
        ImgTxPixValxM  <= roi_gen_val_s0;
      else
        ImgTxPixValxM  <= roi_gen_val_s3;
      end if;
    end if;
  end process;
  
  out_reg: process(SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      if (param_bypass_en_i='1') then
        ImgTxPixDataxD <= roi_gen_data_s0;
        ImgTxPixStatxS <= roi_gen_stat_s0;
      else
        ImgTxPixDataxD <= roi_gen_data_s3;
        ImgTxPixStatxS <= roi_gen_stat_s3;
      end if;
    end if;
  end process;
  
  ImgTxPixValxMO  <= ImgTxPixValxM;
  ImgTxPixStatxSO <= ImgTxPixStatxS;
  ImgTxPixDataxDO <= ImgTxPixDataxD;
  ImgRxPixRdyxMO  <= ImgTxPixRdyxMI;
	
end scale_d_cm_roi_gen_a;

