-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Controller for OpenCamera USER design
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-20  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;

use work.CustomPkg.all;
use work.UserPkg.all;

entity UsrCtrl is
    port (
        -- Module Infrastructure Signals
        SysClkxCI        : in  std_logic;                                       -- Custom Module Clock 100 MHz
        SysRstxRI        : in  std_logic;                                       -- Custom Module Reset

        -- SystemBus Control/Status Port
        UsrCtrlRegxMI    : in  tUSR_CTRL_REG;                                   -- SystemBus USER Control Registers
        UsrStatRegxMO    : out tUSR_STAT_REG;                                   -- SystemBus USER Status Registers

        -- SystemBus Memory Access Port
        SysMemReqxMI     : in  std_logic;                                       -- SystemBus Memory Cycle Request
        SysMemAckxMO     : out std_logic;                                       -- SystemBus Memory Cycle Acknowledge
        SysMemErrxMO     : out std_logic;                                       -- SystemBus Memory Cycle Error
        SysMemWrEnxMI    : in  std_logic;                                       -- SystemBus Memory Write Enable
        SysMemAddrxAI    : in  std_logic_vector(cSYS_ADDR_WIDTH-1 downto 0);    -- SystemBus Memory Address
        SysMemWrDataxDI  : in  std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Memory Write Data
        SysMemRdDataxDO  : out std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Memory Read Data

        -- ImageBus Receive Port
        ImgRxPixValxMI   : in  std_logic;                                       -- ImageBus Receive Pixel Valid
        ImgRxPixRdyxMO   : out std_logic;                                       -- ImageBus Receive Pixel Ready
        ImgRxPixStatxSI  : in  std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);    -- ImageBus Receive Pixel Status
        ImgRxPixDataxDI  : in  std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);    -- ImageBus Receive Pixel Data

        -- ImageBus Transmit Port
        ImgTxPixValxMO   : out std_logic;                                       -- ImageBus Transmit Pixel Valid
        ImgTxPixRdyxMI   : in  std_logic;                                       -- ImageBus Transmit Pixel Ready
        ImgTxPixStatxSO  : out std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Status
        ImgTxPixDataxDO  : out std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);    -- ImageBus Transmit Pixel Data

        -- Custom Module GPIO
        CustIoCtrlxMO    : out std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Control
        CustIoDataOutxDO : out std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Output Data
        CustIoDataInxDI  : in  std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);    -- GPIO Input Data

        -- MemoryBus Command Port
        px_cmd_en_o      : out std_logic;                                       -- MemoryBus Command Valid
        px_cmd_full_i    : in  std_logic;                                       -- MemoryBus Command Buffer Full
        px_cmd_instr_o   : out std_logic_vector(cMEM_CMD_WIDTH-1 downto 0);     -- MemoryBus Command Instruction
        px_cmd_bl_o      : out std_logic_vector(cMEM_BLEN_WIDTH-1 downto 0);    -- MemoryBus Command Burst Length
        px_cmd_addr_o    : out std_logic_vector(cMEM_ADDR_WIDTH-1 downto 0);    -- MemoryBus Command Address

        -- MemoryBus Read Port
        px_rd_en_o       : out std_logic;                                       -- MemoryBus Read Data Valid
        px_rd_empty_i    : in  std_logic;                                       -- MemoryBus Read Data Buffer Empty
        px_rd_data_i     : in  std_logic_vector(cMEM_DATA_WIDTH-1 downto 0);    -- MemoryBus Read Data Bus

        -- MemoryBus Write Port
        px_wr_en_o       : out std_logic;                                       -- MemoryBus Write Data Valid
        px_wr_full_i     : in  std_logic;                                       -- MemoryBus Write Data Buffer Full
        px_wr_data_o     : out std_logic_vector(cMEM_DATA_WIDTH-1 downto 0);    -- MemoryBus Write Data Bus

        -- Debug Logic Port
        DbgLogicPortxTO  : out std_logic_vector(cDEBUG_PORT_WIDTH-1 downto 0)); -- Custom Module Debug Signals
end UsrCtrl;

architecture rtl of UsrCtrl is

    ---------------------------------------------------------------------------
    -- Signal Declarations
    ---------------------------------------------------------------------------
    signal ImgRxPixRdyxM    : std_logic;                                        -- ImageBus Receive Pixel Ready - ignore if cIMG_HANDSHAKE_EN = False
    signal ImgTxPixValxM    : std_logic;                                        -- ImageBus Transmit Pixel Valid
    signal ImgTxPixStatxS   : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);     -- ImageBus Transmit Pixel Status
    signal ImgTxPixDataxD   : std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);     -- ImageBus Transmit Pixel Data

    -- GPIO signals
    signal CustIoCtrlxM     : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);     -- GPIO Output Control
    signal CustIoDataOutxD  : std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);     -- GPIO Output Data

begin

    ----------------------------------------------------------------------------
    -- SystemBus Logic
    ----------------------------------------------------------------------------
    UsrStatRegxMO   <= cUSR_STAT_DEFAULT;
    SysMemAckxMO    <= SysMemReqxMI;
    SysMemErrxMO    <= '0';
    SysMemRdDataxDO <= (others => '0');

    ----------------------------------------------------------------------------
    -- ImageBus Logic
    ----------------------------------------------------------------------------
    ImgTxPixValxM   <= ImgRxPixValxMI; 
    ImgTxPixStatxS  <= ImgRxPixStatxSI;
    ImgTxPixDataxD  <= ImgRxPixDataxDI;
    ImgRxPixRdyxM   <= ImgTxPixRdyxMI;

    -- Assign internally used output ports
    ImgTxPixValxMO  <= ImgTxPixValxM; 
    ImgTxPixStatxSO <= ImgTxPixStatxS;
    ImgTxPixDataxDO <= ImgTxPixDataxD;
    ImgRxPixRdyxMO  <= ImgRxPixRdyxM;

    ----------------------------------------------------------------------------
    -- MemoryBus Logic
    ----------------------------------------------------------------------------
    px_cmd_en_o    <= '0';
    px_cmd_instr_o <= (others => '0');
    px_cmd_bl_o    <= (others => '0');
    px_cmd_addr_o  <= (others => '0');

    px_rd_en_o     <= '0';
    px_wr_en_o     <= '0';
    px_wr_data_o   <= (others => '0');

    ----------------------------------------------------------------------------
    -- GPIO Logic
    ----------------------------------------------------------------------------
    CustIoCtrlxMO    <= (others => '0');
    CustIoDataOutxDo <= (others => '0');

    ----------------------------------------------------------------------------
    -- Debug Logic
    ----------------------------------------------------------------------------
    NoDbgGen: if (cDEBUG_ENABLE = False) generate
        DbgLogicPortxTO <= (others => '0');
    end generate;

    DbgGen: if (cDEBUG_ENABLE = True) generate
        signal DbgLogicPortxT : std_logic_vector(255 downto 0);
    begin
        DbgProc: process (SysClkxCI)
            variable i: natural;
        begin
            if rising_edge(SysClkxCI) then
                i := 0;
                DbgLogicPortxT(255 downto i) <= (others => '0');
            end if;
        end process;

        DbgLogicPortxTO <= DbgLogicPortxT(cDEBUG_PORT_WIDTH-1 downto 0);
    end generate;

end rtl;
