-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : SystemBus Interface for OpenCamera USER design
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-20  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

use work.CustomPkg.all;
use work.UserPkg.all;

entity UsrSysBusIf is
    port (
        -- Custom Module Infrastructure Signals
        SysClkxCI        : in  std_logic;                                       -- Custom Module Clock 100 MHz
        SysRstxRI        : in  std_logic;                                       -- Custom Module Reset
    
        -- SystemBus Control Interface  
        SysReqxMI        : in  std_logic;                                       -- SystemBus Cycle Request
        SysAckxMO        : out std_logic;                                       -- SystemBus Cycle Acknowledge
        SysErrxMO        : out std_logic;                                       -- SystemBus Cycle Error
        SysWrEnxMI       : in  std_logic;                                       -- SystemBus Write Enable
        SysAddrxAI       : in  std_logic_vector(cSYS_ADDR_WIDTH-1 downto 0);    -- SystemBus Address
        SysWrDataxDI     : in  std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Write Data
        SysRdDataxDO     : out std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);    -- SystemBus Read Data
    
        -- SystemBus Control/Status Port    
        UsrCtrlRegxMO    : out tUSR_CTRL_REG;                                   -- SystemBus User Control Registers
        UsrStatRegxMI    : in  tUSR_STAT_REG;                                   -- SystemBus User Status Registers
    
        -- SystemBus Memory Access Port 
        SysMemReqxMO     : out std_logic;                                       -- SystemBus Memory Cycle Request
        SysMemAckxMI     : in  std_logic;                                       -- SystemBus Memory Cycle Acknowledge
        SysMemErrxMI     : in  std_logic;                                       -- SystemBus Memory Cycle Error
        SysMemRdDataxDI  : in  std_logic_vector(cSYS_DATA_WIDTH-1 downto 0));   -- SystemBus Memory Read Data
end UsrSysBusIf;

architecture rtl of UsrSysBusIf is

    signal SysAckxM      : std_logic;
    signal SysErrxM      : std_logic;
    signal SysRegReqxM   : std_logic;
    signal SysMemReqxM   : std_logic;
    signal SysMemAddrxM  : std_logic;
    signal SysRdDataxD   : std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);
    signal UsrCtrlRegxM  : tUSR_CTRL_REG;

begin

    ---------------------------------------------------------------------------
    -- SystemBus Interface and Control/Status Registers
    ---------------------------------------------------------------------------
    SysBusProc: process (SysClkxCI, SysRstxRI)
    begin
        if (SysRstxRI = '1') then
            SysAckxM      <= '0';  
            SysErrxM      <= '0';  
            SysRdDataxD   <= (others => '0');  
            UsrCtrlRegxM  <= cUSR_CTRL_DEFAULT;
        elsif rising_edge(SysClkxCI) then
            if (SysRegReqxM = '1') then
                -- Deassert all register read bits as default.
                SysRdDataxD  <= (others => '0');  

                case SysAddrxAI(4 downto 2) is
                    when "000" =>  -- 0x00 
                        -- Required SystemBus status register on address 0.
                        -- Register must be present - DO NOT MODIFY
                        SysRdDataxD(9 downto 0)   <= std_logic_vector(to_unsigned(cCUSTOM_DESIGN_ID, 10));
                        SysRdDataxD(20 downto 10) <= std_logic_vector(to_unsigned(cVERSION_NUMBER, 11));
                        SysRdDataxD(31 downto 21) <= std_logic_vector(to_unsigned(cUSER_DESIGN_ID, 11));

                    when "001" =>  -- 0x04 
                        -- Masked Control Register: Bit0..15=CtrlReg, Bit 16..31=Mask
                        if (SysWrEnxMI = '1') then
                            if (SysWrDataxDI(16) = '1') then
                                UsrCtrlRegxM.CtrlReg0 <= SysWrDataxDI(0);   
                            end if;
                        else
                            SysRdDataxD(0) <= UsrCtrlRegxM.CtrlReg0;
                        end if;

                    when "010" =>  -- 0x08 
                        -- Non-masked Control Register
                        if (SysWrEnxMI = '1') then
                            UsrCtrlRegxM.CtrlReg1 <= SysWrDataxDI(3 downto 0);   
                        else
                            SysRdDataxD(3 downto 0) <= UsrCtrlRegxM.CtrlReg1;
                        end if;

                    when "011" =>  -- 0x0C - unused
                        -- Read-only status register
                        SysRdDataxD(0)          <= UsrStatRegxMI.StatReg0;
                        SysRdDataxD(7 downto 4) <= UsrStatRegxMI.StatReg1;

                    when "100" =>  -- 0x10 - unused
                    when "101" =>  -- 0x14 - unused
                    when "110" =>  -- 0x18 - unused
                    when "111" =>  -- 0x1C - unused
                    when others => NULL;
                end case;
            end if;

            -- Generate SystemBus cycle acknowledge.
            if (SysRegReqxM = '1') then
                SysAckxM <= '1';
                SysErrxM <= '0';
            elsif (SysMemReqxM = '1') then
                SysAckxM <= SysMemAckxMI;
                SysErrxM <= SysMemErrxMI;

                if (SysMemAckxMI = '1') then
                    SysRdDataxD <= SysMemRdDataxDI;
                end if;
            elsif (SysReqxMI = '0') then
                SysAckxM <= '0';
                SysErrxM <= '0';
            end if;
        end if;
    end process;

    -- Decode System memory address spaces
    SysMemAddrxM <= '1' when ((SysAddrxAI(cSYS_ADDR_WIDTH-1 downto cSYS_ADDR_WIDTH-3) >= cSYSMEM_START_ADDR) and (cSYSMEN_ENABLE = True)) else '0';

    -- SystemBus device select (Register/memory space)
    SysRegReqxM <= '1' when ((SysReqxMI = '1') and (SysMemAddrxM = '0') and (SysAckxM = '0')) else '0';
    SysMemReqxM <= '1' when ((SysReqxMI = '1') and (SysMemAddrxM = '1') and (SysAckxM = '0') and (SysErrxM = '0')) else '0';

    -- Assign Internally used output ports
    SysMemReqxMO  <= SysMemReqxM;
    SysAckxMO     <= SysAckxM;
    SysErrxMO     <= SysErrxM;
    SysRdDataxDO  <= SysRdDataxD;
    UsrCtrlRegxMO <= UsrCtrlRegxM;

end rtl;
