-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : OpenCamera Custom Module Package file
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2013-05-20  1.0      Manfred  Created
--              2015-04-16  2.0      a.kramer Changed for GigEpro
--              2017-03-02  3.0      Manfred  CorSight2 Modifications
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package CustomPkg is

    ----------------------------------------------------------------------------
    -- Custom Module Design-IDs - USER DEFINABLE
    ----------------------------------------------------------------------------
    -- Add new Design-IDs here. New Design-IDs must be > 1023
    constant cNEW_DESIGN_ID     : integer := 1024;      -- New Design ID

    -- Enable/disable the Custom Module Bypass
    constant cCUSTOM_BYPASS_EN  : boolean := True;      -- Custom Module Bypass Select

    ----------------------------------------------------------------------------
    -- Custom Module Design-IDs - DO NOT MODIFY
    ----------------------------------------------------------------------------
    constant cCUSTOM_DESIGN_ID  : integer := 47;        -- Custom Module Design ID
    constant cDEFAULT_DESIGN_ID : integer := 0;         -- Default Design ID
    constant cUSER_DESIGN_ID    : integer := 1;         -- User Design ID
    constant cLUT_DESIGN_ID     : integer := 2;         -- LUT Design ID
    constant cMEMTEST_DESIGN_ID : integer := 3;         -- Memory Test Design ID
    constant cDIFFPIC_DESIGN_ID : integer := 4;         -- DiffPic Design ID
    constant cCANNY_DESIGN_ID   : integer := 5;         -- Canny Filter Design ID
    constant cBLOCK_DEMO_ID     : integer := 6;         -- BlockDemo Design ID
    constant cSCALED_DESIGN_ID  : integer := 7;         -- Image Scaler Design ID
--  constant cRSVD_DESIGN_ID    : integer := 8..1023;   -- Reserved Design IDs (DO NOT USE!)

    ----------------------------------------------------------------------------
    -- SystemBus Definitions - DO NOT MODIFY
    ----------------------------------------------------------------------------
    constant cSYS_ADDR_WIDTH    : integer := 17;                            -- Width of SystemBus Address Bus
    constant cSYS_DATA_WIDTH    : integer := 32;                            -- Width of SystemBus Data Bus
    constant cSYS_GPIO_WIDTH    : integer := 6;                             -- Width of GPIO Bus

    subtype tSYS_ADDR_BUS is std_logic_vector(cSYS_ADDR_WIDTH-1 downto 0);  -- SystemBus Address Bus Definition
    subtype tSYS_DATA_BUS is std_logic_vector(cSYS_DATA_WIDTH-1 downto 0);  -- SystemBus Data Bus Definition
    subtype tSYS_GPIO_BUS is std_logic_vector(cSYS_GPIO_WIDTH-1 downto 0);  -- GPIO Bus Definition

    type tSYS_ADDR_ARRAY  is array (natural range <>) of tSYS_ADDR_BUS;     -- SystemBus Address Bus Array Definition
    type tSYS_DATA_ARRAY  is array (natural range <>) of tSYS_DATA_BUS;     -- SystemBus Data Bus Array Definition
    type tSYS_GPIO_ARRAY  is array (natural range <>) of tSYS_GPIO_BUS;     -- GPIO Bus Array Definition

    ----------------------------------------------------------------------------
    -- ImageBus Definitions - DO NOT MODIFY
    ----------------------------------------------------------------------------
    constant cIMG_FLOW_ENABLE   : boolean := False;                         -- ImageBus Flow Control Enable
    constant cIMG_STAT_WIDTH    : integer := 3;                             -- ImageBus Status Bus Width
    constant cIMG_DATA_WIDTH    : integer := 36;                            -- ImageBus Data Bus Width

    subtype tIMG_STAT_BUS is std_logic_vector(cIMG_STAT_WIDTH-1 downto 0);  -- ImageBus Status Bus Definition
    subtype tIMG_DATA_BUS is std_logic_vector(cIMG_DATA_WIDTH-1 downto 0);  -- ImageBus Data Bus Definition

    type tIMG_STAT_ARRAY  is array (natural range <>) of tIMG_STAT_BUS;     -- ImageBus Status Bus Array Definition
    type tIMG_DATA_ARRAY  is array (natural range <>) of tIMG_DATA_BUS;     -- ImageBus Data Bus Array Definition

    constant cIMG_STAT_NONE     : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0) := "000";    -- No status
    constant cIMG_STAT_SOF      : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0) := "001";    -- Start Of Frame
    constant cIMG_STAT_SOSF     : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0) := "010";    -- Start Of Single-Line Frame (Linescan)
    constant cIMG_STAT_SOL      : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0) := "100";    -- Start Of Line
    constant cIMG_STAT_SOLL     : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0) := "101";    -- Start Of Last Line
    constant cIMG_STAT_EOL      : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0) := "110";    -- End Of Line
    constant cIMG_STAT_ERR      : std_logic_vector(cIMG_STAT_WIDTH-1 downto 0) := "111";    -- Data Error

    ----------------------------------------------------------------------------
    -- MemoryBus Definitions - DO NOT MODIFY
    ----------------------------------------------------------------------------
    constant cMEM_CMD_WIDTH     : integer := 3;                             -- Width of MemoryBus Command Bus
    constant cMEM_BLEN_WIDTH    : integer := 6;                             -- Width of MemoryBus Burst Length Bus
    constant cMEM_ADDR_WIDTH    : integer := 30;                            -- Width of MemoryBus Address Bus
    constant cMEM_DATA_WIDTH    : integer := 32;                            -- Width of MemoryBus Data Bus

    subtype tMEM_CMD_BUS  is std_logic_vector(cMEM_CMD_WIDTH -1 downto 0);  -- MemoryBus Command Bus Definition
    subtype tMEM_BLEN_BUS is std_logic_vector(cMEM_BLEN_WIDTH-1 downto 0);  -- MemoryBus Burst Length Bus Definition
    subtype tMEM_ADDR_BUS is std_logic_vector(cMEM_ADDR_WIDTH-1 downto 0);  -- MemoryBus Address Bus Definition
    subtype tMEM_DATA_BUS is std_logic_vector(cMEM_DATA_WIDTH-1 downto 0);  -- MemoryBus Data Bus Definition

    type tMEM_CMD_ARRAY   is array (natural range <>) of tMEM_CMD_BUS;      -- MemoryBus Command Bus Array Definition
    type tMEM_BLEN_ARRAY  is array (natural range <>) of tMEM_BLEN_BUS;     -- MemoryBus Burst Length Bus Array Definition
    type tMEM_ADDR_ARRAY  is array (natural range <>) of tMEM_ADDR_BUS;     -- MemoryBus Address Bus Array Definition
    type tMEM_DATA_ARRAY  is array (natural range <>) of tMEM_DATA_BUS;     -- MemoryBus Data Bus Array Definition

    constant cMEM_WRITE_COMMAND : std_logic_vector(cMEM_CMD_WIDTH-1 downto 0) := "000";     -- Memory Write Command
    constant cMEM_READ_COMMAND  : std_logic_vector(cMEM_CMD_WIDTH-1 downto 0) := "001";     -- Memory Read Command

end CustomPkg;
