-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Debug Module for OpenCamera design
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-11-23  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

entity DebugModule is
    generic (
        gLOGIC_ENABLE     : boolean := False;
        gCHIPSCOPE_ENABLE : boolean := True;
        gCSP_ICON_ENABLE  : boolean := True;
        gDEBUG_PORT_WIDTH : integer := 256);
    port (
        DbgClkxCI         : in  std_logic;                                          -- Debug Clock
        DbgLogicPortxTI   : in  std_logic_vector(gDEBUG_PORT_WIDTH-1 downto 0));    -- Debug Port
end DebugModule;

architecture structural of DebugModule is

    ---------------------------------------------------------------------------
    -- Component Declarations
    ---------------------------------------------------------------------------
    -- Chipscope Controller
    component Csp_Icon
        port (
            CONTROL0 : inout std_logic_vector(35 downto 0));
    end component;

    -- 128-bit ChipScope Logic Analyzer
    component Csp_Ila_1x128x1024
        port (
            CONTROL : inout std_logic_vector(35 downto 0);
            CLK     : in    std_logic;
            TRIG0   : in    std_logic_vector(127 downto 0));
    end component;

    -- 256-bit ChipScope Logic Analyzer
    component Csp_Ila_1x256x1024
        port (
            CONTROL  : inout std_logic_vector(35 downto 0);
            CLK      : in    std_logic;
            TRIG0    : in    std_logic_vector(255 downto 0));
    end component;

    -- Logic Analyser, 1 probe of 128 signal with 1024 sample depth.
    component LogicAnalyzer1x128x1024
    port (
        clk     : in std_logic;
        probe0  : in std_logic_vector(127 downto 0));
    end component;

    -- Logic Analyser, 1 probe of 256 signal with 1024 sample depth.
    component LogicAnalyzer1x256x1024
    port (
        clk     : in std_logic;
        probe0  : in std_logic_vector(255 downto 0));
    end component;

    ---------------------------------------------------------------------------
    -- Signal Declarations
    ---------------------------------------------------------------------------
    signal DbgLogicPortxT : std_logic_vector(255 downto 0);

begin

    ---------------------------------------------------------------------------
    -- Re-assign the incoming debug port to fixed size signal
    ---------------------------------------------------------------------------
    DbgProc: process(DbgLogicPortxTI)
    begin
        DbgLogicPortxT <= (others => '0');
        DbgLogicPortxT(gDEBUG_PORT_WIDTH-1 downto 0) <= DbgLogicPortxTI;
    end process;

    ---------------------------------------------------------------------------
     -- Instantiate ChipScopePro Analyzer
    ---------------------------------------------------------------------------
    CspGen: if (gCHIPSCOPE_ENABLE = True) generate
        signal CspControlxM   : std_logic_vector(35 downto 0);
    begin
        -- Instantiate ChipScopePro ICON
        IconGen: if (gCSP_ICON_ENABLE = True) generate
            IconInst: Csp_Icon
            port map (
                CONTROL0 => CspControlxM);
        end generate;

        -- Instantiate ChipScopePro 128-bit ILA
        CspIla128Gen: if (gDEBUG_PORT_WIDTH <= 128) generate
            CspIla128Inst: Csp_Ila_1x128x1024
            port map (
                CONTROL  => CspControlxM,
                CLK      => DbgClkxCI,
                TRIG0    => DbgLogicPortxT(127 downto 0));
        end generate;

        -- Instantiate ChipScopePro 256-bit ILA
        CspIla256Gen: if (gDEBUG_PORT_WIDTH > 128) generate
            CspIla256Inst: Csp_Ila_1x256x1024
            port map (
                CONTROL  => CspControlxM,
                CLK      => DbgClkxCI,
                TRIG0    => DbgLogicPortxT(255 downto 0));
        end generate;
    end generate;

    ---------------------------------------------------------------------------
     -- Instantiate Vivado LogicAnalyzer
    ---------------------------------------------------------------------------
    LogGen: if (gLOGIC_ENABLE = True) generate
        -- Instantiate LogicAnalyzer 128-bit ILA
        LogIla128Gen: if (gDEBUG_PORT_WIDTH <= 128) generate
            LogIla128Inst: LogicAnalyzer1x128x1024
            port map (
                clk    => DbgClkxCI,
                probe0 => DbgLogicPortxT(127 downto 0));
        end generate;

        -- Instantiate LogicAnalyzer 256-bit ILA
        LogIla256Gen: if (gDEBUG_PORT_WIDTH > 128) generate
            LogIla256Inst: LogicAnalyzer1x256x1024
            port map (
                clk    => DbgClkxCI,
                probe0 => DbgLogicPortxT(255 downto 0));
        end generate;
    end generate;

end structural;
